<?php
/**
 * Created by PhpStorm.
 * User: PC
 * Date: 26/04/2018
 * Time: 16:01
 */

namespace App\Controller\Rest;


use Doctrine\ORM\Query;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Address;
use App\Entity\CardFamily;
use App\Entity\Customer;
use App\Entity\CustomerPdv;
use App\Entity\Size;
use App\Entity\webSiteUsers;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Intl\Tests\Data\Bundle\Writer\JsonBundleWriterTest;
use Symfony\Component\Validator\Constraints\DateTime;
use Nelmio\ApiDocBundle\Annotation\ApiDoc;
use ElephantIO\Client;
use ElephantIO\Engine\SocketIO\Version1X;


class AddressController extends FOSRestController
{
    /**
     * @Rest\Post("/address")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="streetName", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="streetNumber", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="postalCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="city", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="complementAddress", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="address", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="inputCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="building", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="staircase", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="floor", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="intercom", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="door", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="country", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="customer", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="principal", nullable=true,allowBlank=true,strict=true)
     */
    public function addAddress(Request $request, ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $customer = $this->getDoctrine()->getRepository("App:CustomerPdv")->find($paramFetcher->get('customer'));
        $address = new Address();
        $address->setStreetName($paramFetcher->get('streetName'));
        $address->setStreetNumber($paramFetcher->get('streetNumber'));
        $address->setPostalCode($paramFetcher->get('postalCode'));
        $address->setCity($paramFetcher->get('city'));
        $address->setComplementAddress($paramFetcher->get('complementAddress'));
        $address->setInputCode($paramFetcher->get('inputCode'));
        $address->setBuilding($paramFetcher->get('building'));
        $address->setStaircase($paramFetcher->get('staircase'));
        $address->setFloor($paramFetcher->get('floor'));
        $address->setIntercom($paramFetcher->get('intercom'));
        $address->setDoor($paramFetcher->get('door'));
        $address->setCountry($paramFetcher->get('country'));
        $address->setActive(1);
        if($paramFetcher->get('principal')==true){
            $address->setPrincipal(1);
            $addresses=$customer->getAddresses();
            foreach ($addresses as $a){
                $a->setPrincipal(0);
                $em->persist($a);
            }
        }
        $address->setCustomer($customer);
        $em->persist($address);
        $em->flush();
        return new JsonResponse([
            'id' => $address->getId(),
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/address/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="streetName", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="streetNumber", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="postalCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="city", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="complementAddress", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="address", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="inputCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="building", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="staircase", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="floor", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="intercom", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="door", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="principal", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="country", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="id", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAddress(Request $request, $id, ParamFetcher $paramFetcher)
    {

        $em = $this->getDoctrine()->getManager();
        $address = $this->getDoctrine()->getRepository("App:Address")->find(intval($id));
        $address->setStreetName($paramFetcher->get('streetName'));
        $address->setStreetNumber($paramFetcher->get('streetNumber'));
        $address->setPostalCode($paramFetcher->get('postalCode'));
        $address->setCity($paramFetcher->get('city'));
        $address->setComplementAddress($paramFetcher->get('complementAddress'));
        $address->setInputCode($paramFetcher->get('inputCode'));
        $address->setBuilding($paramFetcher->get('building'));
        $address->setStaircase($paramFetcher->get('staircase'));
        $address->setFloor($paramFetcher->get('floor'));
        $address->setIntercom($paramFetcher->get('intercom'));
        $address->setDoor($paramFetcher->get('door'));
        $address->setCountry($paramFetcher->get('country'));
        if($paramFetcher->get('principal')==true){
            $address->setPrincipal(1);
            $customer = $em->getRepository('App:CustomerPdv')
                ->createQueryBuilder('c')
                ->InnerJoin('c.addresses', 'a')
                ->where('a.id = :address ')
                ->setParameter('address', $id)
                ->setMaxResults(1)
                ->getQuery()
                ->getOneOrNullResult();
            $addresses=$customer->getAddresses();
            foreach ($addresses as $a){
                $a->setPrincipal(0);
                $em->persist($a);
            }
        }
        $address->setPrincipal($paramFetcher->get('principal'));
        $em->persist($address);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);

    }

    /**
     * @Rest\Get("/address/{id}")
     */
    public function getAddress(Request $request, $id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb1 = $em->createQueryBuilder();
        $address = $qb1->select("a")
            ->from("App:Address", "a")
            ->innerJoin('a.customer', 'cp')
            ->where('a.id = :add')
            ->setParameter('add', $id)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($address, 'json');
        return new JsonResponse(json_decode($data));


    }

    /**
     * @Rest\Delete("/address/delete/{id}")
     */
    public function deleteAddress($id)
    {
        $em = $this->getDoctrine()->getManager();
        $a = $em->getRepository('App:Address')->find($id);
        $em->remove($a);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/address/order/{id}")
     */

    public function getAddressByOrder(Request $request, $id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb1 = $em->createQueryBuilder();
        $address = $qb1->select("a.streetName,a.streetNumber,a.city,
        a.postalCode,a.complementAddress,a.inputCode,a.building,a.staircase,a.floor,a.intercom,a.door,a.country")
            ->from("App:Command", "o")
            ->innerJoin('o.address', 'a');
        if ($this->getUser()->getPdv()) {
            $address = $address->innerJoin('o.orderTypePdv', 'mop')
                ->innerJoin('mop.pdv', 'p');
        }
        $address = $address->where('o.id = :order')->setParameter('order', $id);
        if ($this->getUser()->getPdv()) {
            $address = $address->andWhere('p.id =:pdv')->setParameter('pdv', $this->getUser()->getPdv());
        }
        $address = $address->setMaxResults(1)->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($address, 'json');
        return new JsonResponse(json_decode($data));
    }
}



