<?php
/**
 * Created by PhpStorm.
 * User: PC
 * Date: 11/06/2018
 * Time: 12:29
 */

namespace App\Controller\Rest;

use App\Entity\Bench;
use App\Entity\Table;
use Doctrine\ORM\Query\ResultSetMapping;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Request\ParamFetcher;
use Doctrine\ORM\Tools\Pagination\Paginator;

class BenchController extends FOSRestController
{
    /**
     * @Rest\Post("/bench")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="numTable", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="nbrPerson", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="tablesBinding", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     */
    public function addAction(Request $request, ParamFetcher $paramFetcher)
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $tablsBinding = $paramFetcher->get('tablesBinding');
        $em = $this->getDoctrine()->getManager();
        $pdv = $em->getRepository('App:Pdv')->find($this->getUser()->getPdv());
        $bench = new Bench();
        $bench->setNum($paramFetcher->get('numTable'));
        $bench->setNbrPerson($paramFetcher->get('nbrPerson'));
        $bench->setComment($paramFetcher->get('comment'));
        if (is_array($tablsBinding) && count($tablsBinding) > 0) {
            foreach ($tablsBinding as $val){
                $ben = $em->getRepository('App:Bench')->find($val);
                if ($ben) {
                    $bench->addBench($ben);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        }
        $bench->setPdv($pdv);
        $em->persist($bench);
        $em->flush();
        return new JsonResponse([
            'id' => $bench->getId(),
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/bench/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="numTable", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="nbrPerson", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="tablesBinding", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAction(Request $request, ParamFetcher $paramFetcher,$id)
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $tablsBinding = $paramFetcher->get('tablesBinding');
        $em = $this->getDoctrine()->getManager();
        $bench = $em->getRepository('App:Bench')->find($id);
        $bench->setNum($paramFetcher->get('numTable'));
        $bench->setNbrPerson($paramFetcher->get('nbrPerson'));
        $bench->setComment($paramFetcher->get('comment'));
        $bench->setBenchs([]);
        if (is_array($tablsBinding) && count($tablsBinding) > 0) {
            foreach ($tablsBinding as $val) {
                $ben = $em->getRepository('App:Bench')->find($val);
                if ($ben) {
                    $bench->addBench($ben);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }

            }
        }
        $em->persist($bench);
        $em->flush();
        return new JsonResponse([
            'id' => $bench->getId(),
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/bench/all")
     */
    public function getBenchs()
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $pdv = $this->getUser()->getPdv();
        $qb = $em->createQueryBuilder();
        $address = $qb->select("b")
            ->from("App:Bench", "b")
            ->innerJoin('b.pdv', 'p')
            ->where('p.id = :pdv')
            ->setParameter('pdv', $pdv)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($address, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/bench/all/select")
     */
    public function getBenchsForSelect()
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $pdv = $this->getUser()->getPdv();
        $qb = $em->createQueryBuilder();
        $address = $qb->select("b.id,b.num as text")
            ->from("App:Bench", "b")
            ->innerJoin('b.pdv', 'p')
            ->where('p.id = :pdv')
            ->setParameter('pdv', $pdv)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($address, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/bench/{id}")
     */
    public function getBenchById($id)
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $pdv = $this->getUser()->getPdv();
        $qb = $em->createQueryBuilder();
        $address = $qb->select("b")
            ->from("App:Bench", "b")
            ->innerJoin('b.pdv', 'p')
            ->where('p.id = :pdv')
            ->andWhere('b.id = :bench')
            ->setParameter('pdv', $pdv)
            ->setParameter('bench', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($address, 'json');
        return new JsonResponse(json_decode($data));


    }

    /**
     * @Rest\Delete("/bench/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = $em->getRepository('App:Bench')->find($id);
        $em->remove($p);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Post("/tables")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="dateHour", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="nbrPerson", nullable=false, allowBlank=false, strict=false)
     * @RequestParam(name="pdv", nullable=false, allowBlank=false, strict=false)
     */
    function getTables(ParamFetcher $paramFetcher){

        $datetime =  \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('dateHour'));
        $date=$datetime->format('Y-m-d');
        $Heur=$datetime->format('H');
        $d=new \DateTime();
        $pdv=$paramFetcher->get('pdv');
        if($datetime>=$d && $Heur>=7 && $Heur<24){
            $service=0;
            if(7<=$Heur && $Heur<16){
                $service=1;
                $servicelib="matin";
            }elseif (16<=$Heur && $Heur<20) {
                $service=2;
                $servicelib="soir 1";
            }elseif (20<=$Heur && $Heur<24) {
                $service=3;
                $servicelib="soir 2";
            }
            if($service>0){
                $em = $this->getDoctrine()->getManager();
                $query = "SELECT A.*, (select GROUP_CONCAT(B.num) from bench B, bench_binding C 
                          where (B.id = C.benchSec_id AND C.benchInit_id=A.id) or (B.id = C.benchInit_id AND C.benchSec_id=A.id)) as liaison 
                          FROM bench A WHERE A.pdv_id =".$pdv." AND A.id NOT IN ( SELECT rt.table_id FROM reservation_table AS rt, reservation AS r
                          WHERE r.id=rt.reservation_id AND DATE(r.datetime)= '".$date."' AND r.service_id='".$service."')";
                $statement = $em->getConnection()->prepare($query);
                $statement->execute();
                if($statement->rowCount()>0) {
                    $result['tables'] = $statement->fetchAll();
                    $result['service'] = $servicelib;
                    $serializer = $this->get('serializer');
                    $data = $serializer->serialize($result, 'json');
                    return new JsonResponse(json_decode($data));
                }else{
                    return new JsonResponse(array('error'=>true,'code'=>2));
                }

            }else{
                return new JsonResponse(array('error'=>true,'code'=>1));
            }

        }else{
            return new JsonResponse(array('error'=>true,'code'=>3));
        }
    }
}