<?php

namespace App\Controller\Rest;

use App\Service\SendEvent;
use Doctrine\Common\Collections\ArrayCollection;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Card;
use App\Entity\CardFamily;
use App\Entity\CardMenuCategory;
use App\Entity\Ingredient;
use App\Entity\IngredientCatFamSize;
use App\Entity\Pdv;
use App\Entity\Size;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Contact;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;
use ElephantIO\Client;
use ElephantIO\Engine\SocketIO\Version1X;

class CardController extends FOSRestController
{
    /**
     * @Rest\Put("/carte/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="startedValidity", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="stoppedValidity", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="nbrOffersAuthorized", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="nbrOffersAuthorizedSite", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="displayCode", nullable=true,allowBlank=false,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="menuCategories", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="deliveryCharges", nullable=true,allowBlank=false,strict=true)
     * @RequestParam(name="managementFees", nullable=true,allowBlank=false,strict=true)
     * @RequestParam(name="deliveryChargesLabel", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="managementFeesLabel", nullable=true,allowBlank=true,strict=true)
     */
    public function updateCarte(Request $request, $id, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        if(in_array($this->getUser()->getProfil()->getId(),[4,3])){
            if($this->getUser()->getPdv()!=intval($id)){
                return new JsonResponse(null,403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $carte = $this->getDoctrine()->getRepository("App:Card")->find($id);
        $debut = new \DateTime($paramFetcher->get('startedValidity'));
        $fin = new \DateTime($paramFetcher->get('stoppedValidity'));
        $carte->setStartedValidity($debut);
        $carte->setStoppedValidity($fin);
        $carte->setNbrOffersAuthorized($paramFetcher->get('nbrOffersAuthorized'));
        $carte->setNbrOffersAuthorizedSite($paramFetcher->get('nbrOffersAuthorizedSite'));
        $carte->setDisplayCode($paramFetcher->get('displayCode'));
        $carte->setComment($paramFetcher->get('comment'));
        $carte->setDeliveryCharges(floatval($paramFetcher->get('deliveryCharges')));
        $carte->setManagementFees(floatval($paramFetcher->get('managementFees')));
        $carte->setDeliveryChargesLabel($paramFetcher->get('deliveryChargesLabel'));
        $carte->setManagementFeesLabel($paramFetcher->get('managementFeesLabel'));
        foreach ($paramFetcher->get('families') as $value) {
            $family = $em->getRepository('App:Family')->find($value);
            $familyCard = $em->getRepository('App:CardFamily')->findOneBy(array("card" => $carte, "family" => $family));
            if (is_null($familyCard)) {
                $familyCard = new CardFamily();
                $familyCard->setFamily($family);
                $familyCard->setCard($carte);
                $em->persist($familyCard);
            }
        }
        foreach ($carte->getFamilies() as $value) {
            if (!in_array($value->getFamily()->getId(), $paramFetcher->get('families'))) {
                $em->remove($value);
            }
        }
        foreach ($paramFetcher->get('menuCategories') as $value) {
            $menuCategory = $em->getRepository('App:MenuCategory')->find($value);
            $cardMenuCategory = $em->getRepository('App:CardMenuCategory')->findOneBy(array("card" => $carte, "menuCategory" => $menuCategory));
            if (is_null($cardMenuCategory)) {
                $cardMenuCategory = new CardMenuCategory();
                $cardMenuCategory->setMenuCategory($menuCategory);
                $cardMenuCategory->setCard($carte);
                $em->persist($cardMenuCategory);
            }
        }
        foreach ($carte->getMenuCategories() as $value) {
            if (!in_array($value->getMenuCategory()->getId(), $paramFetcher->get('menuCategories'))) {
                $em->remove($value);
            }
        }
        $em->persist($carte);
        $em->flush();
        $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'carte', 'id' => $id]);
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/carte/{id}")
     */
    public function getCarte($id)
    {
        if(in_array($this->getUser()->getProfil()->getId(),[4,3])){
            if($this->getUser()->getPdv()!=intval($id)){
                return new JsonResponse(null,403);
            }
        }
        $carte = $this->getDoctrine()->getRepository("App:Card")->find($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($carte, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/carte/originCarte/{idOriginCarte}/newCarte/{idNewCarte}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     */
    public function copyCarte(Request $request, $idOriginCarte, $idNewCarte, ParamFetcher $paramFetcher)
    {
        $data = $request->request->all();
        $em = $this->getDoctrine()->getManager();
        $originCarte = $this->getDoctrine()->getRepository("App:Card")->find($idOriginCarte);
        $newCarte = $this->getDoctrine()->getRepository("App:Card")->find($idNewCarte);

        /*$newCarte->setStartedValidity($originCarte->getStartedValidity());
          $newCarte->setStoppedValidity($originCarte->getStoppedValidity());
          $newCarte->setNbrOffersAuthorized($originCarte->getNbrOffersAuthorized());
          $newCarte->setDisplayCode($originCarte->getDisplayCode());
          $newCarte->setComment($originCarte->getComment());*/
        /*
                    $newCarte->setFamilies([]);

                    foreach ($originCarte->getFamilies() as $value){
                            $familyCard=new CardFamily();
                            $familyCard->setFamily($value->getFamily());
                            $familyCard->setCard($newCarte);
                            $em->persist($familyCard);
                    }*/

        /*foreach ($newCarte->getFamilies() as $value){
            if (!in_array( $value->getFamily()->getId(), $originCarte->getFamilies())) {
                $em->remove($value);
            }
        }*/

        /*foreach ($originCarte->getMenuCategories() as $value) {
            $cardMenuCategory = new CardMenuCategory();
            $cardMenuCategory->setMenuCategory($value->getMenuCategory());
            $cardMenuCategory->setCard($newCarte);
            $em->persist($cardMenuCategory);

        }*/
        /*
                $ingredients = $this->getDoctrine()->getRepository("App:Ingredient")->findBy(array("card"=>$originCarte));


                foreach ($ingredients as $value) {

                    $families=new ArrayCollection();
                    $ingredient=new Ingredient();
                    foreach ($value->getFamilies() as $family) {
                        $cardFamily = $this->getFamiliesIngredientByCard($idNewCarte,$family->getFamily()->getLabel());
                        $families[]=$cardFamily;
                    }
                    $ingredient->setName($value->getName());
                    $ingredient->setDescription($value->getDescription());
                    $ingredient->setPrice($value->getPrice());
                    $ingredient->setAvailable($value->getAvailable());
                    $ingredient->setIngredientCategory($value->getIngredientCategory());
                    $ip = $request->getClientIp();
                    $ingredient->setCreatedIp($ip);
                    $ingredient->setCreatedUser('0');
                    $ingredient->setUpdatedIp('');
                    $ingredient->setUpdatedUser('0');
                    $ingredient->setFamilies($families);
                    $ingredient->setCard($newCarte);
                    $em->persist($ingredient);
                }*/

////////////////*****************************************************////////////////////
        /*
                $sizes = $this->getDoctrine()->getRepository("App:Size")->findBy(array("card"=>$originCarte));
                foreach ($sizes as $value) {

                    $families=new ArrayCollection();
                    $size=new Size();
                    foreach ($value->getFamilies() as $family) {
                        $cardFamily = $this->getFamiliesIngredientByCard($idNewCarte,$family->getFamily()->getLabel());
                        $families[]=$cardFamily;
                    }
                    $size->setLevel($value->getLevel());
                    $size->setLabel($value->getLabel());
                    $size->setFamilies($families);
                    $size->setCard($newCarte);
                    $em->persist($size);
                }*/
/////////////////////******************************************//////////////////////////////////////
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredientCatFamSize = $qb->select("icfs")
            ->from("App:IngredientCatFamSize", "icfs")
            ->innerJoin('icfs.cardFamily', 'cf')
            ->innerJoin('cf.card', 'c')
            ->where('c.id = :idCard')
            ->setParameter('idCard', $idOriginCarte)
            ->getQuery()->getResult();

        foreach ($ingredientCatFamSize as $value) {
            $cardFamily = $this->getFamiliesIngredientByCard($idNewCarte, $value->getCardFamily()->getFamily()->getLabel());
            $size = $this->getSizeByCard($idNewCarte, $value->getSize()->getLabel());
            $ingredientCatFamSize = new IngredientCatFamSize();
            $ingredientCatFamSize->setCardFamily($cardFamily);
            $ingredientCatFamSize->setIngredientCategory($value->getIngredientCategory());
            $ingredientCatFamSize->setSize($size);
            $ingredientCatFamSize->setPrice($value->getPrice());
            $em->persist($ingredientCatFamSize);
        }

        $em->persist($newCarte);
        $em->flush();
        /* $client = new Client(new Version1X('//127.0.0.1:1337'));
         $client->initialize();
         $client->emit('notif', ['type' => 'persist','entity' => 'carte', 'id' => $id]);
         $client->close();*/
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);

    }

    /**
     * @Rest\Get("/carte/miniInfo/{idCarte}")
     */
    public function getElementsByFamilyCard($idCarte)
    {
        if(in_array($this->getUser()->getProfil()->getId(),[4,3])){
            if($this->getUser()->getPdv()!=intval($idCarte)){
                return new JsonResponse(null,403);
            }
        }
        if($this->getUser()->getPdv() && $this->getUser()->getPdv()!=$idCarte){
            return new JsonResponse('',403);
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $card = $qb->select("c.id,c.comment,c.nbrOffersAuthorized,c.nbrOffersAuthorizedSite,c.displayCode")
            ->from("App:Card", "c")
            ->where('c.id = :card')
            ->setParameter('card', $idCarte)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($card, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/carte/{idcarte}/restaurant")
     */
    public function getRestaurantIdByCarte($idcarte)
    {
        if(in_array($this->getUser()->getProfil()->getId(),[4,3])){
            if($this->getUser()->getPdv()!=intval($idcarte)){
                return new JsonResponse(null,403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("r.id")
            ->from("App:Card", "c")
            ->innerJoin('c.pdv', 'p')
            ->innerJoin('p.restaurant', 'r')
            ->where('c.id =:idCard')
            ->setParameter('idCard', $idcarte)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families[0], 'json');
        return new JsonResponse(json_decode($data));
    }

    public function getFamiliesIngredientByCard($idCarte, $label)
    {
        if(in_array($this->getUser()->getProfil()->getId(),[4,3])){
            if($this->getUser()->getPdv()!=intval($idCarte)){
                return new JsonResponse(null,403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("cf")
            ->from("App:CardFamily", "cf")
            ->innerJoin('cf.card', 'ca')
            ->innerJoin('cf.family', 'f')
            ->where('ca.id = :idCard')
            ->andWhere('f.label = :label')
            ->setParameter('idCard', $idCarte)
            ->setParameter('label', $label)
            ->getQuery()->setMaxResults(1)->getSingleResult();

        return $families;
    }

    public function getSizeByCard($idCarte, $label)
    {
        if(in_array($this->getUser()->getProfil()->getId(),[4,3])){
            if($this->getUser()->getPdv()!=intval($idCarte)){
                return new JsonResponse(null,403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("s")
            ->from("App:Size", "s")
            ->innerJoin('s.card', 'ca')
            ->where('ca.id = :idCard')
            ->andWhere('s.label = :label')
            ->setParameter('idCard', $idCarte)
            ->setParameter('label', $label)
            ->getQuery()->setMaxResults(1)->getSingleResult();
        return $families;
    }

}
