<?php
/**
 * Created by PhpStorm.
 * User: HP
 * Date: 16/02/2018
 * Time: 11:34
 */

namespace App\Controller\Rest;

use Doctrine\ORM\Query;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Category;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;

class CategoryController extends FOSRestController
{
    /**
     * @Rest\Post("/categories")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="label", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     */
    public function addAction(ParamFetcher $paramFetcher)
    {
        $category = new Category();
        $category->setLabel($paramFetcher->get('label'));
        $category->setDescription($paramFetcher->get('description'));
        $category->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $em->persist($category);
        $em->flush();

        return new JsonResponse([
            'id'=>$category->getId(),
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/categories/carte/{idCarte}")
     */
    public function getAllCatAction($idCarte)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $categories = $qb->select("c.id,c.label")
            ->from("App:CardFamily", "cf")
            ->innerJoin('cf.family', 'f')
            ->innerJoin('f.category', 'c')
            ->innerJoin('cf.card', 'ca')
            ->where('ca.id = :idCarte')
            ->setParameter('idCarte', $idCarte)
            ->groupBy('c.id')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($categories, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/categories")
     */
    public function getListAction()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder('t');
        $families = $qb->select("f.id,f.label,c.label as category")
            ->from("App:Family", "f")
            ->innerJoin('f.category', 'c')
            ->getQuery()->getResult();
        $result = array();
        foreach ($families as $element) {
            $index=$this->findIndex($result,$element['category']);
            if($index>-1){
                $result[$index]['families'][] = $element;
            }else{
                $temp = array();
                $temp['category'] = $element['category'];
                $temp['families'][] = $element;
                $result[] = $temp;
            }
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/categories2")
     */
    public function getCategories()
    {
        $em = $this->getDoctrine()->getManager();
        $result = $em->getRepository('App:Category')->findAll();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Delete("/categories/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = $em->getRepository('App:Category')->find($id);
        $em->remove($p);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/categories/{id}")
     */
    public function getCategory($id)
    {
        $em = $this->getDoctrine()->getManager();
        $category = $em->getRepository('App:Category')->find($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($category, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/categories/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="label", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAction($id, ParamFetcher $paramFetcher)
    {
        $category = $this->getDoctrine()->getRepository("App:Category")->find($id);
        $category->setLabel($paramFetcher->get('label'));
        $category->setDescription($paramFetcher->get('description'));
        $category->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $em->persist($category);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    public function findIndex($array, $category)
    {
        $index = -1;
        foreach ($array as $key => $value) {
            if ($value['category'] == $category) {
                $index = $key;
                break;
            }
        }
        return $index;
    }
}
