<?php

namespace App\Controller\Rest;

use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use App\Entity\Villes_france;
use FOS\RestBundle\Request\ParamFetcher;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Restaurant;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Doctrine\ORM\Tools\Pagination\Paginator;
use FOS\RestBundle\Controller\Annotations\RequestParam;

class CityFranceController extends FOSRestController
{

    /**
     * @Rest\Get("/francecities/{count_per_page}/{current_page}/{key}", defaults={"key" = ""})
     */
    public function getCityFranceAction(Request $request,$current_page,$count_per_page,$key)
    {
        $data = [];
        $data['count_per_page'] =  intval($count_per_page);
        $data['current_page'] = intval($current_page);
        $range_start = (intval($data['current_page']) - 1) * intval($data['count_per_page']);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $qb->select("a")
            ->from("App:CityFrance", "a")
            ->where('a.postalCode LIKE :key')
            ->orWhere('a.name LIKE :key')
            ->setParameter('key', '%'.$key.'%') ;
        $query = $qb->getQuery()->setFirstResult($range_start)->setMaxResults($data['count_per_page']);
        $paginator = new Paginator($query, true);
        $data['data'] = $query->execute();
        $data['total_page'] = $paginator->count();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        $response = new Response();
        $response->setContent($data);
        return $response;
    }

    /**
     * @Rest\Get("/cities/postal/{postal}")
     */
    public function getListByPostal($postal)
    {
        $cities = $this->getDoctrine()->getRepository("App:CityFrance")->findBy(array("postalCode"=>$postal));
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($cities, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Post("/authorizedCity")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="postalCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="idPdv", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="city", nullable=false,allowBlank=false,strict=true)
     */

    public function checkAuthorizedCity(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $cities = $qb->select("c.id,cp.amountMiniDelivery")
            ->from("App:CityFrancePdv", "cp")
            ->innerJoin('cp.pdv', 'p')
            ->innerJoin('cp.city', 'c')
            ->where('p.id = :pdv')
            ->andWhere('c.postalCode=:postalCode')
            ->andWhere('c.name=:city')
            ->setParameter('pdv', $paramFetcher->get('idPdv'))
            ->setParameter('postalCode', $paramFetcher->get('postalCode'))
            ->setParameter('city', $paramFetcher->get('city'))
            ->getQuery()->getResult();
        if(count($cities)>0){
            return new JsonResponse(['authorized'=>true,'amountMiniDelivery'=>$cities[0]['amountMiniDelivery']]);
        }else{
            return new JsonResponse(['authorized'=>false]);
        }
    }

}
