<?php

namespace App\Controller\Rest;

use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use App\Entity\Pdv;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Contact;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Request\ParamFetcher;
use Symfony\Component\Validator\Constraints\DateTime;

class ContactController extends FOSRestController
{

    /**
     * @Rest\Post("/contacts")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="firstname", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="lastname", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="email", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="phone", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="function", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(Request $request, ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $idPdv = $paramFetcher->get('pdv');
        $pdv = $em->getRepository('App:Pdv')->find($paramFetcher->get('pdv'));
        $contact = new Contact();
        $qb->select("c")
            ->from("App:Contact", "c")
            ->innerJoin('c.pdvs', 'P')
            ->where('P.id = :pdv')
            ->andWhere('c.phone = :tel OR c.email = :email')
            ->setParameter('tel', $paramFetcher->get('phone'))
            ->setParameter('email', $paramFetcher->get('email'))
            ->setParameter('pdv', $idPdv);
        $c = $qb->getQuery()->getResult();
        if (count($c) > 0) {
            $contact = $c[0];
            $contacts = $em->getRepository('App:Contact')
                ->createQueryBuilder('C')
                ->innerJoin('C.pdvs', 'P')
                ->where('P.id = :pdv')
                ->andWhere('C.id = :contact')
                ->setParameter('pdv', $idPdv)
                ->setParameter('contact', $contact->getId())
                ->getQuery()
                ->getResult();
            if (count($contacts) == 0) {
                $contact->addPdv($pdv);
                $em->persist($contact);
                $em->flush();
                return new JsonResponse([
                    'id' => $contact->getId(),
                    'success' => true,
                    'code' => 200
                ]);
            }else{
                return new JsonResponse([],400);
            }
        } else {
            $contact->setfirstname($paramFetcher->get('firstname'));
            $contact->setlastname($paramFetcher->get('lastname'));
            $contact->setemail($paramFetcher->get('email'));
            $contact->setphone($paramFetcher->get('phone'));
            $contact->setfunction($paramFetcher->get('function'));
            $contact->setUser(false);
            $contact->addPdv($pdv);
            $em->persist($contact);
            $em->flush();
            return new JsonResponse([
                'id' => $contact->getId(),
                'success' => true,
                'code' => 200
            ]);
        }


    }

    /**
     * @Rest\Put("/contacts/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="firstname", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="lastname", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="email", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="phone", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="function", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     */
    public function updateAction(Request $request,$id, ParamFetcher $paramFetcher)
    {
        $contact = $this->getDoctrine()->getRepository("App:Contact")->find(intval($id));
        if(!is_null($contact)){
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $idPdv = $paramFetcher->get('pdv');
            $qb->select("c")
                ->from("App:Contact", "c")
                ->innerJoin('c.pdvs', 'P')
                ->where('P.id = :pdv')
                ->andWhere('c.phone = :tel OR c.email = :email')
                ->andWhere('c.id != :id')
                ->setParameter('tel', $paramFetcher->get('phone'))
                ->setParameter('email', $paramFetcher->get('email'))
                ->setParameter('pdv', $idPdv)
                ->setParameter('id', $id);
            $c = $qb->getQuery()->getResult();
            if (count($c) > 0) {
                return new JsonResponse([],409);
            } else {
                $contact->setfirstname($paramFetcher->get('firstname'));
                $contact->setlastname($paramFetcher->get('lastname'));
                $contact->setemail($paramFetcher->get('email'));
                $contact->setphone($paramFetcher->get('phone'));
                $contact->setfunction($paramFetcher->get('function'));
                $em = $this->getDoctrine()->getManager();
                $em->persist($contact);
                $em->flush();
                return new JsonResponse([
                    'success' => true,
                    'code' => 200
                ]);

            }
        }else{
            return new JsonResponse('',204);
        }


    }

    /**
     * @Rest\Get("/contacts/pdv/{idPdv}")
     */
    public function getConatctsPdv($idPdv)
    {
        $em = $this->getDoctrine()->getManager();
        $qb=$em->createQueryBuilder();
        $contacts =$qb->select("c.id,c.lastname,c.firstname,c.phone,c.email,c.function,u.id as user")
            ->from('App:Contact','c')
            ->leftJoin('App:User', 'u', 'WITH', 'u.contact = c.id AND u.enabled = 1')
            ->innerJoin('c.pdvs', 'P')
            ->where('P.id = :pdv')
            ->setParameter('pdv', $idPdv)
            ->getQuery()
            ->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($contacts, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/contacts/{id}")
     */
    public function getContact($id)
    {
        $contact = $this->getDoctrine()->getRepository("App:Contact")->find($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($contact, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Delete("/contacts/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = $em->getRepository('App:Contact')->find($id);
        $em->remove($p);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

}
