<?php

namespace App\Controller\Rest;

use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use App\Entity\Address;
use App\Entity\CustomerPdv;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Customer;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Controller\Annotations\FileParam;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Doctrine\ORM\Tools\Pagination\Paginator;
use Symfony\Component\Validator\Constraints\DateTime;

class CustomerController extends FOSRestController
{

    /**
     * @Rest\Post("/customers")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="firstname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="lastname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="society", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="phone", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="secondaryPhone", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="smsAuthorized", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="address", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(Request $request, ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $customer = $this->getDoctrine()->getRepository("App:Customer")->findOneBy(array("phone" => $paramFetcher->get('phone'),'indicator'=>NULL));
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($paramFetcher->get('pdv'));
        if(!$pdv){
        	return new JsonResponse('pdv not found', 400);
        }
        if (!is_null($customer)) {
            $customerPdv = $this->getDoctrine()->getRepository("App:CustomerPdv")->findOneBy(array("pdv" => $pdv, "customer" => $customer));
            if (is_null($customerPdv)) {
                $customerPdv = new CustomerPdv();
                $customerPdv->setCustomer($customer);
                $customerPdv->setComment($paramFetcher->get('comment'));
                $customerPdv->setPDV($pdv);
                $em->persist($customerPdv);
                $add = $paramFetcher->get('address');
                $address = new Address();
                if ($add) {
                    $address->setPrincipal(1);
                    $address->setStreetName($add['streetName']);
                    $address->setStreetNumber($add['streetNumber']);
                    $address->setPostalCode($add['postalCode']);
                    $address->setCity($add['city']);
                    $address->setComplementAddress((array_key_exists('complementAddress', $add)) ? $add['complementAddress'] : '');
                    $address->setInputCode((array_key_exists('inputCode', $add)) ? $add['inputCode'] : '');
                    $address->setBuilding(array_key_exists('building', $add) ? $add['building'] : '');
                    $address->setStaircase(array_key_exists('staircase', $add) ? $add['staircase'] : '');
                    $address->setFloor(array_key_exists('floor', $add) ? $add['floor'] : '');
                    $address->setIntercom(array_key_exists('intercom', $add) ? $add['intercom'] : '');
                    $address->setDoor(array_key_exists('door', $add) ? $add['door'] : '');
                    $address->setCountry(array_key_exists('country', $add) ? $add['country'] : '');
                    $address->setCustomer($customerPdv);
                    $em->persist($address);
                }
                $em->flush();
                return new JsonResponse([
                    'id' => $customer->getId(),
                    'customer' => $customerPdv->getId(),
                    'address' => (!is_null($address)) ? $address->getId() : "",
                    'success' => true,
                    'code' => 200
                ]);
            } else {
                return new JsonResponse([], 409);
            }
        } else {
            $customer = new Customer();
            $customer->setFirstname($paramFetcher->get('firstname'));
            $customer->setLastname($paramFetcher->get('lastname'));
            $customer->setSociety($paramFetcher->get('society'));
            $customer->setPhone($paramFetcher->get('phone'));
            $customer->setSecondaryPhone($paramFetcher->get('secondaryPhone'));
            $customer->setEmail($paramFetcher->get('email'));
            $customer->setSmsAuthorized($paramFetcher->get('smsAuthorized'));
            $ip = $request->getClientIp();
            $customer->setCreatedIp($ip);
            $customer->setUpdatedIp('');
            $customer->setCreatedUser($this->getUser());
            $customer->setUpdatedUser(null);
            $customer->setIndicator(null);
            $em = $this->getDoctrine()->getManager();
            $em->persist($customer);
            $customerPdv = new CustomerPdv();
            $customerPdv->setcustomer($customer);
            $customerPdv->setPdv($pdv);
            $customerPdv->setComment($paramFetcher->get('comment'));
            $em->persist($customerPdv);
            $add = $paramFetcher->get('address');
            $address = new Address();
            if ($add) {
                $address->setPrincipal(1);
                $address->setStreetName($add['streetName']);
                $address->setStreetNumber($add['streetNumber']);
                $address->setPostalCode($add['postalCode']);
                $address->setCity($add['city']);
                $address->setComplementAddress((array_key_exists('complementAddress', $add)) ? $add['complementAddress'] : '');
                $address->setInputCode((array_key_exists('inputCode', $add)) ? $add['inputCode'] : '');
                $address->setBuilding(array_key_exists('building', $add) ? $add['building'] : '');
                $address->setStaircase(array_key_exists('staircase', $add) ? $add['staircase'] : '');
                $address->setFloor(array_key_exists('floor', $add) ? $add['floor'] : '');
                $address->setIntercom(array_key_exists('intercom', $add) ? $add['intercom'] : '');
                $address->setDoor(array_key_exists('door', $add) ? $add['door'] : '');
                $address->setCountry(array_key_exists('country', $add) ? $add['country'] : '');
                $address->setCustomer($customerPdv);
                $em->persist($address);
            }
            $em->flush();
            return new JsonResponse([
                'customer' => $customerPdv->getId(),
                'address' => (!is_null($address)) ? $address->getId() : "",
                'success' => true,
                'code' => 200
            ]);
        }

    }

    /**
     * @Rest\Put("/customers")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="firstname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="lastname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="society", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="phone", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="secondaryPhone", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="smsAuthorized", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="id", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="address", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAction(Request $request, ParamFetcher $paramFetcher)
    {
        $customerPdv = $this->getDoctrine()->getRepository("App:CustomerPdv")->find(intval($paramFetcher->get('id')));
        if(!$customerPdv){
            return new JsonResponse([], 400);
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $customer = $qb->select('c.id')
            ->from("App:customerPdv", "pc")
            ->innerJoin("pc.customer","c")
            ->innerJoin("pc.pdv","p")
            ->where('c.phone = :phone')
            ->andWhere("c.id !=:id")
            ->andWhere("p.id=:pdv")
            ->andWhere("c.indicator is NULL")
            ->setParameter('phone', $paramFetcher->get('phone'))
            ->setParameter('id', $customerPdv->getCustomer()->getId())
            ->setParameter('pdv', $customerPdv->getIdPdv())
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
        if (!is_null($customer)) {
            return new JsonResponse([], 409);
        }
        $customer = $customerPdv->getCustomer();
        $customer->setFirstname($paramFetcher->get('firstname'));
        $customer->setLastname($paramFetcher->get('lastname'));
        $customer->setSociety($paramFetcher->get('society'));
        $customer->setPhone($paramFetcher->get('phone'));
        $customer->setSecondaryPhone($paramFetcher->get('secondaryPhone'));
        $customer->setEmail($paramFetcher->get('email'));
        $customer->setSmsAuthorized($paramFetcher->get('smsAuthorized'));
        $ip = $request->getClientIp();
        $customer->setUpdatedIp($ip);
        $customer->setCreatedUser(null);
        $customer->setUpdatedUser($this->getUser());
        $customerPdv->setComment($paramFetcher->get('comment'));
        $add = $paramFetcher->get('address');
        $address = null;
        if($add){
            if(!array_key_exists('id', $add) || intval($add['id'])==0){
                $address = new Address();
            }else{
                $address=$this->getDoctrine()->getRepository("App:Address")->find($add['id']);
            }
            $address->setStreetNumber($add['streetNumber']);
            $address->setStreetName($add['streetName']);
            $address->setPostalCode($add['postalCode']);
            $address->setCity($add['city']);
            $address->setComplementAddress((array_key_exists('complementAddress', $add)) ? $add['complementAddress'] : '');
            $address->setInputCode((array_key_exists('inputCode', $add)) ? $add['inputCode'] : '');
            $address->setBuilding(array_key_exists('building', $add) ? $add['building'] : '');
            $address->setStaircase(array_key_exists('staircase', $add) ? $add['staircase'] : '');
            $address->setFloor(array_key_exists('floor', $add) ? $add['floor'] : '');
            $address->setIntercom(array_key_exists('intercom', $add) ? $add['intercom'] : '');
            $address->setDoor(array_key_exists('door', $add) ? $add['door'] : '');
            $address->setCountry(array_key_exists('country', $add) ? $add['country'] : '');
            $address->setCustomer($customerPdv);
            if(array_key_exists('principal', $add) && $add['principal']==true){
                $addresses=$customerPdv->getAddresses();
                foreach ($addresses as $a){
                    $a->setPrincipal(0);
                    $em->persist($a);
                }
                 $address->setPrincipal(1);
            }
            $em->persist($address);
        }
        $em->persist($customerPdv);
        $em->persist($customer);
        $em->flush();
        return new JsonResponse([
            'customer' => $customerPdv->getId(),
            'address' =>(!is_null($address)) ? $address->getId() : "",
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Post("/customers/import")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @FileParam(name="fileCsv", nullable=false, strict=true)
     * @RequestParam(name="pdv", nullable=false, strict=true)
     */
    public function importcustomers(Request $request, ParamFetcher $paramFetcher)
    {
        $file = $paramFetcher->get('fileCsv');
        $fileName = $this->generateUniqueFileName() . '.' . $file->guessExtension();
        $file->move(
            $this->getParameter('file_directory'),
            $fileName
        );
        $em = $this->getDoctrine()->getManager();
        $idPdv = $paramFetcher->get('pdv');
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($idPdv);
        $uploaddir = $this->getParameter('file_directory') . '/';
        $table = array(
            "lastname",
            "firstname",
            "society",
            "phone",
            "secondaryPhone",
            "email",
            "streetNumber",
            "streetName",
            "postalCode",
            "city",
            "complementAddress",
            "inputCode",
            "building",
            "staircase",
            "floor",
            "intercom",
            "door",
            "smsAuthorized",
            "country",
            "comment"
        );
        $uploadfile = $uploaddir . basename($fileName);
        $objPHPExcel = \PhpOffice\PhpSpreadsheet\IOFactory::load($uploadfile);
        $worksheet = $objPHPExcel->setActiveSheetIndex(0);
        $arrayData = $worksheet->toArray(NULL, false, true, false);
        $key = $arrayData[0];
        unset($arrayData[0]);
        foreach ($arrayData as $a => $b) {
            foreach ($b as $c => $d) {
                $array[$a][$key[$c]] = $arrayData[$a][$c];
            }
        }
        if ($key == $table) {
            $i = 0;
            foreach ($array as $a => $b) {
                $phone = str_replace(' ', '', $array[$a]["phone"]);
                $customer = $this->getDoctrine()->getRepository("App:Customer")->findOneBy(array("phone" => $phone));
                if (!is_null($customer)) {
                    $customerPdv = $this->getDoctrine()->getRepository("App:CustomerPdv")->findOneBy(array("pdv" => $pdv, "customer" => $customer));
                    if (is_null($customerPdv)) {
                        $customerPdv1 = new CustomerPdv();
                        $customerPdv1->setCustomer($customer);
                        $customerPdv1->setComment($array[$a]['comment']);
                        $customerPdv1->setPDV($pdv);
                        $em->persist($customerPdv1);
                        $address = new Address();
                        $address->setStreetNumber($array[$a]['streetNumber']);
                        $address->setStreetName($array[$a]['streetName']);
                        $address->setPostalCode($array[$a]['postalCode']);
                        $address->setCity($array[$a]['city']);
                        $address->setComplementAddress($array[$a]['complementAddress']);
                        $address->setInputCode($array[$a]['inputCode']);
                        $address->setBuilding($array[$a]['building']);
                        $address->setStaircase($array[$a]['staircase']);
                        $address->setFloor($array[$a]['floor']);
                        $address->setIntercom($array[$a]['intercom']);
                        $address->setDoor($array[$a]['door']);
                        $address->setCountry($array[$a]['country']);
                        $address->setPrincipal(1);
                        $address->setCustomer($customerPdv1);
                        $em->persist($address);
                        $i++;
                    }else {
                        $addresses = $em->getRepository('App:Address')
                            ->createQueryBuilder('a')
                            ->InnerJoin('a.customer', 'CP')
                            ->where('CP.id = :cust ');
                        if (is_null($array[$a]["streetNumber"])) {
                            $addresses = $addresses->andWhere('a.streetNumber IS NULL');
                        } else {
                            $addresses = $addresses->andWhere('a.streetNumber = :streetNumber')
                                ->setParameter('streetNumber', $array[$a]["streetNumber"]);
                        }
                        if (is_null($array[$a]["streetName"])) {
                            $addresses = $addresses->andWhere('a.streetName IS NULL');
                        } else {
                            $addresses = $addresses->andWhere('a.streetName = :streetName')
                                ->setParameter('streetName', $array[$a]["streetName"]);
                        }
                        if (is_null($array[$a]["postalCode"])) {
                            $addresses = $addresses->andWhere('a.postalCode IS NULL');
                        } else {
                            $addresses = $addresses->andWhere('a.postalCode = :postalCode')
                                ->setParameter('postalCode', $array[$a]["postalCode"]);
                        }
                        $addresses = $addresses->setParameter('cust', $customerPdv->getId())
                            ->getQuery()
                            ->getResult();
                        if (count($addresses) == 0) {
                            $address = new Address();
                            $address->setStreetNumber($array[$a]['streetNumber']);
                            $address->setStreetName($array[$a]['streetName']);
                            $address->setPostalCode($array[$a]['postalCode']);
                            $address->setCity($array[$a]['city']);
                            $address->setComplementAddress($array[$a]['complementAddress']);
                            $address->setInputCode($array[$a]['inputCode']);
                            $address->setBuilding($array[$a]['building']);
                            $address->setStaircase($array[$a]['staircase']);
                            $address->setFloor($array[$a]['floor']);
                            $address->setIntercom($array[$a]['intercom']);
                            $address->setDoor($array[$a]['door']);
                            $address->setCountry($array[$a]['country']);
                            $address->setCustomer($customerPdv);
                            $em->persist($address);
                            $i++;
                        }
                    }
                } else {
                    $customer = new Customer();
                    $customer->setFirstname($array[$a]['firstname']);
                    $customer->setLastname($array[$a]['lastname']);
                    $customer->setSociety($array[$a]['society']);
                    $customer->setPhone($phone);
                    $customer->setEmail($array[$a]['email']);
                    $customer->setSmsAuthorized(0);
                    $ip = $request->getClientIp();
                    $customer->setCreatedIp($ip);
                    $customer->setUpdatedIp('');
                    $customer->setCreatedUser($this->getUser());
                    $customer->setUpdatedUser(null);
                    $em->persist($customer);
                    $customerPdv = new CustomerPdv();
                    $customerPdv->setcustomer($customer);
                    $customerPdv->setComment($array[$a]['comment']);
                    $customerPdv->setPdv($pdv);
                    $em->persist($customerPdv);
                    $em->flush();
                    $address = new Address();
                    $address->setStreetNumber($array[$a]['streetNumber']);
                    $address->setStreetName($array[$a]['streetName']);
                    $address->setPostalCode($array[$a]['postalCode']);
                    $address->setCity($array[$a]['city']);
                    $address->setComplementAddress($array[$a]['complementAddress']);
                    $address->setInputCode($array[$a]['inputCode']);
                    $address->setBuilding($array[$a]['building']);
                    $address->setStaircase($array[$a]['staircase']);
                    $address->setFloor($array[$a]['floor']);
                    $address->setIntercom($array[$a]['intercom']);
                    $address->setDoor($array[$a]['door']);
                    $address->setCountry($array[$a]['country']);
                    $address->setPrincipal(1);
                    $address->setCustomer($customerPdv);
                    $em->persist($address);
                    $i++;
                }
            }
            if ($i == 0) {
                $message = "fichier déjà importé dans la base de données";
                $code = 1;
            } else {
                $em->flush();
                $code = 0;
                $message = "chargement terminé avec succès";
            }
        } else {
            $message = "les colonnes de fichier ne sont pas identiques a celles de la table";
            $code = 2;
        }
        unlink($uploadfile);
        return new JsonResponse([
            'message' => $message,
            'success' => true,
            'code' => $code
        ]);
    }

    /**
     * @Rest\Get("/customers/pdv/{idPdv}")
     */
    public function getcustomersPdv($idPdv)
    {
        if ($this->getUser()->getPdv() && $this->getUser()->getPdv() != $idPdv) {
            return new JsonResponse('', 403);
        }
        $em = $this->getDoctrine()->getManager();
        $customers = $em->getRepository('App:Customer')
            ->createQueryBuilder('C')
            ->innerJoin('C.pdvs', 'CP')
            ->innerJoin('CP.pdv', 'P')
            ->where('P.id = :pdv')
            ->setParameter('pdv', $idPdv)
            ->getQuery()
            ->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($customers, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/customers/order/{idOrder}")
     */
    public function getCustomersByOrder($idOrder)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $cp = $qb->select("cp.id")
            ->from("App:Command", "co")
            ->InnerJoin('co.customer', 'cp')
            ->where('co.id=:idOrder')
            ->setParameter('idOrder', $idOrder)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $customer = $this->getDoctrine()->getRepository("App:CustomerPdv")->find($cp['id']);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($customer, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/customers/{idcustomer}")
     */
    public function getCustomer($idcustomer)
    {
        $customer = $this->getDoctrine()->getRepository("App:CustomerPdv")->find($idcustomer);
        if (!is_null($customer)) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($customer, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            $response = new Response();
            $response->setStatusCode(204);
            return $response;
        }
    }

    /**
     * @Rest\Get("/customers/{count_per_page}/{current_page}/{idPdv}/{key}", defaults={"key" = ""})
     */
    public function getcustomersAction(Request $request, $current_page, $count_per_page, $idPdv, $key)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idPdv)) {
                return new JsonResponse(null, 403);
            }
        }
        $data = [];
        $data['count_per_page'] = intval($count_per_page);
        $data['current_page'] = intval($current_page);
        $range_start = (intval($data['current_page']) - 1) * intval($data['count_per_page']);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $qb->select("distinct cp")
            ->from("App:CustomerPdv", "cp")
            ->innerJoin('cp.customer', 'c')
            ->leftJoin('cp.addresses', 'a')
            ->innerJoin('cp.pdv', 'p')
            ->where('c.phone LIKE :key
                        or c.lastname LIKE :key 
                        or c.firstname LIKE :key 
                        or c.society LIKE :key
                        or a.city LIKE :key 
                        or a.postalCode LIKE :key')
            ->andWhere("c.indicator is NULL")
            ->andWhere('p.id = :pdv')
            ->setParameter('pdv', $idPdv)
            ->setParameter('key', '%' . $key . '%');
            //->orderBy('c.'.$prop,$dir);
        $query = $qb->getQuery()->setFirstResult($range_start)->setMaxResults($data['count_per_page']);;
        $paginator = new Paginator($query, false);
        $data['data'] = $query->execute();
        $data['total_page'] = $paginator->count();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/customers/cmd/pdv/{idPdv}/phone/{phone}")
     */
    public function getCusByPdvAndPhone($idPdv, $phone)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idPdv)) {
                return new JsonResponse(null, 403);
            }
        }
        if ($this->getUser()->getPdv() && $this->getUser()->getPdv() != $idPdv) {
            return new JsonResponse('', 403);
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $customer = $qb->select("cp")
            ->from("App:CustomerPdv", "cp")
            ->leftJoin('cp.customer', 'c')
            ->leftJoin('cp.pdv', 'p')
            ->where("c.phone = :phone")
            ->andWhere('p.id = :pdv')
            ->andWhere("c.indicator is NULL")
            ->setParameter('pdv', $idPdv)
            ->setParameter('phone',$phone)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($customer, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Delete("/customers/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $customer = $qb->select("C")
            ->from("App:Customer", "C")
            ->innerJoin('C.pdvs', 'cp')
            ->where('cp.id =:idCp')
            ->setParameter('idCp', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $em->remove($customer);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    private function generateUniqueFileName()
    {
        return md5(uniqid());
    }


}


