<?php

namespace App\Controller\Rest;

use App\Entity\CommandPaymentMode;
use App\Service\SendEvent;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Controller\Annotations\QueryParam;
use App\Entity\Address;
use App\Entity\CommandElement;
use App\Entity\Command;
use App\Entity\CommandElementIngredients;
use App\Entity\CommandMenu;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Pdv;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Doctrine\ORM\Tools\Pagination\Paginator;
use DateTime;
use Symfony\Component\HttpFoundation\StreamedResponse;

class DelivrayController extends FOSRestController
{



    /**
     * @Rest\Get("/delivray/orders/day")
     */
    public function getDayOrders()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $now = new DateTime();
        $query = "co.id,co.num,co.discount,co.corrected,co.amount,s.id as status,s.id as oldStatus,t.label as type,mop.minDelay,
        co.orderProg,p.id as pdvId,p.name as pdvName,c.id as customerId,c.firstname as customerFirstname,c.lastname as customerLastname,
        c.phone as customerPhone,c.indicator";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $query .= ",u.id as idUser,u.firstname as userFirstname,u.lastname as userLastname";
        }
        if ($this->getUser()->getPdv()) {
            $query .= ",team.id as teamMember,team.id as oldTeamMember";
        }
        $orders = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $orders = $orders->leftJoin('co.createdUser', 'u');
        }
        if ($this->getUser()->getPdv()) {
            $orders = $orders->leftJoin('co.team', 'team');
        }
        $orders = $orders->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('co.status', 's')
            ->where('co.createdAt >=:date')
            ->andWhere('co.createdAt <=:date2')
            ->andWhere('t.id =1')
            ->andWhere('team IS NULL');
        if ($this->getUser()->getPdv()) {
            $orders = $orders->andWhere('p.id =:pdv')
                ->setParameter('pdv', $this->getUser()->getPdv());
        }
        $orders = $orders->setParameter('date', $now->format('Y-m-d 00:00:00'))
            ->setParameter('date2', $now->format('Y-m-d 23:59:59'))
            ->orderBy('co.orderProg')
            ->getQuery()->getResult();
        foreach ($orders as $key => $order) {
            $ord = $em->getRepository('App:Command')->find($order['id']);
            $orders[$key]['PaymentModes'] = $ord->getCommandPaymentModes();
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($orders, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Post("/delivray/orders/collection")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="team", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="date", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="paymentModes", nullable=true,allowBlank=true,strict=true)
     */
    public function getCollectionOrders(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $now = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('date'));
        $query = "co.id,co.num,co.discount,co.corrected,co.amount,s.id as status,s.id as oldStatus,t.label as type,mop.minDelay,
        co.orderProg,p.id as pdvId,p.name as pdvName,c.id as customerId,c.firstname as customerFirstname,c.lastname as customerLastname,
        c.phone as customerPhone,SUM(cmp.amount) as sumModes, (co.amount - SUM(cmp.amount)) as sumAmount";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $query .= ",u.id as idUser,u.firstname as userFirstname,u.lastname as userLastname";
        }
        if ($this->getUser()->getPdv()) {
            $query .= ",team.id as teamMember,team.designation as teamName";
        }
        $orders = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $orders = $orders->leftJoin('co.createdUser', 'u');
        }
        if ($this->getUser()->getPdv()) {
            $orders = $orders->leftJoin('co.team', 'team');
        }
        $orders = $orders->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('co.status', 's')
            ->innerJoin('co.commandPaymentModes', 'cmp')
            ->innerJoin('cmp.paymentMode', 'mp')
            ->where('co.team IS NOT NULL')
            ->andWhere('t.id = 1')
            ->andWhere('DATE(co.createdAt) =:date');

        if ($paramFetcher->get('team')) {
            $orders = $orders->andWhere('team.id =:tea')
                ->setParameter('tea', $paramFetcher->get('team'));
        }

        if ($paramFetcher->get('paymentModes')) {
            $orders = $orders->andWhere('mp.id IN (:paymentModes)')
                ->setParameter('paymentModes', $paramFetcher->get('paymentModes'));
        }

        if ($this->getUser()->getPdv()) {
            $orders = $orders->andWhere('p.id =:pdv')
                ->setParameter('pdv', $this->getUser()->getPdv());
        }
        $orders = $orders->setParameter('date', $now->format('Y-m-d'))
            ->groupBy('co.id')
            ->getQuery()->getResult();
        foreach ($orders as $key => $order) {
            $ord = $em->getRepository('App:Command')->find($order['id']);
            $orders[$key]['PaymentModes'] = $ord->getCommandPaymentModes();
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($orders, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Put("/delivray/orders/collectionOrder")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="id", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="paymentMode", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="amount", nullable=false,allowBlank=false,strict=true)
     */
    public function updateOrderCollection(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        if ($this->getUser()->hasRole('RULE_COLLECTION') || $this->getUser()->getProfil()->getId() == 3) {
            $order = $em->getRepository('App:Command')->find($paramFetcher->get('id'));
            $pm = $em->getRepository('App:PaymentMode')->find($paramFetcher->get('paymentMode'));
            $orderPaymentMode = $em->getRepository('App:CommandPaymentMode')->findOneBy(array('command' => $order, 'paymentMode' => $pm));
            if (!is_null($orderPaymentMode)) {
                $orderPaymentMode->setAmount($paramFetcher->get('amount'));
                $em->persist($orderPaymentMode);
                $em->flush();
                return new JsonResponse ([], 200);

            } else {
                $commandPaymentMode = new CommandPaymentMode();
                if ($pm) {
                    $commandPaymentMode->setPaymentMode($pm);
                    $commandPaymentMode->setCommand($order);
                    $commandPaymentMode->setAmount($paramFetcher->get('amount'));
                    $em->persist($commandPaymentMode);
                    $em->flush();
                    return new JsonResponse ([], 200);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse('', 403);
        }
    }


    /**
     * @Rest\Put("/delivray/orders/team")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="command", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="team", nullable=true,allowBlank=true,strict=true)
     */
    public function updateOrderTeam(ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {

        if ($this->getUser()->getProfil()->getId() != 3 && $this->getUser()->getProfil()->getId() != 4) {
            return new JsonResponse('', 403);
        } else {
            $em = $this->getDoctrine()->getManager();
            $order = $em->getRepository('App:Command')->find($paramFetcher->get('command'));
            if (intval($paramFetcher->get('team')) > 0) {
                $team = $em->getRepository('App:Team')->find($paramFetcher->get('team'));
            } else {
                $team = null;
            }
            $order->setTeam($team);
            $em->persist($order);
            $em->flush();
            $sendEvent->sendMessage(['type' => 'updateOrderTeam', 'entity' => 'order', 'id' => $order->getId(), 'team' => ($team) ? $team->getId() : '']);
            return new JsonResponse ([
                'success' => true,
                'code' => 200
            ]);
        }
    }


    /**
     * @Rest\Get("/delivray/orders/{id}/paymentModes")
     */
    public function getOrderPaymentModes($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $paymentModes = $qb->select("m.label")
            ->from("App:Command", "co")
            ->innerJoin('co.commandPaymentModes', 'pm')
            ->innerJoin('pm.paymentMode', 'm')
            ->where('co.id = :order')
            ->setParameter('order', $id)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($paymentModes, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/delivray/orders/{id}/info")
     */
    public function getInfoOrder($id,$response=true)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $order = $qb->select("co.num,co.comment,co.corrected,co.createdAt,co.amount,co.discount,co.orderProg,t.label as type,t.id as orderType,p.phone as pdvPhone,p.siret as pdvSiret,
        p.address as pdvAddress,p.name as pdvName,a.streetName,a.streetNumber,a.city,
        a.postalCode,a.building,a.staircase,a.floor,a.door,a.inputCode,a.intercom,a.complementAddress,c.firstname as customerFirstname,c.society as customerSociety,c.lastname as customerLastname,c.phone as customerPhone")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->leftJoin('co.address', 'a')
            ->where('co.id = :order')
            ->setParameter('order', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if($response){
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($order, 'json');
            return new JsonResponse(json_decode($data));
        }else{
            return $order;
        }
    }

    /**
     * @Rest\Get("/delivray/orders/{id}/details")
     */
    public function getOrder($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $order = $qb->select("co as cmd,t.id as mode,p.id as idPdv,cp.id As idCustomer")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('mop.pdv', 'p')
            ->leftJoin('co.address', 'a')
            ->where('co.id = :order')
            ->setParameter('order', $id);
        if ($this->getUser()->getPdv()) {
            $order = $order->andWhere('p.id =:pdv')->setParameter('pdv', $this->getUser()->getPdv());
        }
        $order = $order->setMaxResults(1)->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($order, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/delivray/orders/{id}/elements")
     */
    public function getOrderElements($id, $reponse = true)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("ce.id,ce.quantity,ce.comment,ce.level,ce.total,ce.price + ce.additionalCost as price,ce.additionalCost,ce.total,ce.name,ce.size")
            ->from("App:Command", "co")
            ->innerJoin('co.commandElements', 'ce')
            ->leftJoin('ce.commandMenu', 'cm')
            ->where('co.id = :order')
            ->andWhere('cm.id IS NULL')
            ->setParameter('order', $id)
            ->getQuery()->getResult();

        foreach ($elements as $key => $value) {
            $elements[$key]['optionIngredients'] = $this->getElementComSuppRemovOpt($value['id'], 'opt');
            $elements[$key]['removedCompositions'] = $this->getElementComSuppRemovOpt($value['id'], 'remove');
            $elements[$key]['supplements'] = $this->getElementComSuppRemovOpt($value['id'], 'supp');
        }
        if ($reponse == true) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($elements, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return $elements;
        }

    }


    /**
     * @Rest\Get("/delivray/orders/{id}/menus")
     */
    public function getOrderMenus($id, $reponse = true)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("cm.id,cm.quantity,cm.comment,cm.total,cm.price + cm.additionalCost as price,cm.title")
            ->from("App:CommandMenu", "cm")
            ->innerJoin('cm.command', 'c')
            ->where('c.id = :order')
            ->setParameter('order', $id)
            ->getQuery()->getResult();

        foreach ($menus as $ke => $val) {
            $menus[$ke]['elements'] = $this->getMenuElements($val['id']);
            foreach ($menus[$ke]['elements'] as $key => $value) {
                $menus[$ke]['elements'][$key]['optionIngredients'] = $this->getElementComSuppRemovOpt($value['id'], 'opt');
                //  $elements[$key]['compositions']=$this->getElementComSuppRemovOpt($value['id'],'comp');
                $menus[$ke]['elements'][$key]['removedCompositions'] = $this->getElementComSuppRemovOpt($value['id'], 'remove');
                $menus[$ke]['elements'][$key]['supplements'] = $this->getElementComSuppRemovOpt($value['id'], 'supp');
            }
        }
        if ($reponse) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($menus, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return $menus;
        }

    }


    /**
     * @Rest\Get("/delivray/{type}/orders/{id}")
     */
    public function getDelivrayDetails($id,$type)
    {

        $order = $this->getInfoOrder($id,false);
        $data=array('order' => $order);
        if($type==1){
            $paymentModes = $this->getPaymentModesOfOrder($id);
            $data['paymentModes']=$paymentModes;
        }
        $elements = $this->getOrderElements($id, false);
        $data['elements']=$elements;
        $menus = $this->getOrderMenus($id, false);
        $data['menus']=$menus;
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }


    public function getElementComSuppRemovOpt($idElement, $action)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("c.name,c.cout")
            ->from("App:CommandElementIngredients", "c")
            ->innerJoin('c.commandElement', 'ce')
            ->where('ce.id = :el')
            ->andWhere('c.action = :action')
            ->setParameter('el', $idElement)
            ->setParameter('action', $action)
            ->getQuery()->getResult();

        return $tab;
    }


    public function getMenuElements($idMenu)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("ce.id,ce.quantity,ce.comment,ce.additionalCost,ce.price,ce.name,ce.size,ce.level")
            ->from("App:Command", "co")
            ->innerJoin('co.commandElements', 'ce')
            ->leftJoin('ce.commandMenu', 'cm')
            ->where('cm.id = :menu')
            ->setParameter('menu', $idMenu)
            ->getQuery()->getResult();

        return $tab;
    }


    public function getPaymentModesOfOrder($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $paymentModes = $qb->select("m.label")
            ->from("App:Command", "co")
            ->innerJoin('co.commandPaymentModes', 'pm')
            ->innerJoin('pm.paymentMode', 'm')
            ->where('co.id = :order')
            ->setParameter('order', $id)
            ->getQuery()->getScalarResult("");
        $paymentModes = array_column($paymentModes, "label");
        return $paymentModes;
    }

}
