<?php
/**
 * Created by PhpStorm.
 * User: HP
 * Date: 15/02/2018
 * Time: 18:18
 */

namespace App\Controller\Rest;

use App\Service\SendEvent;
use Doctrine\Common\Collections\ArrayCollection;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Element;
use App\Entity\ElementOrderType;
use App\Entity\ElementSize;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;
use Doctrine\ORM\Tools\Pagination\Paginator;
use Spatie\ImageOptimizer\OptimizerChainFactory;

class ElementController extends FOSRestController
{
    /**
     * @Rest\Post("/elements")
     * @RequestParam(name="code", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="avatar", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="name", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="additionalOption", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="active", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="activeSite", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="hasSupp", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="vat", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="maxNbrIngOptions", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="priceSuppApp", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="family", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="supplements", nullable=false,allowBlank=true,strict=true)
     * @RequestParam(name="sizes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="compositions", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="category", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="authorizedMode", nullable=false,allowBlank=true,strict=true)
     * @RequestParam(name="optionIngredient", nullable=false,allowBlank=true,strict=true)
     * @RequestParam(name="carte", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="custom", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(request $request, ParamFetcher $paramFetcher)
    {
        $idCarte = $paramFetcher->get('carte');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCarte)) {
                return new JsonResponse(null, 403);
            }
        }
        $element = new Element();
        $element->setCode($paramFetcher->get('code'));
        $element->setName($paramFetcher->get('name'));
        $element->setDescription($paramFetcher->get('description'));
        $element->setAdditionalOption($paramFetcher->get('additionalOption'));
        $element->setActive($paramFetcher->get('active'));
        $element->setActiveSite($paramFetcher->get('activeSite'));
        $element->setHasSupp($paramFetcher->get('hasSupp'));
        $element->setCustom($paramFetcher->get('custom'));
        $element->setVat($paramFetcher->get('vat'));
        $element->setMaxNbrIngOptions(intval($paramFetcher->get('maxNbrIngOptions')));
        $element->setPriceSuppApp($paramFetcher->get('priceSuppApp'));
        $element->setCreatedUser('0');
        $ip = $request->getClientIp();
        $element->setCreatedIp($ip);
        $element->setUpdatedIp('');
        $element->setUpdatedUser('0');
        $em = $this->getDoctrine()->getManager();
        $family = $em->getRepository('App:CardFamily')->find(intval($paramFetcher->get('family')));
        $card = $em->getRepository('App:Card')->find(intval($paramFetcher->get('carte')));
        $element->setCard($card);
        $compositions = $paramFetcher->get('compositions');
        $optionsIngredient = $paramFetcher->get('optionIngredient');
        $supplements = $paramFetcher->get('supplements');
        if (is_array($compositions) && count($compositions) > 0) {
            foreach ($compositions as $value) {
                $composition = $em->getRepository('App:Ingredient')->find($value);
                if ($composition) {
                    $element->addElementComposition($composition);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        }

        if (is_array($optionsIngredient) && count($optionsIngredient) > 0 && count($optionsIngredient) >= intval($paramFetcher->get('maxNbrIngOptions')) && intval($paramFetcher->get('maxNbrIngOptions')) > 0) {
            foreach ($optionsIngredient as $value) {
                $optionIngredient = $em->getRepository('App:Ingredient')->find($value);
                if ($optionIngredient) {
                    $element->addElementOption($optionIngredient);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        }


        if (is_array($supplements) && count($supplements) > 0 && $paramFetcher->get('hasSupp')) {
            foreach ($supplements as $value) {
                $supplement = $em->getRepository('App:Ingredient')->find(intval($value));
                if ($supplement) {
                    $element->addElementSupplement($supplement);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        }

        $sizes = $paramFetcher->get('sizes');
        if (is_array($sizes)) {
            foreach ($sizes as $value) {
                if (array_key_exists('id', $value)) {
                    $elementSize = new ElementSize();
                    $size = $em->getRepository('App:Size')->find($value['id']);
                    if ($size) {
                        $elementSize->setSize($size);
                        $elementSize->setElement($element);
                        $elementSize->setAvailable($value['available']);
                        $elementSize->setPriceSupp($value['priceSupp']);
                        $elementSize->setPrice($value['price']);
                        $em->persist($elementSize);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    $response = array(
                        'code' => 0,
                        'message' => 'Champs requis: priceSupp et id',
                        'error' => null,
                        'result' => null
                    );
                    return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 4], Response::HTTP_BAD_REQUEST);
        }

        $orderTypes = $paramFetcher->get('authorizedMode');
        if (is_array($orderTypes)) {
            foreach ($orderTypes as $value) {
                if (array_key_exists('id', $value) && array_key_exists('supplement', $value) && array_key_exists('active', $value)) {
                    $elementOrderType = new ElementOrderType();
                    $orderType = $em->getRepository('App:OrderType')->find($value['id']);
                    if ($orderType) {
                        $elementOrderType->setOrderType($orderType);
                        $elementOrderType->setElement($element);
                        $elementOrderType->setSupplement($value['supplement']);
                        $elementOrderType->setActive($value['active']);
                        $em->persist($elementOrderType);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    $response = array(
                        'code' => 0,
                        'message' => 'Champs requis: supplement et id et active',
                        'error' => null,
                        'result' => null
                    );
                    return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 5], Response::HTTP_BAD_REQUEST);
        }

        if ($paramFetcher->get('avatar') != '') {
            $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
            $f = finfo_open();
            $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
            $split = explode('/', $mime_type);
            $type = $split[1];
            $filename = md5(uniqid()) . '.' . $type;
            $path = $this->getParameter('products_directory') . '/' . $filename;
            $path2 = $this->getParameter('products_directory') . '/platform/' . $filename;
            $element->setAvatar($filename);
            file_put_contents($path, $decoded);
            $optimizerChain = OptimizerChainFactory::create();
            $optimizerChain->optimize($path);
            exec('cwebp -q 100 ' . $path . ' -o ' . $path . '.webp');
            exec('cp ' . $path .' '. $path2);
            exec('convert '.$path2.' -resize 75x50\> '.$path2);
        } else {
            $element->setAvatar('');
        }
        $em->persist($element);
        $element->setFamily($family);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);

    }

    /**
     * @Rest\Get("/elements/carte/{idCard}/{count_per_page}/{current_page}/{key}",defaults={"key" = ""})
     */
    public function getListAction($idCard, $current_page, $count_per_page, $key)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $data = [];
        $data['count_per_page'] = intval($count_per_page);
        $data['current_page'] = intval($current_page);
        $range_start = (intval($data['current_page']) - 1) * intval($data['count_per_page']);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $qb->select("e")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->leftJoin('f.category', 'cat')
            ->innerJoin('e.card', 'c')
            ->where('c.id  =:card')
            ->andWhere('e.name LIKE :key OR cat.label LIKE :key OR f.label LIKE :key')
            ->setParameter('card', $idCard)
            ->setParameter('key', '%' . $key . '%')
            ->orderBy('e.name')
            ->getQuery()->getResult();
        $query = $qb->getQuery()->setFirstResult($range_start)->setMaxResults($data['count_per_page']);
        $paginator = new Paginator($query, false);
        $data['data'] = $query->execute();
        $data['total_page'] = $paginator->count();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        $response = new Response();
        $response->setContent($data);
        return $response;
    }

    /**
     * @Rest\Get("/elements/unavailable/carte/{idCard}")
     */
    public function getUnavailableElements($idCard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id,e.name,s.label as size,es.id as idSize,es.available,es.price,es.priceSupp")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('e.card', 'c')
            ->innerJoin('es.size', 's')
            ->where('es.available=false')
            ->andWhere('c.id = :card')
            ->setParameter('card', $idCard)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Delete("/elements/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $p = $em->getRepository('App:Element')->findOneBy(array('id' => $id, 'card' => $card));
        } else {
            $p = $em->getRepository('App:Element')->find($id);
        }
        if (!is_null($p)) {
            $em->remove($p);
            $em->flush();
            $avatar = $this->getParameter('products_directory') . "/" . $p->getAvatar();
            $avatarWebp = $this->getParameter('products_directory') . "/" . $p->getAvatar() . 'webp';
            if (file_exists($avatar) && is_file($avatar)) unlink($avatar);
            if (file_exists($avatarWebp) && is_file($avatarWebp)) unlink($avatarWebp);
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        }
        return new JsonResponse([
            'success' => true,
            'code' => 204
        ]);
    }

    /**
     * @Rest\Get("/elements/size/{id}")
     */
    public function getElementSize($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id,e.name,s.label as size,es.id as idSize,es.available,es.price,es.priceSupp,c.id as card")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('e.card', 'c')
            ->innerJoin('es.size', 's')
            ->where('es.id=:id');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $elements = $elements->andWhere('c.id=:card')
                ->setParameter('card', $this->getUser()->getPdv());
        }
        $elements = $elements->setParameter('id', $id)
            ->getQuery()->getResult();
        if (count($elements) == 1) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($elements[0], 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
    }

    /**
     * @Rest\Get("/elements/{id}")
     */
    public function Action($id)
    {
        $element = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $element = $em->getRepository('App:Element')->findOneBy(array('id' => $id, 'card' => $card));
        } else {
            $element = $em->getRepository('App:Element')->find($id);
        }
        if (!is_null($element)) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($element, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
    }

    /**
     * @Rest\Put("/elements/{id}")
     * @RequestParam(name="code", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="avatar", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="name", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="additionalOption", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="active", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="activeSite", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="hasSupp", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="vat", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="maxNbrIngOptions", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="priceSuppApp", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="family", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="supplements", nullable=false,allowBlank=true,strict=true)
     * @RequestParam(name="sizes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="compositions", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="category", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="authorizedMode", nullable=false,allowBlank=true,strict=true)
     * @RequestParam(name="optionIngredient", nullable=false,allowBlank=true,strict=true)
     * @RequestParam(name="custom", nullable=false,allowBlank=true,strict=true)
     */
    public function updateAction(request $request, ParamFetcher $paramFetcher, $id)
    {
        $element = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $element = $em->getRepository('App:Element')->findOneBy(array('id' => $id, 'card' => $card));
        } else {
            $element = $em->getRepository('App:Element')->find($id);
        }
        if (!is_null($element)) {
            $element->setCode($paramFetcher->get('code'));
            $element->setName($paramFetcher->get('name'));
            $element->setDescription($paramFetcher->get('description'));
            $element->setAdditionalOption($paramFetcher->get('additionalOption'));
            $element->setActive($paramFetcher->get('active'));
            $element->setActiveSite($paramFetcher->get('activeSite'));
            $element->setVat($paramFetcher->get('vat'));
            $element->setMaxNbrIngOptions($paramFetcher->get('maxNbrIngOptions'));
            $element->setHasSupp($paramFetcher->get('hasSupp'));
            $element->setCustom($paramFetcher->get('custom'));
            $element->setPriceSuppApp($paramFetcher->get('priceSuppApp'));
            $element->setCreatedUser('0');
            $ip = $request->getClientIp();
            $element->setUpdatedUser('0');
            $element->setUpdatedIp($ip);
            $em = $this->getDoctrine()->getManager();
            $family = $em->getRepository('App:CardFamily')->find(intval($paramFetcher->get('family')));
            $compositions = $paramFetcher->get('compositions');
            $optionsIngredient = $paramFetcher->get('optionIngredient');
            $supplements = $paramFetcher->get('supplements');
            $element->setElementComposition([]);
            $element->setElementOption([]);
            $element->setElementSupplement([]);


            //////////************************************************************************//////////////////


            if (is_array($compositions) && count($compositions) > 0) {
                foreach ($compositions as $value) {
                    $composition = $em->getRepository('App:Ingredient')->find($value);
                    if ($composition) {
                        $element->addElementComposition($composition);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                }
            }
            /////////************************************************************************//////////////////

            if (is_array($optionsIngredient) && count($optionsIngredient) > 0 && count($optionsIngredient) >= intval($paramFetcher->get('maxNbrIngOptions')) && intval($paramFetcher->get('maxNbrIngOptions')) > 0) {
                foreach ($optionsIngredient as $value) {
                    $optionIngredient = $em->getRepository('App:Ingredient')->find($value);
                    if ($optionIngredient) {
                        $element->addElementOption($optionIngredient);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                }
            }

/////////************************************************************************//////////////////


            if (is_array($supplements) && count($supplements) > 0 && $paramFetcher->get('hasSupp')) {
                foreach ($supplements as $value) {
                    $supplement = $em->getRepository('App:Ingredient')->find(intval($value));
                    if ($supplement) {
                        $element->addElementSupplement($supplement);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                }
            }
/////////************************************************************************//////////////////
            $sizes = $this->getElementSizes($id);
            $sizes2 = $paramFetcher->get('sizes');
            $sizes3 = new ArrayCollection();

            if (is_array($sizes2)) {
                foreach ($sizes2 as $value) {
                    if (array_key_exists('id', $value)) {
                        $elementSize = new ElementSize();
                        $size = $em->getRepository('App:Size')->find($value['id']);
                        if ($size) {
                            $elementSize->setSize($size);
                            $elementSize->setElement($element);
                            $elementSize->setAvailable($value['available']);
                            $elementSize->setPriceSupp($value['priceSupp']);
                            $elementSize->setPrice($value['price']);
                            $sizes3[] = $elementSize;
                            //  $em->persist($elementSize);
                        } else {
                            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                        }
                    } else {
                        $response = array(
                            'code' => 0,
                            'message' => 'Champs requis: priceSupp et id',
                            'error' => null,
                            'result' => null
                        );
                        return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse(['error' => 4], Response::HTTP_BAD_REQUEST);
            }

            foreach ($sizes as $value) {
                $index = $this->findIndex($sizes3, $value, 2);
                if ($index > -1) {
                    $value->setAvailable($sizes3[$index]->getAvailable());
                    $value->setPriceSupp($sizes3[$index]->getPriceSupp());
                    $value->setPrice($sizes3[$index]->getPrice());
                    $em->persist($value);
                } else {
                    $em->remove($value);
                }
            }

            foreach ($sizes3 as $value) {
                $index = $this->findIndex($sizes, $value, 2);
                if ($index == -1) {
                    $em->persist($value);
                }
            }


/////////************************************************************************//////////////////
            $orderTypes = $paramFetcher->get('authorizedMode');
            if (is_array($orderTypes)) {
                foreach ($orderTypes as $value) {
                    if (array_key_exists('id', $value) && array_key_exists('idElementOrderType', $value) && array_key_exists('supplement', $value) && array_key_exists('active', $value)) {
                        $elementOrderType = $em->getRepository('App:ElementOrderType')->find($value['idElementOrderType']);
                        if (!is_null($elementOrderType)) {
                            $elementOrderType->setSupplement($value['supplement']);
                            $elementOrderType->setActive($value['active']);
                            $em->persist($elementOrderType);
                        } else {
                            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                        }
                    } else {
                        $response = array(
                            'code' => 0,
                            'message' => 'Champs requis: supplement et id',
                            'error' => null,
                            'result' => null
                        );
                        return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse(['error' => 5], Response::HTTP_BAD_REQUEST);
            }

            if ($paramFetcher->get('avatar') != '') {
                if (substr($paramFetcher->get('avatar'), 0, 4) == 'data') {
                    $avatar = $this->getParameter('products_directory') . "/" . $element->getAvatar();
                    $avatarWebp = $this->getParameter('products_directory') . "/" . $element->getAvatar() . 'webp';
                    if (file_exists($avatar) && is_file($avatar)) unlink($avatar);
                    if (file_exists($avatarWebp) && is_file($avatarWebp)) unlink($avatarWebp);
                    $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
                    $f = finfo_open();
                    $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
                    $split = explode('/', $mime_type);
                    $type = $split[1];
                    $filename = md5(uniqid()) . '.' . $type;
                    $path = $this->getParameter('products_directory') . '/' . $filename;
                    $path2 = $this->getParameter('products_directory') . '/platform/' . $filename;
                    $element->setAvatar($filename);
                    file_put_contents($path, $decoded);
                    $optimizerChain = OptimizerChainFactory::create();
                    $optimizerChain->optimize($path);
                    exec('cwebp -q 100 ' . $path . ' -o ' . $path . '.webp');
                    exec('cp ' . $path .' '. $path2);
                    exec('convert '.$path2.' -resize 75x50\> '.$path2);
                }
            } else {
                if ($element->getAvatar() != '') {
                    $avatar = $this->getParameter('products_directory') . "/" . $element->getAvatar();
                    $avatar2 = $this->getParameter('products_directory') . "/platform/" . $element->getAvatar();
                    if (file_exists($avatar))unlink($avatar);
                    if (file_exists($avatar2))unlink($avatar2);
                    $element->setAvatar('');
                }
            }

            $em->persist($element);
            $element->setFamily($family);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse(null, 204);
        }
    }

    /**
     * @Rest\Put("/elements/{id}/status")
     * @RequestParam(name="available", nullable=false,allowBlank=false,strict=false)
     */
    public function updateElementSize(request $request, ParamFetcher $paramFetcher, $id, SendEvent $sendEvent)
    {
        $em = $this->getDoctrine()->getManager();
        $elementSize = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $qb = $em->createQueryBuilder();
            $elementSize = $qb->select("es")
                ->from("App:ElementSize", "es")
                ->innerJoin('es.element', 'e')
                ->innerJoin('e.card', 'c')
                ->where('es.id = :idEs')
                ->andWhere('c.id = :card')
                ->setParameter('idEs', $id)
                ->setParameter('card', $this->getUser()->getPdv())
                ->setMaxResults(1)
                ->getQuery()
                ->getOneOrNullResult();
        } else {
            $elementSize = $this->getDoctrine()->getRepository("App:ElementSize")->find(intval($id));
        }
        if (!is_null($elementSize)) {
            $elementSize->setAvailable($paramFetcher->get('available'));
            $em->persist($elementSize);
            $em->flush();
            $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'elementSize', 'id' => $id]);
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse(null, 204);
        }
    }

    /**
     * @Rest\Put("/elements/{id}/status/site")
     * @RequestParam(name="active", nullable=false,allowBlank=false,strict=false)
     */
    public function updateElementSiteActive(request $request, ParamFetcher $paramFetcher, $id, SendEvent $sendEvent)
    {
        $em = $this->getDoctrine()->getManager();
        $element= null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $qb = $em->createQueryBuilder();
            $element = $qb->select("e")
                ->from("App:Element", "e")
                ->innerJoin('e.card', 'c')
                ->where('e.id = :idE')
                ->andWhere('c.id = :card')
                ->setParameter('idE', $id)
                ->setParameter('card', $this->getUser()->getPdv())
                ->setMaxResults(1)
                ->getQuery()
                ->getOneOrNullResult();
        } else {
            $element = $this->getDoctrine()->getRepository("App:Element")->find(intval($id));
        }
        if (!is_null($element)) {
            $element->setActiveSite(intval($paramFetcher->get('active')));
            $em->persist($element);
            $em->flush();
            $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'element', 'id' => $id]);
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse(null, 204);
        }
    }

    /**
     * @Rest\Post("/elements/copy/{idElement}/number/{number}")
     */
    public function CopyProduct(request $request, $idElement, $number)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $element = null;
        $card = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $element = $em->getRepository('App:Element')->findOneBy(array('id' => $idElement, 'card' => $card));
        } else {
            $element = $this->getDoctrine()->getRepository("App:Element")->find($idElement);
            $card = $qb->select("c")
                ->from("App:Card", "c")
                ->innerJoin('c.elements', 'el')
                ->where('el.id = :element')
                ->setParameter('element', $idElement)
                ->setMaxResults(1)
                ->getQuery()
                ->getOneOrNullResult();
        }
        if (!is_null($element)) {
            $newElementsIds = [];
            for ($i = 0; $i < $number; $i++) {
                $nelement = new Element();
                $nelement->setCode($element->getCode());
                $nelement->setAvatar($element->getAvatar());
                $nelement->setName($element->getName() . ($i + 1));
                $nelement->setDescription($element->getDescription());
                $nelement->setAdditionalOption($element->getAdditionalOption());
                $nelement->setActive($element->getActive());
                $nelement->setActiveSite($element->getActiveSite());
                $nelement->setHasSupp($element->getHasSupp());
                $nelement->setCustom($element->getCustom());
                $nelement->setVat($element->getVat());
                $nelement->setMaxNbrIngOptions($element->getMaxNbrIngOptions());
                $nelement->setPriceSuppApp($element->getPriceSuppApp());
                $nelement->setCard($card);
                $nelement->setCreatedUser('0');
                $ip = $request->getClientIp('');
                $nelement->setUpdatedUser('0');
                $nelement->setCreatedIp($ip);
                $nelement->setUpdatedIp('');
                $nelement->setFamily($element->getFamily());
                $qb = $em->createQueryBuilder();
                $qb1 = $em->createQueryBuilder();
                $qb2 = $em->createQueryBuilder();
                $qb3 = $em->createQueryBuilder();
                //////////************************************************************************//////////////////

                $compositions = $qb->select("ec.id")
                    ->from("App:Element", "e")
                    ->innerJoin('e.elementComposition', 'ec')
                    ->where('e.id = :element')
                    ->setParameter('element', $idElement)
                    ->getQuery()->getResult();

                if (is_array($compositions) && count($compositions) > 0) {
                    foreach ($compositions as $value) {
                        $composition = $em->getRepository('App:Ingredient')->find($value['id']);
                        $nelement->addElementComposition($composition);
                    }
                }


                /////////************************************************************************//////////////////

                $optionsIngredient = $qb1->select("eo.id")
                    ->from("App:Element", "el")
                    ->innerJoin('el.elementOption', 'eo')
                    ->where('el.id = :element')
                    ->setParameter('element', $idElement)
                    ->getQuery()->getResult();
                if (is_array($optionsIngredient) && count($optionsIngredient) > 0) {
                    foreach ($optionsIngredient as $value) {
                        $optionIngredient = $em->getRepository('App:Ingredient')->find($value['id']);
                        $nelement->addElementOption($optionIngredient);
                    }
                }

                /////////************************************************************************//////////////////

                $supplements = $qb2->select("esu.id")
                    ->from("App:Element", "elm")
                    ->innerJoin('elm.elementSupplement', 'esu')
                    ->where('elm.id = :element')
                    ->setParameter('element', $idElement)
                    ->getQuery()->getResult();
                if (is_array($supplements) && count($supplements) > 0) {
                    foreach ($supplements as $value) {
                        $supplement = $em->getRepository('App:Ingredient')->find(intval($value['id']));
                        $nelement->addElementSupplement($supplement);
                    }
                }

                /////////***********************************************************************//////////////////

                $sizes = $qb3->select("s.id ,s.label,es.price,es.available,es.priceSupp")
                    ->from("App:ElementSize", "es")
                    ->leftJoin('es.size', 's')
                    ->innerJoin('es.element', 'elmn')
                    ->where('elmn.id = :idElement')
                    ->setParameter('idElement', $idElement)
                    ->getQuery()->getResult();

                if (is_array($sizes)) {
                    foreach ($sizes as $value) {
                        $elementSize = new ElementSize();
                        $size = $em->getRepository('App:Size')->find($value['id']);
                        $elementSize->setSize($size);
                        $elementSize->setElement($nelement);
                        $elementSize->setAvailable($value['available']);
                        $elementSize->setPriceSupp($value['priceSupp']);
                        $elementSize->setPrice($value['price']);
                        $em->persist($elementSize);
                    }
                }
                /////////************************************************************************//////////////////

                $orderTypes = $element->getElementOrderType();
                foreach ($orderTypes as $value) {
                    $elementOrderType = new ElementOrderType();
                    $orderType = $this->getDoctrine()->getRepository("App:OrderType")->find($value->getOrderType()->getId());
                    $elementOrderType->setOrderType($orderType);
                    $elementOrderType->setElement($nelement);
                    $elementOrderType->setSupplement($value->getSupplement());
                    $elementOrderType->setActive($value->getActive());
                    $em->persist($elementOrderType);
                }
                $em->persist($nelement);
                $em->flush();
                $newElementsIds[] = $nelement->getId();
            }
            return new JsonResponse([
                'ids' => $newElementsIds,
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse(null, 204);
        }
    }


    /**
     * @Rest\Get("/elements/carte/{idCard}/{idFamily}")
     */
    public function getElementsByFamilyAndCard($idCard, $idFamily)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("e.id,e.name")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->innerJoin('e.card', 'c')
            ->where('c.id = :card')
            ->andWhere('cf.id = :cardFamily')
            ->setParameter('card', $idCard)
            ->setParameter('cardFamily', $idFamily)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/elements/family/{idFamily}")
     */
    public function getElementsByFamilyCard($idFamily)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("e.id,e.name")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->where('cf.id = :cardFamily');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $elements = $elements->innerJoin('cf.card', 'c')
                ->andWhere('c.id = :card')
                ->setParameter('card', $this->getUser()->getPdv());
        }
        $elements = $elements->setParameter('cardFamily', $idFamily)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/elements/active/family/{idFamily}/orderType/{idOrderType}")
     */
    public function getActiveElementsByFamilyCard($idFamily, $idOrderType)
    {
        $idCard = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $idCard = $this->getUser()->getPdv();
        }
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $elements = $qb->select("e.id,e.hasSupp,e.custom,e.avatar,e.code,e.name,e.active,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,e.priceSuppApp,SUM(es.available) as available")
                ->from("App:ElementSize", "es")
                ->innerJoin('es.element', 'e')
                ->innerJoin('e.family', 'cf')
                ->where('cf.id = :cardFamily');
            if (!is_null($idCard)) {
                $elements = $elements->innerJoin('e.card', 'c')
                    ->andWhere('c.id = :card')
                    ->setParameter('card', $idCard);
            }
            $elements = $elements->andWhere('e.active = :active')
                ->setParameter('cardFamily', $idFamily)
                ->setParameter('active', 1)
                ->groupBy('e.id')
                ->getQuery()
                ->getResult();
        foreach ($elements as $key => $element) {
            $elements[$key]['dispo'] = intval($this->hasOrderType($element['id'], $idOrderType)['dispo']);
            $elements[$key]['supplement'] = floatval($this->hasOrderType($element['id'], $idOrderType)['supplement']);
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/elements/active/carte/{idCarte}/orderType/{idOrderType}/key/{key}")
     */
    public function getActiveElementsByKey($key, $idOrderType, $idCarte)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCarte)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $elements = $qb->select("e.id,e.code,e.vat,e.name,e.active,1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,e.hasSupp,e.priceSuppApp,SUM(es.available) as available")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('e.card', 'c')
            ->innerJoin('e.family', 'cf')
            ->where('e.active = :active')
            ->andWhere('e.name LIKE :key or e.code LIKE :code')
            ->andWhere('c.id=:card')
            ->setParameter('key', '%' . $key . '%')
            ->setParameter('code', '%' . $key . '%')
            ->setParameter('card', $idCarte)
            ->setParameter('active', 1)
            ->groupBy('e.id')
            ->getQuery()
            ->getResult();
        foreach ($elements as $ke => $element) {
            $elements[$ke]['dispo'] = intval($this->hasOrderType($element['id'], $idOrderType)['dispo']);
            $elements[$ke]['supplement'] = floatval($this->hasOrderType($element['id'], $idOrderType)['supplement']);
        }
        $d = new \DateTime();
        $midi = new \DateTime();
        $midi->setTime(16, 0, 0);
        $em = $this->getDoctrine()->getManager();
        $qb2 = $em->createQueryBuilder();
        if ($d > $midi) {
            $menus = $qb2->select("m.id,m.vat,m.code,m.title,m.description,m.price,m.available,mt.label as menuTime,o.id as offer,
            case when mt.id = 1 then false else true end as validPeriod");
        } else {
            $menus = $qb2->select("m.id,m.vat,m.code,m.title,m.description,m.price,m.available,mt.label as menuTime,o.id as offer,
            case when mt.id = 3 or mt.id=4 then false else true end as validPeriod");
        }
        //var_dump($key,$idOrderType,$idCarte);
        $menus = $menus->from("App:Menu", "m")
            ->leftJoin('m.offer', 'co')
            ->leftJoin('co.offer', 'o')
            ->innerJoin('m.menuTime', 'mt')
            ->innerJoin('m.card', 'ca')
            ->where('ca.id = :card')
            ->andWhere('m.title LIKE :key OR m.code LIKE :code')
            ->setParameter('card', $idCarte)
            ->setParameter('key', '%' . $key . '%')
            ->setParameter('code', '%' . intval($key) . '%')
            ->groupBy('m.id')
            ->getQuery()->getResult();
        foreach ($menus as $k => $menu) {
            $menus[$k]['dispo'] = intval($this->menuHasOrderType($menu['id'], $idOrderType)['dispo']);
            $menus[$k]['supplement'] = floatval($this->menuHasOrderType($menu['id'], $idOrderType)['supplement']);
        }

        $d = array('elements' => $elements, 'menus' => $menus);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($d, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Post("/elements/shearch/ByCompositions")
     * @RequestParam(name="ingredients", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="idOrderType", nullable=false,allowBlank=false,strict=true)
     */
    public function getElements(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $result = new ArrayCollection();
        $ingredients = $paramFetcher->get('ingredients');
        $idOrderType = intval($paramFetcher->get('idOrderType'));
        foreach ($ingredients as $key1 => $ing) {
            $qb = $em->createQueryBuilder();
            $elements = $qb->select("e.priceSuppApp,e.id,e.vat,e.code,e.name,e.active,e.hasSupp,1 as qty,e.description,e.maxNbrIngOptions,SUM(es.available) as available")
                ->from("App:ElementSize", "es")
                ->innerJoin("es.element", "e");
            if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
                $elements = $elements->innerJoin('e.card', 'c')
                    ->where('c.id=:card')
                    ->setParameter('card', $this->getUser()->getPdv());
            }
            $elements = $elements->innerJoin('e.elementComposition', 'ec')
                ->where('ec.id=:ing')
                ->andwhere('e.active=1')
                ->setParameter('ing', $ing)
                ->groupBy('e.id')
                ->getQuery()->getResult();
            foreach ($elements as $key => $element) {
                $elements[$key]['dispo'] = intval($this->hasOrderType($element['id'], $idOrderType)['dispo']);
                $elements[$key]['supplement'] = intval($this->hasOrderType($element['id'], $idOrderType)['supplement']);
            }

            if ($key1 == 0) {
                $result = $elements;
            } else {
                $result = array_intersect(array_map('serialize', $result), array_map('serialize', $elements));
                $result = array_map('unserialize', $result);
            }
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize(array_values($result), 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/elements/additionalOptions/carte/{idCard}")
     */
    public function getAdditionalOptions($idCard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id as idElementSize,e.name,s.label as size")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('es.size', 's')
            ->innerJoin('e.card', 'c')
            ->where('c.id=:card')
            ->andWhere('e.additionalOption=:additionalOption')
            ->setParameter('card', $idCard)
            ->setParameter('additionalOption', 1)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/elements/carte/{idCard}/family/{idFamily}/size/{idSize}")
     */
    public function getElementsByFamilyAndCardAndSize($idCard, $idFamily, $idSize)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id,e.name,s.label as size")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('es.size', 's')
            ->innerJoin('e.family', 'cf')
            ->innerJoin('e.card', 'c')
            ->where('c.id = :card')
            ->andWhere('cf.id = :cardFamily')
            ->andWhere('s.id = :size')
            ->setParameter('card', $idCard)
            ->setParameter('cardFamily', $idFamily)
            ->setParameter('size', $idSize)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    public function findIndex($array, $object, $key)
    {
        $index = -1;
        switch ($key) {
            case 2:
                {
                    foreach ($array as $key => $value) {
                        if ($value->getSize()->getId() == $object->getSize()->getId()) {
                            $index = $key;
                            break;
                        }
                    }
                    break;
                }

        }
        return $index;
    }

    public function getElementSizes($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $element = $em->getRepository('App:Element')->findOneBy(array('id' => $idElement, 'card' => $card));
        } else {
            $element = $this->getDoctrine()->getRepository("App:Element")->find($idElement);
        }
        if (!is_null($element)) {
            $sizes = $this->getDoctrine()->getRepository("App:ElementSize")->findBy(array("element" => $element));
            return $sizes;
        } else {
            return [];
        }

    }
    /////////////////////////////////////////////////////////////////////// api restaurant /////////////////////////////////////////////////////

    /**
     * @Rest\Get("/restaurant/elements/active/family/{idFamily}/orderType/{idOrderType}")
     */
    public function getRestaurantActiveElementsByFamilyCard($idFamily, $idOrderType)
    {
        $idCard = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $idCard = $this->getUser()->getPdv();
        }
        $elements = $this->getDoctrine()->getRepository("App:Element")->getActiveElementsByFamilyCardAndOrderType($idFamily, $idOrderType, $idCard,false);
        foreach ($elements as $key => $element) {
            $elements[$key]['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
            $elements[$key]['compositions'] = $this->getDoctrine()->getRepository("App:Ingredient")->getIngredientComposByEl($element['id']);
            $elements[$key]['size'] = 0;
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/restaurant/elements/active/carte/{idCarte}/orderType/{idOrderType}/key/{key}")
     */
    public function getRestaurantActiveElementsByOrderAndKey($idCarte, $key, $idOrderType)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCarte)) {
                return new JsonResponse(null, 403);
            }
        }
        $elements = $this->getDoctrine()->getRepository("App:Element")->getActiveElementsAndKey($key, $idOrderType);
        $menus = $this->getDoctrine()->getRepository("App:Element")->getMenusByKey($key, $idOrderType, $idCarte);
        foreach ($elements as $key => $element) {
            //$elements[$key]['dispo']=intval($this->hasOrderType($element['id'],$idOrderType)['dispo']);
            // $elements[$key]['supplement']=intval($this->hasOrderType($element['id'],$idOrderType)['supplement']);
            $elements[$key]['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
            $elements[$key]['compositions'] = $this->getDoctrine()->getRepository("App:Ingredient")->getIngredientComposByEl($element['id']);
            $elements[$key]['size'] = 0;
        }
        $da['elements'] = $elements;
        $da['menus'] = $menus;
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($da, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Post("/restaurant/elements/search/compositions")
     * @RequestParam(name="ingredients", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="idOrderType", nullable=false,allowBlank=false,strict=true)
     */
    public function getElementsSiteByComposition(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $result = new ArrayCollection();
        $ingredients = $paramFetcher->get('ingredients');
        $idOrderType = intval($paramFetcher->get('idOrderType'));
        foreach ($ingredients as $key1 => $ing) {
            $qb = $em->createQueryBuilder();
            $elements = $qb->select("e.priceSuppApp,e.id,e.hasSupp,e.avatar,e.code,e.name,eot.supplement,e.active,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp")
                ->from("App:Element", "e")
                ->innerJoin('e.family', 'cf')
                ->innerJoin('cf.family', 'f')
                ->innerJoin('e.elementOrderType', 'eot')
                ->innerJoin('eot.orderType', 'ot')
                ->innerJoin('e.elementComposition', 'ec')
                ->where('e.active=:active');
            if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
                $elements = $elements->innerJoin('e.card', 'c')
                    ->where('c.id = :card')
                    ->setParameter('card', $this->getUser()->getPdv());
            }
            $elements = $elements->andWhere('ot.id=:orderType')
                ->andWhere('eot.active=:active')
                ->andWhere('ec.id=:ing')
                ->setParameter('orderType', $idOrderType)
                ->setParameter('active', 1)
                ->setParameter('ing', $ing)
                ->getQuery()->getResult();
            foreach ($elements as $key => $element) {
                $elements[$key]['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
                $elements[$key]['compositions'] = $this->getDoctrine()->getRepository("App:Ingredient")->getIngredientComposByEl($element['id']);
                $elements[$key]['size'] = 0;
            }
            if ($key1 == 0) {
                $result = $elements;
            } else {
                $result = array_intersect(array_map('serialize', $result), array_map('serialize', $elements));
                $result = array_map('unserialize', $result);
            }
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize(array_values($result), 'json');
        return new JsonResponse(json_decode($data));
    }

    ////////////////////////////////////////////////////////////////////////// functions ///////////////////////////////////////////////////////
    public function hasOrderType($idElement, $idOrderType)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $result = $qb->select("count(ot.id) as dispo,eot.supplement")
            ->from("App:Element", "e")
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->where('e.id = :idElement')
            ->andWhere('ot.id = :idOrderType')
            ->andWhere('eot.active = 1')
            ->setParameter('idOrderType', $idOrderType)
            ->setParameter('idElement', $idElement)
            ->getQuery()
            ->getSingleResult();
        return $result;
    }

    public function menuHasOrderType($idMenu, $idOrderType)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $result = $qb->select("count(ot.id) as dispo,mot.supplement")
            ->from("App:Menu", "m")
            ->innerJoin('m.menuOrderType', 'mot')
            ->innerJoin('mot.orderType', 'ot')
            ->where('m.id = :idMenu')
            ->andWhere('ot.id = :idOrderType')
            ->andWhere('mot.active = 1')
            ->setParameter('idOrderType', $idOrderType)
            ->setParameter('idMenu', $idMenu)
            ->getQuery()
            ->getSingleResult();
        return $result;
    }

}
