<?php
/**
 * Created by PhpStorm.
 * User: HP
 * Date: 16/02/2018
 * Time: 11:34
 */

namespace App\Controller\Rest;

use Doctrine\ORM\Query;
use FOS\RestBundle\Controller\FOSRestController;
use App\Entity\Family;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;

class FamilyController extends FOSRestController
{

    /**
     * @Rest\Post("/families")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="label", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="category", nullable=true,allowBlank=true,strict=true)
     */
    public function addAction(ParamFetcher $paramFetcher)
    {
            $family = new Family();
            $family->setLabel($paramFetcher->get('label'));
            $family->setDescription($paramFetcher->get('description'));
            $family->setActive($paramFetcher->get('active'));
            $em = $this->getDoctrine()->getManager();
            $em->persist($family);
            $category = $em->getRepository('App:Category')->find($paramFetcher->get('category'));
            $family->setCategory($category);
            $em->flush();
            return new JsonResponse([
                'id'=>$family->getId(),
                'success' => true,
                'code' => 200
            ]);
    }

    /**
     * @Rest\Get("/families")
     */
    public function getListAction()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder('t');
        $families = $qb->select("f.id,f.label,c.label as category,f.description,f.active")
            ->from("App:Family", "f")
            ->innerJoin('f.category', 'c')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/families/{id}")
     */
    public function getFamily($id)
    {
        $em = $this->getDoctrine()->getManager();
        $family = $em->getRepository('App:Family')->find($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($family, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Delete("/families/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = $em->getRepository('App:Family')->find($id);
        $em->remove($p);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/families/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="label", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="category", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAction(ParamFetcher $paramFetcher, $id)
    {
        $family = $this->getDoctrine()->getRepository("App:Family")->find($id);
        $family->setLabel($paramFetcher->get('label'));
        $family->setDescription($paramFetcher->get('description'));
        $family->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $em->persist($family);
            $category = $em->getRepository('App:Category')->find($paramFetcher->get('category'));
            $family->setCategory($category);
        $em->persist($family);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);

    }

    /**
     * @Rest\Put("/families/status/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="label", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="category", nullable=true,allowBlank=true,strict=true)
     */
    public function updateActive(ParamFetcher $paramFetcher, $id)
    {
        $family = $this->getDoctrine()->getRepository("App:Family")->find($id);
        $family->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $em->persist($family);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);

    }



    /**
     * @Rest\Get("/families/carte/{id}")
     */
    public function getfamillesCat($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder('t');
        $families = $qb->select("cf.id,f.label,c.label as category")
            ->from("App:CardFamily", "cf")
            ->innerJoin('cf.card', 'ca')
            ->innerJoin('cf.family', 'f')
            ->innerJoin('f.category', 'c')
            ->where('ca.id = :idCard')
            ->setParameter('idCard', $id)
            ->getQuery()->getResult();
        $result = array();
        foreach ($families as $element) {
            $index=$this->findIndex($result,$element['category']);
            if($index>-1){
                $result[$index]['families'][] = $element;
            }else{
                $temp = array();
                $temp['category'] = $element['category'];
                $temp['families'][] = $element;
                $result[] = $temp;
            }
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/families/cmd/carte/{idCarte}")
     */
    public function getfamiliesByPdvAndCard($idCarte)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("cf.id,f.label")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->innerJoin('e.card', 'c')
            ->where('c.id = :card')
            ->andWhere('e.active =1')
            ->setParameter('card', $idCarte)
            ->groupBy('cf.id')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/families/carte/{idCard}/category/{idCategory}")
     */
    public function getCardFamiliesByCat($idCard, $idCategory)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("cf.id,f.label")
            ->from("App:CardFamily", "cf")
            ->innerJoin('cf.family', 'f')
            ->leftJoin('f.category', 'cat')
            ->innerJoin('cf.card', 'C')
            ->where('C.id = :card')
            ->andWhere('cat.id = :category')
            ->setParameter('card', $idCard)
            ->setParameter('category', $idCategory)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }

    public function findIndex($array, $category)
    {
        $index = -1;
        foreach ($array as $key => $value) {
            if ($value['category'] == $category) {
                $index = $key;
                break;
            }
        }
        return $index;
    }

}
