<?php


namespace App\Controller\Rest;

use App\Service\SendEvent;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Ingredient;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;

class IngredientController extends FOSRestController
{

    /**
     * @Rest\Post("/ingredients")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="price", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="available", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="carte", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="ingredientCategory", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="avatar", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="activeSite", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(Request $request, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {

        $idCarte = intval($paramFetcher->get('carte'));
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != $idCarte) {
                return new JsonResponse(null, 403);
            }
        }
        $ingredient = new Ingredient();
        $ingredient->setName($paramFetcher->get('name'));
        $ingredient->setDescription($paramFetcher->get('description'));
        $ingredient->setPrice($paramFetcher->get('price'));
        $ingredient->setAvailable($paramFetcher->get('available'));
        $ip = $request->getClientIp();
        $ingredient->setCreatedIp($ip);
        $ingredient->setCreatedUser('0');
        $ingredient->setUpdatedIp('');
        $ingredient->setUpdatedUser('0');
        $ingredient->setActiveSite(intval($paramFetcher->get('activeSite')));
        $em = $this->getDoctrine()->getManager();
        $em->persist($ingredient);
        $card = $em->getRepository('App:Card')->find($idCarte);
        $ingredient->setcard($card);
        $ingredientCategorie = $em->getRepository('App:IngredientCategory')->find(intval($paramFetcher->get('ingredientCategory')));
        $ingredient->setIngredientCategory($ingredientCategorie);
        $ingredient->setFamilies([]);
        foreach ($paramFetcher->get('families') as $value) {
            $cardFamily = $em->getRepository('App:CardFamily')->find($value);
            $ingredient->addFamily($cardFamily);
        }
        if ($paramFetcher->get('avatar') != '') {
            $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
            $f = finfo_open();
            $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
            $split = explode('/', $mime_type);
            $type = $split[1];
            $filename = md5(uniqid()) . '.' . $type;
            $path = $this->getParameter('ingredients_directory') . '/' . $filename;
            $ingredient->setAvatar($filename);
            file_put_contents($path, $decoded);
            exec('cwebp -q 100 ' . $path . ' -o ' . $path . '.webp');
        } else {
            $ingredient->setAvatar('');
        }
        $em->flush();
        $id = $ingredient->getId();
        $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'ingredient', 'id' => $id]);

        return new JsonResponse([
            'id' => $id,
            'avatar' => $ingredient->getAvatar(),
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/ingredients")
     */
    public function getListAction()
    {
        $ingredients = $this->getDoctrine()->getRepository("App:Ingredient")->findAll();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        $response = new Response();
        $response->setContent($data);
        return $response;
    }

    /**
     * @Rest\Get("/ingredients/carte/{idcard}")
     */
    public function getListIngcardAction($idcard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != $idcard) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredients = $qb->select("i.avatar,i.activeSite,i.id,i.price,i.description,i.available,i.name,C.id as card,IC.id as idCAT,IC.name as category")
            ->from("App:Ingredient", "i")
            ->innerJoin('i.card', 'C')
            ->innerJoin('i.ingredientCategory', 'IC')
            ->where('C.id = :card')
            ->setParameter('card', $idcard)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        return new JsonResponse(json_decode($data));

    }


    /**
     * @Rest\Get("/unavailableIngredients/carte/{idcard}")
     */
    public function getListIngcardUnaiv($idcard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != $idcard) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredients = $qb->select("i.avatar,i.id,i.price,i.description,i.available,i.name,C.id as card,IC.id as idCAT,IC.name as category")
            ->from("App:Ingredient", "i")
            ->innerJoin('i.card', 'C')
            ->innerJoin('i.ingredientCategory', 'IC')
            ->where('C.id = :card')
            ->andWhere('i.available = 0')
            ->setParameter('card', $idcard)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        return new JsonResponse(json_decode($data));

    }


    /**
     * @Rest\Get("/ingredients/carte/{idcard}/family/{idFamily}")
     */
    public function getIngredientsOfcardByFamille($idcard, $idFamily)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != $idcard) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredients = $qb->select("i.id,i.name as text,IC.name as category")
            ->from("App:Ingredient", "i")
            ->innerJoin('i.card', 'C')
            ->innerJoin('i.families', 'F')
            ->innerJoin('i.ingredientCategory', 'IC')
            ->where('C.id = :card')
            ->andWhere('F.id = :family')
            ->setParameter('card', $idcard)
            ->setParameter('family', $idFamily)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Get("/ingredients/compositions/carte/{idCarte}")
     */
    public function getIngredientComposByCarte($idCarte)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != $idCarte) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredients = $qb->select("ec.id,ec.name as text")
            ->from("App:Element", "e")
            ->innerJoin('e.elementComposition', 'ec')
            ->innerJoin('e.card', 'c')
            ->Where('c.id=:card')
            ->setParameter('card', $idCarte)
            ->groupBy('ec.id')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/ingredients/carte2/{idcard}")
     */
    public function getIngredientsOfcard($idcard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != $idcard) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredients = $qb->select("i.id,i.name as text,IC.name as category")
            ->from("App:Ingredient", "i")
            ->innerJoin('i.card', 'C')
            ->innerJoin('i.ingredientCategory', 'IC')
            ->where('C.id = :card')
            ->setParameter('card', $idcard)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Get("/ingredients/supplements/carte/{idcard}")
     */
    public function getSupplementsOfcard($idcard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != $idcard) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $categories = $qb->select("ic.name,ic.id")
            ->from("App:Ingredient", "i")
            ->innerJoin('i.card', 'c')
            ->innerJoin('i.ingredientCategory', 'ic')
            ->where('c.id = :card')
            ->setParameter('card', $idcard)
            ->groupBy('ic.id')
            ->getQuery()->getResult();
        foreach ($categories as $key => $categorie) {
            $qb = $em->createQueryBuilder();
            $ingredients = $qb->select("i.id,i.name")
                ->from("App:Ingredient", "i")
                ->innerJoin('i.card', 'c')
                ->innerJoin('i.ingredientCategory', 'ic')
                ->where('c.id = :card')
                ->andWhere('ic.id = :ingredientCategory')
                ->setParameter('card', $idcard)
                ->setParameter('ingredientCategory', $categorie['id'])
                ->getQuery()->getResult();
            $categories[$key]['ingredients'] = $ingredients;
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($categories, 'json');
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Delete("/ingredients/{id}")
     */
    public function deleteAction($id)
    {
        $p = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $p = $em->getRepository('App:Ingredient')->findOneBy(Array('id' => $id, 'card' => $card));

        } else {
            $p = $em->getRepository('App:Ingredient')->find($id);
        }
        if (is_null($p)) {
            return new JsonResponse(null, 403);
        }
        $em->remove($p);
        $em->flush();
        $avatar = $this->getParameter('ingredients_directory') . "/" . $p->getAvatar();
        $avatarWebp = $this->getParameter('ingredients_directory') . "/" . $p->getAvatar() . 'webp';
        if (file_exists($avatar) && is_file($avatar)) unlink($avatar);
        if (file_exists($avatarWebp) && is_file($avatarWebp)) unlink($avatarWebp);
        /*$client = new Client(new Version1X('//127.0.0.1:1337'));
        $client->initialize();
        $client->emit('notif', ['type' => 'remove','entity' => 'ingredient', 'id' => $id]);
        $client->close();*/
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/ingredients/{id}")
     */
    public function getAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredient = $qb->select("i as ingredient,c.id as card")
            ->from("App:Ingredient", "i")
            ->innerJoin("i.card", "c")
            ->where('i.id = :id');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $ingredient = $ingredient->andWhere('c.id=:card')->setParameter('card', $this->getUser()->getPdv());
        }
        $ingredient = $ingredient->setParameter('id', $id)
            ->getQuery()->getResult()[0];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredient, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/ingredients/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="price", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="available", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="ingredientCategory", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="avatar", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="activeSite", nullable=false,allowBlank=false,strict=true)
     */
    public function updateAction(Request $request, $id, ParamFetcher $paramFetcher)
    {
        $ingredient = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $ingredient = $em->getRepository('App:Ingredient')->findOneBy(Array('id' => $id, 'card' => $card));

        } else {
            $ingredient = $em->getRepository('App:Ingredient')->find($id);
        }
        if (is_null($ingredient)) {
            return new JsonResponse(null, 403);
        }
        $em = $this->getDoctrine()->getManager();
        $ingredient->setName($paramFetcher->get('name'));
        $ingredient->setDescription($paramFetcher->get('description'));
        $ingredient->setPrice($paramFetcher->get('price'));
        $ingredient->setAvailable($paramFetcher->get('available'));
        $ip = $request->getClientIp();
        $ingredient->setUpdatedIp($ip);
        $ingredient->setUpdatedUser($this->getUser()->getId());
        $ingredient->setActiveSite(intval($paramFetcher->get('activeSite')));
        $ingredientCategorie = $em->getRepository('App:IngredientCategory')->find(intval($paramFetcher->get('ingredientCategory')));
        $ingredient->setIngredientCategory($ingredientCategorie);
        $ingredient->setFamilies([]);
        foreach ($paramFetcher->get('families') as $value) {
            $cardFamily = $em->getRepository('App:CardFamily')->find($value);
            $ingredient->addFamily($cardFamily);
        }
        if ($paramFetcher->get('avatar') != '') {
            if (substr($paramFetcher->get('avatar'), 0, 4) == 'data') {
                $avatar = $this->getParameter('ingredients_directory') . "/" . $ingredient->getAvatar();
                $avatarWebp = $this->getParameter('ingredients_directory') . "/" . $ingredient->getAvatar() . 'webp';
                if (file_exists($avatar) && is_file($avatar)) unlink($avatar);
                if (file_exists($avatarWebp) && is_file($avatarWebp)) unlink($avatarWebp);
                $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
                $f = finfo_open();
                $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
                $split = explode('/', $mime_type);
                $type = $split[1];
                $filename = md5(uniqid()) . '.' . $type;
                $path = $this->getParameter('ingredients_directory') . '/' . $filename;
                $ingredient->setAvatar($filename);
                file_put_contents($path, $decoded);
                exec('cwebp -q 100 ' . $path . ' -o ' . $path . '.webp');
            }
        } else {
            if ($ingredient->getAvatar() != '') {
                unlink($this->getParameter('ingredients_directory') . "/" . $ingredient->getAvatar());
                $ingredient->setAvatar('');
            }
        }
        $em->persist($ingredient);
        $em->flush();
        /*  $client = new Client(new Version1X('//127.0.0.1:1337'));
          $client->initialize();
          $client->emit('notif', ['type' => 'persist','entity' => 'ingredient', 'id' => $id]);
          $client->close();*/
        return new JsonResponse([
            'avatar' => $ingredient->getAvatar(),
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/ingredients/{id}/status")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="available", nullable=true,allowBlank=true,strict=true)
     */
    public function updateStatus(Request $request, $id, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        $ingredient = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $ingredient = $em->getRepository('App:Ingredient')->findOneBy(Array('id' => $id, 'card' => $card));

        } else {
            $ingredient = $em->getRepository('App:Ingredient')->find($id);
        }
        if (is_null($ingredient)) {
            return new JsonResponse(null, 403);
        }
        $em = $this->getDoctrine()->getManager();
        $ingredient->setavailable($paramFetcher->get('available'));
        $ip = $request->getClientIp();
        $ingredient->setUpdatedIp($ip);
        $ingredient->setUpdatedUser('0');
        $em->persist($ingredient);
        $em->flush();
        $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'ingredient', 'id' => $id]);
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/ingredients/{id}/status/site")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="available", nullable=true,allowBlank=true,strict=true)
     */
    public function updateSiteStatus(Request $request, $id, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        $ingredient = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $ingredient = $em->getRepository('App:Ingredient')->findOneBy(Array('id' => $id, 'card' => $card));

        } else {
            $ingredient = $em->getRepository('App:Ingredient')->find($id);
        }
        if (is_null($ingredient)) {
            return new JsonResponse(null, 403);
        }
        $em = $this->getDoctrine()->getManager();
        $ingredient->setActiveSite($paramFetcher->get('available'));
        $ip = $request->getClientIp();
        $ingredient->setUpdatedIp($ip);
        $ingredient->setUpdatedUser($this->getUser()->getId());
        $em->persist($ingredient);
        $em->flush();
        $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'ingredient', 'id' => $id]);
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/ingredients/compositions/element/{idElement}")
     */
    public function getIngredientComposByEl($idElement)
    {
        $card=null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card=$this->getUser()->getPdv();
        }
        $compositions = $this->getDoctrine()->getRepository("App:Ingredient")->getIngredientComposByEl($idElement,$card);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($compositions, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/ingredients/supplements/element/{idElement}")
     */
    public function getIngredientSuppByEl($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("ec.id,ec.name as text,ec.price,ic.id as category,ic.order,ec.available,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementSupplement', 'ec')
            ->innerJoin('ec.ingredientCategory', 'ic')
            ->where('e.id = :element');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $families = $families->innerJoin('e.card', 'c')
                ->andWhere('c.id=:card')->setParameter('card', $this->getUser()->getPdv());
        }
        $families = $families->setParameter('element', $idElement)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/ingredients/optionsIngredient/element/{idElement}")
     */
    public function getOptionsIngredientByEl($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("ec.id,ec.name as text,ec.available,ic.order,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementOption', 'ec')
            ->innerJoin('ec.ingredientCategory', 'ic')
            ->where('e.id = :element');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $families = $families->innerJoin('e.card', 'c')
                ->andWhere('c.id=:card')->setParameter('card', $this->getUser()->getPdv());
        }
        $families = $families->setParameter('element', $idElement)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }

}
