<?php
/**
 * Created by PhpStorm.
 * User: HP
 * Date: 15/02/2018
 * Time: 15:32
 */

namespace App\Controller\Rest;


use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Ingredient;
use App\Entity\LibraryIngredients;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;


class LibraryIngredientController extends FOSRestController
{

    /**
     * @Rest\Post("/libraryingredients")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="suggested_price", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="category", nullable=true,allowBlank=true,strict=true)
     */
    public function addAction(ParamFetcher $paramFetcher)
    {
        $ingredient = new LibraryIngredients();
        $ingredient->setName($paramFetcher->get('name'));
        $ingredient->setDescription($paramFetcher->get('description'));
        $ingredient->setSuggestedPrice(floatval($paramFetcher->get('suggested_price')));
        $ingredient->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $em->persist($ingredient);
        $category = $em->getRepository('App:IngredientCategory')->find(intval($paramFetcher->get('category')));
        $ingredient->setIngredientCategory($category);
        $em->flush();
        $id = $ingredient->getId();
        return new JsonResponse([
            'id' => $id,
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Post("/libraryingredients/newCarteIngredient")
     * @RequestParam(name="category", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="ingredients", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="carte", nullable=true,allowBlank=true,strict=true)
     */
    public function setNewCarteIngredient(request $request, ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $cnt = 0;
        foreach ($paramFetcher->get('ingredients') as $key => $data) {
            $ingredient = new Ingredient();
            $ing = $em->getRepository('App:LibraryIngredients')->find($data);
            $ingredient->setName($ing->getName());
            $ingredient->setDescription($ing->getDescription());
            $ingredient->setPrice($ing->getSuggestedPrice());
            $ingredient->setAvailable('1');
            $ip = $request->getClientIp();
            $ingredient->setCreatedIp($ip);
            $ingredient->setCreatedUser('0');
            $ingredient->setUpdatedIp('');
            $ingredient->setUpdatedUser('0');
            $card = $em->getRepository('App:Card')->find(intval($paramFetcher->get('carte')));
            $ingredient->setCard($card);
            $ingredient->setActiveSite(1);
            $ingredient->setIngredientCategory($ing->getIngredientCategory());
            $em->persist($ingredient);
            $cnt++;
        }
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }


    /**
     * @Rest\Get("/libraryingredients")
     */
    public function getListAction()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredients = $qb->select("i.id,i.suggestedPrice,i.name,i.active,i.description,C.name as category")
            ->from("App:LibraryIngredients", "i")
            ->innerJoin('i.ingredientCategory', 'C')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        $response = new Response();
        $response->setContent($data);
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Get("/libraryingredients/{id}")
     */
    public function getActionById($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $p = $em->getRepository('App:LibraryIngredients')->find($id);
     /*   $ingredients = $qb->select("i.id,i.suggestedPrice,i.name,i.active,i.description,C as category")
            ->from("App:LibraryIngredients", "i")
            ->innerJoin('i.ingredientCategory', 'C')
            ->where('i.id =:id')
            ->setParameter('id', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();*/
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($p, 'json');
        $response = new Response();
        $response->setContent($data);
        return new JsonResponse(json_decode($data));

    }


    /**
     * @Rest\Delete("/libraryingredients/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = $em->getRepository('App:LibraryIngredients')->find($id);
        $em->remove($p);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/libraryingredients/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="suggested_price", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="category", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAction(Request $request, paramFetcher $paramFetcher, $id)
    {
        $ingredient = $this->getDoctrine()->getRepository("App:LibraryIngredients")->find($id);
        $ingredient->setName($paramFetcher->get('name'));
        $ingredient->setDescription($paramFetcher->get('description'));
        $ingredient->setSuggestedPrice(floatval($paramFetcher->get('suggested_price')));
        $ingredient->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $em->persist($ingredient);
        $Category = $em->getRepository('App:IngredientCategory')->find(intval($paramFetcher->get('category')));
        $ingredient->setIngredientCategory($Category);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }


}