<?php
/**
 * Created by PhpStorm.
 * User: PC
 * Date: 02/08/2018
 * Time: 13:08
 */

namespace App\Controller\Rest;

use App\Entity\LibrarySizes;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Ingredient;
use App\Entity\LibraryIngredients;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;

class LibrarySizeController extends FOSRestController
{

    /**
     * @Rest\Post("/librarysizes")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="level", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(ParamFetcher $paramFetcher)
    {
        $size = new LibrarySizes();
        $size->setName($paramFetcher->get('name'));
        $size->setLevel($paramFetcher->get('level'));
        $size->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        foreach ($paramFetcher->get('families') as $value) {
            $family = $em->getRepository('App:Family')->find($value);
            $size->addFamily($family);
        }
        $em->persist($size);
        $em->flush();
        $families = $size->getFamilies();
        $result = [
            'families' => $families,
            'id' => $size->getId(),
            'success' => true,
            'code' => 200
        ];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/librarysizes/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="level", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAction($id, ParamFetcher $paramFetcher)
    {
        $size = $this->getDoctrine()->getRepository("App:LibrarySizes")->find($id);
        $size->setName($paramFetcher->get('name'));
        $size->setLevel($paramFetcher->get('level'));
        $size->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $size->setFamilies([]);
        foreach ($paramFetcher->get('families') as $value) {
            $Family = $em->getRepository('App:Family')->find($value);
            $size->addFamily($Family);
        }
        $families = $size->getFamilies();
        $result = [
            'families' => $families,
            'success' => true,
            'code' => 200
        ];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        $em->persist($size);
        $em->flush();
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Put("/librarysizes/status/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="level", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="active", nullable=true,allowBlank=true,strict=true)
     */
    public function updateStatus($id, ParamFetcher $paramFetcher)
    {
        $size = $this->getDoctrine()->getRepository("App:LibrarySizes")->find($id);
        $size->setActive($paramFetcher->get('active'));
        $em = $this->getDoctrine()->getManager();
        $result = [
            'success' => true,
            'code' => 200
        ];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        $em->persist($size);
        $em->flush();
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Get("/librarysizes/{id}")
     */
    public function getAction($id, ParamFetcher $paramFetcher)
    {
        $size = $this->getDoctrine()->getRepository("App:LibrarySizes")->find($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($size, 'json');
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Get("/librarysizes/carte/{idCarte}")
     */
    public function getListActionByCarte($idCarte)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $qb1 = $em->createQueryBuilder();
        $qb2 = $em->createQueryBuilder();
        $sizes = $qb->select("s.label")
            ->from("App:Size", "s")
            ->innerJoin('s.card', 'c')
            ->where('c.id = :idCard')
            ->setParameter('idCard', $idCarte)
            ->getQuery()
            ->getScalarResult();
        $familes = $qb1->select("f.id")
            ->from("App:CardFamily", "cf")
            ->innerJoin('cf.family', 'f')
            ->innerJoin('cf.card', 'c')
            ->where('c.id = :idCard')
            ->setParameter('idCard', $idCarte)
            ->getQuery()
            ->getScalarResult();

        $sizesLibrary = $qb2->select("sl")
            ->from("App:LibrarySizes", "sl")
            ->innerJoin('sl.families', 'f')
            ->where('f.id IN (:families)')
            ->andWhere('sl.name NOT IN (:sizes)')
            ->setParameter('families', $familes)
            ->setParameter('sizes', $sizes)
            ->getQuery()
            ->getResult();

        $serializer = $this->get('serializer');
        $data = $serializer->serialize($sizesLibrary, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/librarysizes")
     */
    public function getListAction()
    {
        $size = $this->getDoctrine()->getRepository("App:LibrarySizes")->findAll();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($size, 'json');
        return new JsonResponse(json_decode($data));
    }

}