<?php
/**
 * Created by PhpStorm.
 * User: HP
 * Date: 15/02/2018
 * Time: 18:18
 */

namespace App\Controller\Rest;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Query;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\Element;
use App\Entity\ElementOrderType;
use App\Entity\ElementSize;
use App\Entity\Menu;
use App\Entity\MenuElementSize;
use App\Entity\MenuExclusion;
use App\Entity\MenuFamilySize;
use App\Entity\MenuOption;
use App\Entity\MenuOrderType;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;

class MenuController extends FOSRestController
{
    /**
     * @Rest\Post("/menus")
     * @RequestParam(name="category", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="code", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="avatar", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="title", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="offer", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="price", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="vat", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="available", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="modes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="menuTime", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="elements", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="excludedProducts", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="additionalOptions", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="carte", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="activeSite", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(request $request, ParamFetcher $paramFetcher)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($paramFetcher->get('carte'))) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $menu = new Menu();
        $menuCategory = $em->getRepository('App:MenuCategory')->find(intval($paramFetcher->get('category')));
        if (!is_null($menuCategory)) {
            $menu->setMenuCategory($menuCategory);
        } else {
            return new JsonResponse(['error' => 1], Response::HTTP_BAD_REQUEST);
        }
        $menu->setCode($paramFetcher->get('code'));
        $menu->setActiveSite($paramFetcher->get('activeSite'));
        if (trim($paramFetcher->get('title')) != "") {
            $menu->setTitle($paramFetcher->get('title'));
        } else {
            return new JsonResponse(['error' => 2], Response::HTTP_BAD_REQUEST);
        }
        if (intval($paramFetcher->get('offer')) == 0) {
            $menu->setOffer(null);
        } else {
            $offer = $em->getRepository('App:CardOffer')->find(intval($paramFetcher->get('offer')));
            if (!is_null($offer)) {
                $menu->setOffer($offer);
            } else {
                return new JsonResponse(['error' => 3], Response::HTTP_BAD_REQUEST);
            }
        }
        $menu->setPrice(floatval($paramFetcher->get('price')));
        if ($paramFetcher->get('vat') != 10 || $paramFetcher->get('vat') != 5.5) {
            $menu->setVat($paramFetcher->get('vat'));
        } else {
            return new JsonResponse(['error' => 5], Response::HTTP_BAD_REQUEST);
        }
        $menu->setDescription($paramFetcher->get('description'));
        if (intval($paramFetcher->get('available')) != 0 || intval($paramFetcher->get('available')) != 1) {
            $menu->setAvailable(intval($paramFetcher->get('available')));
        } else {
            return new JsonResponse(['error' => 6], Response::HTTP_BAD_REQUEST);
        }
        $menuTime = $em->getRepository('App:MenuTime')->find(intval($paramFetcher->get('menuTime')));
        if (!is_null($menuTime)) {
            $menu->setMenuTime($menuTime);
        } else {
            return new JsonResponse(['error' => 7], Response::HTTP_BAD_REQUEST);
        }

        $orderTypes = $paramFetcher->get('modes');
        if (is_array($orderTypes)) {
            foreach ($orderTypes as $value) {
                if (array_key_exists('id', $value) && array_key_exists('active', $value) && array_key_exists('supplement', $value)) {
                    $menuOrderType = new MenuOrderType();
                    $orderType = $em->getRepository('App:OrderType')->find($value['id']);
                    if ($orderType) {
                        $menuOrderType->setOrderType($orderType);
                        $menuOrderType->setMenu($menu);
                        $menuOrderType->setSupplement($value['supplement']);
                        $menuOrderType->setActive($value['active']);
                        $em->persist($menuOrderType);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    $response = array(
                        'code' => 0,
                        'message' => 'Champs requis: supplement et id',
                        'error' => null,
                        'result' => null
                    );
                    return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 8], Response::HTTP_BAD_REQUEST);
        }

        $elements = $paramFetcher->get('elements');

        if (is_array($elements)) {
            foreach ($elements as $element) {
                if (
                    array_key_exists('element_id', $element) &&
                    array_key_exists('size_id', $element) &&
                    array_key_exists('quantity', $element) &&
                    array_key_exists('choice', $element)
                ) {
                    $elementSize = $em->getRepository('App:ElementSize')->find(intval($element['size_id']));
                    if (!is_null($elementSize)) {
                        $menuElementSize = new MenuElementSize();
                        $menuElementSize->setMenu($menu);
                        $menuElementSize->setElementSize($elementSize);
                        $menuElementSize->setQuantity($element['quantity']);
                        $menuElementSize->setChoice($element['choice']);
                        $em->persist($menuElementSize);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    return new JsonResponse(['error' => 9], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 10], Response::HTTP_BAD_REQUEST);
        }

        $families = $paramFetcher->get('families');

        if (is_array($families)) {
            foreach ($families as $family) {
                if (
                    array_key_exists('family_id', $family) &&
                    array_key_exists('size_id', $family) &&
                    array_key_exists('quantity', $family) &&
                    array_key_exists('choice', $family)
                ) {
                    $cardFamily = $em->getRepository('App:CardFamily')->find(intval($family['family_id']));
                    $size = $em->getRepository('App:Size')->find(intval($family['size_id']));
                    if (!is_null($family) && !is_null($size)) {
                        $menuFamilySize = new MenuFamilySize();
                        $menuFamilySize->setMenu($menu);
                        $menuFamilySize->setCardFamily($cardFamily);
                        $menuFamilySize->setSize($size);
                        $menuFamilySize->setQuantity($family['quantity']);
                        $menuFamilySize->setChoice($family['choice']);
                        $em->persist($menuFamilySize);
                    } else {
                        return new JsonResponse(['error' => 11], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    return new JsonResponse(['error' => 12], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 13], Response::HTTP_BAD_REQUEST);
        }

        $excludedProducts = $paramFetcher->get('excludedProducts');
        if (is_array($excludedProducts)) {
            foreach ($excludedProducts as $idElementSize) {
                $elementSize = $em->getRepository('App:ElementSize')->find(intval($idElementSize['id']));
                if (!is_null($elementSize)) {
                    $menu->addExcludedProduct($elementSize);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        }

        $additionalOptions = $paramFetcher->get('additionalOptions');
        if (is_array($additionalOptions)) {
            foreach ($additionalOptions as $element) {
                if (
                    array_key_exists('idElementSize', $element) &&
                    array_key_exists('price', $element)
                ) {
                    $elementSize = $em->getRepository('App:ElementSize')->find(intval($element['idElementSize']));
                    if (!is_null($elementSize)) {
                        $menuOption = new MenuOption();
                        $menuOption->setMenu($menu);
                        $menuOption->setElementSize($elementSize);
                        $menuOption->setPrice($element['price']);
                        $em->persist($menuOption);
                    } else {
                        return new JsonResponse(['error' => 14], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    return new JsonResponse(['error' => 15], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 16], Response::HTTP_BAD_REQUEST);
        }

        $card = $em->getRepository('App:Card')->find(intval($paramFetcher->get('carte')));
        if (!is_null($card)) {
            $menu->setCard($card);
        } else {
            return new JsonResponse(['error' => 17], Response::HTTP_BAD_REQUEST);
        }

        if ($paramFetcher->get('avatar') != '') {
            $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
            $f = finfo_open();
            $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
            $split = explode('/', $mime_type);
            $type = $split[1];
            $filename = md5(uniqid()) . '.' . $type;
            $path = $this->getParameter('menus_directory') . '/' . $filename;
            $path2 = $this->getParameter('menus_directory') . '/platform/' . $filename;
            $menu->setAvatar($filename);
            file_put_contents($path, $decoded);
            exec('cwebp -q 100 ' . $path . ' -o ' . $path . '.webp');
            exec('cp ' . $path .' '. $path2);
            exec('convert '.$path2.' -resize 75x50\> '.$path2);
        } else {
            $menu->setAvatar('');
        }

        $menu->setCreatedUser('0');
        $ip = $request->getClientIp();
        $menu->setCreatedIp($ip);
        $menu->setUpdatedIp('');
        $menu->setUpdatedUser('0');
        $em->persist($menu);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/menus/carte/{idCard}")
     */
    public function getListAction($idCard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("m.id,m.activeSite,m.avatar,m.title,m.price,m.vat,m.available,c.name as category")
            ->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->innerJoin('m.card', 'ca')
            ->where('ca.id = :card')
            ->setParameter('card', $idCard)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/menus/unavailable/carte/{idCard}")
     */
    public function getListActionUnav($idCard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("m.id,m.avatar,m.title,m.price,m.vat,m.available,c.name as category")
            ->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->innerJoin('m.card', 'ca')
            ->where('ca.id = :card')
            ->andWhere('m.available=false')
            ->setParameter('card', $idCard)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/menus/{id}")
     */
    public function getAction($id)
    {
        $menu = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $this->getDoctrine()->getRepository("App:Card")->find($this->getUser()->getPdv());
            $menu = $this->getDoctrine()->getRepository("App:Menu")->findOneBy(array("id" => $id, "card" => $card));
        } else {
            $menu = $em->getRepository('App:Menu')->find($id);
        }
        if (is_null($menu)) {
            return new JsonResponse(null, 403);
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($menu, 'json');
        $response = new Response();
        $response->setContent($data);
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/menus/{id}/elements")
     */
    public function getCompositionElements($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $query = $qb->select("mes.id,cat.id as category_id,cat.label as category_label,cf.id as family_id,
        f.label as family_label,e.id as element_id,e.name as element_name,es.id as size_id,
        s.label as size_label,mes.quantity,mes.choice")
            ->from("App:MenuElementSize", "mes")
            ->leftJoin('mes.elementSize', 'es')
            ->leftJoin('es.element', 'e')
            ->leftJoin('es.size', 's')
            ->leftJoin('e.family', 'cf')
            ->leftJoin('cf.family', 'f')
            ->leftJoin('f.category', 'cat')
            ->leftJoin('mes.menu', 'm')
            ->where('m.id  =:idMenu');
        if ($this->getUser()->getPdv()) {
            $query = $query->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $query = $query->setParameter('idMenu', $id)
            ->getQuery();

        $results = $query->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($results, 'json');
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Get("/menus/{id}/families")
     */
    public function getCompositionFamilies($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $query = $qb->select("mfs.id,cat.id as category_id,cat.label as category_label,cf.id as family_id,f.label as family_label,s.id as size_id,s.label as size_label,mfs.quantity,mfs.choice")
            ->from("App:MenuFamilySize", "mfs")
            ->leftJoin('mfs.size', 's')
            ->leftJoin('mfs.cardFamily', 'cf')
            ->leftJoin('cf.family', 'f')
            ->leftJoin('f.category', 'cat')
            ->leftJoin('mfs.menu', 'm')
            ->where('m.id  =:idMenu');
        if ($this->getUser()->getPdv()) {
            $query = $query->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $query = $query->setParameter('idMenu', $id)->getQuery();

        $results = $query->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($results, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/menus/{id}/additionalOptions")
     */
    public function getOption($id)
    {
        $results = $this->getOptionAction($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($results, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/menus/{id}/excludedProducts")
     */
    public function getExcludedProducts($id)
    {
        /*$em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $results = $qb->select("es.id")
            ->from("App:Menu", "m")
            ->innerJoin('m.excludedProducts', 'es')
            ->where('m.id LIKE :idMenu')
            ->setParameter('idMenu', $id)
            ->getQuery()
            ->getResult();*/
        $results = $this->getExcludedProductsAction($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($results, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/menus")
     * @RequestParam(name="id", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="category", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="avatar", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="code", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="title", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="offer", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="price", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="vat", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="description", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="available", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="modes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="menuTime", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="elements", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="excludedProducts", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="additionalOptions", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="activeSite", nullable=false,allowBlank=false,strict=true)
     */
    public function updateAction(request $request, ParamFetcher $paramFetcher)
    {
        $menu = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $this->getDoctrine()->getRepository("App:Card")->find($this->getUser()->getPdv());
            $menu = $this->getDoctrine()->getRepository("App:Menu")->findOneBy(array("id" => $paramFetcher->get('id'), "card" => $card));
        } else {
            $menu = $em->getRepository('App:Menu')->find($paramFetcher->get('id'));
        }

        if (is_null($menu)) {
            return new JsonResponse(null, 403);
        }
        if (!is_null($menu)) {
            //$menu = new Menu();
            $menuCategory = $em->getRepository('App:MenuCategory')->find(intval($paramFetcher->get('category')));
            if (!is_null($menuCategory)) {
                $menu->setMenuCategory($menuCategory);
            } else {
                return new JsonResponse(['error' => 1], Response::HTTP_BAD_REQUEST);
            }
            $menu->setCode($paramFetcher->get('code'));
            $menu->setActiveSite($paramFetcher->get('activeSite'));
            if (trim($paramFetcher->get('title')) != "") {
                $menu->setTitle($paramFetcher->get('title'));
            } else {
                return new JsonResponse(['error' => 2], Response::HTTP_BAD_REQUEST);
            }
            if (intval($paramFetcher->get('offer')) == 0) {
                $menu->setOffer(null);
            } else {
                $offer = $em->getRepository('App:CardOffer')->find(intval($paramFetcher->get('offer')));
                if (!is_null($offer)) {
                    $menu->setOffer($offer);
                } else {
                    return new JsonResponse(['error' => 3], Response::HTTP_BAD_REQUEST);
                }
            }
            $menu->setPrice(floatval($paramFetcher->get('price')));
            if ($paramFetcher->get('vat') != 10 || $paramFetcher->get('vat') != 5.5) {
                $menu->setVat($paramFetcher->get('vat'));
            } else {
                return new JsonResponse(['error' => 5], Response::HTTP_BAD_REQUEST);
            }
            $menu->setDescription($paramFetcher->get('description'));
            if (intval($paramFetcher->get('available')) != 0 || intval($paramFetcher->get('available')) != 1) {
                $menu->setAvailable(intval($paramFetcher->get('available')));
            } else {
                return new JsonResponse(['error' => 6], Response::HTTP_BAD_REQUEST);
            }
            $menuTime = $em->getRepository('App:MenuTime')->find(intval($paramFetcher->get('menuTime')));
            if (!is_null($menuTime)) {
                $menu->setMenuTime($menuTime);
            } else {
                return new JsonResponse(['error' => 7], Response::HTTP_BAD_REQUEST);
            }

            /////////************************************************************************//////////////////
            $orderTypes = $paramFetcher->get('modes');
            if (is_array($orderTypes)) {
                foreach ($orderTypes as $value) {
                    if (array_key_exists('id', $value) && array_key_exists('idMenuOrderType', $value) && array_key_exists('supplement', $value) && array_key_exists('active', $value)) {
                        $menuOrderType = $em->getRepository('App:MenuOrderType')->find($value['idMenuOrderType']);
                        if (!is_null($menuOrderType)) {
                            $menuOrderType->setSupplement($value['supplement']);
                            $menuOrderType->setActive($value['active']);
                            $em->persist($menuOrderType);
                        } else {
                            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                        }
                    } else {
                        $response = array(
                            'code' => 0,
                            'message' => 'Champs requis: supplement et id et idMenuOrderType et active',
                            'error' => null,
                            'result' => null
                        );
                        return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse(['error' => 5], Response::HTTP_BAD_REQUEST);
            }
            /////////************************************************************************//////////////////

            $elements = $em->getRepository('App:MenuElementSize')->findBy(array('menu' => $menu));
            $elements2 = $paramFetcher->get('elements');
            if (is_array($elements2)) {
                foreach ($elements2 as $element) {
                    if (
                        array_key_exists('element_id', $element) &&
                        array_key_exists('size_id', $element) &&
                        array_key_exists('quantity', $element) &&
                        array_key_exists('choice', $element)
                    ) {
                        /*if(array_key_exists('id', $element)){
                            $menuElementSize = $em->getRepository('App:MenuElementSize')->findOneBy(array('id'=>intval($element['id']),'menu'=>$menu));
                            if (!is_null($menuElementSize)){
                                $menuElementSize->setQuantity($element['quantity']);
                                $menuElementSize->setChoice($element['choice']);
                                $em->persist($menuElementSize);
                            }else{
                                return new JsonResponse(['error' => 'element size'], Response::HTTP_BAD_REQUEST);
                            }
                        }*/
                        if (!array_key_exists('id', $element)) {
                            $elementSize = $em->getRepository('App:ElementSize')->find(intval($element['size_id']));
                            if (!is_null($elementSize)) {
                                $menuElementSize = new MenuElementSize();
                                $menuElementSize->setMenu($menu);
                                $menuElementSize->setElementSize($elementSize);
                                $menuElementSize->setQuantity($element['quantity']);
                                $menuElementSize->setChoice($element['choice']);
                                $em->persist($menuElementSize);
                            } else {
                                return new JsonResponse(['error' => 'element size'], Response::HTTP_BAD_REQUEST);
                            }
                        }
                    } else {
                        return new JsonResponse(['error' => 6], Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse(['error' => 7], Response::HTTP_BAD_REQUEST);
            }

            foreach ($elements as $value) {
                $index = $this->findIndex($elements2, $value);
                if ($index == -1) {
                    $em->remove($value);
                }
            }
            /////////************************************************************************//////////////////
            $families = $paramFetcher->get('families');
            if (is_array($families)) {
                foreach ($families as $family) {
                    if (
                        array_key_exists('family_id', $family) &&
                        array_key_exists('size_id', $family) &&
                        array_key_exists('quantity', $family) &&
                        array_key_exists('choice', $family)
                    ) {
                        /*if(array_key_exists('id', $family)){
                            $menuFamilySize = $em->getRepository('App:MenuFamilySize')->findOneBy(array('id'=>intval($family['id']),'menu'=>$menu));
                            if (!is_null($menuFamilySize)){
                                $menuFamilySize->setQuantity($family['quantity']);
                                $menuFamilySize->setChoice($family['choice']);
                                $em->persist($menuFamilySize);
                            }else{
                                return new JsonResponse(['error' => 8], Response::HTTP_BAD_REQUEST);
                            }
                        }*/
                        if (!array_key_exists('id', $family)) {
                            $cardFamily = $em->getRepository('App:CardFamily')->find(intval($family['family_id']));
                            $size = $em->getRepository('App:Size')->find(intval($family['size_id']));
                            if (!is_null($family) && !is_null($size)) {
                                $menuFamilySize = new MenuFamilySize();
                                $menuFamilySize->setMenu($menu);
                                $menuFamilySize->setCardFamily($cardFamily);
                                $menuFamilySize->setSize($size);
                                $menuFamilySize->setQuantity($family['quantity']);
                                $menuFamilySize->setChoice($family['choice']);
                                $em->persist($menuFamilySize);
                            } else {
                                return new JsonResponse(['error' => 8], Response::HTTP_BAD_REQUEST);
                            }
                        }
                    } else {
                        return new JsonResponse(['error' => 9], Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse(['error' => 10], Response::HTTP_BAD_REQUEST);
            }
            $families2 = $em->getRepository('App:MenuFamilySize')->findBy(array('menu' => $menu));

            foreach ($families2 as $value) {
                $index = $this->findIndex($families, $value);
                if ($index == -1) {
                    $em->remove($value);
                }
            }

            $menu->setExcludedProducts([]);
            $excludedProducts = $paramFetcher->get('excludedProducts');
            if (is_array($excludedProducts)) {
                foreach ($excludedProducts as $idElementSize) {
                    $elementSize = $em->getRepository('App:ElementSize')->find(intval($idElementSize['id']));
                    if (!is_null($elementSize)) {
                        $menu->addExcludedProduct($elementSize);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                }
            }

            $additionalOptions = $em->getRepository('App:MenuOption')->findBy(array('menu' => $menu));
            $additionalOptions2 = $paramFetcher->get('additionalOptions');
            if (is_array($additionalOptions2)) {
                foreach ($additionalOptions2 as $element) {
                    if (
                        array_key_exists('idElementSize', $element) &&
                        array_key_exists('price', $element)
                    ) {
                        if (!array_key_exists('id', $element)) {
                            $elementSize = $em->getRepository('App:ElementSize')->find(intval($element['idElementSize']));
                            if (!is_null($elementSize)) {
                                $menuOption = new MenuOption();
                                $menuOption->setMenu($menu);
                                $menuOption->setElementSize($elementSize);
                                $menuOption->setPrice($element['price']);
                                $em->persist($menuOption);
                            } else {
                                return new JsonResponse(['error' => 10], Response::HTTP_BAD_REQUEST);
                            }
                        }
                    } else {
                        return new JsonResponse(['error' => 8], Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse(['error' => 12], Response::HTTP_BAD_REQUEST);
            }

            foreach ($additionalOptions as $value) {
                $index = $this->findIndex($additionalOptions2, $value);
                if ($index == -1) {

                    $em->remove($value);
                }
            }

            if ($paramFetcher->get('avatar') != '') {
                if (substr($paramFetcher->get('avatar'), 0, 4) == 'data') {
                    $avatar = $this->getParameter('menus_directory') . "/" . $menu->getAvatar();
                    $avatarWebp = $this->getParameter('menus_directory') . "/" . $menu->getAvatar() . 'webp';
                    if (file_exists($avatar) && is_file($avatar)) unlink($avatar);
                    if (file_exists($avatarWebp) && is_file($avatarWebp)) unlink($avatarWebp);
                    $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
                    $f = finfo_open();
                    $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
                    $split = explode('/', $mime_type);
                    $type = $split[1];
                    $filename = md5(uniqid()) . '.' . $type;
                    $path = $this->getParameter('menus_directory') . '/' . $filename;
                    $path2 = $this->getParameter('menus_directory') . '/platform/' . $filename;
                    $menu->setAvatar($filename);
                    file_put_contents($path, $decoded);
                    exec('cwebp -q 100 ' . $path . ' -o ' . $path . '.webp');
                    exec('cp ' . $path .' '. $path2);
                    exec('convert '.$path2.' -resize 75x50\> '.$path2);
                }
            } else {
                if ($menu->getAvatar() != '') {
                    $avatar = $this->getParameter('menus_directory') . "/" . $menu->getAvatar();
                    $avatar2 = $this->getParameter('menus_directory') . "/platform/" . $menu->getAvatar();
                    if (file_exists($avatar))unlink($avatar);
                    if (file_exists($avatar2))unlink($avatar2);
                    $menu->setAvatar('');
                }
            }

            $ip = $request->getClientIp();
            $menu->setUpdatedIp($ip);
            $menu->setUpdatedUser('0');
            $em->persist($menu);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
        }

    }

    /**
     * @Rest\Put("/menus/{id}/status")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="available", nullable=false,allowBlank=false,strict=true)
     */
    public function updateStatus(Request $request, $id, ParamFetcher $paramFetcher)
    {
        $menu = $this->getDoctrine()->getRepository("App:Menu")->find(intval($id));
        if (!is_null($menu)) {
            $menu->setavailable($paramFetcher->get('available'));
            $ip = $request->getClientIp();
            $menu->setUpdatedIp($ip);
            $menu->setUpdatedUser('0');
            $em = $this->getDoctrine()->getManager();
            $em->persist($menu);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
        }


        /*  $client = new Client(new Version1X('//127.0.0.1:1337'));
          $client->initialize();
          $client->emit('notif', ['type' => 'persist','entity' => 'ingredient', 'id' => $id]);
          $client->close();*/

    }

    /**
     * @Rest\Put("/menus/{id}/status/site")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="active", nullable=false,allowBlank=false,strict=true)
     */
    public function updateStatusSite(Request $request, $id, ParamFetcher $paramFetcher)
    {
        $menu = $this->getDoctrine()->getRepository("App:Menu")->find(intval($id));
        if (!is_null($menu)) {
            $menu->setActiveSite(intval($paramFetcher->get('active')));
            $ip = $request->getClientIp();
            $menu->setUpdatedIp($ip);
            $menu->setUpdatedUser($this->getUser()->getId());
            $em = $this->getDoctrine()->getManager();
            $em->persist($menu);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
        }
    }

    /**
     * @Rest\Delete("/menus/{id}")
     */
    public function deleteAction($id)
    {
        $menu = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $this->getDoctrine()->getRepository("App:Card")->find($this->getUser()->getPdv());
            $menu = $this->getDoctrine()->getRepository("App:Menu")->findOneBy(array("id" => $id, "card" => $card));
        } else {
            $menu = $em->getRepository('App:Menu')->find($id);
        }
        if (is_null($menu)) {
            return new JsonResponse(null, 403);
        }
        $em->remove($menu);
        $em->flush();
        $avatar = $this->getParameter('menus_directory') . "/" . $menu->getAvatar();
        $avatarWebp = $this->getParameter('menus_directory') . "/" . $menu->getAvatar() . 'webp';
        if (file_exists($avatar) && is_file($avatar)) unlink($avatar);
        if (file_exists($avatarWebp) && is_file($avatarWebp)) unlink($avatarWebp);
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/menus/carte/{idCarte}/category/{idCategory}/orderType/{idOrderType}")
     */
    public function getMenuByCategoryAndCard($idCarte, $idCategory, $idOrderType)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCarte)) {
                return new JsonResponse(null, 403);
            }
        }
        $d = new \DateTime();
        $midi = new \DateTime();
        $midi->setTime(16, 0, 0);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        if ($d > $midi) {
            $menus = $qb->select("m.id,m.avatar,m.code,m.vat,m.title,m.description,m.price,m.available,mt.label as menuTime,o.id as offer,
            case when mt.id = 1 then false else true end as validPeriod");
        } else {
            $menus = $qb->select("m.id,m.avatar,m.code,m.vat,m.title,m.description,m.price,m.available,mt.label as menuTime,o.id as offer,
            case when mt.id = 3 or mt.id=4 then false else true end as validPeriod");
        }
        $menus = $menus->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->leftJoin('m.offer', 'co')
            ->leftJoin('co.offer', 'o')
            ->innerJoin('m.menuTime', 'mt')
            ->innerJoin('m.card', 'ca')
            ->where('ca.id = :card')
            ->andWhere('c.id = :idCat')
            ->setParameter('card', $idCarte)
            ->setParameter('idCat', $idCategory)
            ->groupBy('m.id')
            ->getQuery()->getResult();
        foreach ($menus as $key => $menu) {
            $menus[$key]['dispo'] = intval($this->hasOrderType($menu['id'], $idOrderType)['dispo']);
            $menus[$key]['supplement'] = floatval($this->hasOrderType($menu['id'], $idOrderType)['supplement']);
        }

        $serializer = $this->get('serializer');
        $data = $serializer->serialize($menus, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/menus/{id}/compositions")
     */
    public function getElementsSize($id)
    {
        $elements = array();
        $families = array();
        $additionalOptions = $this->getOptionAction($id);
        for ($i = 0; $i < 6; $i++) {
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $query = $qb->select("cf.id as family,f.label as familyName,es.id,es.priceSupp,es.price,es.available,e.id as element,e.hasSupp,e.avatar,e.priceSuppApp,e.name,e.maxNbrIngOptions,s.id as idSize,s.label as size,s.level,mes.quantity,mes.choice")
                ->from("App:Menu", "m")
                ->innerJoin('m.elements', 'mes')
                ->innerJoin('mes.elementSize', 'es')
                ->innerJoin('es.element', 'e')
                ->innerJoin('e.family', 'cf')
                ->innerJoin('cf.family', 'f')
                ->innerJoin('es.size', 's');
            if ($this->getUser()->getPdv()) {
                $query = $query->innerJoin('m.card', 'c')
                    ->andWhere('c.id  =:idCard')
                    ->setParameter('idCard', $this->getUser()->getPdv());
            }
            $query = $query->where('mes.menu = :idMenu')
                ->andWhere('mes.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            foreach ($results as $key => $value) {
                $results[$key]['compositions'] = $this->getIngredientComposByEl($value['element']);
                $results[$key]['supplements'] = $this->getIngredientSuppByEl($value['element']);
                $results[$key]['optionsIngredient'] = $this->getOptionsIngredientByEl($value['element']);
            }
            $elements[$i] = $results;
            $qb2 = $em->createQueryBuilder();
            $query = $qb2->select("mfs.id,cf.id as cardFamily,f.id as family,f.label,s.label as size,s.level,s.id as sizeId,mfs.quantity,mfs.choice")
                ->from("App:Menu", "m")
                ->leftJoin('m.families', 'mfs')
                ->leftJoin('mfs.cardFamily', 'cf')
                ->leftJoin('cf.family', 'f')
                ->leftJoin('mfs.size', 's');
            if ($this->getUser()->getPdv()) {
                $query = $query->innerJoin('m.card', 'c')
                    ->andWhere('c.id  =:idCard')
                    ->setParameter('idCard', $this->getUser()->getPdv());
            }
            $query = $query->where('mfs.menu = :idMenu')
                ->andWhere('mfs.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            foreach ($results as $keyFam => $family) {
                $results[$keyFam]['elements'] = array();
                $elms = $this->getElementsByFamilyAndSize($family['cardFamily'], $family['sizeId'], $id);
                for ($j = 0; $j < $family['quantity']; $j++) {
                    $results[$keyFam]['elements'][$j] = $elms;
                }
            }
            $families[$i] = $results;
        }
        $data = array('elements' => $elements, 'families' => $families, 'additionalOptions' => $additionalOptions);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }

    public function getIngredientComposByEl($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $compositions = $qb->select("ec.id,ec.available,ec.name as text,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementComposition', 'ec');
        if ($this->getUser()->getPdv()) {
            $compositions = $compositions->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $compositions = $compositions->where('e.id =:element')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();
        return $compositions;
    }

    public function getIngredientSuppByEl($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $supp = $qb->select("ec.id,ic.id as category,ec.name as text,ec.price,ec.available,case when ec.available = 1 then 'false' else 'true' end  as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementSupplement', 'ec')
            ->innerJoin('ec.ingredientCategory', 'ic');
        if ($this->getUser()->getPdv()) {
            $supp = $supp->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $supp = $supp->where('e.id =:element')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();

        return $supp;
    }

    public function getOptionsIngredientByEl($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $options = $qb->select("ec.id,ec.name as text,ec.available,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementOption', 'ec');
        if ($this->getUser()->getPdv()) {
            $options = $options->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $options = $options->where('e.id =:element')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();

        return $options;
    }

    public function getElementsByFamilyAndSize($idFamily, $idSize, $idMenu)
    {
        $exledProducts = $this->getExcludedProductsAction($idMenu);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id,es.available,e.name,e.avatar,e.id as elementId,es.priceSupp,e.hasSupp,cf.id as family,es.price,e.maxNbrIngOptions,e.priceSuppApp,s.label as size,s.level,s.id as idSize,1 as quantity")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e');
        if ($this->getUser()->getPdv()) {
            $elements = $elements->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $elements = $elements->innerJoin('es.size', 's')
            ->innerJoin('e.family', 'cf')
            ->where('cf.id = :cardFamily')
            ->andWhere('s.id = :size');
        if (count($exledProducts) > 0) {
            $elements = $elements->andWhere('es.id not in (:exledProducts)');
            $elements = $elements->setParameter('exledProducts', $exledProducts);
        }
        $elements = $elements->setParameter('cardFamily', $idFamily)
            ->setParameter('size', $idSize)
            ->groupBy('es.id')
            ->getQuery()
            ->getResult();
        return $elements;
    }


    public function hasOrderType($idMenu, $idOrderType)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $result = $qb->select("count(ot.id) as dispo,mot.supplement")
            ->from("App:Menu", "m")
            ->innerJoin('m.menuOrderType', 'mot')
            ->innerJoin('mot.orderType', 'ot');
        if ($this->getUser()->getPdv()) {
            $result = $result->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $result = $result->where('m.id = :idMenu')
            ->andWhere('ot.id = :idOrderType')
            ->andWhere('mot.active=1')
            ->setParameter('idOrderType', $idOrderType)
            ->setParameter('idMenu', $idMenu)
            ->getQuery()
            ->getSingleResult();
        return $result;
    }

    public function getExcludedProductsAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $results = $qb->select("es.id")
            ->from("App:Menu", "m")
            ->innerJoin('m.excludedProducts', 'es')
            ->where('m.id  =:idMenu');
        if ($this->getUser()->getPdv()) {
            $results = $results->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $results = $results->setParameter('idMenu', $id)
            ->getQuery()
            ->getResult();
        return $results;
    }

    public function getOptionAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $results = $qb->select("mo.id,es.id as idElementSize,e.name,s.label as size,mo.price,es.available")
            ->from("App:MenuOption", "mo")
            ->leftJoin('mo.elementSize', 'es')
            ->leftJoin('es.element', 'e')
            ->leftJoin('es.size', 's')
            ->leftJoin('mo.menu', 'm');
        if ($this->getUser()->getPdv()) {
            $results = $results->innerJoin('m.card', 'c')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $results = $results->where('m.id  =:idMenu')
            ->setParameter('idMenu', $id)
            ->getQuery()
            ->getResult();
        return $results;
    }

    public function findIndex($array, $object)
    {
        $index = -1;
        foreach ($array as $key => $value) {
            if (array_key_exists('id', $value) && $value['id'] && $value['id'] == $object->getId()) {
                $index = $key;
                break;
            }
        }
        return $index;
    }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * @Rest\Get("/restaurant/menus/carte/{idCarte}/category/{idCategory}/orderType/{idOrderType}")
     */
    public function getRestaurantMenuByCategoryAndCard($idCarte, $idCategory, $idOrderType)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCarte)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("m.id,mot.supplement,m.avatar,m.code,m.vat,m.title,m.description,m.price,m.available,c.name as category,mt.id as menuTime,o.id as offer,
            case when mt.id = 1 then false else true end as validPeriod,case when m.avatar<>'' then m.avatar else 'no-image.webp' end avatar");
        $menus = $menus->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->leftJoin('m.offer', 'co')
            ->leftJoin('co.offer', 'o')
            ->innerJoin('m.menuTime', 'mt')
            ->innerJoin('m.card', 'ca')
            ->innerJoin('m.menuOrderType', 'mot')
            ->innerJoin('mot.orderType', 'ot')
            ->where('ca.id = :card')
            ->andWhere('ot.id=:orderType')
            ->andWhere('m.available=1')
            ->andWhere('c.id = :idCat')
            ->setParameter('card', $idCarte)
            ->setParameter('orderType', $idOrderType)
            ->setParameter('idCat', $idCategory)
            ->groupBy('m.id')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($menus, 'json');
        return new JsonResponse(json_decode($data));
    }


}
