<?php
/**
 * Created by PhpStorm.
 * User: HP
 * Date: 16/02/2018
 * Time: 11:34
 */

namespace App\Controller\Rest;

use Doctrine\ORM\Query;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\CardOffer;
use App\Entity\Category;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;

class OfferController extends FOSRestController
{
    /**
     * @Rest\Post("/offers/carte")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="idOffer", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="carte", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(ParamFetcher $paramFetcher)
    {
        $idCarte = $paramFetcher->get('carte');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCarte)) {
                return new JsonResponse(null, 403);
            }
        }
        $offer = $this->getDoctrine()->getRepository("App:Offer")->find($paramFetcher->get('idOffer'));
        $card = $this->getDoctrine()->getRepository("App:Card")->find($paramFetcher->get('carte'));
        $cardOffer = new CardOffer();
        $cardOffer->setCard($card);
        $cardOffer->setOffer($offer);
        $em = $this->getDoctrine()->getManager();
        $em->persist($cardOffer);
        $em->flush();
        return new JsonResponse([
            'success' => true,
            'id' => $cardOffer->getId(),
            'label' => $offer->getLabel(),
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/offers/carte/{idCard}")
     */
    public function getOffersByCarte($idCard)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("co.id,o.id as idOffer,o.label")
            ->from("App:CardOffer", "co")
            ->leftJoin('co.offer', 'o')
            ->innerJoin('co.card', 'c')
            ->where('c.id = :card')
            ->setParameter('card', $idCard)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/offers/{id}")
     */
    public function getOffer($id)
    {
        $offer = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $offer = $qb->select("co")
                ->from("App:CardOffer", "co")
                ->innerJoin('co.card', 'c')
                ->where('c.id = :card')
                ->setParameter('card', $this->getUser()->getPdv())
                ->setMaxResuts(1)
                ->getQuery()
                ->getOneOrNullResult();
        } else {
            $offer = $this->getDoctrine()->getRepository("App:CardOffer")->find($id);
        }
        if (!is_null($offer)) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($offer, 'json');
            $response = new Response();
            $response->setContent($data);
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse(null, 204);
        }
    }

    /**
     * @Rest\Get("/carteOffer/{id}")
     */
    public function getOffersByCardOffer($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("o.id as offer")
            ->from("App:CardOffer", "co")
            ->leftJoin('co.offer', 'o')
            ->where('co.id = :offer');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $elements = $elements->innerJoin('co.card', 'c')
                ->andWhere('c.id = :card')
                ->setParameter('card', $this->getUser()->getPdv());
        }
        $elements = $elements->setParameter('offer', $id)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/offers")
     */
    public function getAllCatAction()
    {
        $offers = $this->getDoctrine()->getRepository("App:Offer")->findAll();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($offers, 'json');
        $response = new Response();
        $response->setContent($data);
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Delete("/offers/{id}")
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $em->getRepository('App:Card')->find($this->getUser()->getPdv());
            $p = $em->getRepository('App:CardOffer')->findOneBy(array('id' => $id, 'card' => $card));
        } else {
            $p = $em->getRepository('App:CardOffer')->find($id);
        }
        if (!is_null($p)) {
            $em->remove($p);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse(null, 204);
        }

    }


}
