<?php

namespace App\Controller\Rest;

use App\Entity\CommandPaymentMode;
use App\Service\SendEvent;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Controller\Annotations\QueryParam;
use App\Entity\Address;
use App\Entity\CommandElement;
use App\Entity\Command;
use App\Entity\CommandElementIngredients;
use App\Entity\CommandMenu;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Pdv;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Doctrine\ORM\Tools\Pagination\Paginator;
use DateTime;
use Symfony\Component\HttpFoundation\StreamedResponse;

class OrderController extends FOSRestController
{
    /**
     * @Rest\Post("/orders")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="amount", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="address", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="orderProg", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="remise", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="type", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="user", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="orderType", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="elements", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="menus", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="idCustomer", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="idAddress", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="payementModes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="displayCode", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="token", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="fees", nullable=true,allowBlank=true,strict=true)
     */
    public function addOrder(Request $request, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {

        $idPdv = $paramFetcher->get('pdv');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idPdv)) {
                return new JsonResponse(null, 403);
            }
        }

        $em = $this->getDoctrine()->getManager();
        $order = new Command();
        $order->setAmount($paramFetcher->get('amount'));
        $order->setComment($paramFetcher->get('comment'));
        $order->setCorrected(0);
        $date = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('orderProg'));
        $remise = ($paramFetcher->get('remise')) ? $paramFetcher->get('remise') : 0;
        $order->setDiscount($remise);
        $order->setOrderProg($date);
        $ip = $request->getClientIp();
        $order->setCreatedIp($ip);
        $order->setCreatedUser($this->getUser());
        $order->setUpdatedIp('');
        $order->setUpdatedUser(null);
        $customer = intval($paramFetcher->get('idCustomer'));
        $customerPdv = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($idPdv);
            $customerPdv = $this->getDoctrine()->getRepository("App:CustomerPdv")->findOneBy(array('id' => $customer, 'pdv' => $pdv));
        } else {
            $customerPdv = $this->getDoctrine()->getRepository("App:CustomerPdv")->find($customer);
        }
        if (!is_null($customerPdv)) {
            $order->setCustomer($customerPdv);
        } else {
            return new JsonResponse('no customer selected', 400);
        }
        $orderPdv = $this->getOrderPdvByPdvAndOrder($paramFetcher->get('pdv'), $paramFetcher->get('orderType'));
        $order->setOrderTypePdv($orderPdv);
        if ($paramFetcher->get('orderType') == 1) {
            $idAddress = $paramFetcher->get('idAddress');
            $address = $em->getRepository('App:Address')->findOneBy(array('id' => $idAddress, 'customer' => $customerPdv));
            if (is_null($address)) {
                return new JsonResponse('error in order address', 400);
            }
            $order->setAddress($address);
        }
        $elements = $paramFetcher->get('elements');
        $menus = $paramFetcher->get('menus');
        $fees = $paramFetcher->get('fees');
        if ((is_array($elements) && count($elements) > 0) || (is_array($menus) && count($menus) > 0)) {
            if (count($elements) > 0) {
                foreach ($elements as $value) {
                    if (
                        array_key_exists('price', $value['size']) &&
                        array_key_exists('size', $value['size']) &&
                        array_key_exists('name', $value) &&
                        array_key_exists('qty', $value) &&
                        array_key_exists('comment', $value) &&
                        array_key_exists('total', $value) &&
                        array_key_exists('vat', $value) &&
                        array_key_exists('additionalCost', $value)
                    ) {
                        $commandElement = new CommandElement();
                        if ($paramFetcher->get('displayCode') && $value['code']) {
                            $commandElement->setName($value['code'] . "-" . $value['name']);
                        } else {
                            $commandElement->setName($value['name']);
                        }
                        $commandElement->setQuantity($value['qty']);
                        $commandElement->setComment($value['comment']);
                        $commandElement->setTotal($value['total']);
                        $commandElement->setPrice($value['size']['price']);
                        $commandElement->setVat($value['vat']);
                        $commandElement->setSize($value['size']['size']);
                        if (array_key_exists('level', $value['size'])) {
                            $commandElement->setLevel($value['size']['level']);
                        }
                        $commandElement->setAdditionalCost($value['additionalCost']);
                        $commandElement->setCommand($order);
                        if (array_key_exists('id', $value['size']) && intval($value['size']['id']) > 0) {
                            $elementSize = intval($value['size']['id']);
                            $commandElement->setElementSize($elementSize);
                        }
                        $commandElement->setCode($value['code']);
                        $em->persist($commandElement);
                        /*if (array_key_exists('compositions', $value)) {
                            foreach ($value['compositions'] as $valu) {
                                $commandElementIngC = new CommandElementIngredients();
                                $ingredient= $em->getRepository('App:Ingredient')->find($valu['id']);
                                $commandElementIngC->setIngredient($ingredient);
                                $commandElement->setElementSize($elementSize);
                                $commandElementIngC->setName($valu['text']);
                                $commandElementIngC->setCout(0);
                                $commandElementIngC->setAction('comp');
                                $commandElementIngC->setCommandElement($commandElement);
                                $em->persist($commandElementIngC);
                            }
                        }*/
                        if (array_key_exists('removedCompositions', $value)) {
                            foreach ($value['removedCompositions'] as $val) {
                                $commandElementIngR = new CommandElementIngredients();
                                $ingredient = intval($val['id']);
                                $commandElementIngR->setIngredient($ingredient);
                                $commandElementIngR->setName($val['text']);
                                $commandElementIngR->setCout(0);
                                $commandElementIngR->setAction('remove');
                                $commandElementIngR->setCommandElement($commandElement);
                                $em->persist($commandElementIngR);
                            }
                        }
                        if (array_key_exists('supplements', $value)) {
                            foreach ($value['supplements'] as $va) {
                                $commandElementIngS = new CommandElementIngredients();
                                $ingredient = intval($va['id']);
                                $commandElementIngS->setIngredient($ingredient);
                                $commandElementIngS->setName($va['text']);
                                $commandElementIngS->setCout($va['price']);
                                $commandElementIngS->setAction('supp');
                                $commandElementIngS->setCommandElement($commandElement);
                                $em->persist($commandElementIngS);
                            }
                        }
                        if (array_key_exists('optionIngredients', $value)) {
                            foreach ($value['optionIngredients'] as $v) {
                                $commandElementIngO = new CommandElementIngredients();
                                $ingredient = intval($v['id']);
                                $commandElementIngO->setIngredient($ingredient);
                                $commandElementIngO->setName($v['text']);
                                $commandElementIngO->setCout(0);
                                $commandElementIngO->setAction('opt');
                                $commandElementIngO->setCommandElement($commandElement);
                                $em->persist($commandElementIngO);
                            }
                        }

                    } else {
                        return new JsonResponse(['error' => 'One of this fileds (size(size,price),name,qty,comment,total,vat,additionalCost) are messing'], 400);
                    }
                }
            }
            if (count($menus) > 0) {
                foreach ($menus as $menu) {
                    if (
                        array_key_exists('price', $menu) &&
                        array_key_exists('comment', $menu) &&
                        array_key_exists('qty', $menu) &&
                        array_key_exists('title', $menu) &&
                        array_key_exists('total', $menu) &&
                        array_key_exists('additionalCost', $menu) &&
                        array_key_exists('vat', $menu) &&
                        array_key_exists('elements', $menu) &&
                        is_array($menu['elements']) &&
                        count($menu['elements']) > 0
                    ) {
                        $commandMenu = new CommandMenu();
                        if ($paramFetcher->get('displayCode') && $menu['code']) {
                            $commandMenu->setTitle($menu['code'] . "-" . $menu['title']);
                        } else {
                            $commandMenu->setTitle($menu['title']);
                        }
                        $commandMenu->setQuantity($menu['qty']);
                        $commandMenu->setComment($menu['comment']);
                        $commandMenu->setTotal($menu['total']);
                        $commandMenu->setPrice($menu['price']);
                        $commandMenu->setVat($menu['vat']);
                        $commandMenu->setAdditionalCost($menu['additionalCost']);
                        $commandMenu->setCommand($order);
                        $commandMenu->setCode($menu['code']);
                        $m = intval($menu['id']);
                        $commandMenu->setMenu($m);
                        $em->persist($commandMenu);
                        foreach ($menu['elements'] as $element) {
                            if (
                                array_key_exists('price', $element) &&
                                array_key_exists('name', $element) &&
                                array_key_exists('qty', $element) &&
                                array_key_exists('comment', $element) &&
                                array_key_exists('label', $element['size'])
                            ) {
                                $commandElement = new CommandElement();
                                $commandElement->setName($element['name']);
                                $commandElement->setQuantity($element['qty']);
                                $commandElement->setComment($element['comment']);
                                $commandElement->setTotal(0);
                                $commandElement->setPrice(0);
                                $commandElement->setVat(0);
                                $commandElement->setSize($element['size']['label']);
                                $commandElement->setLevel($element['size']['level']);
                                $commandElement->setAdditionalCost(0);
                                if (array_key_exists('id', $element['size'])) {
                                    $elementSize = intval($element['size']['id']);
                                    $commandElement->setElementSize($elementSize);
                                }
                                $commandElement->setCommandMenu($commandMenu);
                                $commandElement->setCommand($order);
                                $em->persist($commandElement);
                                /*if (array_key_exists('compositions', $element)) {
                                    foreach ($element['compositions'] as $valu) {
                                        $commandElementIngC = new CommandElementIngredients();
                                        $ingredient = $em->getRepository('App:Ingredient')->find($valu['id']);
                                        $commandElementIngC->setIngredient($ingredient);
                                        $commandElementIngC->setName($valu['text']);
                                        $commandElementIngC->setCout(0);
                                        $commandElementIngC->setAction('comp');
                                        $commandElementIngC->setCommandElement($commandElement);
                                        $em->persist($commandElementIngC);
                                    }
                                }*/
                                if (array_key_exists('removedCompositions', $element)) {
                                    foreach ($element['removedCompositions'] as $val) {
                                        $commandElementIngR = new CommandElementIngredients();
                                        $ingredient = intval($val['id']);
                                        $commandElementIngR->setIngredient($ingredient);
                                        $commandElementIngR->setName($val['text']);
                                        $commandElementIngR->setCout(0);
                                        $commandElementIngR->setAction('remove');
                                        $commandElementIngR->setCommandElement($commandElement);
                                        $em->persist($commandElementIngR);
                                    }
                                }
                                if (array_key_exists('supplements', $element)) {
                                    foreach ($element['supplements'] as $va) {
                                        $commandElementIngS = new CommandElementIngredients();
                                        $ingredient = intval($va['id']);
                                        $commandElementIngS->setIngredient($ingredient);
                                        $commandElementIngS->setName($va['text']);
                                        $commandElementIngS->setCout($va['price']);
                                        $commandElementIngS->setAction('supp');
                                        $commandElementIngS->setCommandElement($commandElement);
                                        $em->persist($commandElementIngS);
                                    }
                                }
                                if (array_key_exists('optionIngredients', $element)) {
                                    foreach ($element['optionIngredients'] as $v) {
                                        $commandElementIngO = new CommandElementIngredients();
                                        $ingredient = intval($v['id']);
                                        $commandElementIngO->setIngredient($ingredient);
                                        $commandElementIngO->setName($v['text']);
                                        $commandElementIngO->setCout(0);
                                        $commandElementIngO->setAction('opt');
                                        $commandElementIngO->setCommandElement($commandElement);
                                        $em->persist($commandElementIngO);
                                    }
                                }
                            } else {
                                return new JsonResponse(['error' => 2], 400);
                            }
                        }
                        if (
                            array_key_exists('additionalOptions', $menu) &&
                            is_array($menu['additionalOptions']) &&
                            count($menu['additionalOptions']) > 0
                        ) {
                            foreach ($menu['additionalOptions'] as $element) {
                                if (
                                    array_key_exists('price', $element) &&
                                    array_key_exists('name', $element) &&
                                    array_key_exists('size', $element)
                                ) {
                                    $commandElement = new CommandElement();
                                    $commandElement->setName($element['name']);
                                    $commandElement->setQuantity(1);
                                    $commandElement->setComment('');
                                    $commandElement->setTotal(0);
                                    $commandElement->setPrice($element['price']);
                                    $commandElement->setVat(0);
                                    $commandElement->setSize($element['size']);
                                    //$commandElement->setLevel($element['level']);
                                    $commandElement->setAdditionalCost(0);
                                    $commandElement->setCommandMenu($commandMenu);
                                    $commandElement->setCommand($order);
                                    $em->persist($commandElement);
                                }
                            }
                        }
                    }
                }
            }
        } else {
            return new JsonResponse(['error' => 3], 400);
        }
        if($fees){
                        $commandElement = new CommandElement();
                        $commandElement->setName($fees['name']);
                        $commandElement->setQuantity(0);
                        $commandElement->setComment('');
                        $commandElement->setTotal($fees['price']);
                        $commandElement->setPrice($fees['price']);
                        $commandElement->setVat(0);
                        $commandElement->setSize('');
                        $commandElement->setAdditionalCost(0);
                        $commandElement->setCommand($order);
                        $commandElement->setCode('');
                        $em->persist($commandElement);
        }
        $payementModes = $paramFetcher->get('payementModes');
        if (is_array($payementModes) && count($payementModes) > 0) {
            foreach ($payementModes as $val) {
                $payementMode = $em->getRepository('App:PaymentMode')->find($val['id']);
                $commandPaymentMode = new CommandPaymentMode();
                if ($payementMode) {
                    $commandPaymentMode->setPaymentMode($payementMode);
                    $commandPaymentMode->setCommand($order);
                    $commandPaymentMode->setAmount(0);
                    $em->persist($commandPaymentMode);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }

            }
        } else {
            return new JsonResponse(['error' => 5], 400);
        }
        $now = new DateTime();
        $qb = $em->createQueryBuilder();
        $res = $qb->select("co.num as num")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'otp')
            ->innerJoin('otp.pdv', 'pdv')
            ->where('co.createdAt > :date_start')
            ->andWhere('co.createdAt < :date_end')
            ->andWhere('pdv.id=:pdv')
            ->setParameter('date_start', $now->format('Y-m-d 00:00:00'))
            ->setParameter('date_end', $now->format('Y-m-d 23:59:59'))
            ->setParameter('pdv', intval($paramFetcher->get('pdv')))
            ->setMaxResults(1)
            ->orderBy('co.id', 'DESC')
            ->getQuery()
            ->getOneOrNullResult();
        $statut = $em->getRepository('App:Status')->find(8);
        $order->setStatus($statut);
        $order->setNum(intval($res['num']) + 1);
        $platform = $em->getRepository('App:Platform')->find(1);
        $order->setPlatform($platform);
        $em->persist($order);
        $em->flush();
        $sendEvent->sendMessage(['type' => 'new', 'entity' => 'order', 'id' => $order->getId(),'num'=>$order->getNum(),'pdv' => $paramFetcher->get('pdv')]);
        /////////////////////////////////////////////////////////////////////////
        $result = [
            'id' => $order->getId(),
            'success' => true,
            'code' => 200
        ];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/orders")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="id", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="amount", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="remise", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="user", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="elements", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="menus", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="payementModes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="token", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="displayCode", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="displayCode", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="orderType", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="idAddress", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="fees", nullable=true,allowBlank=true,strict=true)
     */
    public function updateOrder(Request $request, SendEvent $sendEvent, ParamFetcher $paramFetcher)
    {
        if (!$this->getUser()->hasRole('RULE_EDITORDER')) {
            return new JsonResponse('', 403);
        } else {
            $id = intval($paramFetcher->get('id'));
            $em = $this->getDoctrine()->getManager();
            if ($this->getUser()->getPdv()) {
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $order = $qb->select('co')
                    ->from("App:Command", "co")
                    ->innerJoin('co.orderTypePdv', 'mop')
                    ->innerJoin('mop.pdv', 'p')
                    ->where('p.id =:pdv')
                    ->setParameter('pdv', $this->getUser()->getPdv())
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getOneOrNullResult();
            } else {
                $order = $em->getRepository('App:Command')->find($id);
            }
            if (!is_null($order)) {
                $order->setAmount($paramFetcher->get('amount'));
                $order->setComment($paramFetcher->get('comment'));
                $order->setCorrected(1);
                $discount = ($paramFetcher->get('remise')) ? $paramFetcher->get('remise') : 0;
                $order->setDiscount($discount);
                $ip = $request->getClientIp();
                $order->setUpdatedIp($ip);
                $order->setUpdatedUser($this->getUser());
                $statut = $em->getRepository('App:Status')->find(8);
                $order->setStatus($statut);
                /*$orderPdv = $this->getOrderPdvByPdvAndOrder($paramFetcher->get('pdv'), $paramFetcher->get('orderType'));
                $order->setOrderTypePdv($orderPdv);
                if ($paramFetcher->get('orderType') == 1) {
                    $idAddress = $paramFetcher->get('idAddress');
                    $address = $em->getRepository('App:Address')->find($idAddress);
                    $order->setAddress($address);
                }*/
                $oldElements = $em->getRepository('App:CommandElement')
                    ->createQueryBuilder('ce')
                    ->InnerJoin('ce.command', 'c')
                    ->where('c.id = :id ')
                    ->setParameter('id', $id)
                    ->getQuery()
                    ->getResult();
                $oldMenu = $em->getRepository('App:CommandMenu')
                    ->createQueryBuilder('cm')
                    ->InnerJoin('cm.command', 'c')
                    ->where('c.id = :id ')
                    ->setParameter('id', $id)
                    ->getQuery()
                    ->getResult();
                $order->setCommandPaymentModes([]);
                foreach (array_merge($oldElements, $oldMenu) as $e) {
                    $em->remove($e);
                }
                $elements = $paramFetcher->get('elements');
                $menus = $paramFetcher->get('menus');
                $fees = $paramFetcher->get('fees');
                if ((is_array($elements) && count($elements) > 0) || (is_array($menus) && count($menus) > 0)) {
                    if (count($elements) > 0) {
                        foreach ($elements as $value) {
                            if (
                                array_key_exists('price', $value['size']) &&
                                array_key_exists('size', $value['size']) &&
                                array_key_exists('name', $value) &&
                                array_key_exists('qty', $value) &&
                                array_key_exists('comment', $value) &&
                                array_key_exists('total', $value) &&
                                array_key_exists('vat', $value) &&
                                array_key_exists('additionalCost', $value)
                            ) {
                                $commandElement = new CommandElement();
                                if ($paramFetcher->get('displayCode') && $value['code']) {
                                    $commandElement->setName($value['code'] . "-" . $value['name']);
                                } else {
                                    $commandElement->setName($value['name']);
                                }
                                $commandElement->setQuantity($value['qty']);
                                $commandElement->setComment($value['comment']);
                                $commandElement->setTotal($value['total']);
                                $commandElement->setPrice($value['size']['price']);
                                $commandElement->setVat($value['vat']);
                                $commandElement->setSize($value['size']['size']);
                                if (array_key_exists('level', $value['size'])) {
                                    $commandElement->setLevel($value['size']['level']);
                                }
                                $commandElement->setAdditionalCost($value['additionalCost']);
                                $commandElement->setCommand($order);
                                if (array_key_exists('id', $value['size']) && intval($value['size']['id']) > 0) {
                                    $elementSize = intval($value['size']['id']);
                                    $commandElement->setElementSize($elementSize);
                                }
                                $commandElement->setCode($value['code']);
                                $em->persist($commandElement);
                                /*if (array_key_exists('compositions', $value)) {
                                    foreach ($value['compositions'] as $valu) {
                                        $commandElementIngC = new CommandElementIngredients();
                                        $ingredient= $em->getRepository('App:Ingredient')->find($valu['id']);
                                        $commandElementIngC->setIngredient($ingredient);
                                        $commandElement->setElementSize($elementSize);
                                        $commandElementIngC->setName($valu['text']);
                                        $commandElementIngC->setCout(0);
                                        $commandElementIngC->setAction('comp');
                                        $commandElementIngC->setCommandElement($commandElement);
                                        $em->persist($commandElementIngC);
                                    }
                                }*/
                                if (array_key_exists('removedCompositions', $value)) {
                                    foreach ($value['removedCompositions'] as $val) {
                                        $commandElementIngR = new CommandElementIngredients();
                                        $ingredient = intval($val['id']);
                                        $commandElementIngR->setIngredient($ingredient);
                                        $commandElementIngR->setName($val['text']);
                                        $commandElementIngR->setCout(0);
                                        $commandElementIngR->setAction('remove');
                                        $commandElementIngR->setCommandElement($commandElement);
                                        $em->persist($commandElementIngR);
                                    }
                                }
                                if (array_key_exists('supplements', $value)) {
                                    foreach ($value['supplements'] as $va) {
                                        $commandElementIngS = new CommandElementIngredients();
                                        $ingredient = intval($va['id']);
                                        $commandElementIngS->setIngredient($ingredient);
                                        $commandElementIngS->setName($va['text']);
                                        $commandElementIngS->setCout($va['price']);
                                        $commandElementIngS->setAction('supp');
                                        $commandElementIngS->setCommandElement($commandElement);
                                        $em->persist($commandElementIngS);
                                    }
                                }
                                if (array_key_exists('optionIngredients', $value)) {
                                    foreach ($value['optionIngredients'] as $v) {
                                        $commandElementIngO = new CommandElementIngredients();
                                        $ingredient = intval($v['id']);
                                        $commandElementIngO->setIngredient($ingredient);
                                        $commandElementIngO->setName($v['text']);
                                        $commandElementIngO->setCout(0);
                                        $commandElementIngO->setAction('opt');
                                        $commandElementIngO->setCommandElement($commandElement);
                                        $em->persist($commandElementIngO);
                                    }
                                }

                            } else {
                                return new JsonResponse(['error' => 1], 400);
                            }
                        }
                    }
                    if (count($menus) > 0) {
                        foreach ($menus as $menu) {
                            if (
                                array_key_exists('price', $menu) &&
                                array_key_exists('comment', $menu) &&
                                array_key_exists('qty', $menu) &&
                                array_key_exists('title', $menu) &&
                                array_key_exists('total', $menu) &&
                                array_key_exists('additionalCost', $menu) &&
                                array_key_exists('vat', $menu) &&
                                array_key_exists('elements', $menu) &&
                                is_array($menu['elements']) &&
                                count($menu['elements']) > 0
                            ) {
                                $commandMenu = new CommandMenu();
                                if ($paramFetcher->get('displayCode') && $menu['code']) {
                                    $commandMenu->setTitle($menu['code'] . "-" . $menu['title']);
                                } else {
                                    $commandMenu->setTitle($menu['title']);
                                }
                                $commandMenu->setQuantity($menu['qty']);
                                $commandMenu->setComment($menu['comment']);
                                $commandMenu->setTotal($menu['total']);
                                $commandMenu->setPrice($menu['price']);
                                $commandMenu->setVat($menu['vat']);
                                $commandMenu->setAdditionalCost($menu['additionalCost']);
                                $commandMenu->setCommand($order);
                                $commandMenu->setCode($menu['code']);
                                $m = intval($menu['id']);
                                $commandMenu->setMenu($m);
                                $em->persist($commandMenu);
                                foreach ($menu['elements'] as $element) {
                                    if (
                                        array_key_exists('price', $element) &&
                                        array_key_exists('name', $element) &&
                                        array_key_exists('qty', $element) &&
                                        array_key_exists('comment', $element) &&
                                        array_key_exists('label', $element['size'])
                                    ) {
                                        $commandElement = new CommandElement();
                                        $commandElement->setName($element['name']);
                                        $commandElement->setQuantity($element['qty']);
                                        $commandElement->setComment($element['comment']);
                                        $commandElement->setTotal(0);
                                        $commandElement->setPrice(0);
                                        $commandElement->setVat(0);
                                        $commandElement->setSize($element['size']['label']);
                                        $commandElement->setLevel($element['size']['level']);
                                        $commandElement->setAdditionalCost(0);
                                        if (array_key_exists('id', $element['size'])) {
                                            $elementSize = intval($element['size']['id']);
                                            $commandElement->setElementSize($elementSize);
                                        }
                                        $commandElement->setCommandMenu($commandMenu);
                                        $commandElement->setCommand($order);
                                        $em->persist($commandElement);
                                        if (array_key_exists('removedCompositions', $element)) {
                                            foreach ($element['removedCompositions'] as $val) {
                                                $commandElementIngR = new CommandElementIngredients();
                                                $ingredient = intval($val['id']);
                                                $commandElementIngR->setIngredient($ingredient);
                                                $commandElementIngR->setName($val['text']);
                                                $commandElementIngR->setCout(0);
                                                $commandElementIngR->setAction('remove');
                                                $commandElementIngR->setCommandElement($commandElement);
                                                $em->persist($commandElementIngR);
                                            }
                                        }
                                        if (array_key_exists('supplements', $element)) {
                                            foreach ($element['supplements'] as $va) {
                                                $commandElementIngS = new CommandElementIngredients();
                                                $ingredient = intval($va['id']);
                                                $commandElementIngS->setIngredient($ingredient);
                                                $commandElementIngS->setName($va['text']);
                                                $commandElementIngS->setCout($va['price']);
                                                $commandElementIngS->setAction('supp');
                                                $commandElementIngS->setCommandElement($commandElement);
                                                $em->persist($commandElementIngS);
                                            }
                                        }
                                        if (array_key_exists('optionIngredients', $element)) {
                                            foreach ($element['optionIngredients'] as $v) {
                                                $commandElementIngO = new CommandElementIngredients();
                                                $ingredient = intval($v['id']);
                                                $commandElementIngO->setIngredient($ingredient);
                                                $commandElementIngO->setName($v['text']);
                                                $commandElementIngO->setCout(0);
                                                $commandElementIngO->setAction('opt');
                                                $commandElementIngO->setCommandElement($commandElement);
                                                $em->persist($commandElementIngO);
                                            }
                                        }
                                    } else {
                                        return new JsonResponse(['error' => 2], 400);
                                    }
                                }
                                if (
                                    array_key_exists('additionalOptions', $menu) &&
                                    is_array($menu['additionalOptions']) &&
                                    count($menu['additionalOptions']) > 0
                                ) {
                                    foreach ($menu['additionalOptions'] as $element) {
                                        if (
                                            array_key_exists('price', $element) &&
                                            array_key_exists('name', $element) &&
                                            array_key_exists('qty', $element) &&
                                            array_key_exists('size', $element)
                                        ) {
                                            $commandElement = new CommandElement();
                                            $commandElement->setName($element['name']);
                                            $commandElement->setQuantity($element['qty']);
                                            $commandElement->setComment('');
                                            $commandElement->setTotal(0);
                                            $commandElement->setPrice($element['price']);
                                            $commandElement->setVat(0);
                                            $commandElement->setSize($element['size']);
                                            //$commandElement->setLevel($element['level']);
                                            $commandElement->setAdditionalCost(0);
                                            $commandElement->setCommandMenu($commandMenu);
                                            $commandElement->setCommand($order);
                                            $em->persist($commandElement);
                                        }
                                    }
                                }
                            }
                        }
                    }
                } else {
                    return new JsonResponse(['error' => 3], 400);
                }
                if($fees){
                        $commandElement = new CommandElement();
                        $commandElement->setName($fees['name']);
                        $commandElement->setQuantity(0);
                        $commandElement->setComment('');
                        $commandElement->setTotal($fees['price']);
                        $commandElement->setPrice($fees['price']);
                        $commandElement->setVat(0);
                        $commandElement->setSize('');
                        $commandElement->setAdditionalCost(0);
                        $commandElement->setCommand($order);
                        $commandElement->setCode('');
                        $em->persist($commandElement);
                }
                $payementModes = $paramFetcher->get('payementModes');
                if (is_array($payementModes) && count($payementModes) > 0) {
                    foreach ($payementModes as $val) {
                        $payementMode = $em->getRepository('App:PaymentMode')->find($val['id']);
                        $commandPaymentMode = new CommandPaymentMode();
                        if ($payementMode) {
                            $commandPaymentMode->setPaymentMode($payementMode);
                            $commandPaymentMode->setCommand($order);
                            $commandPaymentMode->setAmount(0);
                            $em->persist($commandPaymentMode);
                        } else {
                            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                        }
                    }
                    $commandPaymentModes = $em->getRepository('App:commandPaymentMode')->findBy(array('command' => $order));
                    foreach ($commandPaymentModes as $v) {
                        $em->remove($v);
                    }

                } else {
                    return new JsonResponse(['error' => 5], 400);
                }
                $em->persist($order);
                $em->flush();
                $qb = $em->createQueryBuilder();
                $pdv2 = $qb->select("pdv.id,pdv.name,pdv.email")
                    ->from("App:Command", "co")
                    ->innerJoin('co.orderTypePdv', 'otp')
                    ->innerJoin('otp.pdv', 'pdv')
                    ->where('co.id=:order')
                    ->setParameter('order', $id)
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getOneOrNullResult();
                $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'order', 'num'=>$order->getNum(), 'id' => $id, 'pdv' => $pdv2['id']]);
                $subject = $pdv2['name'] . ' - Modification de commande n°' . $order->getNum();
                $to = $pdv2['email'];
                $template = $this->renderView('cmd.html.twig', array('id' => $order->getNum()));
                $this->sendEmail($to, $subject, $template);
                $result = [
                    'id' => $order->getId(),
                    'success' => true,
                    'code' => 200
                ];
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($result, 'json');
                return new JsonResponse(json_decode($data));
            }
        }
    }

    /**
     * @Rest\Get("/orders/processed")
     */
    public function getProcessedOrders()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $now = new DateTime();
        $query = "co.id,co.num,co.discount,co.corrected,co.amount,t.label as type,mop.minDelay,co.createdAt,p.id as pdvId,
        p.name as pdvName,c.id as customerId,c.firstname as customerFirstname,c.lastname as customerLastname,c.phone as customerPhone,c.indicator";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $query .= ",u.id as idUser,u.firstname as userFirstname,u.lastname as userLastname";
        }
        $orders = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $orders = $orders->leftJoin('co.createdUser', 'u');
        }
        $orders = $orders->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('co.status', 's')
            ->where('s.id =:status')
            ->andWhere('co.createdAt >=:date')
            ->andWhere('co.createdAt <=:date2');
        if ($this->getUser()->getPdv()) {
            $orders = $orders->andWhere('p.id =:pdv')
                ->setParameter('pdv', $this->getUser()->getPdv());
        }
        $orders = $orders->setParameter('status', 8)
            ->setParameter('date', $now->format('Y-m-d 00:00:00'))
            ->setParameter('date2', $now->format('Y-m-d 23:59:59'))
            ->orderBy('co.createdAt')
            ->getQuery()->getResult();
        foreach ($orders as $key => $order) {
            $ord = $em->getRepository('App:Command')->find($order['id']);
            $orders[$key]['PaymentModes'] = $ord->getCommandPaymentModes();
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($orders, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/orders/day")
     */
    public function getDayOrders()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $now = new DateTime();
        $query = "co.id,co.num,co.discount,co.corrected,co.amount,s.id as status,s.id as oldStatus,t.label as type,mop.minDelay,
        co.createdAt,p.id as pdvId,p.name as pdvName,c.id as customerId,c.firstname as customerFirstname,c.lastname as customerLastname,
        c.phone as customerPhone,c.indicator";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $query .= ",u.id as idUser,u.firstname as userFirstname,u.lastname as userLastname";
        }
        if ($this->getUser()->getPdv()) {
            $query .= ",team.id as teamMember,team.id as oldTeamMember";
        }
        $orders = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $orders = $orders->leftJoin('co.createdUser', 'u');
        }
        if ($this->getUser()->getPdv()) {
            $orders = $orders->leftJoin('co.team', 'team');
        }
        $orders = $orders->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('co.status', 's')
            ->where('co.createdAt >=:date')
            ->andWhere('co.createdAt <=:date2');
        if ($this->getUser()->getPdv()) {
            $orders = $orders->andWhere('p.id =:pdv')
                ->setParameter('pdv', $this->getUser()->getPdv());
        }
        $orders = $orders->setParameter('date', $now->format('Y-m-d 00:00:00'))
            ->setParameter('date2', $now->format('Y-m-d 23:59:59'))
            ->orderBy('co.createdAt')
            ->getQuery()->getResult();
        foreach ($orders as $key => $order) {
            $ord = $em->getRepository('App:Command')->find($order['id']);
            $orders[$key]['PaymentModes'] = $ord->getCommandPaymentModes();
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($orders, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Post("/orders/collection")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="team", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="date", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="paymentModes", nullable=true,allowBlank=true,strict=true)
     */
    public function getCollectionOrders(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $now = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('date'));
        $query = "co.id,co.num,co.discount,co.corrected,co.amount,s.id as status,s.id as oldStatus,t.label as type,mop.minDelay,
        co.createdAt,p.id as pdvId,p.name as pdvName,c.id as customerId,c.firstname as customerFirstname,c.lastname as customerLastname,
        c.phone as customerPhone,SUM(cmp.amount) as sumModes, (co.amount - SUM(cmp.amount)) as sumAmount";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $query .= ",u.id as idUser,u.firstname as userFirstname,u.lastname as userLastname";
        }
        if ($this->getUser()->getPdv()) {
            $query .= ",team.id as teamMember,team.designation as teamName";
        }
        $orders = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $orders = $orders->leftJoin('co.createdUser', 'u');
        }
        if ($this->getUser()->getPdv()) {
            $orders = $orders->leftJoin('co.team', 'team');
        }
        $orders = $orders->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('co.status', 's')
            ->innerJoin('co.commandPaymentModes', 'cmp')
            ->innerJoin('cmp.paymentMode', 'mp')
            ->where('co.team IS NOT NULL')
            ->andWhere('co.createdAt >=:date')
            ->andWhere('co.createdAt <=:date2');

        if ($paramFetcher->get('team')) {
            $orders = $orders->andWhere('team.id =:tea')
                ->setParameter('tea', $paramFetcher->get('team'));
        }

        if ($paramFetcher->get('paymentModes')) {
            $orders = $orders->andWhere('mp.id IN (:paymentModes)')
                ->setParameter('paymentModes', $paramFetcher->get('paymentModes'));
        }

        if ($this->getUser()->getPdv()) {
            $orders = $orders->andWhere('p.id =:pdv')
                ->setParameter('pdv', $this->getUser()->getPdv());
        }
        $orders = $orders->setParameter('date', $now->format('Y-m-d 00:00:00'))
            ->setParameter('date2', $now->format('Y-m-d 23:59:59'))
            ->groupBy('co.id')
            ->getQuery()->getResult();
        foreach ($orders as $key => $order) {
            $ord = $em->getRepository('App:Command')->find($order['id']);
            $orders[$key]['PaymentModes'] = $ord->getCommandPaymentModes();
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($orders, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/orders/collectionOrder")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="id", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="paymentMode", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="amount", nullable=false,allowBlank=false,strict=true)
     */
    public function updateOrderCollection(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        if ($this->getUser()->hasRole('RULE_COLLECTION') || $this->getUser()->getProfil()->getId() == 3) {
            $order = $em->getRepository('App:Command')->find($paramFetcher->get('id'));
            $pm = $em->getRepository('App:PaymentMode')->find($paramFetcher->get('paymentMode'));
            $orderPaymentMode = $em->getRepository('App:CommandPaymentMode')->findOneBy(array('command' => $order, 'paymentMode' => $pm));
            if (!is_null($orderPaymentMode)) {
                $orderPaymentMode->setAmount($paramFetcher->get('amount'));
                $em->persist($orderPaymentMode);
                $em->flush();
                return new JsonResponse ([], 200);

            } else {
                $commandPaymentMode = new CommandPaymentMode();
                if ($pm) {
                    $commandPaymentMode->setPaymentMode($pm);
                    $commandPaymentMode->setCommand($order);
                    $commandPaymentMode->setAmount($paramFetcher->get('amount'));
                    $em->persist($commandPaymentMode);
                    $em->flush();
                    return new JsonResponse ([], 200);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse('', 403);
        }


    }

    /**
     * @Rest\Put("/orders/collectionOrder/global")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="idsOrders", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="idPaymentMode", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="amount", nullable=false,allowBlank=false,strict=true)
     */
    public function updateOrderCollectionForGlobal(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        if ($this->getUser()->hasRole('RULE_COLLECTION') || $this->getUser()->getProfil()->getId() == 3) {
            $amount = $paramFetcher->get('amount');
            $idsOrders = $paramFetcher->get('idsOrders');
            $idMode = $paramFetcher->get('idPaymentMode');
            $tabOrders = $this->getTabOrdersForCollGlobal($idsOrders);
            $pm = $em->getRepository('App:PaymentMode')->find($idMode);
            $price = $amount;
            if (count($tabOrders) > 0) {
                foreach ($tabOrders as $key => $order) {
                    $cp = 0;
                    $M = $order['amount'] - $order['sumModes'];
                    if ($M > 0 && $M < $order['amount'] && $price > 0 || $M == $order['amount'] && $price > 0) {
                        if ($M > 0 && $M < $order['amount']) {
                            $amountAtt = ($M > $amount) ? $amount : $M;
                            $price = $amount - $M;
                        }
                        if ($M == $order['amount']) {
                            $amountAtt = ($amount > $order['amount']) ? $order['amount'] : $amount;
                            $price = $amount - $order['amount'];
                        }
                        foreach ($order['PaymentModes'] as $p) {
                            if ($p->getPaymentMode()->getId() == $idMode) {
                                $p->setAmount($amountAtt);
                                $em->persist($p);
                                $em->flush();
                                $cp++;
                            }
                        }
                        if ($cp == 0) {
                            $commandPaymentMode = new CommandPaymentMode();
                            $ord = $em->getRepository('App:Command')->find($order['id']);
                            $commandPaymentMode->setPaymentMode($pm);
                            $commandPaymentMode->setCommand($ord);
                            $commandPaymentMode->setAmount($amountAtt);
                            $em->persist($commandPaymentMode);
                            $em->flush();
                        }
                        $amount = $price;
                        if ($price == 0 || $key == (count($tabOrders) - 1) && $price > 0) {
                            if ($key == (count($tabOrders) - 1) && $price > 0) {
                                $ct = 0;
                                $qb = $em->createQueryBuilder();
                                $paymentModesLastOrder = $qb->select("cpm")
                                    ->from("App:CommandPaymentMode", "cpm")
                                    ->innerJoin('cpm.paymentMode', 'p')
                                    ->innerJoin('cpm.command', 'c')
                                    ->where('c.id =:idOrder')
                                    ->setParameter('idOrder', $tabOrders[count($tabOrders) - 1]['id'])
                                    ->getQuery()->getResult();
                                foreach ($paymentModesLastOrder as $p) {
                                    if ($p->getPaymentMode()->getId() == $idMode) {
                                        $pr = $p->getAmount() + $price;
                                        $p->setAmount($pr);
                                        $em->persist($p);
                                        $ct++;
                                    }
                                }
                                if ($ct == 0) {
                                    $commandPaymentMode = new CommandPaymentMode();
                                    $ord = $em->getRepository('App:Command')->find($order['id']);
                                    $commandPaymentMode->setPaymentMode($pm);
                                    $commandPaymentMode->setCommand($ord);
                                    $commandPaymentMode->setAmount($price);
                                    $em->persist($commandPaymentMode);
                                }
                            }
                            break;
                        }
                    }
                }
                $em->flush();
                return new JsonResponse([], 200);
            } else {
                return new JsonResponse([], Response::HTTP_BAD_REQUEST);
            }
        } else {
            return new JsonResponse('', 403);
        }
    }


    /**
     * @Rest\Put("/orders/collectionOrder/clearMode")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="idsOrders", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="idPaymentMode", nullable=false,allowBlank=false,strict=true)
     */
    public function clearModeForGlobal(ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        $em = $this->getDoctrine()->getManager();
        if ($this->getUser()->hasRole('RULE_COLLECTION') || $this->getUser()->getProfil()->getId() == 3) {
            $idsOrders = $paramFetcher->get('idsOrders');
            $idMode = $paramFetcher->get('idPaymentMode');
            if (count($idsOrders) > 0) {
                foreach ($idsOrders as $key => $idOrder) {
                    $qb = $em->createQueryBuilder();
                    $paymentModes = $qb->select("cpm")
                        ->from("App:CommandPaymentMode", "cpm")
                        ->innerJoin('cpm.paymentMode', 'p')
                        ->innerJoin('cpm.command', 'c')
                        ->where('c.id =:idOrder')
                        ->setParameter('idOrder', $idOrder)
                        ->getQuery()->getResult();
                    foreach ($paymentModes as $p) {
                        if ($p->getPaymentMode()->getId() == $idMode) {
                            $p->setAmount(0);
                            $em->persist($p);
                        }
                    }
                }
                $em->flush();
                return new JsonResponse ([], 200);
            } else {
                return new JsonResponse([], Response::HTTP_BAD_REQUEST);
            }
        } else {
            return new JsonResponse('', 403);
        }
    }

    public function getTabOrdersForCollGlobal($ids)
    {
        $tabOrders = [];
        foreach ($ids as $idOrder) {
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $query = "co.id,co.amount,SUM(cmp.amount) as sumModes, (co.amount - SUM(cmp.amount)) as sumAmount";
            $order = $qb->select($query)
                ->from("App:Command", "co")
                ->innerJoin('co.commandPaymentModes', 'cmp')
                ->innerJoin('cmp.paymentMode', 'mp')
                ->where('co.id =:order')
                ->setParameter('order', $idOrder)
                ->setMaxResults(1)
                ->getQuery()->getOneOrNullResult();
            $qb1 = $em->createQueryBuilder();
            $paymentModesOrder = $qb1->select("cpm")
                ->from("App:CommandPaymentMode", "cpm")
                ->innerJoin('cpm.paymentMode', 'p')
                ->innerJoin('cpm.command', 'c')
                ->where('c.id =:idOrder')
                ->setParameter('idOrder', $idOrder)
                ->getQuery()->getResult();
            $order['PaymentModes'] = $paymentModesOrder;
            $tabOrders[] = $order;
        }
        return $tabOrders;
    }

    /**
     * @Rest\Put("/orders/status")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="command", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="status", nullable=false,allowBlank=false,strict=true)
     */
    public function updateOrderStatus(ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        $em = $this->getDoctrine()->getManager();
        $idOrder = intval($paramFetcher->get('command'));
        $idStatus = intval($paramFetcher->get('status'));
        $order = $em->getRepository('App:Command')->find($idOrder);
        $qb = $em->createQueryBuilder();
        $idPdv = $qb->select("pdv.id")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'otp')
            ->innerJoin('otp.pdv', 'pdv')
            ->where('co.id=:order')
            ->setParameter('order', $idOrder)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult()['id'];
        $status = $em->getRepository('App:Status')->find($idStatus);
        $order->setStatus($status);
        $em->persist($order);
        $em->flush();
        $sendEvent->sendMessage(['type' => 'updateOrderStatus', 'entity' => 'order', 'id' => $idOrder, 'status' => $idStatus, 'pdv' => $idPdv]);
        return new JsonResponse ([
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/orders/team")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="command", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="team", nullable=true,allowBlank=true,strict=true)
     */
    public function updateOrderTeam(ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {

        if ($this->getUser()->getProfil()->getId() != 3 && $this->getUser()->getProfil()->getId() != 4) {
            return new JsonResponse('', 403);
        } else {
            $em = $this->getDoctrine()->getManager();
            $order = $em->getRepository('App:Command')->find($paramFetcher->get('command'));
            if (intval($paramFetcher->get('team')) > 0) {
                $team = $em->getRepository('App:Team')->find($paramFetcher->get('team'));
            } else {
                $team = null;
            }
            $order->setTeam($team);
            $em->persist($order);
            $em->flush();
            $sendEvent->sendMessage(['type' => 'updateOrderTeam', 'entity' => 'order', 'id' => $order->getId(), 'team' => ($team) ? $team->getId() : '']);
            return new JsonResponse ([
                'success' => true,
                'code' => 200
            ]);
        }
    }

    /**
     * @Rest\Get("/orders/historic")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @QueryParam(name="amountMin", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="amountMax", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="customer", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="dateD", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="dateF", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="pdv", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="status", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="user", nullable=true,allowBlank=true,strict=true,default=-1)
     * @QueryParam(name="current_page", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="count_per_page", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="search", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="mode", nullable=true,allowBlank=true,strict=true)
     * @QueryParam(name="type", nullable=true,allowBlank=true,strict=true)
     */
    public function getHistoricOrdersByPage(Request $request, ParamFetcher $paramFetcher)
    {
        $data = [];
        $data['count_per_page'] = intval($paramFetcher->get('count_per_page'));
        $data['current_page'] = intval($paramFetcher->get('current_page'));
        $range_start = (intval($data['current_page']) - 1) * intval($data['count_per_page']);
        $qb=$this->getHistoricOrders($paramFetcher);
        $query = $qb->groupBy('co.id')->getQuery()->setFirstResult($range_start)->setMaxResults($data['count_per_page']);
        $paginator = new Paginator($query, false);
        $data['data'] = $query->execute();
        $data['total_page'] = $paginator->count();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        $response = new Response();
        $response->setContent($data);
        return $response;
    }

    /**
     * @Rest\Post("/orders/historic/export")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="amountMin", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="amountMax", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="customer", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="dateD", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="dateF", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="pdv", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="status", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="user", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="search", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="mode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="type", nullable=true,allowBlank=true,strict=true)
     */
    public function exportHistoricOrders(ParamFetcher $paramFetcher)
    {
        $isNotRestaurateur = in_array($this->getUser()->getProfil()->getId(), [1, 5, 6]);
        $qb=$this->getexportHistoricOrders($paramFetcher);
        $query = $qb->groupBy('co.id')->getQuery();
        $data = $query->execute();
        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $header = ['Pdv', 'Date', 'Client', 'Type', 'Montant', 'Modes de réglement', 'Statut', 'Prise par'];
        } else {
            $header = ['Date', 'Client', 'Type', 'Montant', 'Modes de réglement', 'Statut'];
        }
        array_unshift($data, $header);
        $sheet->fromArray($data, NULL);
        $sheet->fromArray($data, NULL);
        $cells = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I'];
        foreach ($cells as $c) {
            $sheet->getColumnDimension($c)->setAutoSize(true);
        }
        $writer = new Xlsx($spreadsheet);
        $reponse = new StreamedResponse(
            function () use ($writer) {
                $writer->save('php://output');
            }
        );
        $reponse->headers->set('Content-Type', 'application/vnd.ms-excel');
        return $reponse;
    }


    /**
     * @Rest\Get("/orders/{id}/paymentModes")
     */
    public function getOrderPaymentModes($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $paymentModes = $qb->select("m.label")
            ->from("App:Command", "co")
            ->innerJoin('co.commandPaymentModes', 'pm')
            ->innerJoin('pm.paymentMode', 'm')
            ->where('co.id = :order')
            ->setParameter('order', $id)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($paymentModes, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/orders/{id}/info")
     */
    public function getInfoOrder($id,$response=true)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $order = $qb->select("co.num,co.comment,co.corrected,co.createdAt,co.amount,co.discount,co.orderProg,t.label as type,t.id as orderType,p.phone as pdvPhone,p.siret as pdvSiret,
        p.address as pdvAddress,p.name as pdvName,p.printer,a.streetName,a.streetNumber,a.city,
        a.postalCode,a.building,a.staircase,a.floor,a.door,a.inputCode,a.intercom,a.complementAddress,c.firstname as customerFirstname,c.society as customerSociety,c.lastname as customerLastname,c.phone as customerPhone,c.secondaryPhone as customerSecondaryPhone,u.id as user")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('co.createdUser', 'u')
            ->leftJoin('co.address', 'a')
            ->where('co.id = :order')
            ->setParameter('order', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if($response){
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($order, 'json');
            return new JsonResponse(json_decode($data));
        }else{
            return $order;
        }
    }

    /**
     * @Rest\Get("/orders/{id}/details")
     */
    public function getOrder($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $order = $qb->select("co as cmd,t.id as mode,p.id as idPdv,cp.id As idCustomer")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('mop.pdv', 'p')
            ->leftJoin('co.address', 'a')
            ->where('co.id = :order')
            ->setParameter('order', $id);
        if ($this->getUser()->getPdv()) {
            $order = $order->andWhere('p.id =:pdv')->setParameter('pdv', $this->getUser()->getPdv());
        }
        $order = $order->setMaxResults(1)->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($order, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/orders/{id}/forProcessed")
     */
    public function getOrderForProc($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $query="co.id,co.num,co.discount,co.corrected,co.amount,s.id as status,t.label as type,mop.minDelay,co.createdAt,p.id as pdvId,p.name as pdvName,c.id as customerId,c.firstname as customerFirstname,c.lastname as customerLastname,c.phone as customerPhone,c.indicator, co.orderProg";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $query .= ",u.id as idUser,u.firstname as userFirstname,u.lastname as userLastname";
        }
        if ($this->getUser()->getPdv()) {
            $query .= ",team.id as teamMember,team.id as oldTeamMember";
        }
        $order = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('co.status', 's')
            ->andWhere('co.id = :order');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $order = $order->leftJoin('co.createdUser', 'u');
        }
        if ($this->getUser()->getPdv()) {
            $order = $order->leftJoin('co.team', 'team');
        }
        if ($this->getUser()->getProfil()->getId() == 3 || $this->getUser()->getProfil()->getId() == 4) {
            $order = $order->andWhere('p.id = :idPdv')
                ->setParameter('idPdv', $this->getUser()->getPdv());
        }
        $order = $order->setParameter('order', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if($order){
                $ord = $em->getRepository('App:Command')->find($order['id']);
                $order['PaymentModes'] = $ord->getCommandPaymentModes();
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($order, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/orders/{id}/elements")
     */
    public function getOrderElements($id, $reponse = true,$fees=false)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("ce.id,ce.quantity,ce.comment,ce.level,ce.total,ce.price + ce.additionalCost as price,ce.additionalCost,ce.total,ce.name,ce.size")
            ->from("App:Command", "co")
            ->innerJoin('co.commandElements', 'ce')
            ->leftJoin('ce.commandMenu', 'cm')
            ->where('co.id = :order')
            ->andWhere('cm.id IS NULL');
        if($fees){
            $elements =$elements->andWhere('ce.quantity=0');
        }else{
            $elements =$elements->andWhere('ce.quantity!=0');
        }
        $elements =$elements->setParameter('order', $id)
            ->getQuery()->getResult();

        foreach ($elements as $key => $value) {
            $elements[$key]['optionIngredients'] = $this->getElementComSuppRemovOpt($value['id'], 'opt');
            $elements[$key]['removedCompositions'] = $this->getElementComSuppRemovOpt($value['id'], 'remove');
            $elements[$key]['supplements'] = $this->getElementComSuppRemovOpt($value['id'], 'supp');
        }
        if ($reponse == true) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($elements, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return $elements;
        }

    }

    /**
     * @Rest\Get("/orders/{id}/details/elements")
     */
    public function getOrderDetailsElements($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("e.id,ce.additionalCost,ce.id as commandElement,es.id as elementSize,e.hasSupp,e.code,e.name,e.vat, ce.quantity as qty,ce.total,ce.comment,e.maxNbrIngOptions,cf.id as family,e.priceSuppApp,ce.price+ce.additionalCost as priceWithSupp")
            ->from("App:CommandElement", "ce")
            ->innerJoin('ce.command', 'c');
        if ($this->getUser()->getPdv()) {
            $elements = $elements->innerJoin('c.orderTypePdv', 'mop')
                ->innerJoin('mop.pdv', 'p');
        }
        $elements = $elements->innerJoin('App:ElementSize', 'es', 'WITH', 'es.id = ce.elementSize')
            ->innerJoin('es.element', 'e')
            ->innerJoin('e.family', 'cf')
            ->where('c.id=:id')
            ->andWhere('ce.elementSize is NOT NULL')
            ->andWhere('ce.commandMenu is NULL')
            ->setParameter('id', $id);
        if ($this->getUser()->getPdv()) {
            $elements = $elements->andWhere('p.id =:pdv')->setParameter('pdv', $this->getUser()->getPdv());
        }
        $elements = $elements->getQuery()->getResult();
        $qb = $em->createQueryBuilder();
        $elements2 = $qb->select("ce.additionalCost,ce.id as commandElement,ce.code,ce.name,ce.vat, ce.quantity as qty,ce.total,ce.comment,ce.price+ce.additionalCost as priceWithSupp,ce.price,ce.size,1 as customized")
            ->from("App:CommandElement", "ce")
            ->innerJoin('ce.command', 'c');
        if ($this->getUser()->getPdv()) {
            $elements2 = $elements2->innerJoin('c.orderTypePdv', 'mop')
                ->innerJoin('mop.pdv', 'p');
        }
        $elements2 = $elements2->where('c.id=:id')
            ->andWhere('ce.elementSize is NULL')
            ->andWhere('ce.commandMenu is NULL')
            ->setParameter('id', $id);
        if ($this->getUser()->getPdv()) {
            $elements2 = $elements2->andWhere('p.id =:pdv')->setParameter('pdv', $this->getUser()->getPdv());
        }
        $elements2 = $elements2->getQuery()->getResult();
        foreach ($elements2 as $key => $value) {
            $elements2[$key]['size'] = array("id" => "", "size" => $elements2[$key]['size'], "price" => $elements2[$key]['price']);
            $elements2[$key]['optionIngredients'] = array();
            $elements2[$key]['removedCompositions'] = array();
            $elements2[$key]['supplements'] = array();
        }
        foreach ($elements as $key => $value) {
            $qb = $em->createQueryBuilder();
            $elements[$key]['size'] = $qb->select("es.id,s.id as idSize,s.label as size,s.level,es.price,es.available,es.priceSupp")
                ->from("App:ElementSize", "es")
                ->innerJoin('es.size', 's')
                ->where('es.id=:id')
                ->setParameter('id', $value['elementSize'])
                ->setMaxResults(1)
                ->getQuery()
                ->getOneOrNullResult();
            $elements[$key]['optionIngredients'] = $this->getCommandElementComSuppRemovOpt($value['commandElement'], 'opt');
            $elements[$key]['removedCompositions'] = $this->getCommandElementComSuppRemovOpt($value['commandElement'], 'remove');
            $elements[$key]['supplements'] = $this->getCommandElementComSuppRemovOpt($value['commandElement'], 'supp');
        }
        $result = array_merge($elements, $elements2);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/orders/{id}/menus")
     */
    public function getOrderMenus($id, $reponse = true)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("cm.id,cm.quantity,cm.comment,cm.total,cm.price + cm.additionalCost as price,cm.title")
            ->from("App:CommandMenu", "cm")
            ->innerJoin('cm.command', 'c')
            ->where('c.id = :order')
            ->setParameter('order', $id)
            ->getQuery()->getResult();

        foreach ($menus as $ke => $val) {
            $menus[$ke]['elements'] = $this->getMenuElements($val['id']);
            foreach ($menus[$ke]['elements'] as $key => $value) {
                $menus[$ke]['elements'][$key]['optionIngredients'] = $this->getElementComSuppRemovOpt($value['id'], 'opt');
                //  $elements[$key]['compositions']=$this->getElementComSuppRemovOpt($value['id'],'comp');
                $menus[$ke]['elements'][$key]['removedCompositions'] = $this->getElementComSuppRemovOpt($value['id'], 'remove');
                $menus[$ke]['elements'][$key]['supplements'] = $this->getElementComSuppRemovOpt($value['id'], 'supp');
            }
        }
        if ($reponse) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($menus, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return $menus;
        }

    }

    /**
     * @Rest\Get("/orders/{id}/details/menus")
     */
    public function getOrderDetailsMenus($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("m.id,o.id as offer,cm.id as commandMenu,cm.quantity as qty,cm.comment,cm.total,cm.price,cm.vat,cm.code,cm.additionalCost,cm.price + cm.additionalCost as priceWithSupp,cm.title")
            ->from("App:CommandMenu", "cm")
            ->innerJoin('cm.command', 'c');
        if ($this->getUser()->getPdv()) {
            $menus = $menus->innerJoin('c.orderTypePdv', 'mop')
                ->innerJoin('mop.pdv', 'p');
        }
        $menus = $menus->innerJoin('App:Menu', 'm', 'WITH', 'm.id = cm.menu')
            ->leftJoin('m.offer', 'o')
            ->where('c.id = :order')
            ->setParameter('order', $id);
        if ($this->getUser()->getPdv()) {
            $menus = $menus->andWhere('p.id =:pdv')->setParameter('pdv', $this->getUser()->getPdv());
        }
        $menus = $menus->getQuery()->getResult();

        foreach ($menus as $ke => $val) {
            $menus[$ke]['elements'] = $this->getCommandMenuElements($val['commandMenu']);
            $menus[$ke]['additionalOptions'] = $this->getCommandMenuOptions($val['commandMenu']);
            foreach ($menus[$ke]['elements'] as $key => $value) {
                $menus[$ke]['elements'][$key]['size'] = array('id' => $value['id'], 'idSize' => $value['sizeId'], 'label' => $value['sizeLabel'], 'level' => $value['sizeLevel']);
                unset($menus[$ke]['elements'][$key]['sizeId']);
                unset($menus[$ke]['elements'][$key]['sizeLabel']);
                unset($menus[$ke]['elements'][$key]['sizeLevel']);
                $menus[$ke]['elements'][$key]['optionIngredients'] = $this->getCommandElementComSuppRemovOpt($value['commandElement'], 'opt');
                //  $elements[$key]['compositions']=$this->getElementComSuppRemovOpt($value['id'],'comp');
                $menus[$ke]['elements'][$key]['removedCompositions'] = $this->getCommandElementComSuppRemovOpt($value['commandElement'], 'remove');
                $menus[$ke]['elements'][$key]['supplements'] = $this->getCommandElementComSuppRemovOpt($value['commandElement'], 'supp');
            }
        }

        $serializer = $this->get('serializer');
        $data = $serializer->serialize($menus, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/ticket/{type}/orders/{id}")
     */
    public function getTicketClient($id,$type)
    {

        $order = $this->getInfoOrder($id,false);
        $data=array('order' => $order);
        if($type==1){
            $paymentModes = $this->getPaymentModesOfOrder($id);
            $data['paymentModes']=$paymentModes;
        }
        $elements = $this->getOrderElements($id, false,false);
        $fees= $this->getOrderElements($id, false,true);
        $data['fees']=$fees;
        $data['elements']=$elements;
        $menus = $this->getOrderMenus($id, false);
        $data['menus']=$menus;
        $temp=($type==1)?'ticket.html.twig':'ticketcuisine.html.twig';
        $template = $this->renderView($temp,$data);
        return new JsonResponse(array('template'=>$template,'num'=>$order['num'],'printer'=>$order['printer']));
    }

    public function getElementComSuppRemovOpt($idElement, $action)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("c.name,c.cout")
            ->from("App:CommandElementIngredients", "c")
            ->innerJoin('c.commandElement', 'ce')
            ->where('ce.id = :el')
            ->andWhere('c.action = :action')
            ->setParameter('el', $idElement)
            ->setParameter('action', $action)
            ->getQuery()->getResult();

        return $tab;
    }

    public function getCommandElementComSuppRemovOpt($idCmdElement, $action)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $query = "i.id,c.cout as price,i.name,i.name as label,i.name as text";
        //$query .= ($action != "remove") ? "i.name as label" : "i.name as text";
        $tab = $qb->select($query)
            ->from("App:CommandElementIngredients", "c")
            ->innerJoin('c.commandElement', 'ce')
            ->innerJoin('App:Ingredient', 'i', 'WITH', 'i.id = c.ingredient')
            ->where('ce.id = :ce')
            ->andWhere('c.action = :action')
            ->setParameter('ce', $idCmdElement)
            ->setParameter('action', $action)
            ->getQuery()->getResult();

        return $tab;
    }

    public function getMenuElements($idMenu)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("ce.id,ce.quantity,ce.comment,ce.additionalCost,ce.price,ce.name,ce.size,ce.level")
            ->from("App:Command", "co")
            ->innerJoin('co.commandElements', 'ce')
            ->leftJoin('ce.commandMenu', 'cm')
            ->where('cm.id = :menu')
            ->setParameter('menu', $idMenu)
            ->getQuery()->getResult();

        return $tab;
    }

    public function getCommandMenuElements($idMenu)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("ce.id as commandElement,es.id,ce.quantity as qty,ce.comment,ce.price,ce.name,s.id as sizeId,ce.size as sizeLabel,ce.level as sizeLevel,f.id as family")
            ->from("App:CommandElement", "ce")
            ->innerJoin('App:ElementSize', 'es', 'WITH', 'es.id = ce.elementSize')
            ->innerJoin('es.element', 'e')
            ->innerJoin('es.size', 's')
            ->innerJoin('e.family', 'f')
            ->innerJoin('ce.commandMenu', 'cm')
            ->where('cm.id = :menu')
            ->setParameter('menu', $idMenu)
            ->getQuery()->getResult();
        return $tab;
    }

    public function getCommandMenuOptions($idMenu)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("ce.id as commandElement,ce.quantity as qty,ce.price,ce.name,ce.size")
            ->from("App:CommandElement", "ce")
            ->innerJoin('ce.commandMenu', 'cm')
            ->where('ce.elementSize is NULL')
            ->andWhere('cm.id = :menu')
            ->setParameter('menu', $idMenu)
            ->getQuery()->getResult();
        return $tab;
    }

    public function getOrderPdvByPdvAndOrder($idPdv, $idOrder)
    {
        $em = $this->getDoctrine()->getManager();
        $pdv = $em->getRepository('App:Pdv')->find($idPdv);
        $orderType = $em->getRepository('App:OrderType')->find($idOrder);
        $orderTypePdv = $em->getRepository('App:OrderTypePdv')->findOneBy(array('pdv' => $pdv, 'orderType' => $orderType));
        return $orderTypePdv;
    }

    public function getCustmoerPdvByPdvAndCustomer($idPdv, $idCustomer)
    {
        $em = $this->getDoctrine()->getManager();
        $pdv = $em->getRepository('App:Pdv')->find($idPdv);
        $customer = $em->getRepository('App:Customer')->find($idCustomer);
        $customerPdv = $em->getRepository('App:CustomerPdv')->findOneBy(array('pdv' => $pdv, 'customer' => $customer));
        return $customerPdv;
    }

    public function getPaymentModesOfOrder($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $paymentModes = $qb->select("m.label")
            ->from("App:Command", "co")
            ->innerJoin('co.commandPaymentModes', 'pm')
            ->innerJoin('pm.paymentMode', 'm')
            ->where('co.id = :order')
            ->setParameter('order', $id)
            ->getQuery()->getResult();
            if(is_array($paymentModes) && count($paymentModes)>0){
                 $paymentModes = array_column($paymentModes, "label");
             }else{
                $paymentModes=[];
             }
       
        return $paymentModes;
    }

    private function sendEmail($to, $subject, $template)
    {
        $transport = new \Swift_SmtpTransport('ssl0.ovh.net', 465, 'ssl');
        $transport->setUsername('notifications@helpyfood.com');
        $transport->setPassword('wZ433miLyAZ9');
        $mailer = new \Swift_Mailer($transport);
        $message = (new \Swift_Message($subject))
            ->setTo($to)
            ->setFrom('notifications@helpyfood.com')
            ->setBody(strip_tags($template))
            ->addPart($template, 'text/html');
        $mailer->send($message);
    }

    private function getHistoricOrders(ParamFetcher $paramFetcher){
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();

        $query = "co.num,co.id,co.createdAt,co.amount,co.discount,GROUP_CONCAT(mp.label SEPARATOR ', ') as paymentModes,s.label as status,s.id as idStatus,t.label as type,p.name as pdvName,c.firstname as customerFirstname,c.lastname as customerLastname,c.indicator";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $query .= ",u.id as idUser,u.firstname as userFirstname,u.lastname as userLastname";
        }
        $qb = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.status', 's')
            ->leftJoin('co.commandPaymentModes', 'cmp')
            ->leftJoin('cmp.paymentMode', 'mp');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $qb = $qb->leftJoin('co.createdUser', 'u');
        }
        $qb = $qb->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->andWhere('c.firstname LIKE :customer OR c.lastname LIKE :customer');
        if ($this->getUser()->getPdv()) {
            $qb = $qb->andWhere('p.id =:pdv')
                ->setParameter('pdv', $this->getUser()->getPdv());
        }
        if (intval($paramFetcher->get('user')) > 0 && in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $qb = $qb->andWhere('u.id =:user')
                ->setParameter('user', intval($paramFetcher->get('user')));
        }
        $qb = $qb->setParameter('customer', '%' . $paramFetcher->get('customer') . '%');
        if ($paramFetcher->get('pdv')) {
            $qb = $qb->andWhere('p.id = :pdv')
                ->setParameter('pdv', $paramFetcher->get('pdv'));
        }
        if ($paramFetcher->get('status')) {
            $qb = $qb->andWhere('s.id = :status')
                ->setParameter('status', $paramFetcher->get('status'));
        }
        if ($paramFetcher->get('mode')) {
            $qb = $qb->andWhere('mp.id = :mode')
                ->setParameter('mode', $paramFetcher->get('mode'));
        }
        if ($paramFetcher->get('type')) {
            $qb = $qb->andWhere('t.id = :type')
                ->setParameter('type', $paramFetcher->get('type'));
        }
        if ($paramFetcher->get('amountMin')) {
            $qb = $qb->andWhere('co.amount >= :amountMin')
                ->setParameter('amountMin', $paramFetcher->get('amountMin'));
        }
        if ($paramFetcher->get('amountMax')) {
            $qb = $qb->andWhere('co.amount <= :amountMax')
                ->setParameter('amountMax', $paramFetcher->get('amountMax'));
        }
        if ($paramFetcher->get('dateD')) {
            $dateD = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('dateD'));
            $qb = $qb->andWhere('co.createdAt >= :date_start')
                ->setParameter('date_start', $dateD->format('Y-m-d 00:00:00'));
        }
        if ($paramFetcher->get('dateF')) {
            $dateF = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('dateF'));
            $qb = $qb->andWhere('co.createdAt <= :date_end')
                ->setParameter('date_end', $dateF->format('Y-m-d 23:59:59'));
        }
        //$qb=$qb->andWhere('s.id<>3');
        if ($paramFetcher->get('dateD') || $paramFetcher->get('dateF')) {
            $qb = $qb->orderBy('co.createdAt', 'ASC');
        } else {
            $qb = $qb->orderBy('co.createdAt', 'DESC');
        }
        return $qb;
    }

    private function getexportHistoricOrders(ParamFetcher $paramFetcher){
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $query="";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
           $query .= "p.name as pdvName,";
        }
        $query .="
         DATE_FORMAT(co.createdAt, '%d/%m/%Y %H:%m'),
         CONCAT(c.firstname, ' ', c.lastname) AS customer,
         t.label as type,
         co.amount,
         GROUP_CONCAT(mp.label SEPARATOR ', ') as paymentModes,
         s.label as status";
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
           $query .= ",CONCAT(u.firstname, ' ', u.lastname) as user";
        }
        $qb = $qb->select($query)
            ->from("App:Command", "co")
            ->innerJoin('co.status', 's')
            ->leftJoin('co.commandPaymentModes', 'cmp')
            ->leftJoin('cmp.paymentMode', 'mp');
        if (in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $qb = $qb->leftJoin('co.createdUser', 'u');
        }
        $qb = $qb->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->andWhere('c.firstname LIKE :customer OR c.lastname LIKE :customer');
        if ($this->getUser()->getPdv()) {
            $qb = $qb->andWhere('p.id =:pdv')
                ->setParameter('pdv', $this->getUser()->getPdv());
        }
        if (intval($paramFetcher->get('user')) > 0 && in_array($this->getUser()->getProfil()->getId(), [1, 5, 6])) {
            $qb = $qb->andWhere('u.id =:user')
                ->setParameter('user', intval($paramFetcher->get('user')));
        }
        $qb = $qb->setParameter('customer', '%' . $paramFetcher->get('customer') . '%');
        if ($paramFetcher->get('pdv')) {
            $qb = $qb->andWhere('p.id = :pdv')
                ->setParameter('pdv', $paramFetcher->get('pdv'));
        }
        if ($paramFetcher->get('status')) {
            $qb = $qb->andWhere('s.id = :status')
                ->setParameter('status', $paramFetcher->get('status'));
        }
        if ($paramFetcher->get('mode')) {
            $qb = $qb->andWhere('mp.id = :mode')
                ->setParameter('mode', $paramFetcher->get('mode'));
        }
        if ($paramFetcher->get('type')) {
            $qb = $qb->andWhere('t.id = :type')
                ->setParameter('type', $paramFetcher->get('type'));
        }
        if ($paramFetcher->get('amountMin')) {
            $qb = $qb->andWhere('co.amount >= :amountMin')
                ->setParameter('amountMin', $paramFetcher->get('amountMin'));
        }
        if ($paramFetcher->get('amountMax')) {
            $qb = $qb->andWhere('co.amount <= :amountMax')
                ->setParameter('amountMax', $paramFetcher->get('amountMax'));
        }
        if ($paramFetcher->get('dateD')) {
            $dateD = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('dateD'));
            $qb = $qb->andWhere('co.createdAt >= :date_start')
                ->setParameter('date_start', $dateD->format('Y-m-d 00:00:00'));
        }
        if ($paramFetcher->get('dateF')) {
            $dateF = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('dateF'));
            $qb = $qb->andWhere('co.createdAt <= :date_end')
                ->setParameter('date_end', $dateF->format('Y-m-d 23:59:59'));
        }
        if ($paramFetcher->get('dateD') || $paramFetcher->get('dateF')) {
            $qb = $qb->orderBy('co.createdAt', 'ASC');
        } else {
            $qb = $qb->orderBy('co.createdAt', 'DESC');
        }
        return $qb;
    }

}
