<?php

namespace App\Controller\Rest;

use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class OrderTypeController extends FOSRestController
{

    /**
     * @Rest\Get("/ordertypes")
     */
    public function getOrderTypeAction(Request $request)
    {
        $typesCommandes = $this->getDoctrine()->getRepository("App:OrderType")->findAll();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($typesCommandes, 'json');
        $response = new Response();
        $response->setContent($data);
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/ordertypes/carte/{idCarte}")
     */
    public function getModeCmdByCarte($idCarte)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdvs = $qb->select("t.id,t.label")
            ->from("App:Card", "C")
            ->innerJoin('C.pdv', 'P')
            ->innerJoin('P.orderTypes', 'tp')
            ->innerJoin('tp.orderType', 't')
            ->where('C.id = :carte')
            ->setParameter('carte', $idCarte)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdvs, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/ordertypes/pdv/{idPdv}")
     */
    public function getModeCmdByPdv($idPdv)
    {
        if($this->getUser()->getPdv() && $this->getUser()->getPdv()!=$idPdv){
            return new JsonResponse('',403);
        }
        $d = new \DateTime();
        $midi = new \DateTime();
        $midi->setTime(16, 0, 0);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        if ($d > $midi) {
            $pdvs = $qb->select("t.id,t.label,tp.minDelay,tp.maxDelay,tp.active,case when TIME(av.eveningEnd) > :da and TIME(av.eveningStart) < :da then 'true' else 'false' end available");
        }else{
            $pdvs = $qb->select("t.id,t.label,tp.minDelay,tp.maxDelay,tp.active,case when TIME(av.middayEnd) > :da and TIME(av.middayStart) < :da then 'true' else 'false' end available");
        }
        $pdvs = $pdvs
            ->from("App:Pdv", "P")
            ->innerJoin('P.orderTypes', 'tp')
            ->innerJoin('tp.availability', 'av')
            ->innerJoin('av.day', 'd')
            ->innerJoin('tp.orderType', 't')
            ->where('P.id = :pdv')
            ->andWhere('d.id=:day')
            ->setParameter('pdv', $idPdv)
            ->setParameter('da', $d->format('H:i:s'))
            ->setParameter('day', $d->format('N'))
            ->orderBy('t.level')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdvs, 'json');
        return new JsonResponse(json_decode($data));
    }


}
