<?php

namespace App\Controller\Rest;

use App\Service\SendEvent;
use Doctrine\Common\Collections\ArrayCollection;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations as Rest;
use App\Entity\Card;
use App\Entity\CityFrancePdv;
use App\Entity\Jour_ouverturePdv;
use App\Entity\OpeningDayPdv;
use App\Entity\OrderTypePdv;
use App\Entity\OrderTypePdvDay;
use App\Entity\Type_commandePdv;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Pdv;
use App\Entity\Villes_francePdv;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Request\ParamFetcher;
use Spatie\ImageOptimizer\OptimizerChainFactory;

class PdvController extends FOSRestController
{

    /**
     * @Rest\Post("/pdvs")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="siret", nullable=true,strict=true)
     * @RequestParam(name="number", nullable=true,strict=true)
     * @RequestParam(name="address", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="postalCode", nullable=true,strict=true)
     * @RequestParam(name="city", nullable=true,strict=true)
     * @RequestParam(name="country", nullable=true,strict=true)
     * @RequestParam(name="amountMiniOrder", nullable=false, strict=true)
     * @RequestParam(name="amountMiniSociety", nullable=false, strict=true)
     * @RequestParam(name="amountMiniMandatory", nullable=false, strict=true )
     * @RequestParam(name="phone", nullable=false, strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="principal", nullable=false, strict=true)
     * @RequestParam(name="active", nullable=false, strict=true)
     * @RequestParam(name="paymentModes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="orderModes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="authorizedCities", nullable=false,strict=true)
     * @RequestParam(name="restaurant", nullable=false,allowBlank=false,strict=true)
     */
    public function addAction(Request $request, ParamFetcher $paramFetcher)
    {
        $pdv = new Pdv();
        $pdv->setName($paramFetcher->get('name'));
        $pdv->setSiret($paramFetcher->get('siret'));
        $pdv->setNumber($paramFetcher->get('number'));
        $pdv->setAddress($paramFetcher->get('address'));
        $pdv->setPostalCode($paramFetcher->get('postalCode'));
        $pdv->setCity($paramFetcher->get('city'));
        $pdv->setCountry($paramFetcher->get('country'));
        $pdv->setResetDelay(1);
        $pdv->setAmountMiniOrder($paramFetcher->get('amountMiniOrder'));
        $pdv->setAmountMiniSociety($paramFetcher->get('amountMiniSociety'));
        $pdv->setAmountMiniMandatory($paramFetcher->get('amountMiniMandatory'));
        $pdv->setPhone($paramFetcher->get('phone'));
        $pdv->setEmail($paramFetcher->get('email'));
        $pdv->setPrincipal($paramFetcher->get('principal'));
        $pdv->setActive($paramFetcher->get('active'));
        $pdv->setAutomaticPrinting(0);
        $pdv->setAutomaticPrinting2(0);
        $pdv->setPrinter('cuisine');
        $pdv->setComment('');
        $pdv->setImage('');
        $ip = $request->getClientIp();
        $pdv->setCreatedIp($ip);
        $pdv->setCreatedUser('0');
        $pdv->setUpdatedIp('');
        $pdv->setUpdatedUser('0');
        $em = $this->getDoctrine()->getManager();
        $paymentModes = $paramFetcher->get('paymentModes');
        $pdv->setCloseSite(1);
        $pdv->setDisplayBestSales(0);
        $pdv->setDDisplayBestOffers(0);
        $pdv->setCloseComment('Bonjour, nous sommes exceptionnellement fermé. Veuillez nous excuser pour la gêne occasionnée.');
        if (is_array($paymentModes) && count($paymentModes) > 0) {
            foreach ($paymentModes as $value) {
                $paymentMode = $em->getRepository('App:PaymentMode')->find($value['id']);
                if ($paymentMode) {
                    $pdv->addPaymentMode($paymentMode);
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 1], Response::HTTP_BAD_REQUEST);
        }

        $orderModes = $paramFetcher->get('orderModes');
        if (is_array($orderModes) && count($orderModes) > 0) {
            foreach ($orderModes as $value) {
                $orderModePdv = new OrderTypePdv();
                if (
                    array_key_exists('minDelay', $value) &&
                    array_key_exists('maxDelay', $value) &&
                    array_key_exists('active', $value) &&
                    array_key_exists('id', $value) &&
                    is_array($value['availability']) && count($value['availability']) == 7
                ) {
                    $orderModePdv->setMinDelay($value['minDelay']);
                    $orderModePdv->setMaxDelay($value['maxDelay']);
                    $orderModePdv->setDefaultMinDelay($value['minDelay']);
                    $orderModePdv->setDefaultMaxDelay($value['maxDelay']);
                    $orderModePdv->setPdv($pdv);
                    $orderModePdv->setActive($value['active']);
                    $orderMode = $em->getRepository('App:OrderType')->find($value['id']);
                    $orderModePdv->setOrderType($orderMode);
                    foreach ($value['availability'] as $key => $availability) {
                        if (
                            array_key_exists('day', $availability) &&
                            array_key_exists('id', $availability['day']) &&
                            array_key_exists('middayStart', $availability) &&
                            array_key_exists('middayEnd', $availability) &&
                            array_key_exists('eveningStart', $availability) &&
                            array_key_exists('eveningEnd', $availability)
                        ) {
                            $orderTypePdvDay = new OrderTypePdvDay();
                            $day = $em->getRepository('App:Day')->find($availability['day']['id']);
                            $orderTypePdvDay->setDay($day);
                            $orderTypePdvDay->setOrderTypePdv($orderModePdv);
                            $middayStart = ($availability['middayStart'] != "" && !is_null($availability['middayStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayStart']) : null;
                            $middayEnd = ($availability['middayEnd'] != "" && !is_null($availability['middayEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayEnd']) : null;
                            $eveningStart = ($availability['eveningStart'] != "" && !is_null($availability['eveningStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningStart']) : null;
                            $eveningEnd = ($availability['eveningEnd'] != "" && !is_null($availability['eveningEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningEnd']) : null;
                            $orderTypePdvDay->setMiddayStart($middayStart);
                            $orderTypePdvDay->setMiddayEnd($middayEnd);
                            $orderTypePdvDay->setEveningStart($eveningStart);
                            $orderTypePdvDay->setEveningEnd($eveningEnd);
                            $orderModePdv->addOrderTypePdvDay($orderTypePdvDay);
                        } else {
                            return new JsonResponse(['error' => 5], Response::HTTP_BAD_REQUEST);
                        }
                    }
                    $em->persist($orderModePdv);
                } else {
                    return new JsonResponse(['error' => 2], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 3], Response::HTTP_BAD_REQUEST);
        }

        $authorizedCities = $paramFetcher->get('authorizedCities');
        if (is_array($authorizedCities)) {
            foreach ($authorizedCities as $value) {
                if (array_key_exists('id', $value) && array_key_exists('amountMiniDelivery', $value)) {
                    $cityFrancePdv = new CityFrancePdv();
                    $cityFrance = $em->getRepository('App:CityFrance')->find($value['id']);
                    if ($cityFrance) {
                        $cityFrancePdv->setCity($cityFrance);
                        $cityFrancePdv->setPdv($pdv);
                        $cityFrancePdv->setAmountMiniDelivery($value['amountMiniDelivery']);
                        $em->persist($cityFrancePdv);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    $response = array(
                        'code' => 0,
                        'message' => 'Champs requis: amountMiniDelivery et id',
                        'error' => null,
                        'result' => null
                    );
                    return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse(['error' => 4], Response::HTTP_BAD_REQUEST);
        }
        $restaurant = $em->getRepository('App:Restaurant')->find(intval($paramFetcher->get('restaurant')));
        if ($restaurant) {
            $pdv->setRestaurant($restaurant);
            $em->persist($pdv);
        } else {
            return new JsonResponse(['error' => 7], Response::HTTP_BAD_REQUEST);
        }

        $key = $this->dec_enc('encrypt', $pdv->getId());
        $pdv->setPdvKey($key);
        $em->flush();
        $id = $pdv->getId();
        $card = new Card();
        $start = new \DateTime();
        $end = new \DateTime();
        $end->modify('+100 year');
        $card->setId($pdv->getId());
        $card->setStartedValidity($start);
        $card->setStoppedValidity($end);
        $card->setNbrOffersAuthorized(1);
        $card->setNbrOffersAuthorizedSite(1);
        $card->setDisplayCode(0);
        $card->setComment('');
        $card->setPdv($pdv);
        $em->persist($card);
        $metadata = $em->getClassMetaData(get_class($card));
        $metadata->setIdGenerator(new \Doctrine\ORM\Id\AssignedGenerator());
        $metadata->setIdGeneratorType(\Doctrine\ORM\Mapping\ClassMetadata::GENERATOR_TYPE_NONE);
        $em->flush();
        /* $pdvDirectory=$this->getParameter('img_directory') . "/".$id;
         $productsDirectory=$this->getParameter('products_directory') . "/".$id;
         $menusDirectory=$this->getParameter('menus_directory') . "/".$id;
         $ingredientsDirectory=$this->getParameter('ingredients_directory') . "/".$id;
         mkdir($pdvDirectory, 0755, true);
         mkdir($productsDirectory, 0755, true);
         mkdir($menusDirectory, 0755, true);
         mkdir($ingredientsDirectory, 0755, true);*/
        return new JsonResponse([
            'id' => $id,
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Get("/pdvs/restaurant/{idRest}")
     */
    public function getListPdvs($idRest)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdvs = $qb->select("p")
            ->from("App:Pdv", "p")
            ->innerJoin('p.restaurant', 'r')
            ->where('r.id=:restaurant')
            ->setParameter('restaurant', $idRest)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdvs, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/pdvs/get/setting")
     */
    public function getPdvForSetting()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $idPdv = $this->getUser()->getPdv();
        $pdvs = $qb->select("op.id,o.label,op.minDelay,op.maxDelay,p.automaticPrinting,p.resetDelay,p.automaticPrinting2,p.printer")
            ->from("App:OrderTypePdv", "op")
            ->innerJoin('op.pdv', 'p')
            ->innerJoin('op.orderType', 'o')
            ->where('p.id=:pdv')
            ->setParameter('pdv', $idPdv)
            ->getQuery()
            ->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdvs, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/pdvName/{id}")
     */
    public function getPdvName($id)
    {
        $id2 = ($this->getUser()->getProfil()->getId() == 3) ? $this->getUser()->getPdv() : $id;
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdv = $qb->select("p.name,p.email")
            ->from("App:Pdv", "p")
            ->where('p.id=:pdv')
            ->setParameter('pdv', $id2)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
        if ($pdv) {
            $pdv['hash'] = hash_hmac("sha256", $pdv['email'], "f58c96af526796f8d7240e9af7312f0f64072e5f");
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($pdv, 'json');
            return new JsonResponse(json_decode($data));
        }
        return new JsonResponse('', 204);
    }

    /**
     * @Rest\Get("/pdvs/{id}")
     */

    public function getPdv($id)
    {
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($id);
        if ($pdv) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($pdv, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], Response::HTTP_NO_CONTENT);
        }
    }

    /**
     * @Rest\Delete("/pdvs/{id}")
     */
    public function deletePdv($id)
    {
        $em = $this->getDoctrine()->getManager();
        $p = $em->getRepository('App:Pdv')->find($id);
        if ($p) {
            $em->remove($p);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse([], Response::HTTP_NO_CONTENT);
        }


    }

    /**
     * @Rest\Put("/pdvs/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="siret", nullable=true,strict=true))
     * @RequestParam(name="number", nullable=true,strict=true)
     * @RequestParam(name="address", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="postalCode", nullable=true,strict=true)
     * @RequestParam(name="city", nullable=true,strict=true)
     * @RequestParam(name="country", nullable=true,strict=true)
     * @RequestParam(name="amountMiniOrder", nullable=false, strict=true, requirements="\d+(\.\d{1,2})?")
     * @RequestParam(name="amountMiniSociety", nullable=false, strict=true, requirements="\d+(\.\d{1,2})?")
     * @RequestParam(name="amountMiniMandatory", nullable=false, strict=true )
     * @RequestParam(name="phone", nullable=false, strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true, strict=true))
     * @RequestParam(name="principal", nullable=false, strict=true, requirements="^(true|false|1|0)$")
     * @RequestParam(name="active", nullable=false, strict=true, requirements="^(true|false|1|0)$")
     * @RequestParam(name="paymentModes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="orderModes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="authorizedCities", nullable=false,strict=true)
     */
    public function updateAction(Request $request, ParamFetcher $paramFetcher, $id)
    {
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($id);
        ///////////////////////////////// Modifier les information du pdv ////////////////////////////////////////////
        if ($pdv) {
            $pdv->setName($paramFetcher->get('name'));
            $pdv->setSiret($paramFetcher->get('siret'));
            $pdv->setNumber($paramFetcher->get('number'));
            $pdv->setAddress($paramFetcher->get('address'));
            $pdv->setPostalCode($paramFetcher->get('postalCode'));
            $pdv->setCity($paramFetcher->get('city'));
            $pdv->setCountry($paramFetcher->get('country'));
            $pdv->setAmountMiniOrder($paramFetcher->get('amountMiniOrder'));
            $pdv->setAmountMiniSociety($paramFetcher->get('amountMiniSociety'));
            $pdv->setAmountMiniMandatory($paramFetcher->get('amountMiniMandatory'));
            $pdv->setPhone($paramFetcher->get('phone'));
            $pdv->setEmail($paramFetcher->get('email'));
            $pdv->setPrincipal($paramFetcher->get('principal'));
            $pdv->setActive($paramFetcher->get('active'));
            $ip = $request->getClientIp();
            $pdv->setUpdatedIp($ip);
            $pdv->setUpdatedUser('0');
            $pdv->setPaymentModes([]);
            $em = $this->getDoctrine()->getManager();
            $paymentModes = $paramFetcher->get('paymentModes');
            if (is_array($paymentModes) && count($paymentModes) > 0) {
                foreach ($paymentModes as $value) {
                    $paymentMode = $em->getRepository('App:PaymentMode')->find($value['id']);
                    if ($paymentMode) {
                        $pdv->addPaymentMode($paymentMode);
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse([], Response::HTTP_BAD_REQUEST);
            }
            ///////////////////////////////// Modifier les types de commande////////////////////////////////////////////
            $orderTypes = $paramFetcher->get('orderModes');
            if (is_array($orderTypes) && count($orderTypes) > 0) {
                foreach ($orderTypes as $value) {
                    if (
                        array_key_exists('minDelay', $value) &&
                        array_key_exists('maxDelay', $value) &&
                        array_key_exists('active', $value) &&
                        array_key_exists('id', $value) &&
                        array_key_exists('idorderTypePdv', $value) &&
                        array_key_exists('availability', $value) &&
                        is_array($value['availability']) && count($value['availability']) == 7
                    ) {
                        $orderTypesPdv = $em->getRepository('App:OrderTypePdv')->findOneBy(array('id' => $value['idorderTypePdv'], 'pdv' => $pdv));
                        if (!is_null($orderTypesPdv)) {
                            $orderTypesPdv->setMinDelay($value['minDelay']);
                            $orderTypesPdv->setMaxDelay($value['maxDelay']);
                            $orderTypesPdv->setDefaultMinDelay($value['minDelay']);
                            $orderTypesPdv->setDefaultMaxDelay($value['maxDelay']);
                            $orderTypesPdv->setActive($value['active']);
                            $em->persist($orderTypesPdv);
                            foreach ($value['availability'] as $key => $availability) {
                                if (
                                    array_key_exists('day', $availability) &&
                                    array_key_exists('id', $availability['day']) &&
                                    array_key_exists('middayStart', $availability) &&
                                    array_key_exists('middayEnd', $availability) &&
                                    array_key_exists('eveningStart', $availability) &&
                                    array_key_exists('eveningEnd', $availability) &&
                                    array_key_exists('id', $availability)
                                ) {
                                    $orderTypePdvDay = $em->getRepository('App:OrderTypePdvDay')->find($availability['id']);
                                    if (!is_null($orderTypePdvDay)) {
                                        $middayStart = ($availability['middayStart'] != "" && !is_null($availability['middayStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayStart']) : null;
                                        $middayEnd = ($availability['middayEnd'] != "" && !is_null($availability['middayEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayEnd']) : null;
                                        $eveningStart = ($availability['eveningStart'] != "" && !is_null($availability['eveningStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningStart']) : null;
                                        $eveningEnd = ($availability['eveningEnd'] != "" && !is_null($availability['eveningEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningEnd']) : null;
                                        $orderTypePdvDay->setMiddayStart($middayStart);
                                        $orderTypePdvDay->setMiddayEnd($middayEnd);
                                        $orderTypePdvDay->setEveningStart($eveningStart);
                                        $orderTypePdvDay->setEveningEnd($eveningEnd);
                                        $em->persist($orderTypePdvDay);
                                    } else {
                                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                                    }
                                } else {
                                    return new JsonResponse(['error' => 5], Response::HTTP_BAD_REQUEST);
                                }
                            }
                        } else {
                            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                        }
                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse([], Response::HTTP_BAD_REQUEST);
            }

            ///////////////////////////////// Modifier les villes autorisee////////////////////////////////////////////

            $franceCities = $pdv->getCitiesFrance();
            $franceCities2 = $paramFetcher->get('authorizedCities');
            $franceCities3 = new ArrayCollection();

            if (is_array($franceCities2)) {
                foreach ($franceCities2 as $value) {
                    if (array_key_exists('id', $value) && array_key_exists('amountMiniDelivery', $value)) {
                        $cityFrancePdv = new CityFrancePdv();
                        $cityFrance = $em->getRepository('App:CityFrance')->find($value['id']);
                        if ($cityFrance) {
                            $cityFrancePdv->setCity($cityFrance);
                            $cityFrancePdv->setPdv($pdv);
                            $cityFrancePdv->setAmountMiniDelivery($value['amountMiniDelivery']);
                            $franceCities3[] = $cityFrancePdv;
                        } else {
                            return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                        }
                    } else {
                        $response = array(
                            'code' => 0,
                            'message' => 'Champs requis: montantville et id',
                            'error' => null,
                            'result' => null
                        );
                        return new JsonResponse($response, Response::HTTP_BAD_REQUEST);
                    }
                }
            } else {
                return new JsonResponse([], Response::HTTP_BAD_REQUEST);
            }

            foreach ($franceCities as $value) {
                $index = $this->findIndex($franceCities3, $value, 2);
                if ($index > -1) {
                    $value->setAmountMiniDelivery($franceCities3[$index]->getAmountMiniDelivery());
                    $em->persist($value);
                } else {
                    $pdv->removeCityFrance($value);
                    $em->remove($value);
                }
            }

            foreach ($franceCities3 as $value) {
                $index = $this->findIndex($franceCities, $value, 2);
                if ($index == -1) {
                    $em->persist($value);
                }
            }

            /////////////////////////////////////////////////////////////////////////////////////////////////////////////
            $key = $this->dec_enc('encrypt', $id);
            $pdv->setPdvKey($key);
            $em->persist($pdv);
            $em->flush();
            $id = $pdv->getId();
            return new JsonResponse([
                'id' => $id,
                'success' => true,
                'code' => 200
            ]);
        } else {
            $response = new Response();
            $response->setStatusCode(400);
            return $response;
        }
    }

    /**
     * @Rest\Put("/pdvs/put/setting/time")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="automaticPrinting", nullable=true,strict=true)
     * @RequestParam(name="resetDelay", nullable=true,strict=true))
     * @RequestParam(name="orderModes", nullable=true,allowBlank=true,strict=true)
     */
    public function updateSettingPdvTime(SendEvent $sendEvent, ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $id = $this->getUser()->getPdv();
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($id);
        $orderTypes = $paramFetcher->get('orderModes');
        if (is_array($orderTypes) && count($orderTypes) > 0) {
            $pdv->setResetDelay($paramFetcher->get('resetDelay'));
            foreach ($orderTypes as $value) {
                if (
                    array_key_exists('minDelay', $value) &&
                    array_key_exists('maxDelay', $value) &&
                    array_key_exists('id', $value)
                ) {
                    $orderTypesPdv = $em->getRepository('App:OrderTypePdv')->findOneBy(array('id' => $value['id'], 'pdv' => $pdv));
                    if (!is_null($orderTypesPdv)) {
                        $orderTypesPdv->setMinDelay($value['minDelay']);
                        $orderTypesPdv->setMaxDelay($value['maxDelay']);
                        $em->persist($orderTypesPdv);


                    } else {
                        return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                    }
                } else {
                    return new JsonResponse([], Response::HTTP_BAD_REQUEST);
                }
            }
        } else {
            return new JsonResponse([], 400);
        }
        $em->persist($pdv);
        $em->flush();
        return new JsonResponse([], 200);
    }

    /**
     * @Rest\Put("/pdvs/put/setting/autoprinting")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="automaticPrinting", nullable=true,strict=true)
     * @RequestParam(name="automaticPrinting2", nullable=true,strict=true)
     * @RequestParam(name="printer", nullable=true,strict=true)
     */
    public function updateSettingPdvAutoprinting(SendEvent $sendEvent, ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $id = $this->getUser()->getPdv();
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($id);
        $pdv->setAutomaticPrinting(intval($paramFetcher->get('automaticPrinting')));
        $pdv->setAutomaticPrinting2(intval($paramFetcher->get('automaticPrinting2')));
        $pdv->setPrinter($paramFetcher->get('printer'));
        $em->persist($pdv);
        $sendEvent->sendMessage([
            'pdv' => $id,
            'type' => 'updatePdvAutoPrinting',
            'entity' => 'pdv',
            'autoPrinting' => $paramFetcher->get('automaticPrinting'),
            'autoPrinting2' => $paramFetcher->get('automaticPrinting2'),
            'printer' => $paramFetcher->get('printer')
        ]);
        $em->flush();
        return new JsonResponse([], 200);
    }

    /**
     * @Rest\Put("/pdvs/{id}/status")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="active", nullable=false, strict=true, requirements="^(true|false|1|0)$")
     */
    public function updatePdvStatus(Request $request, ParamFetcher $paramFetcher, $id)
    {
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($id);
        if ($pdv) {
            $pdv->setActive($paramFetcher->get('active'));
            $ip = $request->getClientIp();
            $pdv->setUpdatedIp($ip);
            $pdv->setUpdatedUser($this->getUser()->getId());
            $em = $this->getDoctrine()->getManager();
            $em->persist($pdv);
            $em->flush();
            $id = $pdv->getId();
            return new JsonResponse([
                'id' => $id,
                'success' => true,
                'code' => 200
            ]);
        } else {
            $response = new Response();
            $response->setStatusCode(400);
            return $response;
        }
    }

    /**
     * @Rest\Put("/pdvs/{idPdv}/mode/{idMode}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="active", nullable=false, strict=true, requirements="^(true|false|1|0)$")
     */
    public function updateModeStatus(Request $request, ParamFetcher $paramFetcher, $idPdv, $idMode)
    {
        //  $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($idPdv);
        $em = $this->getDoctrine()->getManager();
        $modes = $em->getRepository('App:OrderTypePdv')
            ->createQueryBuilder('O')
            ->leftJoin('O.pdv', 'P')
            ->where('P.id = :pdv')
            ->Andwhere('O.id = :idMode')
            ->setParameter('pdv', $idPdv)
            ->setParameter('idMode', $idMode)
            ->getQuery()
            ->getResult();
        if (count($modes) > 0) {
            $mode = $modes[0];
            $mode->setActive($paramFetcher->get('active'));
            $em->persist($mode);
            $em->flush();
            /*  $client = new Client(new Version1X('//127.0.0.1:1337'));
              $client->initialize();
              $client->emit('notif', ['type' => 'persist','entity' => 'pdv', 'id' => $idPdv]);
              $client->close();*/
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            $response = new Response();
            $response->setStatusCode(400);
            return $response;
        }
    }

    /**
     * @Rest\Put("/pdvs/site/info")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="id", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="image", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="nameRestaurant", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="urlSite", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="facebook", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="phoneContact", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="closeSite", nullable=false,allowBlank=false, strict=true)
     * @RequestParam(name="displayBestOffers", nullable=false,allowBlank=false, strict=true)
     * @RequestParam(name="displayBestSales", nullable=false,allowBlank=false, strict=true)
     * @RequestParam(name="availability", nullable=true,allowBlank=true, strict=true)
     * @RequestParam(name="siteType", nullable=true,allowBlank=true, strict=true)
     */
    public function updateSiteInfo(Request $request, ParamFetcher $paramFetcher)
    {
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find(intval($paramFetcher->get('id')));
        $em = $this->getDoctrine()->getManager();
        if ($pdv) {
            $pdv->setComment($paramFetcher->get('comment'));
            $pdv->setEmailContact($paramFetcher->get('email'));
            $pdv->setCloseSite($paramFetcher->get('closeSite'));
            $pdv->setNameRestaurant($paramFetcher->get('nameRestaurant'));
            $pdv->setUrlSite($paramFetcher->get('urlSite'));
            $pdv->setFacebook($paramFetcher->get('facebook'));
            $pdv->setphoneContact($paramFetcher->get('phoneContact'));
            $pdv->setDisplayBestSales($paramFetcher->get('displayBestSales'));
            $pdv->setDDisplayBestOffers($paramFetcher->get('displayBestOffers'));
            $pdv->setSiteType(intval($paramFetcher->get('siteType')));
            $availabilitys = $paramFetcher->get('availability');
            if (is_array($availabilitys) && (count($availabilitys) == 21 || count($availabilitys) == 14 || count($availabilitys) == 7)) {
                foreach ($availabilitys as $key => $availability) {
                    if (
                        array_key_exists('id', $availability) &&
                        array_key_exists('middayStart', $availability) &&
                        array_key_exists('middayEnd', $availability) &&
                        array_key_exists('eveningStart', $availability) &&
                        array_key_exists('eveningEnd', $availability)
                    ) {
                        $orderTypePdvDay = $em->getRepository('App:OrderTypePdvDay')->find($availability['id']);
                        $middayStart = ($availability['middayStart'] != "" && !is_null($availability['middayStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayStart']) : null;
                        $middayEnd = ($availability['middayEnd'] != "" && !is_null($availability['middayEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayEnd']) : null;
                        $eveningStart = ($availability['eveningStart'] != "" && !is_null($availability['eveningStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningStart']) : null;
                        $eveningEnd = ($availability['eveningEnd'] != "" && !is_null($availability['eveningEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningEnd']) : null;
                        $orderTypePdvDay->setMiddayStartSite($middayStart);
                        $orderTypePdvDay->setMiddayEndSite($middayEnd);
                        $orderTypePdvDay->setEveningStartSite($eveningStart);
                        $orderTypePdvDay->setEveningEndSite($eveningEnd);
                        $em->persist($orderTypePdvDay);
                    }
                }
            }
            if ($paramFetcher->get('image') != '') {
                if (substr($paramFetcher->get('image'), 0, 4) == 'data') {
                    if ($pdv->getImage() != '') {
                        $avatar = $this->getParameter('img_directory') . "/" . $pdv->getImage();
                        $avatarWebp = $this->getParameter('img_directory') . "/" . $pdv->getImage() . 'webp';
                        if (file_exists($avatar)) unlink($avatar);
                        if (file_exists($avatarWebp)) unlink($avatarWebp);
                    }
                    $decoded = base64_decode(substr($paramFetcher->get('image'), 22));
                    $f = finfo_open();
                    $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
                    $split = explode('/', $mime_type);
                    $type = $split[1];
                    $filename = md5(uniqid()) . '.' . $type;
                    $path = $this->getParameter('img_directory') . '/' . $filename;
                    $pdv->setImage($filename);
                    file_put_contents($path, $decoded);
                    $optimizerChain = OptimizerChainFactory::create();
                    $optimizerChain->optimize($path);
                    exec('cwebp -q 100 ' . $path . ' -o ' . $path . '.webp');
                }
            } else {
                if ($pdv->getImage() != '') {
                    $avatar = $this->getParameter('img_directory') . "/" . $pdv->getImage();
                    if (file_exists($avatar))
                        unlink($avatar);
                    $pdv->setImage('');
                }
            }
            $em->persist($pdv);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse([], 404);
        }
    }

    /**
     * @Rest\Get("/pdvs/{id}/site/info")
     */
    public function GetSiteInfo($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdv = $qb->select("p.id,p.closeComment,p.displayBestOffers,p.displayBestSales,p.image,p.facebook,p.comment,p.emailContact as email,p.closeSite,p.nameRestaurant,p.urlSite,p.phoneContact,p.siteType")
            ->from("App:Pdv", "p")
            ->where('p.id = :id')
            ->setParameter('id', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdv, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/availablePdvs")
     */
    public function getAvailablePdvs()
    {
        if ($this->getUser()->getPdv()) {
            return new JsonResponse('', 403);
        }
        $d = new \DateTime();
        $midi = new \DateTime();
        $midi->setTime(16, 0, 0);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $query = $qb->select("p.id,p.name,r.name as restaurant, p.phone, p.address")
            ->from("App:Pdv", "p")
            ->innerJoin('p.restaurant', 'r')
            ->innerJoin('p.orderTypes', 'otp')
            ->innerJoin('otp.availability', 'av')
            ->innerJoin('av.day', 'd')
            ->where('d.id=:day')
            ->andWhere('p.active = 1')
            ->andWhere('r.active = 1');
        if ($d < $midi) {
            $query = $query->andWhere('TIME(av.middayEnd) > :da')->andWhere('TIME(av.middayStart) < :da');
        } else {
            $query = $query->andWhere('TIME(av.eveningEnd) > :da')->andWhere('TIME(av.eveningStart) < :da');;
        }
        $query = $query->setParameter('da', $d->format('H:i:s'));
        $query = $query->setParameter('day', $d->format('N'));
        $query = $query->orderBy('r.id')->groupBy('p.id')->getQuery();
        $pdvs = $query->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdvs, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/pdv/{id}")
     */
    public function getOrderPdv($id)
    {
        if ($this->getUser()->getPdv() && $this->getUser()->getPdv() != $id) {
            return new JsonResponse('', 403);
        } else {
            if ($this->getUser()->hasRole('RULE_NEWORDER')) {
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $pdv = $query = $qb->select("p.id,p.name, r.name as restaurant, p.phone,p.amountMiniOrder,p.amountMiniSociety,p.amountMiniMandatory, p.address,c.displayCode")
                    ->from("App:Card", "c")
                    ->innerJoin('c.pdv', 'p')
                    ->innerJoin('p.restaurant', 'r')
                    ->innerJoin('p.orderTypes', 'otp')
                    ->innerJoin('otp.availability', 'av')
                    ->andWhere('p.active = 1')
                    ->andWhere('r.active = 1')
                    ->andWhere('p.id =:pdv')
                    ->setParameter('pdv', $id)
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getOneOrNullResult();
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($pdv, 'json');
                return new JsonResponse(json_decode($data));
            } else {
                return new JsonResponse('', 403);
            }
        }

    }

    /**
     * @Rest\Get("/pdvs")
     */
    public function getPdvs()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdvs = $qb->select("p.id,p.name")
            ->from("App:Pdv", "p")
            ->orderBy("p.name")
            ->getQuery()
            ->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdvs, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/status")
     */
    public function getStatus()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $status = $qb->select("s.id,s.label")
            ->from("App:Status", "s")
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($status, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/pdv/{id}/miniInfo")
     */
    public function getPdvMiniInfo(Request $request, $id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdv = $qb->select("c.deliveryCharges,c.managementFees,c.deliveryChargesLabel,c.managementFeesLabel,c.nbrOffersAuthorized,c.displayCode,c.comment,p.name,p.amountMiniMandatory,p.amountMiniOrder,p.amountMiniSociety")
            ->from("App:Card", "c")
            ->innerJoin("c.pdv", "p")
            ->where('p.id=:pdv')
            ->setParameter('pdv', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if ($pdv) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($pdv, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
        return new JsonResponse([], 204);
    }

    /**
     * @Rest\Get("/pdv/{idPdv}/authorizedCities")
     */
    public function getAuthorizedCities($idPdv)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $cities = $qb->select("c.name,c.postalCode,cp.amountMiniDelivery")
            ->from("App:CityFrancePdv", "cp")
            ->innerJoin('cp.pdv', 'p')
            ->innerJoin('cp.city', 'c')
            ->where('p.id=:pdv')
            ->setParameter('pdv', $idPdv)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($cities, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/pdv/{idPdv}/modes/site")
     */
    public function getSiteWebModes($idPdv)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $cities = $qb->select("otpd.id,d.label as day,ot.label as mode,otpd.middayStartSite,otpd.middayEndSite,otpd.eveningStartSite,otpd.eveningEndSite")
            ->from("App:OrderTypePdvDay", "otpd")
            ->innerJoin('otpd.orderTypePdv', 'otp')
            ->innerJoin('otp.orderType', 'ot')
            ->innerJoin('otp.pdv', 'p')
            ->innerJoin('otpd.day', 'd')
            ->where('p.id=:pdv')
            ->andwhere('otp.active=1')
            ->setParameter('pdv', $idPdv)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($cities, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/refresh/pdvs")
     */
    public function refresh(SendEvent $sendEvent)
    {
        $sendEvent->sendMessage(['type' => 'refresh']);
        return new JsonResponse('');
    }

    public function findIndex($array, $object, $key)
    {
        $index = -1;
        switch ($key) {
            case 2:
                {
                    foreach ($array as $key => $value) {
                        if ($value->getCity()->getId() == $object->getCity()->getId()) {
                            $index = $key;
                            break;
                        }
                    }
                    break;
                }
            case 3:
                {
                    foreach ($array as $key => $value) {
                        if ($value->getDay()->getId() == $object->getDay()->getId()) {
                            $index = $key;
                            break;
                        }
                    }
                    break;
                }

        }
        return $index;
    }

    private function dec_enc($action, $string)
    {
        $output = false;
        $encrypt_method = "AES-256-CBC";
        $secret_key = 'helpyfood';
        $secret_iv = 'helpyfood';
        $key = hash('sha256', $secret_key);
        $iv = substr(hash('sha256', $secret_iv), 0, 16);
        if ($action == 'encrypt') {
            $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
            $output = base64_encode($output);
        } else if ($action == 'decrypt') {
            $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        }
        return $output;
    }

}
