<?php

namespace App\Controller\Rest;

use App\Entity\Card;
use App\Entity\CityFrancePdv;
use App\Entity\Contact;
use App\Entity\OrderTypePdv;
use App\Entity\OrderTypePdvDay;
use App\Entity\Pdv;
use App\Entity\Restaurant;
use App\Service\SendEvent;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Controller\FOSRestController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Validator\Constraints\DateTime;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\QueryParam;
use Nelmio\ApiDocBundle\Annotation\ApiDoc;

class RestaurantController extends FOSRestController
{

    /**
     * @Rest\Post("/restaurants")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="restaurant", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false, strict=true)
     * @RequestParam(name="conatc", nullable=false,allowBlank=false, strict=false)
     */
    public function addRestaurant(Request $request, ParamFetcher $paramFetcher)
    {
        /*$restaurant = new Restaurant();
        $restaurant->setName($paramFetcher->get('name'));
        $restaurant->setWebsite($paramFetcher->get('website'));
        if ($paramFetcher->get('avatar') != '') {
            $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
            $f = finfo_open();
            $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
            $split = explode('/', $mime_type);
            $type = $split[1];
            $filename = md5(uniqid()) . '.' . $type;
            $path = $this->getParameter('img_directory') . '/' . $filename;
            $restaurant->setAvatar($filename);
            file_put_contents($path, $decoded);
        } else {
            $restaurant->setAvatar('');
        }
        $restaurant->setActive(1);
        $ip = $request->getClientIp();
        $restaurant->setCreatedIp($ip);
        $restaurant->setCreatedUser('0');
        $restaurant->setUpdatedIp('');
        $restaurant->setUpdatedUser('0');
        $em = $this->getDoctrine()->getManager();
        $em->persist($restaurant);
        $em->flush();
        $id = $restaurant->getId();
        return new JsonResponse([
            'id' => $id,
        ], 201);*/
        $content = $request->getContent();
        if (!empty($content)) {
            $params = json_decode($content, true);
            if (array_key_exists('restaurant', $params) && array_key_exists('pdv', $params) && array_key_exists('conatct', $params)) {
                $resturantdata = $params['restaurant'];
                $pdvdata = $params['pdv'];
                $contactdata = $params['conatct'];
                $resp1 = $this->addRest($resturantdata, $request);
                $resp2 = $this->addPdv($pdvdata, $request);
                $resp3 = $this->addContact($contactdata);
                if ($resp1 instanceof JsonResponse) {
                    return $resp1;
                }
                if ($resp2 instanceof JsonResponse) {
                    return $resp2;
                }
                if ($resp3 instanceof JsonResponse) {
                    return $resp3;
                }
                //////////////// add Restaurant ///////////////////
                $restaurant=$resp1;
                $em = $this->getDoctrine()->getManager();
                $em->persist($restaurant);
                $em->flush();
                ////////////////// add Pdv ///////////////////////
                $pdv=$resp2;
                $pdv->setRestaurant($restaurant);
                $em->persist($pdv);
                $key = $this->dec_enc('encrypt', $pdv->getId());
                $pdv->setPdvKey($key);
                $em->persist($pdv);
                $em->flush();
                ////////////////// add Card ////////////////////
                $card=$this->addCard();
                $card->setId($pdv->getId());
                $card->setPdv($pdv);
                $metadata = $em->getClassMetaData(get_class($card));
                $metadata->setIdGenerator(new \Doctrine\ORM\Id\AssignedGenerator());
                $metadata->setIdGeneratorType(\Doctrine\ORM\Mapping\ClassMetadata::GENERATOR_TYPE_NONE);
                $em->persist($card);
                $em->flush();
                ////////////////// add Contact ////////////////
                $contact=$resp3;
                $contact->addPdv($pdv);
                $em->persist($contact);
                $em->flush();
                $this->addUser($contact,$pdv->getId());
                return new JsonResponse("restaurant created successfuly", 201);
            } else {
                return new JsonResponse('one of this fileds (restaurant,pdv,conatct) is null', 400);
            }
        } else {
            return new JsonResponse('no content found', 400);
        }
        //var_dump($paramFetcher->get('pdv'));
        //var_dump($paramFetcher->get('contact'));
    }

    /**
     * @Rest\Get("/restaurants/{id}")
     * @ApiDoc(
     *      resource=true,
     *      description="Recuperer les informations d'un restauratant",
     *      parameters={
     *         {"name"="id", "dataType"="Integer", "required"=true, "description"="identifiant du restaurateur"}
     *      },
     *      output={"class"="Restaurant"}
     * )
     */
    public function getRestaurant($id)
    {
        $restaurant = $this->getDoctrine()->getRepository("App:Restaurant")->find($id);
        if ($restaurant) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($restaurant, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
    }

    /**
     * @Rest\Get("/restaurants")
     * @ApiDoc(
     *      resource=true,
     *      description="Recuperer la liste des restaurateurs",
     *      output={"collection"=true, "collectionName"="classes", "class"="Restaurant"}
     * )
     */
    public function getRestaurants(Request $request)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $restaurants = $qb->select("r.id,r.name,r.avatar,r.website,r.active,p.id as card")
            ->from("App:Pdv", "p")
            ->innerJoin("p.restaurant", "r")
            ->groupBy('r.id')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($restaurants, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Put("/restaurants/{id}/status")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="active", nullable=false, strict=true, description="ACTIFREST.")
     *
     */
    public function updateRestaurantStatus(Request $request, $id, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        $restaurant = $this->getDoctrine()->getRepository("App:Restaurant")->find(intval($id));
        if (!is_null($restaurant)) {
            $status = $paramFetcher->get('active');
            $restaurant->setActive($status);
            $ip = $request->getClientIp();
            $restaurant->setUpdatedIp($ip);
            $restaurant->setUpdatedUser('0');
            $em = $this->getDoctrine()->getManager();
            $em->persist($restaurant);
            $em->flush();
            $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'restaurant', 'fileds' => 'status', 'status' => $status, 'id' => $id]);
            if($status==false){
                $qb = $em->createQueryBuilder();
                $users = $qb->select("u")
                ->from("App:User", "u")
                ->innerJoin('App:Pdv', 'p', 'WITH', 'p.id = u.pdv')
                ->innerJoin("p.restaurant", "r")
                ->where('r.id=:id')
                ->setParameter('id', $id)
                ->getQuery()
                ->getResult();
                foreach ($users as $user) {
                    $user->setEnabled(0);
                    $em->persist($restaurant);
                    $sendEvent->sendMessage(['type' => 'refreshRest','pdv'=>$user->getPdv()]);
                }
                $em->flush();
            }
            return new JsonResponse('', 200);
        } else {
            return new JsonResponse('', 204);
        }
    }

    /**
     * @Rest\Put("/restaurants/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=false,allowBlank=false,strict=true, description="NOM.")
     * @RequestParam(name="avatar", nullable=false,allowBlank=true,strict=true, description="AVTAR.")
     * @RequestParam(name="website", nullable=false,allowBlank=true,strict=true, description="SITE_WEB.")
     * @RequestParam(name="active", nullable=false,allowBlank=false, strict=true, description="ACTIFREST." , requirements="(true|false|1|0)")
     * @Rest\Put("/restaurants/{id}")
     */
    public function updateAction(Request $request, $id, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        $restaurant = $this->getDoctrine()->getRepository("App:Restaurant")->find(intval($id));
        if (!is_null($restaurant)) {
            $restaurant->setName($paramFetcher->get('name'));
            $restaurant->setWebsite($paramFetcher->get('website'));
            if ($paramFetcher->get('avatar') != '') {
                if (substr($paramFetcher->get('avatar'), 0, 4) == 'data') {
                    $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
                    $f = finfo_open();
                    $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
                    $split = explode('/', $mime_type);
                    $type = $split[1];
                    $filename = md5(uniqid()) . '.' . $type;
                    $path = $this->getParameter('img_directory') . '/' . $filename;
                    $restaurant->setAvatar($filename);
                    file_put_contents($path, $decoded);
                }
            } else {
                if ($restaurant->getAvatar() != '') {
                    $avatar = $this->getParameter('img_directory') . "/" . $restaurant->getAvatar();
                    if (file_exists($avatar))
                        unlink($avatar);
                    $restaurant->setAvatar('');
                }
            }
            $restaurant->setActive($paramFetcher->get('active'));
            $ip = $request->getClientIp();
            $restaurant->setUpdatedIp($ip);
            $restaurant->setUpdatedUser('0');
            $em = $this->getDoctrine()->getManager();
            $em->persist($restaurant);
            $em->flush();
            $sendEvent->sendMessage(['type' => 'persist', 'entity' => 'restaurant', 'id' => $id]);
            return new JsonResponse('', 200);
        } else {
            return new JsonResponse('', 204);
        }
    }

    /**
     * @Rest\Delete("/restaurants/{id}")
     * * @ApiDoc(
     *      resource=true,
     *      description="Ajouter un restaurateur",
     *      parameters={
     *         {"name"="id", "dataType"="Integer", "required"=true, "description"="identifiant du restaurateur"}
     *      }
     * )
     */
    public function deleteAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $restaurant = $em->getRepository('App:Restaurant')->find($id);
        if ($restaurant) {
            $em->remove($restaurant);
            $em->flush();
            return new JsonResponse([], 200);
        } else {
            return new JsonResponse([], 204);
        }
    }

    /**
     * @Rest\Get("/restaurants/{id}/name")
     */
    public function getRestaurantName($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $restaurant = $qb->select("r.name")
            ->from("App:Restaurant", "r")
            ->where('r.id = :id')
            ->setParameter('id', $id)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($restaurant, 'json');
        return new JsonResponse(json_decode($data));
    }

    private function addRest($data, Request $request)
    {
        $name = (array_key_exists('name', $data)) ? $data['name'] : null;
        $website = (array_key_exists('website', $data)) ? $data['website'] : '';
        $avatar = (array_key_exists('avatar', $data)) ? $data['avatar'] : '';
        if ($name) {
            $restaurant = new Restaurant();
            $restaurant->setName($name);
            $restaurant->setWebsite($website);
            if ($avatar != '') {
                $decoded = base64_decode(substr($avatar, 22));
                $f = finfo_open();
                $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
                $split = explode('/', $mime_type);
                $type = $split[1];
                $filename = md5(uniqid()) . '.' . $type;
                $path = $this->getParameter('img_directory') . '/' . $filename;
                $restaurant->setAvatar($filename);
                file_put_contents($path, $decoded);
            } else {
                $restaurant->setAvatar('');
            }
            $restaurant->setActive(1);
            $ip = $request->getClientIp();
            $restaurant->setCreatedIp($ip);
            $restaurant->setCreatedUser('0');
            $restaurant->setUpdatedIp('');
            $restaurant->setUpdatedUser('0');
            return $restaurant;
        } else {
            return new JsonResponse('name of restaurant undefined', 400);
        }
    }

    public function addPdv($data, Request $request)
    {
        $name = (array_key_exists('name', $data)) ? $data['name'] : null;
        $siret = (array_key_exists('siret', $data)) ? $data['siret'] : '';
        $number = (array_key_exists('number', $data)) ? $data['number'] : null;
        $address = (array_key_exists('address', $data)) ? $data['address'] : null;
        $postalCode = (array_key_exists('postalCode', $data)) ? $data['postalCode'] : null;
        $city = (array_key_exists('city', $data)) ? $data['city'] : null;
        $country = (array_key_exists('country', $data)) ? $data['country'] : null;
        $amountMiniOrder = (array_key_exists('amountMiniOrder', $data)) ? $data['amountMiniOrder'] : null;
        $amountMiniSociety = (array_key_exists('amountMiniSociety', $data)) ? $data['amountMiniSociety'] : null;
        $amountMiniMandatory = (array_key_exists('amountMiniMandatory', $data)) ? $data['amountMiniMandatory'] : null;
        $phone = (array_key_exists('phone', $data)) ? $data['phone'] : null;
        $email = (array_key_exists('email', $data)) ? $data['email'] : '';
        $principal = (array_key_exists('principal', $data)) ? $data['principal'] : null;
        $active = (array_key_exists('active', $data)) ? $data['active'] : null;
        $paymentModes = (array_key_exists('paymentModes', $data)) ? $data['paymentModes'] : null;
        $orderModes = (array_key_exists('orderModes', $data)) ? $data['orderModes'] : null;
        $authorizedCities = (array_key_exists('authorizedCities', $data)) ? $data['authorizedCities'] : null;
        if (
            is_null($name) || empty($name) ||
            is_null($address) || empty($address) ||
            is_null($phone) || empty($phone) || !preg_match('/^\d{10}$/', $phone) ||
            !is_array($paymentModes) || count($paymentModes) == 0 ||
            !is_array($orderModes) || count($paymentModes) == 0 ||
            !is_array($authorizedCities)
        ) {
            return new JsonResponse('one of this fileds (name,address,phone,paymentModes,orderModes,authorizedCities) is null or invalide', 400);
        }
        $pdv = new Pdv();
        $pdv->setName($name);
        $pdv->setSiret($siret);
        $pdv->setNumber($number);
        $pdv->setAddress($address);
        $pdv->setPostalCode($postalCode);
        $pdv->setCity($city);
        $pdv->setCountry($country);
        $pdv->setResetDelay(1);
        $pdv->setAmountMiniOrder(floatval($amountMiniOrder));
        $pdv->setAmountMiniSociety(floatval($amountMiniSociety));
        $pdv->setAmountMiniMandatory(intval($amountMiniMandatory));
        $pdv->setPhone($phone);
        $pdv->setEmail($email);
        $pdv->setPrincipal(intval($principal));
        $pdv->setActive(intval($active));
        $pdv->setAutomaticPrinting(0);
        $pdv->setAutomaticPrinting2(0);
        $pdv->setPrinter('cuisine');
        $pdv->setComment('');
        $pdv->setImage('');
        $ip = $request->getClientIp();
        $pdv->setCreatedIp($ip);
        $pdv->setCreatedUser($this->getUser()->getId());
        $pdv->setUpdatedIp('');
        $pdv->setUpdatedUser('0');
        $em = $this->getDoctrine()->getManager();
        $pdv->setCloseSite(1);
        $pdv->setDisplayBestSales(0);
        $pdv->setDDisplayBestOffers(0);
        $pdv->setSiteType(0);
        $pdv->setCloseComment('Bonjour, nous sommes exceptionnellement fermé. Veuillez nous excuser pour la gêne occasionnée.');
        foreach ($paymentModes as $value) {
            $paymentMode = $em->getRepository('App:PaymentMode')->find($value['id']);
            if ($paymentMode) {
                $pdv->addPaymentMode($paymentMode);
            } else {
                return new JsonResponse("payament mode invalide", 400);
            }
        }
        foreach ($orderModes as $value) {
            $orderModePdv = new OrderTypePdv();
            if (
                array_key_exists('minDelay', $value) &&
                array_key_exists('maxDelay', $value) &&
                array_key_exists('active', $value) &&
                array_key_exists('id', $value) &&
                is_array($value['availability']) && count($value['availability']) == 7
            ) {
                $orderModePdv->setMinDelay($value['minDelay']);
                $orderModePdv->setMaxDelay($value['maxDelay']);
                $orderModePdv->setDefaultMinDelay($value['minDelay']);
                $orderModePdv->setDefaultMaxDelay($value['maxDelay']);
                $orderModePdv->setPdv($pdv);
                $orderModePdv->setActive($value['active']);
                $orderMode = $em->getRepository('App:OrderType')->find($value['id']);
                $orderModePdv->setOrderType($orderMode);
                foreach ($value['availability'] as $key => $availability) {
                    if (
                        array_key_exists('day', $availability) &&
                        array_key_exists('id', $availability['day']) &&
                        array_key_exists('middayStart', $availability) &&
                        array_key_exists('middayEnd', $availability) &&
                        array_key_exists('eveningStart', $availability) &&
                        array_key_exists('eveningEnd', $availability)
                    ) {
                        $orderTypePdvDay = new OrderTypePdvDay();
                        $day = $em->getRepository('App:Day')->find($availability['day']['id']);
                        $orderTypePdvDay->setDay($day);
                        $orderTypePdvDay->setOrderTypePdv($orderModePdv);
                        $middayStart = ($availability['middayStart'] != "" && !is_null($availability['middayStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayStart']) : null;
                        $middayEnd = ($availability['middayEnd'] != "" && !is_null($availability['middayEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['middayEnd']) : null;
                        $eveningStart = ($availability['eveningStart'] != "" && !is_null($availability['eveningStart'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningStart']) : null;
                        $eveningEnd = ($availability['eveningEnd'] != "" && !is_null($availability['eveningEnd'])) ? \DateTime::createFromFormat('D M d Y H:i:s e+', $availability['eveningEnd']) : null;
                        $orderTypePdvDay->setMiddayStart($middayStart);
                        $orderTypePdvDay->setMiddayEnd($middayEnd);
                        $orderTypePdvDay->setEveningStart($eveningStart);
                        $orderTypePdvDay->setEveningEnd($eveningEnd);
                        $orderModePdv->addOrderTypePdvDay($orderTypePdvDay);
                    } else {
                        return new JsonResponse("availability (day,id,middayStart,middayEnd,eveningStart,eveningEnd) missing data", 400);
                    }
                }
                $pdv->addOrderType($orderModePdv);
            } else {
                return new JsonResponse("orderModes (id,minDelay,maxDelay,active) missing data", 400);
            }
        }
        foreach ($authorizedCities as $value) {
            if (array_key_exists('id', $value) && array_key_exists('amountMiniDelivery', $value)) {
                $cityFrancePdv = new CityFrancePdv();
                $cityFrance = $em->getRepository('App:CityFrance')->find($value['id']);
                if ($cityFrance) {
                    $cityFrancePdv->setCity($cityFrance);
                    $cityFrancePdv->setPdv($pdv);
                    $cityFrancePdv->setAmountMiniDelivery($value['amountMiniDelivery']);
                    $pdv->addCityFrance($cityFrancePdv);
                    //$em->persist($cityFrancePdv);
                } else {
                    return new JsonResponse("city not found", 400);
                }
            } else {
                return new JsonResponse("authorizedCities (id,amountMiniDelivery) missing data", 400);
            }
        }
        return $pdv;
    }

    private function addCard()
    {
        $card = new Card();
        $start = new \DateTime();
        $end = new \DateTime();
        $end->modify('+100 year');
        $card->setStartedValidity($start);
        $card->setStoppedValidity($end);
        $card->setNbrOffersAuthorized(1);
        $card->setNbrOffersAuthorizedSite(1);
        $card->setDisplayCode(0);
        $card->setComment('');
        $card->setDeliveryCharges(0);
        $card->setManagementFees(0);
        return $card;
    }

    private function addContact($data)
    {
        $firstname = (array_key_exists('firstname', $data)) ? $data['firstname'] : null;
        $lastname = (array_key_exists('lastname', $data)) ? $data['lastname'] : null;
        $phone = (array_key_exists('phone', $data)) ? $data['phone'] : null;
        $email = (array_key_exists('email', $data)) ? $data['email'] : null;
        $function = (array_key_exists('function', $data)) ? $data['function'] : null;
        $emailregex = "/^[a-zA-Z0-9._-]+@[a-z0-9._-]{2,}\.[a-z]{2,4}$/";
        $phoneregex = "/^\d{10}$/";
        if (
            is_null($firstname) || empty($firstname) ||
            is_null($lastname) || empty($lastname) ||
            is_null($email) || empty($email) || !preg_match($emailregex, $email) ||
            is_null($function) || empty($function) ||
            is_null($phone) || empty($phone) || !preg_match($phoneregex, $phone)
        ) {
            return new JsonResponse('one of this fileds (firstname,lastname,phone,email,function) is null or invalide', 400);
        }
        $userManager = $this->get('fos_user.user_manager');
        $user2 = $userManager->findUserByEmail($email);
        if ($user2) {
            return new JsonResponse('user already exist', 400);
        }
        $contact = new Contact();
        $contact->setfirstname($firstname);
        $contact->setlastname($lastname);
        $contact->setemail($email);
        $contact->setphone($phone);
        $contact->setfunction($function);
        $contact->setUser(false);
        return $contact;
    }

    private function addUser(Contact $contact,$pdv)
    {
        if ($this->getUser()->getProfil()->getId() == 3 || $this->getUser()->getProfil()->getId() == 4) {
            return new JsonResponse([], 403);
        }
        $userManager = $this->get('fos_user.user_manager');
        $user = $userManager->createUser();
        $rules=$this->getBaserights(3);
        $user->setRoles($rules);
        $profil = $this->getDoctrine()->getRepository("App:Profil")->find(3);
        $user->setProfil($profil);
        $user->setUsername($contact->getEmail());
        $user->setFirstname($contact->getFirstname());
        $user->setLastname($contact->getLastname());
        $user->setEmail($contact->getEmail());
        $user->setEmailCanonical($contact->getEmail());
        $user->setEnabled(1);
        $user->setContact($contact->getId());
        $user->setPdv($pdv);
        $user->setPlainPassword('&Helpy2018!');
        $userManager->updateUser($user);
    }

    private function dec_enc($action, $string)
    {
        $output = false;
        $encrypt_method = "AES-256-CBC";
        $secret_key = 'helpyfood';
        $secret_iv = 'helpyfood';
        $key = hash('sha256', $secret_key);
        $iv = substr(hash('sha256', $secret_iv), 0, 16);
        if ($action == 'encrypt') {
            $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
            $output = base64_encode($output);
        } else if ($action == 'decrypt') {
            $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        }
        return $output;
    }

    private function getBaserights($profil){
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $r=$qb->select("r.rule")
            ->from("App:ProfilRights", "pr")
            ->innerJoin('pr.rights', 'r')
            ->innerJoin('pr.profil', 'p')
            ->where('p.id = :idP')
            ->andWhere('pr.base = 1')
            ->setParameter('idP', $profil)
            ->getQuery()->getResult();
        $rules = array_column($r, "rule");
        return $rules;
    }

}
