<?php

namespace App\Controller\Rest;


use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use App\Entity\CardFamily;
use App\Entity\Size;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\DateTime;
use Nelmio\ApiDocBundle\Annotation\ApiDoc;


class SizeController extends FOSRestController
{
    /**
     * @Rest\Post("/sizes")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="label", nullable=false,allowBlank=false,strict=true,description="Nom du taille")
     * @RequestParam(name="level", nullable=false,allowBlank=false,strict=true,description="Niveau du taille")
     * @RequestParam(name="carte", nullable=false,allowBlank=false,strict=true,description="Identifiant du taille")
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true,description="tableau des identifiants des familles possibles")
     * @ApiDoc(
     *      resource=true,
     *      description="Ajouter une taille",
     *
     * )
     */
    public function addAction(Request $request, ParamFetcher $paramFetcher)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($paramFetcher->get('carte'))) {
                return new JsonResponse(null, 403);
            }
        }
        $size = new Size();
        $size->setLabel($paramFetcher->get('label'));
        $size->setLevel($paramFetcher->get('level'));
        $em = $this->getDoctrine()->getManager();
        $carte = $em->getRepository('App:Card')->find(intval($paramFetcher->get('carte')));
        $size->setCard($carte);
        foreach ($paramFetcher->get('families') as $value) {
            $family = $em->getRepository('App:CardFamily')->find($value);
            $size->addFamily($family);
        }
        $em->persist($size);
        $em->flush();
        /*$client = new Client(new Version1X('//127.0.0.1:1337'));
        $client->initialize();
        $client->emit('notif', ['type' => 'persist','entity' => 'size', 'id' => $size->getId()]);
        $client->close();*/
        $families = $size->getFamilies();
        $result = [
            'families' => $families,
            'id' => $size->getId(),
            'success' => true,
            'code' => 200
        ];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/sizes")
     */
    public function getListAction()
    {
        $size = $this->getDoctrine()->getRepository("App:Size")->findAll();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($size, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/sizes/carte/{idCarte}")
     */
    public function getListByCarteAction($idCarte)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCarte)) {
                return new JsonResponse(null, 403);
            }
        }
        $card = $this->getDoctrine()->getRepository("App:Card")->find($idCarte);
        $sizes = $this->getDoctrine()->getRepository("App:Size")->findBy(array("card" => $card));
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($sizes, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/sizes/element/{idElement}")
     */
    public function getListByElementAction($idElement)
    {
        $sizes = $this->getDoctrine()->getRepository("App:Size")->getListByElement($idElement);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($sizes, 'json');
        return new JsonResponse(json_decode($data));
    }


    /**
     * @Rest\Get("/sizes/family/{idFamily}")
     */
    public function getListByFamily($idFamily)
    {

        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $sizes = $qb->select("s.id,s.label")
            ->from("App:Size", "s")
            ->innerJoin('s.families', 'f')
            ->where('f.id = :idFamily');
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $sizes = $sizes->innerJoin('s.card', 'c')
                ->andWhere('c.id = :idCard')
                ->setParameter('idCard', $this->getUser()->getPdv());
        }
        $sizes = $sizes
            ->setParameter('idFamily', $idFamily)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($sizes, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/sizes/carte/{idCard}/family/{idFamily}")
     */
    public function getSizeOfCarteByFamily($idCard, $idFamily)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($idCard)) {
                return new JsonResponse(null, 403);
            }
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $sizes = $qb->select("s.id,s.label")
            ->from("App:Size", "s")
            ->leftJoin('s.families', 'cf')
            ->innerJoin('s.card', 'C')
            ->where('C.id = :card')
            ->andWhere('cf.id = :family')
            ->setParameter('card', $idCard)
            ->setParameter('family', $idFamily)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($sizes, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Delete("/sizes/{id}")
     */
    public function deleteAction($id)
    {
        $p = null;
        $em = $this->getDoctrine()->getManager();
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $this->getDoctrine()->getRepository("App:Card")->find($this->getUser()->getPdv());
            $p = $this->getDoctrine()->getRepository("App:Size")->findOneBy(array("id" => $id, "card" => $card));
        } else {
            $p = $em->getRepository('App:Size')->find($id);
        }
        if (is_null($p)) {
            return new JsonResponse(null, 403);
        }
        $em->remove($p);
        $em->flush();
        /*  $client = new Client(new Version1X('//127.0.0.1:1337'));
          $client->initialize();
          $client->emit('notif', ['type' => 'remove','entity' => 'size', 'id' => $id]);
          $client->close();*/
        return new JsonResponse([
            'success' => true,
            'code' => 200
        ]);

    }

    /**
     * @Rest\Put("/sizes/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="label", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="level", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="families", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAction($id, ParamFetcher $paramFetcher)
    {
        $size = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $this->getDoctrine()->getRepository("App:Card")->find($this->getUser()->getPdv());
            $size = $this->getDoctrine()->getRepository("App:Size")->findOneBy(array("id" => $id, "card" => $card));
        } else {
            $size = $this->getDoctrine()->getRepository("App:Size")->find($id);
        }
        if (is_null($size)) {
            return new JsonResponse(null, 403);
        }

        $size = $this->getDoctrine()->getRepository("App:Size")->find($id);
        $size->setLabel($paramFetcher->get('label'));
        $size->setLevel($paramFetcher->get('level'));
        $em = $this->getDoctrine()->getManager();
        $size->setFamilies([]);
        foreach ($paramFetcher->get('families') as $value) {
            $cardFamily = $em->getRepository('App:CardFamily')->find($value);
            $size->addFamily($cardFamily);
        }
        $families = $size->getFamilies();
        $result = [
            'families' => $families,
            'success' => true,
            'code' => 200
        ];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        $em->persist($size);
        $em->flush();
        /* $client = new Client(new Version1X('//127.0.0.1:1337'));
         $client->initialize();
         $client->emit('notif', ['type' => 'persist','entity' => 'size', 'id' => $id]);
         $client->close();*/
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Get("/sizes/{id}")
     */
    public function getAction($id, ParamFetcher $paramFetcher)
    {
        $size = null;
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            $card = $this->getDoctrine()->getRepository("App:Card")->find($this->getUser()->getPdv());
            $size = $this->getDoctrine()->getRepository("App:Size")->findOneBy(array("id" => $id, "card" => $card));
        } else {
            $size = $this->getDoctrine()->getRepository("App:Size")->find($id);
        }
        if (is_null($size)) {
            return new JsonResponse(null, 403);
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($size, 'json');
        return new JsonResponse(json_decode($data));

    }

    /**
     * @Rest\Post("/sizes/setNewCarteSize")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="size", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="carte", nullable=false,allowBlank=false,strict=true)
     */
    public function addNewSizeWithImport(Request $request, ParamFetcher $paramFetcher)
    {
        if (in_array($this->getUser()->getProfil()->getId(), [4, 3])) {
            if ($this->getUser()->getPdv() != intval($paramFetcher->get('carte'))) {
                return new JsonResponse(null, 403);
            }
        }
        $sizeImport = $paramFetcher->get('size');
        $em = $this->getDoctrine()->getManager();
        if (is_null($sizeImport)) {
            return new JsonResponse([], 400);
        }
        foreach ($sizeImport as $value) {
            $size = new Size();
            $carte = $em->getRepository('App:Card')->find(intval($paramFetcher->get('carte')));
            $size->setCard($carte);
            $sizeI = $this->getDoctrine()->getRepository("App:LibrarySizes")->find($value);
            $size->setLabel($sizeI->getName());
            $size->setLevel($sizeI->getLevel());
            $families = $sizeI->getFamilies();
            foreach ($families as $fam) {
                $family = $em->getRepository('App:CardFamily')->findOneBy(array('family' => $fam, 'card' => $carte));
                if ($family)
                    $size->addFamily($family);
            }
            $em->persist($size);
            $em->flush();
        }
        return new JsonResponse([], 200);
    }
}
