<?php

namespace App\Controller\Rest;

use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations\Route;
use FOS\RestBundle\Controller\Annotations as Rest;
use App\Entity\Pdv;
use App\Entity\Team;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\Entity\Contact;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use FOS\RestBundle\Request\ParamFetcher;
use Doctrine\ORM\Tools\Pagination\Paginator;
use Symfony\Component\Validator\Constraints\DateTime;

class TeamController extends FOSRestController
{

    /**
     * @Rest\Post("/team")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="firstname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="lastname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="phone", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="designation", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="active", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="quality", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="avatar", nullable=true, strict=true)
     */
    public function addAction(ParamFetcher $paramFetcher)
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $pdv = $em->getRepository('App:Pdv')->find($this->getUser()->getPdv());
        $team = new Team();
        $team->setfirstname($paramFetcher->get('firstname'));
        $team->setlastname($paramFetcher->get('lastname'));
        $team->setemail($paramFetcher->get('email'));
        $team->setphone($paramFetcher->get('phone'));
        $team->setDesignation($paramFetcher->get('designation'));
        $team->setActive($paramFetcher->get('active'));
        $team->setQuality($paramFetcher->get('quality'));
        $team->setUser(0);
        if ($paramFetcher->get('avatar') != '') {
            $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
            $f = finfo_open();
            $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
            $split = explode('/', $mime_type);
            $type = $split[1];
            $filename = md5(uniqid()) . '.' . $type;
            $path = $this->getParameter('img_directory') . '/' . $filename;
            $team->setAvatar($filename);
            file_put_contents($path, $decoded);
        } else {
            $team->setAvatar('');
        }
        $team->setPdv($pdv);
        $em->persist($team);
        $em->flush();
        return new JsonResponse([
            'id' => $team->getId(),
            'success' => true,
            'code' => 200
        ]);
    }

    /**
     * @Rest\Put("/team")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="firstname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="lastname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="phone", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="designation", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="active", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="quality", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="avatar", nullable=true, strict=true)
     * @RequestParam(name="id", nullable=false,allowBlank=false,strict=true)
     */
    public function updateAction(Request $request, ParamFetcher $paramFetcher)
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $team = $this->getDoctrine()->getRepository("App:Team")->find($paramFetcher->get('id'));
        if (!is_null($team)) {
            $team->setfirstname($paramFetcher->get('firstname'));
            $team->setlastname($paramFetcher->get('lastname'));
            $team->setemail($paramFetcher->get('email'));
            $team->setphone($paramFetcher->get('phone'));
            $team->setDesignation($paramFetcher->get('designation'));
            $team->setActive($paramFetcher->get('active'));
            $team->setQuality($paramFetcher->get('quality'));
            if ($paramFetcher->get('avatar') != '') {
                if (substr($paramFetcher->get('avatar'), 0, 4) == 'data') {
                    $decoded = base64_decode(substr($paramFetcher->get('avatar'), 22));
                    $f = finfo_open();
                    $mime_type = finfo_buffer($f, $decoded, FILEINFO_MIME_TYPE);
                    $split = explode('/', $mime_type);
                    $type = $split[1];
                    $filename = md5(uniqid()) . '.' . $type;
                    $path = $this->getParameter('img_directory') . '/' . $filename;
                    $team->setAvatar($filename);
                    file_put_contents($path, $decoded);
                }
            } else {
                if ($team->getAvatar() != '') {
                    $avatar = $this->getParameter('img_directory') . "/" . $team->getAvatar();
                    if (file_exists($avatar))
                        unlink($avatar);
                    $team->setAvatar('');
                }
            }
            $em = $this->getDoctrine()->getManager();
            $em->persist($team);
            $em->flush();
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        }else{
            return new JsonResponse([],400);
        }
    }

    /**
     * @Rest\Get("/team/personalQuality")
     */
    public function getPersonalQuality()
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $qua = $qb->select("q.id,q.label")
            ->from("App:PersonalQuality", "q")
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($qua, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/team")
     */
    public function getTeamPdv()
    {
        if (!$this->getUser()->hasRole('RULE_MANAGEMENTTEAM') && $this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $data = $qb->select("t as team,u.id as user")
            ->from("App:Team", "t")
            ->leftJoin('App:User', 'u', 'WITH', 'u.team = t.id AND u.enabled = 1')
            ->innerJoin('t.pdv', 'p')
            ->where('p.id = :pdv')
            ->setParameter('pdv', $this->getUser()->getPdv())
            ->getQuery()
            ->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/team/{id}")
     */
    public function getTeamById($id)
    {
        if ($this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $team = $qb->select("t as team,u.id as user")
            ->from("App:Team", "t")
            ->leftJoin('App:User', 'u', 'WITH', 'u.team = t.id AND u.enabled = 1')
            ->innerJoin('t.pdv', 'p')
            ->where('t.id = :id')
            ->andWhere('p.id = :pdv')
            ->setParameter('id', $id)
            ->setParameter('pdv', $this->getUser()->getPdv())
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($team, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/team/pdv/active")
     */
    public function getActivateTeamPdv()
    {
        if (!$this->getUser()->hasRole('RULE_MANAGEMENTTEAM') && $this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $data = $qb->select("t.id,t.designation,t.firstname,t.lastname")
            ->from("App:Team", "t")
            ->innerJoin('t.pdv', 'p')
            ->where('t.active=1')
            ->andWhere('p.id = :pdv')
            ->setParameter('pdv', $this->getUser()->getPdv())
            ->getQuery()
            ->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/team/pdv/all")
     */
    public function getTeamByPdv()
    {
        if (!$this->getUser()->hasRole('RULE_COLLECTION') && $this->getUser()->getProfil()->getId() != 3) {
            return new JsonResponse([], 403);
        }
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $data = $qb->select("t.id,t.designation,t.firstname,t.lastname")
            ->from("App:Team", "t")
            ->innerJoin('t.pdv', 'p')
            ->where('p.id = :pdv')
            //  ->andWhere('t.quality = 1')
            ->setParameter('pdv', $this->getUser()->getPdv())
            ->getQuery()
            ->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }

}
