<?php
/**
 * Created by PhpStorm.
 * User: HP
 * Date: 07/06/2018
 * Time: 14:19
 */

namespace App\Controller\Rest;

use Symfony\Component\Validator\Constraints as Assert;
use App\Entity\Address;
use App\Entity\Command;
use App\Entity\CommandElement;
use App\Entity\CommandElementIngredients;
use App\Entity\CommandMenu;
use App\Entity\Customer;
use App\Entity\CustomerPdv;
use App\Entity\CommandPaymentMode;
use App\Entity\webSiteUsers;
use App\Service\SendEvent;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Query;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Controller\FOSRestController;
use Symfony\Component\HttpFoundation\JsonResponse;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Controller\Annotations\RequestParam;
use \Firebase\JWT\JWT;
use Symfony\Component\HttpFoundation\Request;

class WebSiteController extends FOSRestController
{
    /**
     * @Rest\Get("/website/pdv")
     */
    public function getPdv(Request $request)
    {
        $id = $request->headers->get('pdv');
        $id = intval($this->dec_enc('decrypt', $id));
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdv = $qb->select("p.name,p.address,p.phone,p.amountMiniMandatory,p.amountMiniOrder")
            ->from("App:Card", "c")
            ->where('c.id = :card')
            ->setParameter('card', $id)
            ->getQuery()->getResult();
        if ($pdv) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($pdv, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
        return new JsonResponse([], 204);
    }

    /**
     * @Rest\Get("/website/pdv/availability")
     */
    public function getPdvAvailability(Request $request)
    {
        $idPdv = $request->headers->get('pdv');
        $idPdv = $this->dec_enc('decrypt', $idPdv);
        $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($idPdv);
        if ($pdv) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($pdv->getOrderTypes(), 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
    }

    /**
     * @Rest\Get("/website/pdv/miniInfo")
     */
    public function getPdvMiniInfo(Request $request)
    {
        $idPdv = $request->headers->get('pdv');
        $idPdv = intval($this->dec_enc('decrypt', $idPdv));
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $pdv = $qb->select("p.discount,p.discountFrom,p.city,p.closeComment,p.nameRestaurant as name,c.nbrOffersAuthorizedSite,p.displayBestSales,p.displayBestOffers,p.facebook,p.address,p.phone,p.emailContact,p.image,p.comment,p.amountMiniMandatory,p.amountMiniOrder,p.closeSite")
            ->from("App:Card", "c")
            ->innerJoin("c.pdv", "p")
            ->where('p.id = :pdv')
            ->setParameter('pdv', $idPdv)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if ($pdv) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($pdv, 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
        return new JsonResponse([], 204);

        /*$idCarte = $this->dec_enc('decrypt', $idCarte);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $card = $qb->select("c.nbrOffersAuthorizedSite")
            ->from("App:Card", "c")
            ->where('c.id LIKE :card')
            ->setParameter('card', $idCarte)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($card, 'json');
        return new JsonResponse(json_decode($data));*/
    }

    /**
     * @Rest\Get("/website/ingredients/compositions")
     */
    public function getIngredientComposByCarte(Request $request)
    {
        $idCarte = $request->headers->get('pdv');
        $idCarte = $this->dec_enc('decrypt', $idCarte);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $ingredients = $qb->select("ec.id,ec.name as text")
            ->from("App:Element", "e")
            ->innerJoin('e.elementComposition', 'ec')
            ->innerJoin('e.card', 'c')
            ->Where('c.id=:card')
            ->setParameter('card', $idCarte)
            ->groupBy('ec.id')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($ingredients, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/families/cmd/carte")
     */
    public function getfamiliesByPdvAndCard(Request $request)
    {
        $idCarte = $request->headers->get('pdv');
        $idCarte = $this->dec_enc('decrypt', $idCarte);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("cf.id,f.label,e.avatar,cf.order")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->innerJoin('e.card', 'c')
            ->where('c.id = :card')
            ->andWhere('e.active =1')
            ->andWhere('e.activeSite =1')
            ->setParameter('card', $idCarte)
            ->groupBy('cf.id')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/menucategories/cmd/carte")
     */
    public function getMenuCategoriesByPdv(Request $request)
    {
        $idCarte = $request->headers->get('pdv');
        $idCarte = $this->dec_enc('decrypt', $idCarte);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder('c');
        $elements = $qb->select("c.name,c.id,m.avatar")
            ->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->innerJoin('m.card', 'ca')
            ->where('ca.id = :card')
            ->andWhere('m.activeSite=1')
            ->setParameter('card', $idCarte)
            ->groupBy('c.id')
            ->getQuery()->getResult();
        foreach ($elements as $key=>$element) {
            $qb1 = $em->createQueryBuilder();
            $el=$qb1->select("cm.order")
            ->from("App:CardMenuCategory", "cm")
            ->innerJoin('cm.menuCategory', 'c')
            ->innerJoin('cm.card', 'ca')
            ->where('ca.id = :card')
            ->andWhere('c.id = :cat')
            ->setParameter('card', $idCarte)
            ->setParameter('cat', $element['id'])
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
            $elements[$key]['order']=$el['order'];
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/ingredientsCatFamSize/category/{idCategory}/family/{idFamily}/size/{idSize}")
     */
    public function getIngredientsCatFamSizeBySize($idCategory, $idFamily, $idSize)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $sizes = $qb->select("icf.id,icf.price AS price")
            ->from("App:IngredientCatFamSize", "icf")
            ->innerJoin('icf.cardFamily', 'cf')
            ->innerJoin('icf.size', 't')
            ->innerJoin('icf.ingredientCategory', 'ic')
            ->where('cf.id = :idCardFamily')
            ->andWhere('ic.id = :category')
            ->andWhere('t.id = :size')
            ->setParameter('category', $idCategory)
            ->setParameter('size', $idSize)
            ->setParameter('idCardFamily', $idFamily)
            ->getQuery()->getResult();

        $serializer = $this->get('serializer');
        $data = $serializer->serialize($sizes, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/menus/carte/category/{idCategory}/orderType/{idOrderType}")
     */
    public function getRestaurantMenuByCategoryAndCard(Request $request, $idCategory, $idOrderType)
    {
        $idCarte = $request->headers->get('pdv');
        $idCarte = $this->dec_enc('decrypt', $idCarte);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("m.id,mot.supplement,m.avatar,m.code,m.vat,m.title,m.description,
        m.price,m.available,c.name as category,mt.id as menuTime,o.id as offer,
        case when mt.id = 1 then false else true end as validPeriod,mot.active as dispo");
        $menus = $menus->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->leftJoin('m.offer', 'co')
            ->leftJoin('co.offer', 'o')
            ->innerJoin('m.menuTime', 'mt')
            ->innerJoin('m.card', 'ca')
            ->innerJoin('m.menuOrderType', 'mot')
            ->innerJoin('mot.orderType', 'ot')
            ->where('ca.id = :card')
            ->andWhere('ot.id=:orderType')
            ->andWhere('m.available=1')
            ->andWhere('m.activeSite=1')
            ->andWhere('c.id = :idCat')
            ->setParameter('card', $idCarte)
            ->setParameter('orderType', $idOrderType)
            ->setParameter('idCat', $idCategory)
            ->groupBy('m.id')
            ->getQuery()->getResult();
        foreach ($menus as $key => $menu) {
            $menus[$key]['compositions'] = $this->getCompositionsMenu($menu['id']);
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($menus, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/bestoffers/orderType/{idOrderType}")
     */
    public function getGetBestOffers(Request $request,$idOrderType)
    {
        $idCarte = $request->headers->get('pdv');
        $idCarte = $this->dec_enc('decrypt', $idCarte);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("m.id,mot.supplement,m.avatar,m.code,m.vat,m.title,m.description,
        m.price,m.available,c.name as category,mt.id as menuTime,o.id as offer,
        case when mt.id = 1 then false else true end as validPeriod,mot.active as dispo");
        $menus = $menus->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->innerJoin('m.offer', 'co')
            ->innerJoin('co.offer', 'o')
            ->innerJoin('m.menuTime', 'mt')
            ->innerJoin('m.card', 'ca')
            ->innerJoin('m.menuOrderType', 'mot')
            ->innerJoin('mot.orderType', 'ot')
            ->where('ca.id = :card')
            ->andWhere('ot.id=:orderType')
            ->andWhere('m.available=1')
            ->andWhere('m.activeSite=1')
            ->setParameter('card', $idCarte)
            ->setParameter('orderType', $idOrderType)
            ->groupBy('m.id')
            ->getQuery()->getResult();
        foreach ($menus as $key => $menu) {
            $menus[$key]['compositions'] = $this->getCompositionsMenu($menu['id']);
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($menus, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/menus/{id}/orderType/{idOrderType}/compositions")
     */
    public function getElementsSize2(Request $request,$id,$idOrderType)
    {
        $elements = array();
        $families = array();
        $additionalOptions = $this->getOptionAction($id);
        $idPdv = $request->headers->get('pdv');
        $idPdv = $this->dec_enc('decrypt', $idPdv);
        for ($i = 0; $i < 6; $i++) {
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $query = $qb->select("cf.id as family,f.label as familyName,es.id,es.priceSupp,es.price,es.available,e.id as element,e.hasSupp,e.avatar,e.priceSuppApp,e.name,e.maxNbrIngOptions,s.id as idSize,s.label as size,s.level,mes.quantity,mes.choice")
                ->from("App:Menu", "m")
                ->innerJoin('m.elements', 'mes')
                ->innerJoin('mes.elementSize', 'es')
                ->innerJoin('es.element', 'e')
                ->innerJoin('e.family', 'cf')
                ->innerJoin('cf.family', 'f')
                ->innerJoin('es.size', 's')
                ->innerJoin('e.card', 'c')
                ->where('mes.menu = :idMenu')
                ->andWhere('c.id  =:idCard')
                ->andWhere('mes.choice = :choice')
                ->andWhere('e.activeSite=1')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->setParameter('idCard', $idPdv)
                ->getQuery();
            $results = $query->getResult();
            foreach ($results as $key => $value) {
                $results[$key]['compositions'] = $this->getIngredientComposByEl2($value['element']);
                $results[$key]['supplements'] = $this->getIngredientSuppByEl2($value['element']);
                $results[$key]['optionsIngredient'] = $this->getOptionsIngredientByEl2($value['element']);
            }
            $elements[$i] = $results;
            $qb2 = $em->createQueryBuilder();
            $query = $qb2->select("mfs.id,cf.id as cardFamily,f.id as family,f.label,s.label as size,s.level,s.id as sizeId,mfs.quantity,mfs.choice")
                ->from("App:Menu", "m")
                ->leftJoin('m.families', 'mfs')
                ->leftJoin('mfs.cardFamily', 'cf')
                ->leftJoin('cf.family', 'f')
                ->leftJoin('mfs.size', 's')
                ->innerJoin('m.card', 'c')
                ->where('mfs.menu = :idMenu')
                ->andWhere('mfs.choice = :choice')
                ->andWhere('c.id  =:idCard')
                ->setParameter('idCard', $idPdv)
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            foreach ($results as $keyFam => $family) {
                $results[$keyFam]['elements'] = array();
                $elms = $this->getElementsByFamilyAndSize2($family['cardFamily'], $family['sizeId'], $id,$idOrderType);
                for ($j = 0; $j < $family['quantity']; $j++) {
                    $results[$keyFam]['elements'][$j] = $elms;
                }
            }
            $families[$i] = $results;
        }
        $data = array('elements' => $elements, 'families' => $families, 'additionalOptions' => $additionalOptions);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }
    
    /**
     * @Rest\Get("/website/menus/{id}/compositions")
     */
    public function getElementsSize($id)
    {
        $elements = array();
        $families = array();
        $additionalOptions = $this->getOptionAction($id);
        for ($i = 0; $i < 6; $i++) {
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $query = $qb->select("cf.id as family,f.label as familyName,es.id,es.priceSupp,es.price,es.available,e.id as element,e.hasSupp,e.avatar,e.priceSuppApp,e.name,e.maxNbrIngOptions,s.id as idSize,s.label as size,s.level,mes.quantity,mes.choice")
                ->from("App:Menu", "m")
                ->innerJoin('m.elements', 'mes')
                ->innerJoin('mes.elementSize', 'es')
                ->innerJoin('es.element', 'e')
                ->innerJoin('e.family', 'cf')
                ->innerJoin('cf.family', 'f')
                ->innerJoin('es.size', 's')
                ->where('mes.menu = :idMenu')
                ->andWhere('mes.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            foreach ($results as $key => $value) {
                $results[$key]['compositions'] = $this->getIngredientComposByEl2($value['element']);
                $results[$key]['supplements'] = $this->getIngredientSuppByEl2($value['element']);
                $results[$key]['optionsIngredient'] = $this->getOptionsIngredientByEl2($value['element']);
            }
            $elements[$i] = $results;
            $qb2 = $em->createQueryBuilder();
            $query = $qb2->select("mfs.id,cf.id as cardFamily,f.id as family,f.label,s.label as size,s.level,s.id as sizeId,mfs.quantity,mfs.choice")
                ->from("App:Menu", "m")
                ->leftJoin('m.families', 'mfs')
                ->leftJoin('mfs.cardFamily', 'cf')
                ->leftJoin('cf.family', 'f')
                ->leftJoin('mfs.size', 's')
                ->where('mfs.menu = :idMenu')
                ->andWhere('mfs.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            foreach ($results as $keyFam => $family) {
                $results[$keyFam]['elements'] = array();
                $elms = $this->getElementsByFamilyAndSize($family['cardFamily'], $family['sizeId'], $id);
                for ($j = 0; $j < $family['quantity']; $j++) {
                    $results[$keyFam]['elements'][$j] = $elms;
                }
            }
            $families[$i] = $results;
        }
        $data = array('elements' => $elements, 'families' => $families, 'additionalOptions' => $additionalOptions);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($data, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/paymentmodes")
     */
    public function getPaymentModeByPdv(Request $request)
    {
        $idPdv = $request->headers->get('pdv');
        $idPdv = $this->dec_enc('decrypt', $idPdv);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $modesPaimenents = $qb->select("pm.id, pm.label as text")
            ->from("App:Pdv", "p")
            ->innerJoin('p.paymentModes', 'pm')
            ->where('p.id = :pdv')
            ->setParameter('pdv', $idPdv)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($modesPaimenents, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/ordertypes")
     */
    public function getModeCmdByPdv(Request $request)
    {
        $idPdv = $request->headers->get('pdv');
        $idPdv = $this->dec_enc('decrypt', $idPdv);
        //$d = new \DateTime();
        //$midi = new \DateTime();
        //$midi->setTime(16, 0, 0);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        //if ($d > $midi) {
        $pdvs = $qb->select("t.id,t.label,tp.minDelay,tp.maxDelay");
        //} else {
        //$pdvs = $qb->select("t.id,t.label,tp.minDelay,tp.maxDelay,tp.active,case when av.middayEnd > :da and av.middayStart < :da then 'true' else 'false' end available");
        //}
        $pdvs = $pdvs
            ->from("App:Pdv", "P")
            ->innerJoin('P.orderTypes', 'tp')
            //->innerJoin('tp.availability', 'av')
            //->innerJoin('av.day', 'd')
            ->innerJoin('tp.orderType', 't')
            ->where('P.id = :pdv')
            ->andwhere('tp.active = 1')
            //andWhere('d.id=:day')
            ->setParameter('pdv', $idPdv)
            //->setParameter('da', $d->format('H:i:s'))
            //->setParameter('day', $d->format('N'))
            ->orderBy('t.level')
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($pdvs, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Post("/website/authorizedCity")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="postalCode", nullable=true,allowBlank=true,strict=true)
     */
    public function checkAuthorizedCity(Request $request, ParamFetcher $paramFetcher)
    {
        $idPdv = $request->headers->get('pdv');
        $pdv = $this->dec_enc('decrypt', $idPdv);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $cities = $qb->select("c.id,cp.amountMiniDelivery")
            ->from("App:CityFrancePdv", "cp")
            ->innerJoin('cp.pdv', 'p')
            ->innerJoin('cp.city', 'c')
            ->where('p.id = :pdv')
            ->andWhere('c.postalCode=:postalCode')
            ->setParameter('pdv', $pdv)
            ->setParameter('postalCode', $paramFetcher->get('postalCode'))
            ->getQuery()->getResult();
        if (count($cities) > 0) {
            return new JsonResponse(['authorized' => true, 'amountMiniDelivery' => $cities[0]['amountMiniDelivery']]);
        } else {
            return new JsonResponse(['authorized' => false]);
        }
    }

    /**
     * @Rest\Get("/website/authorizedCities")
     */
    public function getAuthorizedCities(Request $request)
    {
        $idPdv = $request->headers->get('pdv');
        $pdv = $this->dec_enc('decrypt', $idPdv);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $cities = $qb->select("c.name,c.postalCode,cp.amountMiniDelivery")
            ->from("App:CityFrancePdv", "cp")
            ->innerJoin('cp.pdv', 'p')
            ->innerJoin('cp.city', 'c')
            ->where('p.id=:pdv')
            ->setParameter('pdv', $pdv)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($cities, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/elements/{id}")
     */
    public function Action($id)
    {
        $element = $this->getDoctrine()->getRepository("App:Element")->find($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($element, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/order/elements/{id}")
     */
    public function getOrderElement($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $element = $qb->select("
            e.id,e.hasSupp,e.custom,e.code,e.name,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp,e.avatar")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->andWhere('e.activeSite=:active')
            ->andWhere('e.id=:id')
            ->setParameter('id', $id)
            ->setParameter('active', 1)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
        $element['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
        $element['compositions'] = $this->getDoctrine()->getRepository("App:Ingredient")->getSiteIngredientComposByEl($element['id'], null);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($element, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/ingredients/compositions/element/{idElement}")
     */
    public function getIngredientComposByEl($idElement)
    {
        $compositions = $this->getDoctrine()->getRepository("App:Ingredient")->getSiteIngredientComposByEl($idElement, null);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($compositions, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/ingredients/supplements/element/{idElement}")
     */
    public function getIngredientSuppByEl($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("ec.id,ec.avatar,ec.name as text,ec.price,ic.id as category,ic.name as categoryName,ec.available,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementSupplement', 'ec')
            ->innerJoin('ec.ingredientCategory', 'ic')
            ->where('e.id = :element')
            ->andWhere('ec.activeSite=1')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/ingredients/optionsIngredient/element/{idElement}")
     */
    public function getOptionsIngredientByEl($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $families = $qb->select("ec.id,ec.avatar,ec.name as text,ec.available,ic.id as category,ic.name as categoryName,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementOption', 'ec')
            ->innerJoin('ec.ingredientCategory', 'ic')
            ->where('e.id = :element')
            ->andWhere('ec.activeSite=1')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($families, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/elements/size/{id}")
     */
    public function getElementSize($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id,e.name,s.label as size,es.id as idSize,es.available,es.price,es.priceSupp,c.id as card")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('e.card', 'c')
            ->innerJoin('es.size', 's')
            ->where('es.id=:id')
            ->setParameter('id', $id)
            ->getQuery()->getResult();
        if (count($elements) == 1) {
            $serializer = $this->get('serializer');
            $data = $serializer->serialize($elements[0], 'json');
            return new JsonResponse(json_decode($data));
        } else {
            return new JsonResponse([], 204);
        }
    }

    /**
     * @Rest\Get("/website/elements/additionalOptions")
     */
    public function getAdditionalOptions(Request $request)
    {
        $idCard = $request->headers->get('pdv');
        $idCard = $this->dec_enc('decrypt', $idCard);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id as idElementSize,e.name,s.label as size")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('es.size', 's')
            ->innerJoin('e.card', 'c')
            ->where('c.id = :card')
            ->andWhere('e.additionalOption = :additionalOption')
            ->setParameter('card', $idCard)
            ->setParameter('additionalOption', 1)
            ->getQuery()->getResult();
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/elements/active/family/{idFamily}/orderType/{idOrderType}")
     */
    public function getRestaurantActiveElementsByFamilyCard($idFamily, $idOrderType)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("
            e.id,e.hasSupp,e.custom,e.code,e.name,eot.supplement,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp,e.avatar,0 as size")
            ->from("App:Element", "e")
            ->leftJoin('e.elementComposition', 'ec')
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->where('cf.id = :cardFamily')
            ->andWhere('e.active=1')
            ->andWhere('e.activeSite=1')
            ->andWhere('ot.id=:orderType')
            ->andWhere('eot.active=1')
            ->setParameter('cardFamily', $idFamily)
            ->setParameter('orderType', $idOrderType)
            ->groupBy('e.id')
            ->orderBy('e.name')
            ->getQuery()
            ->getResult();
        foreach ($elements as $key => $element) {
            $elements[$key]['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
            $qb = $em->createQueryBuilder();
            $elements[$key]['comp']=$this->getCompositionsElement($element['id']);
            $elements[$key]['size'] = 0;
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/elements/{id}/sizes")
     */
    public function getElementSizes($id)
    {
        $sizes = $this->getDoctrine()->getRepository("App:Size")->getListByElement($id);
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($sizes, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/elements/active/orderType/{idOrderType}/key/{key}")
     */
    public function getRestaurantActiveElementsByOrderAndKey(Request $request, $key, $idOrderType)
    {
        $idCarte = $request->headers->get('pdv');
        $idCarte = $this->dec_enc('decrypt', $idCarte);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("e.id,e.hasSupp,e.custom,e.code,e.name,eot.supplement,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp,e.avatar,0 as size")
            ->from("App:Element", "e")
            ->innerJoin('e.card', 'c')
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->where('e.name LIKE :key')
            ->andWhere('e.activeSite=:active')
            ->andWhere('ot.id=:orderType')
            ->andWhere('eot.active=:active')
            ->andWhere('c.id=:card')
            ->setParameter('key', '%' . $key . '%')
            ->setParameter('orderType', $idOrderType)
            ->setParameter('card', $idCarte)
            ->setParameter('active', 1)
            ->getQuery()
            ->getResult();
        $qb2 = $em->createQueryBuilder();
        $menus = $qb2->select("m.id,mot.supplement,m.avatar,m.code,m.vat,m.title,m.description,m.price,m.available,mt.id as menuTime,c.name as category,o.id as offer,
            case when mt.id = 1 then false else true end as validPeriod,mot.active as dispo");
        $menus = $menus->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->leftJoin('m.offer', 'co')
            ->leftJoin('co.offer', 'o')
            ->innerJoin('m.menuTime', 'mt')
            ->innerJoin('m.card', 'ca')
            ->innerJoin('m.menuOrderType', 'mot')
            ->innerJoin('mot.orderType', 'ot')
            ->where('ca.id = :card')
            ->andWhere('ot.id=:orderType')
            ->andWhere('m.available=1')
            ->andWhere('m.title LIKE :key or m.code LIKE :key')
            ->setParameter('card', $idCarte)
            ->setParameter('orderType', $idOrderType)
            ->setParameter('key', '%' . $key . '%')
            ->groupBy('m.id')
            ->getQuery()->getResult();
        foreach ($menus as $key => $menu){
            $menus[$key]['compositions']=$this->getCompositionsMenu($menu['id']);
        }
        foreach ($elements as $key => $element) {
        	$elements[$key]['comp'] = $this->getCompositionsElement($element['id']);
            $elements[$key]['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
        }
        $da['elements'] = $elements;
        $da['menus'] = $menus;
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($da, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Post("/website/elements/search/compositions")
     * @RequestParam(name="ingredients", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="idOrderType", nullable=false,allowBlank=false,strict=true)
     */
    public function getElementsSiteByComposition(ParamFetcher $paramFetcher)
    {
        $em = $this->getDoctrine()->getManager();
        $result = new ArrayCollection();
        $ingredients = $paramFetcher->get('ingredients');
        $idOrderType = intval($paramFetcher->get('idOrderType'));
        foreach ($ingredients as $key1 => $ing) {
            $qb = $em->createQueryBuilder();
            $elements = $qb->select("e.id,e.hasSupp,e.avatar,e.code,e.name,eot.supplement,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp,0 as size")
            	->from("App:Element", "e")
           		->leftJoin('e.elementComposition', 'ec')
                ->innerJoin('e.family', 'cf')
                ->innerJoin('cf.family', 'f')
                ->innerJoin('e.elementOrderType', 'eot')
                ->innerJoin('eot.orderType', 'ot')
                ->where('e.activeSite=:active')
                ->andWhere('ot.id=:orderType')
                ->andWhere('eot.active=:active')
                ->andWhere('ec.id=:ing')
                ->setParameter('orderType', $idOrderType)
                ->setParameter('active', 1)
                ->setParameter('ing', $ing)
                ->groupBy('e.id')
                ->getQuery()->getResult();
            foreach ($elements as $key => $element) {
                $elements[$key]['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
                $qb = $em->createQueryBuilder();
                $elements[$key]['comp']=$this->getCompositionsElement($element['id']);
            }
            if ($key1 == 0) {
                $result = $elements;
            } else {
                $result = array_intersect(array_map('serialize', $result), array_map('serialize', $elements));
                $result = array_map('unserialize', $result);
            }
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize(array_values($result), 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Get("/website/elements/bestSales/orderType/{idOrderType}")
     */
    public function getRestaurantBestSales(Request $request, $idOrderType)
    {
        $idPdv = $request->headers->get('pdv');
        $idPdv = $this->dec_enc('decrypt', $idPdv);
        $em = $this->getDoctrine()->getManager();
        $elements = $this->getBestSalles(true, $idOrderType, $idPdv);
        foreach ($elements as $key => $element) {
            $elements[$key]['sizes'] = $this->getDoctrine()->getRepository("App:Size")->getListByElement($element['id']);
            $qb = $em->createQueryBuilder();
            $elements[$key]['comp']=$this->getCompositionsElement($element['id']);
        }
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($elements, 'json');
        return new JsonResponse(json_decode($data));
    }

    /**
     * @Rest\Post("/website/login")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="email", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="password", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=true,allowBlank=true,strict=false)
     */
    public function login(ParamFetcher $paramFetcher,Request $request)
    {
        $password = $this->dec_enc('encrypt', $paramFetcher->get('password'));
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        if($paramFetcher->get('pdv')){
             $idPdv = $this->dec_enc('decrypt', $paramFetcher->get('pdv'));
        }else{
             $idPdv = $this->dec_enc('decrypt', $request->headers->get('pdv'));
        }
        $user = $qb->select("u.id,cp.id as customer,u.email,u.enabled")
            ->from("App:webSiteUsers", "u")
            ->innerJoin('u.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->innerJoin('cp.pdv', 'p')
            ->where('u.email = :email')
            ->andWhere('u.password = :password')
            ->andWhere('p.id = :pdv')
            ->andWhere('c.indicator = :indicator')
            ->setParameter('email', $paramFetcher->get('email'))
            ->setParameter('pdv', $idPdv)
            ->setParameter('password', $password)
            ->setParameter('indicator', 'S')
            ->getQuery()->getResult(Query::HYDRATE_ARRAY);
        if (count($user) > 0) {
            if($user[0]['enabled']==0){
                 return new JsonResponse("user disabled", 400);
            }
            $jwt = JWT::encode($user[0], 'devfood');
            return new JsonResponse([
                'token' => $jwt,
                'success' => true,
                'code' => 1
            ]);
        } else {
            return new JsonResponse([], 400);
        }
    }

    /**
     * @Rest\Post("/website/orders")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="amount", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="comment", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="orderProg", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="orderType", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="elements", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="menus", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="idAddress", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="payementModes", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="token", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="platform", nullable=true,allowBlank=true,strict=true)
     */
    public function addOrder(Request $request, ParamFetcher $paramFetcher, SendEvent $sendEvent)
    {
        $token = $request->headers->get('token');
        $idPdv = $request->headers->get('pdv');
        $pdv = $this->dec_enc('decrypt', $idPdv);
        if ($token) {
            try {
                $token = JWT::decode($token, 'devfood', array('HS256'));
                $customer = $token->customer;
                $id = $token->id;
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $user = $qb->select("u")
                    ->from("App:webSiteUsers", "u")
                    ->innerJoin('u.customer', 'cp')
                    ->innerJoin('cp.pdv', 'p')
                    ->innerJoin('cp.customer', 'c')
                    ->where('u.id = :user')
                    ->andWhere('p.id = :pdv')
                    ->setParameter('user', $id)
                    ->setParameter('pdv', $pdv)
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getOneOrNullResult();
                if ($user) {
                    if ($user->getConfirmed() == 0) {
                        return new JsonResponse(['success' => false, 'code' => 3, "compte n'est pas confirmé"], 200);
                    }
                    if ($user->getEnabled() == 0) {
                         return new JsonResponse(['success' => false, 'code' => 4, "compte désactivé"], 200);
                    }
                } else {
                    return new JsonResponse('', 401);
                }
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
        $pdv = $this->dec_enc('decrypt', $paramFetcher->get('pdv'));
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $closeSite = $qb->select("p.closeSite")
            ->from("App:Pdv", "p")
            ->where('p.id=:pdv')
            ->setParameter('pdv', $pdv)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
        if ($closeSite) {
            //var_dump($closeSite['closeSite']==1);
            if ($closeSite['closeSite'] == 1) {
                return new JsonResponse(["code" => 2, 'success' => false, "message" => "restaurant close"], 200);
            }
        } else {
            return new JsonResponse([], 400);
        }
        $order = new Command();
        $order->setAmount($paramFetcher->get('amount'));
        $order->setComment($paramFetcher->get('comment'));
        $order->setCorrected(0);
        $date = \DateTime::createFromFormat('D M d Y H:i:s e+', $paramFetcher->get('orderProg'));
        $order->setDiscount(0);
        $order->setOrderProg($date);
        $ip = $request->getClientIp();
        $order->setCreatedIp($ip);
        //$order->setCreatedUser($this->getUser());
        $order->setUpdatedIp('');
        $order->setUpdatedUser(null);
        $statut = $em->getRepository('App:Status')->find(8);
        $customerPdv = $em->getRepository('App:CustomerPdv')->find($customer);
        $user = $em->getRepository('App:User')->find(58);
        $order->setCreatedUser($user);
        $order->setCustomer($customerPdv);
        $order->setStatus($statut);
        $orderPdv = $this->getOrderPdvByPdvAndOrder($pdv, $paramFetcher->get('orderType'));
        $order->setOrderTypePdv($orderPdv);
        if ($paramFetcher->get('orderType') == 1) {
            $idAddress = $paramFetcher->get('idAddress');
            $address = $em->getRepository('App:Address')->find($idAddress);
            $order->setAddress($address);
        }
        $elements = $paramFetcher->get('elements');
        $menus = $paramFetcher->get('menus');
        if ((is_array($elements) && count($elements) > 0) || (is_array($menus) && count($menus) > 0)) {
            if (count($elements) > 0) {
                foreach ($elements as $value) {
                    if (
                        array_key_exists('price', $value['size']) &&
                        array_key_exists('size', $value['size']) &&
                        array_key_exists('name', $value) &&
                        array_key_exists('qty', $value) &&
                        array_key_exists('comment', $value) &&
                        array_key_exists('total', $value) &&
                        array_key_exists('vat', $value) &&
                        array_key_exists('additionalCost', $value)
                    ) {
                        $commandElement = new CommandElement();
                        $commandElement->setName($value['name']);
                        $commandElement->setQuantity($value['qty']);
                        $commandElement->setComment($value['comment']);
                        $commandElement->setTotal($value['total']);
                        $commandElement->setPrice($value['size']['price']);
                        $commandElement->setVat($value['vat']);
                        $commandElement->setSize($value['size']['size']);
                        if (array_key_exists('level', $value['size'])) {
                            $commandElement->setLevel($value['size']['level']);
                        }
                        $commandElement->setAdditionalCost($value['additionalCost']);
                        $commandElement->setCommand($order);
                        if (array_key_exists('id', $value['size'])) {
                            $elementSize = intval($value['size']['id']);
                            $commandElement->setElementSize($elementSize);
                        }
                        $commandElement->setCode($value['code']);
                        $em->persist($commandElement);
                        if (array_key_exists('removedCompositions', $value)) {
                            foreach ($value['removedCompositions'] as $val) {
                                $commandElementIngR = new CommandElementIngredients();
                                $ingredient = intval($val['id']);
                                $commandElementIngR->setIngredient($ingredient);
                                $commandElementIngR->setName($val['text']);
                                $commandElementIngR->setCout(0);
                                $commandElementIngR->setAction('remove');
                                $commandElementIngR->setCommandElement($commandElement);
                                $em->persist($commandElementIngR);
                            }
                        }
                        if (array_key_exists('supplements', $value)) {
                            foreach ($value['supplements'] as $va) {
                                $commandElementIngS = new CommandElementIngredients();
                                $ingredient = intval($va['id']);
                                $commandElementIngS->setIngredient($ingredient);
                                $commandElementIngS->setName($va['label']);
                                $commandElementIngS->setCout($va['price']);
                                $commandElementIngS->setAction('supp');
                                $commandElementIngS->setCommandElement($commandElement);
                                $em->persist($commandElementIngS);
                            }
                        }
                        if (array_key_exists('optionIngredients', $value)) {
                            foreach ($value['optionIngredients'] as $v) {
                                $commandElementIngO = new CommandElementIngredients();
                                $ingredient = intval($v['id']);
                                $commandElementIngO->setIngredient($ingredient);
                                $commandElementIngO->setName($v['label']);
                                $commandElementIngO->setCout(0);
                                $commandElementIngO->setAction('opt');
                                $commandElementIngO->setCommandElement($commandElement);
                                $em->persist($commandElementIngO);
                            }
                        }

                    } else {
                        return new JsonResponse(['error' => 'One of this fileds (size(size,price),name,qty,comment,total,vat,additionalCost) are messing'], 400);
                    }
                }
            }
            if (count($menus) > 0) {
                foreach ($menus as $menu) {
                    if (
                        array_key_exists('price', $menu) &&
                        array_key_exists('comment', $menu) &&
                        array_key_exists('qty', $menu) &&
                        array_key_exists('title', $menu) &&
                        array_key_exists('total', $menu) &&
                        array_key_exists('additionalCost', $menu) &&
                        array_key_exists('vat', $menu) &&
                        array_key_exists('elements', $menu) &&
                        is_array($menu['elements']) &&
                        count($menu['elements']) > 0
                    ) {
                        $commandMenu = new CommandMenu();
                        $commandMenu->setTitle($menu['title']);
                        $commandMenu->setQuantity($menu['qty']);
                        $commandMenu->setComment($menu['comment']);
                        $commandMenu->setTotal($menu['total']);
                        $commandMenu->setPrice($menu['price']);
                        $commandMenu->setVat($menu['vat']);
                        $commandMenu->setAdditionalCost($menu['additionalCost']);
                        $commandMenu->setCommand($order);
                        $commandMenu->setCode($menu['code']);
                        $m = intval($menu['id']);
                        $commandMenu->setMenu($m);
                        $em->persist($commandMenu);
                        foreach ($menu['elements'] as $element) {
                            if (
                                array_key_exists('price', $element) &&
                                array_key_exists('name', $element) &&
                                array_key_exists('qty', $element) &&
                                array_key_exists('comment', $element) &&
                                array_key_exists('label', $element['size'])
                            ) {
                                $commandElement = new CommandElement();
                                $commandElement->setName($element['name']);
                                $commandElement->setQuantity($element['qty']);
                                $commandElement->setComment($element['comment']);
                                $commandElement->setTotal(0);
                                $commandElement->setPrice(0);
                                $commandElement->setVat(0);
                                $commandElement->setSize($element['size']['label']);
                                $commandElement->setLevel($element['size']['level']);
                                $commandElement->setAdditionalCost(0);
                                if (array_key_exists('id', $element['size'])) {
                                    $elementSize = intval($element['size']['id']);
                                    $commandElement->setElementSize($elementSize);
                                }
                                $commandElement->setCommandMenu($commandMenu);
                                $commandElement->setCommand($order);
                                $em->persist($commandElement);
                                /*if (array_key_exists('compositions', $element)) {
                                    foreach ($element['compositions'] as $valu) {
                                        $commandElementIngC = new CommandElementIngredients();
                                        $ingredient = $em->getRepository('App:Ingredient')->find($valu['id']);
                                        $commandElementIngC->setIngredient($ingredient);
                                        $commandElementIngC->setName($valu['text']);
                                        $commandElementIngC->setCout(0);
                                        $commandElementIngC->setAction('comp');
                                        $commandElementIngC->setCommandElement($commandElement);
                                        $em->persist($commandElementIngC);
                                    }
                                }*/
                                if (array_key_exists('removedCompositions', $element)) {
                                    foreach ($element['removedCompositions'] as $val) {
                                        $commandElementIngR = new CommandElementIngredients();
                                        $ingredient = intval($val['id']);
                                        $commandElementIngR->setIngredient($ingredient);
                                        $commandElementIngR->setName($val['text']);
                                        $commandElementIngR->setCout(0);
                                        $commandElementIngR->setAction('remove');
                                        $commandElementIngR->setCommandElement($commandElement);
                                        $em->persist($commandElementIngR);
                                    }
                                }
                                if (array_key_exists('supplements', $element)) {
                                    foreach ($element['supplements'] as $va) {
                                        $commandElementIngS = new CommandElementIngredients();
                                        $ingredient = intval($va['id']);
                                        $commandElementIngS->setIngredient($ingredient);
                                        $commandElementIngS->setName($va['label']);
                                        $commandElementIngS->setCout($va['price']);
                                        $commandElementIngS->setAction('supp');
                                        $commandElementIngS->setCommandElement($commandElement);
                                        $em->persist($commandElementIngS);
                                    }
                                }
                                if (array_key_exists('optionIngredients', $element)) {
                                    foreach ($element['optionIngredients'] as $v) {
                                        $commandElementIngO = new CommandElementIngredients();
                                        $ingredient = intval($v['id']);
                                        $commandElementIngO->setIngredient($ingredient);
                                        $commandElementIngO->setName($v['text']);
                                        $commandElementIngO->setCout(0);
                                        $commandElementIngO->setAction('opt');
                                        $commandElementIngO->setCommandElement($commandElement);
                                        $em->persist($commandElementIngO);
                                    }
                                }
                            } else {
                                return new JsonResponse(['error' => 2], 400);
                            }
                        }
                        if (
                            array_key_exists('additionalOptions', $menu) &&
                            is_array($menu['additionalOptions']) &&
                            count($menu['additionalOptions']) > 0
                        ) {
                            foreach ($menu['additionalOptions'] as $element) {
                                if (
                                    array_key_exists('price', $element) &&
                                    array_key_exists('name', $element) &&
                                    array_key_exists('qty', $element) &&
                                    array_key_exists('size', $element)
                                ) {
                                    $commandElement = new CommandElement();
                                    $commandElement->setName($element['name']);
                                    $commandElement->setQuantity($element['qty']);
                                    $commandElement->setComment('');
                                    $commandElement->setTotal(0);
                                    $commandElement->setPrice($element['price']);
                                    $commandElement->setVat(0);
                                    $commandElement->setSize($element['size']);
                                    //$commandElement->setLevel($element['level']);
                                    $commandElement->setAdditionalCost(0);
                                    $commandElement->setCommandMenu($commandMenu);
                                    $commandElement->setCommand($order);
                                    $em->persist($commandElement);
                                }
                            }
                        }
                    }
                }
            }
        } else {
            return new JsonResponse(['error' => 3], 400);
        }
        $payementModes = $paramFetcher->get('payementModes');
        if (is_array($payementModes) && count($payementModes) > 0) {
            foreach ($payementModes as $val) {
                if (array_key_exists('id', $val)) {
                    $payementMode = $em->getRepository('App:PaymentMode')->find($val['id']);
                    $commandPaymentMode = new CommandPaymentMode();
                    if ($payementMode) {
                        $commandPaymentMode->setPaymentMode($payementMode);
                        $commandPaymentMode->setCommand($order);
                        $commandPaymentMode->setAmount(0);
                        $em->persist($commandPaymentMode);
                    } else {
                        return new JsonResponse([], 400);
                    }
                } else {
                    return new JsonResponse(['error' => 'incorrect format of payement mode (id,label)'], 400);
                }
            }
        } else {
            return new JsonResponse(['error' => 'no payement mode selected'], 400);
        }
        $now = new \DateTime();
        $res = $qb->select("co.num as num")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'otp')
            ->innerJoin('otp.pdv', 'pdv')
            ->where('co.createdAt > :date_start')
            ->andWhere('co.createdAt < :date_end')
            ->andWhere('pdv.id=:pdv')
            ->setParameter('date_start', $now->format('Y-m-d 00:00:00'))
            ->setParameter('date_end', $now->format('Y-m-d 23:59:59'))
            ->setParameter('pdv', $pdv)
            ->setMaxResults(1)
            ->orderBy('co.id', 'DESC')
            ->getQuery()
            ->getOneOrNullResult();
        if (!is_null($res)) {
            $order->setNum(intval($res['num']) + 1);
        } else {
            $order->setNum(1);
        }
        $platforms=[1,2,3,4];
        $pl=intval($paramFetcher->get('platform'));
        $mobile=2;
        if(in_array($pl,$platforms)){
            $mobile=$pl;
        }else{
            $mobile=($this->isMobile())?3:2;
        }
        $platform = $em->getRepository('App:Platform')->find($mobile);
        $order->setPlatform($platform);
        if($pdv==80 && $order->getAmount()>=20){
            $order->setDiscount(10);
        }
        $em->persist($order);
        $em->flush();
        $sendEvent->sendMessage(['type' => 'new', 'entity' => 'order', 'id' => $order->getId(), 'pdv' => $pdv,'from'=>'web']);
        $result = [
            'id' => $order->getId(),
            'success' => true,
            'code' => 1
        ];
        $serializer = $this->get('serializer');
        $data = $serializer->serialize($result, 'json');
        $email = $token->email;
        $idC = $token->customer;
        /*$this->sentOrderParEmail($order->getId(),$email,$idC);*/
        $response = new JsonResponse(json_decode($data));
        $response->setCallback($this->sentOrderParEmail($order->getId(), $email, $idC));
        return $response;
    }

    /**
     * @Rest\Get("/website/orders")
     */
    public function getUserOrders(Request $request)
    {
        $token = $request->headers->get('token');
        $idPdv = $request->headers->get('pdv');
        $pdv = $this->dec_enc('decrypt', $idPdv);
        if ($token) {
            try {
                $em = $this->getDoctrine()->getManager();
                $id = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $qb = $em->createQueryBuilder();
                $orders = $qb->select("co.id,s.id as idStatus,co.num,co.discount,co.corrected,co.amount,t.label as type,s.label as status,mop.minDelay,co.createdAt,p.id as pdvId,p.name as pdvName,c.id as customerId,c.firstname as customerFirstname,c.lastname as customerLastname,c.phone as customerPhone")
                    ->from("App:Command", "co")
                    ->innerJoin('co.orderTypePdv', 'mop')
                    ->innerJoin('mop.orderType', 't')
                    ->innerJoin('mop.pdv', 'p')
                    ->innerJoin('co.customer', 'cp')
                    ->innerJoin('cp.customer', 'c')
                    ->innerJoin('co.status', 's')
                    ->where('cp.id=:cust')
                    ->andWhere('c.indicator=:ind')
                    ->andWhere('p.id=:pdv')
                    ->setParameter('cust', $id)
                    ->setParameter('ind', 'S')
                    ->setParameter('pdv', $pdv)
                    ->getQuery()->getResult();
                foreach ($orders as $key => $order) {
                    $ord = $em->getRepository('App:Command')->find($order['id']);
                    $orders[$key]['PaymentModes'] = $ord->getCommandPaymentModes();
                }
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($orders, 'json');
                return new JsonResponse(json_decode($data));
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Get("/website/orders/{id}/info")
     */
    public function getInfoOrder(Request $request, $id)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $idC = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $order = $this->getOrderInfo($id, $idC);
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($order, 'json');
                return new JsonResponse(json_decode($data));
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Get("/website/orders/{id}/elements")
     */
    public function getOrderElements(Request $request, $id)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $idC = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $elements = $this->getElementsOfOrder($id, $idC);
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($elements, 'json');
                return new JsonResponse(json_decode($data));
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Get("/website/orders/{id}/menus")
     */
    public function getOrderMenus(Request $request, $id)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $idC = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $menus = $this->getMenusOfOrder($id, $idC);
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($menus, 'json');
                return new JsonResponse(json_decode($data));
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Get("/website/orders/{id}/paymentModes")
     */
    public function getOrderPaymentModes(Request $request, $id)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $idC = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $paymentModes = $this->getPyementsModeOfOdrder($id, $idC);
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($paymentModes, 'json');
                return new JsonResponse(json_decode($data));
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Get("/website/user/info")
     */
    public function getInfo(Request $request)
    {
        $token = $request->headers->get('token');
        $idPdv = $request->headers->get('pdv');
        $pdv = $this->dec_enc('decrypt', $idPdv);
        if ($token) {
            try {
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $idUser = JWT::decode($token, 'devfood', array('HS256'))->id;
                $user = $qb->select("c.email, c.firstname, c.lastname, c.phone")
                    ->from("App:webSiteUsers", "u")
                    ->innerJoin('u.customer', 'cp')
                    ->innerJoin('cp.pdv', 'p')
                    ->innerJoin('cp.customer', 'c')
                    ->where('u.id = :user')
                    ->andWhere('p.id = :pdv')
                    ->setParameter('user', $idUser)
                    ->setParameter('pdv', $pdv)
                    ->getQuery()->getResult();
                if ($user) {
                    $serializer = $this->get('serializer');
                    $data = $serializer->serialize($user, 'json');
                    return new JsonResponse(json_decode($data));
                } else {
                    return new JsonResponse('', 401);
                }

            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Get("/website/address")
     */
    public function getAddress(Request $request)
    {
        $token = $request->headers->get('token');
        $idPdv = $request->headers->get('pdv');
        $pdv = $this->dec_enc('decrypt', $idPdv);
        if ($token) {
            try {
                $em = $this->getDoctrine()->getManager();
                $qb1 = $em->createQueryBuilder();
                $customer = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $address = $qb1->select("a")
                    ->from("App:Address", "a")
                    ->innerJoin('a.customer', 'cp')
                    ->innerJoin('cp.pdv', 'pdv')
                    ->where('cp.id = :customer')
                    ->andWhere('a.active = 1')
                    ->andWhere('pdv.id=:pdv')
                    ->setParameter('customer', $customer)
                    ->setParameter('pdv', $pdv)
                    ->getQuery()->getResult();
                $serializer = $this->get('serializer');
                $data = $serializer->serialize($address, 'json');
                return new JsonResponse(json_decode($data));
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);

        }
    }

    /**
     * @Rest\Post("/website/address/add")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="streetName", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="streetNumber", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="postalCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="city", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="complementAddress", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="address", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="inputCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="building", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="staircase", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="floor", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="intercom", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="door", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="country", nullable=true,allowBlank=true,strict=true)
     */
    public function addAddress(Request $request, ParamFetcher $paramFetcher)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $em = $this->getDoctrine()->getManager();
                $idUser = JWT::decode($token, 'devfood', array('HS256'))->id;
                $user = $this->getDoctrine()->getRepository("App:webSiteUsers")->find($idUser);
                $address = new Address();
                $address->setStreetName($paramFetcher->get('streetName'));
                $address->setStreetNumber($paramFetcher->get('streetNumber'));
                $address->setPostalCode($paramFetcher->get('postalCode'));
                $address->setCity($paramFetcher->get('city'));
                $address->setComplementAddress($paramFetcher->get('complementAddress'));
                $address->setInputCode($paramFetcher->get('inputCode'));
                $address->setBuilding($paramFetcher->get('building'));
                $address->setStaircase($paramFetcher->get('staircase'));
                $address->setFloor($paramFetcher->get('floor'));
                $address->setIntercom($paramFetcher->get('intercom'));
                $address->setDoor($paramFetcher->get('door'));
                $address->setCountry($paramFetcher->get('country'));
                $address->setCustomer($user->getCustomer());
                $address->setActive(1);
                $em->persist($address);
                $em->flush();
                return new JsonResponse([
                    'id' => $address->getId(),
                    'success' => true,
                    'code' => 200
                ]);
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Delete("/website/address/{id}")
     */
    public function deleteAddress(Request $request, $id)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $customer = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $address = $qb->select("a")
                    ->from("App:Address", "a")
                    ->innerJoin('a.customer', 'cp')
                    ->where('cp.id = :customer')
                    ->andWhere('a.id = :id')
                    ->setParameter('customer', $customer)
                    ->setParameter('id', $id)
                    ->setMaxResults(1)
                    ->getQuery()->getOneOrNullResult();
                if (!is_null($address)) {
                    $qb = $em->createQueryBuilder();
                    $orders = $qb->select("count(a) as number")
                        ->from("App:Command", "c")
                        ->innerJoin('c.address', 'a')
                        ->where('a.id = :id')
                        ->setParameter('id', $id)
                        ->setMaxResults(1)
                        ->getQuery()->getOneOrNullResult();
                    if ($orders['number'] == 0) {
                        $em->remove($address);
                    } else {
                        $address->setActive(0);
                        $em->persist($address);
                    }
                    $em->flush();
                    return new JsonResponse([
                        'success' => true,
                        'code' => 200
                    ]);
                }
                return new JsonResponse('', 403);
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Put("/website/address/update/{id}")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="streetName", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="streetNumber", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="postalCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="city", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="complementAddress", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="address", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="inputCode", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="building", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="staircase", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="floor", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="intercom", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="door", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="country", nullable=true,allowBlank=true,strict=true)
     */
    public function updateAddress(Request $request, ParamFetcher $paramFetcher, $id)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $customer = JWT::decode($token, 'devfood', array('HS256'))->customer;
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $address = $qb->select("a")
                    ->from("App:Address", "a")
                    ->innerJoin('a.customer', 'cp')
                    ->where('cp.id = :customer')
                    ->andWhere('a.id = :id')
                    ->setParameter('customer', $customer)
                    ->setParameter('id', intval($id))
                    ->setMaxResults(1)
                    ->getQuery()->getOneOrNullResult();
                if (!is_null($address)) {
                    $address->setStreetName($paramFetcher->get('streetName'));
                    $address->setStreetNumber($paramFetcher->get('streetNumber'));
                    $address->setPostalCode($paramFetcher->get('postalCode'));
                    $address->setCity($paramFetcher->get('city'));
                    $address->setComplementAddress($paramFetcher->get('complementAddress'));
                    $address->setInputCode($paramFetcher->get('inputCode'));
                    $address->setBuilding($paramFetcher->get('building'));
                    $address->setStaircase($paramFetcher->get('staircase'));
                    $address->setFloor($paramFetcher->get('floor'));
                    $address->setIntercom($paramFetcher->get('intercom'));
                    $address->setDoor($paramFetcher->get('door'));
                    $address->setCountry($paramFetcher->get('country'));
                    $em->persist($address);
                    $em->flush();
                    return new JsonResponse([
                        'success' => true,
                        'code' => 200
                    ]);
                }
                return new JsonResponse('', 403);
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);

        }
    }

    /**
     * @Rest\Post("/website/websiteuser")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="lastname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="phone", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="password", requirements={@Assert\Length(min=8)})
     */
    public function addCustomer(Request $request, ParamFetcher $paramFetcher)
    {
        $idPdv = $this->dec_enc('decrypt', $paramFetcher->get('pdv'));
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $user = $qb->select("u")
            ->from("App:webSiteUsers", "u")
            ->InnerJoin('u.customer', 'cp')
            ->InnerJoin('cp.pdv', 'p')
            ->where('u.email = :email')
            ->andWhere('p.id=:pdv')
            ->setParameter('email', $paramFetcher->get('email'))
            ->setParameter('pdv', $idPdv)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if (is_null($user)) {
            $pdv = $this->getDoctrine()->getRepository("App:Pdv")->find($idPdv);
            $customer = new Customer();
            $customer->setLastname($paramFetcher->get('lastname'));
            $customer->setSociety('');
            $customer->setPhone($paramFetcher->get('phone'));
            $customer->setSecondaryPhone('');
            $customer->setEmail($paramFetcher->get('email'));
            $customer->setSmsAuthorized(0);
            $ip = $request->getClientIp();
            $customer->setCreatedIp($ip);
            $customer->setUpdatedIp('');
            $customer->setCreatedUser(null);
            $customer->setUpdatedUser(null);
            $customer->setIndicator('S');
            $em->persist($customer);
            $customerPdv = new CustomerPdv();
            $customerPdv->setCustomer($customer);
            $customerPdv->setPdv($pdv);
            $customerPdv->setComment('');
            $em->persist($customerPdv);
            $password = $paramFetcher->get('password');
            $newPass = $this->dec_enc('encrypt', $password);
            $em = $this->getDoctrine()->getManager();
            $user = new webSiteUsers();
            $key = md5(microtime(TRUE) * 100000);
            $user->setConfirmationKey($key);
            $user->setEmail($paramFetcher->get('email'));
            $user->setPassword($newPass);
            $user->setCustomer($customerPdv);
            $user->setConfirmed(0);
            $user->setEnabled(1);
            $em->persist($user);
            $em->flush();
            $qb = $em->createQueryBuilder();
            $data = $qb->select("p.nameRestaurant as name,p.urlSite as website,p.smtpemail,p.smtppassword,p.smtptype")
                ->from("App:Pdv", "p")
                ->where('p.id = :idPdv')
                ->setParameter('idPdv', $idPdv)
                ->setMaxResults(1)
                ->getQuery()->getOneOrNullResult();
            $data['lastname'] = $paramFetcher->get('lastname');
            $data['firstname'] = "";
            $data['phone'] = $pdv->getPhone();
            $data['emailContact'] = $pdv->getEmailContact();
            $data['address'] = $pdv->getAddress();
            $data['password'] = $password;
            $data['color'] = '#000';
            $data['restaurant'] = $data['name'];
            $data['link'] = $data['website'] . "/compte/confirme/user/" . $user->getId() . "/key/" . $key;
            $email = $paramFetcher->get('email');
            $data['email'] = $email;
            $subject = 'Pour finaliser votre inscription, merci de valider votre adresse mail';
            $template = $this->renderView('confirmation.html.twig', $data);
            $restaurant = $data['name'];
            $smtpemail = $data['smtpemail'];
            $smtppassword = $data['smtppassword'];
            $type = $data['smtptype'];
            $this->sendEmail($smtpemail, $smtppassword, $email, $subject, $template, $restaurant, $type);
            $u = Array('id' => $user->getId(), 'customer' => $customerPdv->getId(), 'email' => $email);
            $jwt = JWT::encode($u, 'devfood');
            return new JsonResponse([
                'token' => $jwt,
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse([], 409);
        }
    }

    /**
     * @Rest\Put("/website/info/update")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="firstname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="lastname", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="phone", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="email", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="oldPassword", nullable=true,allowBlank=true,strict=true)
     */
    public function updateInfo(Request $request, ParamFetcher $paramFetcher)
    {
        $token = $request->headers->get('token');
        if ($token) {
            try {
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $id = JWT::decode($token, 'devfood', array('HS256'))->id;
                $password = $this->dec_enc('encrypt', $paramFetcher->get('oldPassword'));
                $user = $qb->select("c.id as customer")
                    ->from("App:webSiteUsers", "u")
                    ->innerJoin('u.customer', 'cp')
                    ->innerJoin('cp.customer', 'c')
                    ->where('u.id = :user')
                    ->andWhere('u.password=:password')
                    ->setParameter('user', $id)
                    ->setParameter('password', $password)
                    ->getQuery()->getResult(Query::HYDRATE_ARRAY);
                if (count($user) > 0) {
                    $customer = $this->getDoctrine()->getRepository("App:Customer")->find($user[0]['customer']);
                    $customer->setFirstname($paramFetcher->get('firstname'));
                    $customer->setLastname($paramFetcher->get('lastname'));
                    $customer->setPhone($paramFetcher->get('phone'));
                    $customer->setEmail($paramFetcher->get('email'));
                    $ip = $request->getClientIp();
                    $customer->setUpdatedIp($ip);
                    $em = $this->getDoctrine()->getManager();
                    $em->persist($customer);
                    $em->flush();
                    return new JsonResponse([
                        'success' => true,
                        'code' => 200
                    ]);
                } else {
                    return new JsonResponse(['worng password'], 400);
                }
            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Put("/website/updatePassword")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="password", nullable=true,allowBlank=true,strict=true)
     * @RequestParam(name="oldPassword", nullable=true,allowBlank=true,strict=true)
     */
    public function updatePassword(Request $request, ParamFetcher $paramFetcher)
    {

        $token = $request->headers->get('token');
        if ($token) {

            try {
                $em = $this->getDoctrine()->getManager();
                $qb = $em->createQueryBuilder();
                $password = $this->dec_enc('encrypt', $paramFetcher->get('oldPassword'));
                $id = JWT::decode($token, 'devfood', array('HS256'))->id;
                $user = $qb->select("u")
                    ->from("App:webSiteUsers", "u")
                    ->where('u.id = :user')
                    ->andWhere('u.password=:password')
                    ->setParameter('user', $id)
                    ->setParameter('password', $password)
                    ->getQuery()->getResult(Query::HYDRATE_ARRAY);
                if (count($user) > 0) {
                    $user = $this->getDoctrine()->getRepository("App:webSiteUsers")->find($id);
                    $newPassword = $this->dec_enc('encrypt', $paramFetcher->get('password'));
                    $user->setPassword($newPassword);
                    $em->persist($user);
                    $em->flush();
                    return new JsonResponse();
                } else {
                    return new JsonResponse([], 400);
                }

            } catch (\Firebase\JWT\SignatureInvalidException $e) {
                return new JsonResponse([], 401);
            }
        } else {
            return new JsonResponse([], 401);
        }
    }

    /**
     * @Rest\Get("/website/forgetPassword")
     */
    public function forgetPassword(Request $request)
    {
        $email = $request->query->get('email');
        $pdv = $request->headers->get('pdv');
        $idPdv = $this->dec_enc('decrypt', $pdv);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $data = $qb->select("c.firstname,c.lastname,u.password,p.smtptype,p.smtpemail,p.smtppassword,p.nameRestaurant as name,p.emailContact,p.phoneContact as phone,p.address,p.urlSite as website")
            ->from("App:webSiteUsers", "u")
            ->innerJoin("u.customer", "cp")
            ->innerJoin("cp.customer", "c")
            ->innerJoin("cp.pdv", "p")
            ->innerJoin("p.restaurant", "r")
            ->where('u.email = :email')
            ->andwhere('p.id = :idPdv')
            ->setParameter('email', $email)
            ->setParameter('idPdv', $idPdv)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if (!is_null($data)) {
            $data['password'] = $this->dec_enc('decrypt', $data['password']);
            $data['color'] = '#000';
            $restaurant = $data['name'];
            $subject = 'Récupération de votre mot de passe';
            $template = $this->renderView('forgetPassword.html.twig', $data);
            $smtpemail = $data['smtpemail'];
            $smtppassword = $data['smtppassword'];
            $type = $data['smtptype'];
            $this->sendEmail($smtpemail, $smtppassword, $email, $subject, $template, $restaurant, $type);
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse([
                'success' => false,
                'message' => 'user not found',
                'code' => 200
            ]);
        }
    }

    public function sentOrderParEmail($id, $email, $idC)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $order = $qb->select("co.num,co.comment,co.createdAt,co.amount,co.discount,co.orderProg,t.label as type,t.id as orderType,p.phone as pdvPhone,p.siret as pdvSiret,
                                      p.address as pdvAddress,p.name as pdvName,a.streetName as customerStreetName,a.streetNumber as customerStreetNumber,
                                      a.city as customerCity,a.postalCode as customerPostalCode,c.firstname as customerFirstname,
                                      c.society as customerSociety,c.lastname as customerLastname,c.phone as customerPhone")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->leftJoin('co.address', 'a')
            ->where('co.id = :order')
            ->setParameter('order', $id)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $data = $qb->select("c.firstname,c.lastname,p.smtptype,p.smtpemail,p.smtppassword,p.emailContact,p.phoneContact as phone ,p.address,p.urlSite as website,p.nameRestaurant as restaurant")
            ->from("App:CustomerPdv", "cp")
            ->innerJoin("cp.customer", "c")
            ->innerJoin("cp.pdv", "p")
            ->where('cp.id = :idC')
            ->setParameter('idC', $idC)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        $data['color'] = '#000';
        $data['order'] = $order;
        $data['elements'] = $this->getElementsOfOrder($id, $idC);
        $data['menus'] = $menus = $this->getMenusOfOrder($id, $idC);
        $data['paymentModes'] = $this->getPyementsModeOfOdrder($id, $idC);
        $subject = 'Récapitulatif de votre commande du ' . $order['createdAt']->format('d M');;
        $template = $this->renderView('order.html.twig', $data);
        $restaurant = $data['restaurant'];
        $smtpemail = $data['smtpemail'];
        $smtppassword = $data['smtppassword'];
        $type = $data['smtptype'];
        $this->sendEmail($smtpemail, $smtppassword, $email, $subject, $template, $restaurant, $type);
    }

    /**
     * @Rest\Post("/website/confirme")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="key", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="user", nullable=false,allowBlank=false,strict=true)
     */
    public function confirmeAccount(Request $request, ParamFetcher $paramFetcher)
    {
        $id = $paramFetcher->get('user');
        $key = $paramFetcher->get('key');
        $user = $this->getDoctrine()->getRepository("App:webSiteUsers")->findOneBy(Array('id' => $id, "confirmationKey" => $key));
        if ($user && !$user->getConfirmed()) {
            $user->setConfirmed(1);
            $em = $this->getDoctrine()->getManager();
            $em->persist($user);
            $em->flush();
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $data = $qb->select("c.firstname,c.lastname,p.smtptype,p.smtpemail,p.smtppassword,p.nameRestaurant as restaurant,p.urlSite as website,p.address,p.emailContact,p.phoneContact as phone")
                ->from("App:webSiteUsers", "u")
                ->InnerJoin('u.customer', 'cp')
                ->InnerJoin('cp.customer', 'c')
                ->InnerJoin('cp.pdv', 'p')
                ->where('u.id = :id')
                ->setParameter('id', $id)
                ->setMaxResults(1)
                ->getQuery()->getOneOrNullResult();
            $data['password'] = $this->dec_enc('decrypt', $user->getPassword());;
            $data['color'] = '#000';
            $email = $user->getEmail();
            $data['email'] = $email;
            $restaurant = $data['restaurant'];
            $subject = 'Félicitations, votre compte vient d\'être crée!';
            $template = $this->renderView('newAccount.html.twig', $data);
            $smtpemail = $data['smtpemail'];
            $smtppassword = $data['smtppassword'];
            $type = $data['smtptype'];
            $this->sendEmail($smtpemail, $smtppassword, $email, $subject, $template, $restaurant, $type);
            return new JsonResponse(["code" => 1, 'mesage' => 'account confirmed with success'], 200);
        } else {
            if (!$user) {
                return new JsonResponse(['error' => 'user not found'], 400);
            } else {
                return new JsonResponse(["code" => 2, 'message' => 'compte aleardy confirmed'], 200);
            }
        }
    }

    /**
     * @Rest\Post("/website/contactus")
     * @param ParamFetcher $paramFetcher Paramfetcher
     * @RequestParam(name="name", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="email", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="subject", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="message", nullable=false,allowBlank=false,strict=true)
     * @RequestParam(name="pdv", nullable=false,allowBlank=false,strict=true)
     */
    public function contactUs(Request $request, ParamFetcher $paramFetcher)
    {
        $name = $paramFetcher->get('name');
        $subject = $paramFetcher->get('subject');
        $email = $paramFetcher->get('email');
        $message = $paramFetcher->get('message');
        $pdv = $paramFetcher->get('pdv');
        $idPdv = $this->dec_enc('decrypt', $pdv);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $data = $qb->select("p.nameRestaurant as name,p.smtptype,p.smtpemail,p.smtppassword,p.emailContact,p.phoneContact as phone,p.address,p.urlSite as website")
            ->from("App:Pdv", "p")
            ->where('p.id = :idPdv')
            ->setParameter('idPdv', $idPdv)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        if (!is_null($data)) {
            $data['color'] = '#000';
            $restaurant = $data['name'];
            $data['customerName'] = $name;
            $data['customerEmail'] = $email;
            $data['customerSubject'] = $subject;
            $data['customerMessage'] = $message;
            $subject = 'Un client de votre site internet vous a envoyé un message';
            $template = $this->renderView('contact.html.twig', $data);
            $smtpemail = $data['smtpemail'];
            $smtppassword = $data['smtppassword'];
            $type = $data['smtptype'];
            $this->sendEmail($smtpemail, $smtppassword, $email, $subject, $template, $restaurant, $type);
            return new JsonResponse([
                'success' => true,
                'code' => 200
            ]);
        } else {
            return new JsonResponse('', 400);
        }
    }

    /**
     * @Rest\Post("/website/login/google")
     * @RequestParam(name="googoleToken", nullable=false,allowBlank=false,strict=true)
     */
    public function authgoogle(ParamFetcher $paramFetcher)
    {
        $jwt = $paramFetcher->get('googoleToken');
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://www.googleapis.com/oauth2/v1/certs",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_POSTFIELDS => "",
            CURLOPT_HTTPHEADER => array(
                "Postman-Token: 04992799-0b34-47c6-8ae3-3c049685f516",
                "cache-control: no-cache"
            ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            echo "cURL Error #:" . $err;
        } else {
            $resp = (Array)json_decode($response);
            $publicKey = reset($resp);
            try {
                $decoded = JWT::decode($jwt, $publicKey, array('RS256'));
                return new JsonResponse($decoded, 400);
            } catch (\Exception $e) {
                return new JsonResponse('Error token invalide', 400);
            }
        }


    }

    private function getOptionAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $results = $qb->select("mo.id,es.id as idElementSize,e.name,s.label as size,mo.price,es.available")
            ->from("App:MenuOption", "mo")
            ->leftJoin('mo.elementSize', 'es')
            ->leftJoin('es.element', 'e')
            ->leftJoin('es.size', 's')
            ->leftJoin('mo.menu', 'm')
            ->where('m.id = :idMenu')
            ->setParameter('idMenu', $id)
            ->getQuery()
            ->getResult();
        return $results;
    }

    private function getIngredientComposByEl2($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $compositions = $qb->select("ec.id,ec.available,ec.name as text,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementComposition', 'ec')
            ->where('e.id = :element')
            ->andwhere('ec.activeSite = 1')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();
        return $compositions;
    }

    private function getIngredientSuppByEl2($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $supp = $qb->select("ec.id,ic.id as category,ic.name as categoryName,ec.name as text,ec.price,ec.available,case when ec.available = 1 then 'false' else 'true' end  as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementSupplement', 'ec')
            ->innerJoin('ec.ingredientCategory', 'ic')
            ->where('e.id = :element')
            ->andwhere('ec.activeSite = 1')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();

        return $supp;
    }

    private function getOptionsIngredientByEl2($idElement)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $options = $qb->select("ec.id,ec.name as text,ic.id as category,ic.name as categoryName,ec.available,case when ec.available = 1 then 'false' else 'true' end as disabled")
            ->from("App:Element", "e")
            ->innerJoin('e.elementOption', 'ec')
            ->innerJoin('ec.ingredientCategory', 'ic')
            ->where('e.id = :element')
            ->andwhere('ec.activeSite = 1')
            ->setParameter('element', $idElement)
            ->getQuery()->getResult();

        return $options;
    }

    private function getElementsByFamilyAndSize($idFamily, $idSize, $idMenu)
    {
        $exledProducts = $this->getExcludedProductsAction($idMenu);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id,es.available,e.name,e.avatar,e.id as element,es.priceSupp,e.hasSupp,cf.id as family,es.price,e.maxNbrIngOptions,e.priceSuppApp,s.label as size,s.level,s.id as idSize,1 as quantity")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('es.size', 's')
            ->innerJoin('e.family', 'cf')
            ->andWhere('cf.id = :cardFamily')
            ->andWhere('s.id = :size');
        if (count($exledProducts) > 0) {
            $elements = $elements->andWhere('es.id not in (:exledProducts)');
            $elements = $elements->setParameter('exledProducts', $exledProducts);
        }
        $elements = $elements->setParameter('cardFamily', $idFamily)
            ->setParameter('size', $idSize)
            ->groupBy('es.id')
            ->getQuery()
            ->getResult();
        foreach ($elements as $key => $element) {
            $elements[$key]['compositions'] = $this->getIngredientComposByEl2($element['element']);
        }
        return $elements;
    }

    public function getElementsByFamilyAndSize2($idFamily, $idSize, $idMenu,$idOrderType)
    {
        $exledProducts = $this->getExcludedProductsAction($idMenu);
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("es.id,es.available,e.name,e.avatar,e.id as element,es.priceSupp,e.hasSupp,cf.id as family,es.price,e.maxNbrIngOptions,e.priceSuppApp,s.label as size,s.level,s.id as idSize,1 as quantity")
            ->from("App:ElementSize", "es")
            ->innerJoin('es.element', 'e')
            ->innerJoin('es.size', 's')
            ->innerJoin('e.family', 'cf')
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->andWhere('ot.id = :idOrderType')
            //->andWhere('eot.active = 1')
            ->andWhere('e.activeSite = 1')
            ->andWhere('cf.id = :cardFamily')
            ->andWhere('s.id = :size');
        if (count($exledProducts) > 0) {
            $elements = $elements->andWhere('es.id not in (:exledProducts)');
            $elements = $elements->setParameter('exledProducts', $exledProducts);
        }
        $query = $elements->setParameter('cardFamily', $idFamily)
            ->setParameter('size', $idSize)
            ->setParameter('idOrderType', $idOrderType)
            ->groupBy('es.id')
            ->getQuery();
        //var_dump($query->getSql());
        $elements=$query->getResult();
        foreach ($elements as $key => $element) {
            $elements[$key]['compositions'] = $this->getIngredientComposByEl2($element['element']);
            $elements[$key]['supplement'] = floatval($this->hasOrderType2($element['element'], $idOrderType)['supplement']);
        }
        return $elements;
    }

    private function dec_enc($action, $string)
    {
        $output = false;
        $encrypt_method = "AES-256-CBC";
        $secret_key = 'helpyfood';
        $secret_iv = 'helpyfood';
        $key = hash('sha256', $secret_key);
        $iv = substr(hash('sha256', $secret_iv), 0, 16);
        if ($action == 'encrypt') {
            $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
            $output = base64_encode($output);
        } else if ($action == 'decrypt') {
            $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        }
        return $output;
    }

    private function getOrderPdvByPdvAndOrder($idPdv, $idOrder)
    {
        $em = $this->getDoctrine()->getManager();
        $pdv = $em->getRepository('App:Pdv')->find($idPdv);
        $orderType = $em->getRepository('App:OrderType')->find($idOrder);
        $orderTypePdv = $em->getRepository('App:OrderTypePdv')->findOneBy(array('pdv' => $pdv, 'orderType' => $orderType));
        return $orderTypePdv;
    }

    private function getElementComSuppRemovOpt($idElement, $action)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("c.name,c.cout")
            ->from("App:CommandElementIngredients", "c")
            ->innerJoin('c.commandElement', 'ce')
            ->where('ce.id = :el')
            ->andWhere('c.action = :action')
            ->setParameter('el', $idElement)
            ->setParameter('action', $action)
            ->getQuery()->getResult();

        return $tab;
    }

    private function getMenuElements($idMenu)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $tab = $qb->select("ce.id,ce.quantity,ce.comment,ce.additionalCost,ce.price,ce.name,ce.size,ce.level")
            ->from("App:Command", "co")
            ->innerJoin('co.commandElements', 'ce')
            ->leftJoin('ce.commandMenu', 'cm')
            ->where('cm.id = :menu')
            ->setParameter('menu', $idMenu)
            ->getQuery()->getResult();

        return $tab;
    }

    private function getOrderInfo($id, $idC)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $order = $qb->select("co.num,co.comment,co.createdAt,co.amount,co.discount,co.orderProg,t.label as type,t.id as orderType,p.phone as pdvPhone,p.siret as pdvSiret,
                                      p.address as pdvAddress,p.name as pdvName,a.streetName as customerStreetName,a.streetNumber as customerStreetNumber,
                                      a.city as customerCity,a.postalCode as customerPostalCode,a.building,a.staircase,a.floor,a.door,a.inputCode,a.intercom,a.complementAddress,c.firstname as customerFirstname,
                                      c.society as customerSociety,c.lastname as customerLastname,c.phone as customerPhone")
            ->from("App:Command", "co")
            ->innerJoin('co.orderTypePdv', 'mop')
            ->innerJoin('mop.orderType', 't')
            ->innerJoin('mop.pdv', 'p')
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('cp.customer', 'c')
            ->leftJoin('co.address', 'a')
            ->where('co.id = :order')
            ->andWhere('cp.id=:cust')
            ->setParameter('order', $id)
            ->setParameter('cust', $idC)
            ->setMaxResults(1)
            ->getQuery()->getOneOrNullResult();
        return $order;
    }

    private function getElementsOfOrder($id, $idC)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("ce.id,ce.quantity,ce.comment,ce.level,ce.total,ce.price + ce.additionalCost as price,ce.additionalCost,ce.total,ce.name,ce.size")
            ->from("App:Command", "co")
            ->innerJoin('co.commandElements', 'ce')
            ->innerJoin('co.customer', 'cp')
            ->leftJoin('ce.commandMenu', 'cm')
            ->where('co.id = :order')
            ->andWhere('cm.id IS NULL')
            ->andWhere('cp.id=:cust')
            ->setParameter('order', $id)
            ->setParameter('cust', $idC)
            ->getQuery()->getResult();

        foreach ($elements as $key => $value) {
            $elements[$key]['optionIngredients'] = $this->getElementComSuppRemovOpt($value['id'], 'opt');
            $elements[$key]['removedCompositions'] = $this->getElementComSuppRemovOpt($value['id'], 'remove');
            $elements[$key]['supplements'] = $this->getElementComSuppRemovOpt($value['id'], 'supp');
        }
        return $elements;
    }

    private function getMenusOfOrder($id, $idC)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("cm.id,cm.quantity,cm.comment,cm.total,cm.price + cm.additionalCost as price,cm.title")
            ->from("App:CommandMenu", "cm")
            ->innerJoin('cm.command', 'c')
            ->innerJoin('c.customer', 'cp')
            ->where('c.id = :order')
            ->andWhere('cp.id=:cust')
            ->setParameter('order', $id)
            ->setParameter('cust', $idC)
            ->getQuery()->getResult();
        foreach ($menus as $ke => $val) {
            $menus[$ke]['elements'] = $this->getMenuElements($val['id']);
            foreach ($menus[$ke]['elements'] as $key => $value) {
                $menus[$ke]['elements'][$key]['optionIngredients'] = $this->getElementComSuppRemovOpt($value['id'], 'opt');
                //  $elements[$key]['compositions']=$this->getElementComSuppRemovOpt($value['id'],'comp');
                $menus[$ke]['elements'][$key]['removedCompositions'] = $this->getElementComSuppRemovOpt($value['id'], 'remove');
                $menus[$ke]['elements'][$key]['supplements'] = $this->getElementComSuppRemovOpt($value['id'], 'supp');
            }
        }
        return $menus;
    }

    private function getPyementsModeOfOdrder($id, $idC)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $paymentModes = $qb->select("pm.label")
            ->from("App:Command", "co")
            ->innerJoin('co.customer', 'cp')
            ->innerJoin('co.commandPaymentModes', 'cpm')
            ->innerJoin('cpm.paymentMode', 'pm')
            ->where('co.id = :order')
            ->andWhere('cp.id=:cust')
            ->setParameter('order', $id)
            ->setParameter('cust', $idC)
            ->getQuery()->getResult();
        return $paymentModes;
    }

    private function sendEmail($email, $password, $to, $subject, $template, $restaurant, $type)
    {
        if ($email != "" && $password != "") {
            if ($type == 2) {
                $transport = new \Swift_SmtpTransport('pro1.mail.ovh.net', 587, 'tls');
            } else {
                $transport = new \Swift_SmtpTransport('ssl0.ovh.net', 465, 'ssl');
            }
        } else {
            $transport = new \Swift_SmtpTransport('ssl0.ovh.net', 465, 'ssl');
            $email = "siteweb@helpyfood.com";
            $password = "=gszE9P4*V";
        }
        $transport->setUsername($email);
        $transport->setPassword($password);
        $mailer = new \Swift_Mailer($transport);
        $message = (new \Swift_Message($subject))
            ->setTo($to)
            ->setFrom([$email => $restaurant])
            ->setBody(strip_tags($template))
            ->addPart($template, 'text/html');
        $mailer->send($message);
        return new JsonResponse();
    }

    private function getExcludedProductsAction($id)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $results = $qb->select("es.id")
            ->from("App:Menu", "m")
            ->innerJoin('m.excludedProducts', 'es')
            ->where('m.id = :idMenu')
            ->setParameter('idMenu', $id)
            ->getQuery()
            ->getResult();
        return $results;
    }

    public function getCompositionsMenu($id)
    {
        $elements = array();
        $families = array();
        for ($i = 0; $i < 6; $i++) {
            $em = $this->getDoctrine()->getManager();
            $qb = $em->createQueryBuilder();
            $query = $qb->select("mes.quantity,e.name,s.label as size")
                ->from("App:Menu", "m")
                ->innerJoin('m.elements', 'mes')
                ->innerJoin('mes.elementSize', 'es')
                ->innerJoin('es.element', 'e')
                ->innerJoin('es.size', 's')
                ->where('mes.menu = :idMenu')
                ->andWhere('mes.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            $elements[$i] = $results;
            $qb2 = $em->createQueryBuilder();
            $query = $qb2->select("mfs.quantity,f.label,s.label as size")
                ->from("App:Menu", "m")
                ->leftJoin('m.families', 'mfs')
                ->leftJoin('mfs.cardFamily', 'cf')
                ->leftJoin('cf.family', 'f')
                ->leftJoin('mfs.size', 's')
                ->where('mfs.menu = :idMenu')
                ->andWhere('mfs.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            $families[$i] = $results;
        }
        $data = array('elements' => $elements, 'families' => $families);
        return $data;
    }

    public function getCompositionsElement($id)
    {
    	$em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $data = $qb->select("GROUP_CONCAT(ec.name) as comp")
            ->from("App:Element", "e")
            ->leftJoin('e.elementComposition', 'ec')
            ->where('ec.activeSite=1')
            ->andWhere('e.id=:id')
            ->setParameter('id', $id)
            ->groupBy('e.id')
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
        return $data['comp'];
    }

    public function getBestSalles($site=false, $idOrderType, $idPdv){
        $em = $this->getDoctrine()->getManager();
        $families = $em->createQueryBuilder('f')
            ->select('cf.id,SUM(ce.quantity) as quantity')
            ->from("App:CommandElement", "ce")
            ->innerJoin("ce.command", 'co')
            ->innerJoin("co.customer", 'cup')
            ->innerJoin("cup.customer", 'cu')
            ->innerJoin("cup.pdv", 'pdv')
            ->innerJoin('App:Element', 'e', 'WITH', 'e.name = ce.name')
            ->innerJoin('e.family', 'cf')
            ->innerJoin('e.card', 'c')
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->andWhere('e.activeSite = :active')
            ->andWhere('pdv.id=:idPdv')
            ->andWhere('eot.active=:active')
            ->andWhere('ot.id=:orderType')
            ->andWhere('ce.commandMenu is null')
            ->andWhere('c.id=:idPdv')
            ->setParameter('active', 1)
            ->setParameter('orderType', $idOrderType)
            ->setParameter('idPdv', $idPdv);
        if($site){
            $families = $families->andWhere('cu.indicator=:indicator')->setParameter('indicator', 'S');
        }
        $families = $families->groupBy('cf.id')
            ->orderBy('quantity', 'DESC')
            ->setMaxResults(4)
            ->getQuery()
            ->getResult();
        $elements = array();
        foreach ($families as $f) {
            $el = $em->createQueryBuilder('r')
                ->select('e.id,e.hasSupp,e.custom,e.avatar,e.code,e.name,eot.supplement,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp,SUM(ce.quantity) as quantity,0 as size')
                ->from("App:CommandElement", "ce")
                ->innerJoin("ce.command", 'co')
                ->innerJoin("co.customer", 'cup')
                ->innerJoin("cup.customer", 'cu')
                ->innerJoin("cup.pdv", 'pdv')
                ->innerJoin('App:Element', 'e', 'WITH', 'e.name = ce.name')
                ->innerJoin('e.family', 'cf')
                ->innerJoin('cf.family', 'f')
                ->innerJoin('e.card', 'c')
                ->innerJoin('e.elementOrderType', 'eot')
                ->innerJoin('eot.orderType', 'ot')
                ->andWhere('e.activeSite = :active')
                ->andWhere('c.id=:idCarte')
                ->andWhere('pdv.id=:idCarte')
                ->andWhere('eot.active=:active')
                ->andWhere('ot.id=:orderType')
                ->andWhere('cf.id=:family')
                ->andWhere('ce.commandMenu is null')
                ->setParameter('active', 1)
                ->setParameter('orderType', $idOrderType)
                ->setParameter('idCarte', $idPdv)
                ->setParameter('family', $f['id']);
            if($site){
                $el = $el->andWhere('cu.indicator=:indicator')->setParameter('indicator', 'S');
            }
            $el = $el->groupBy('e.id')
                ->orderBy('quantity', 'DESC')
                ->setMaxResults(1)
                ->getQuery()
                ->getOneOrNullResult();
            if ($el) {
                $elements[] = $el;
            }
        }
        return $elements;
    }
    private function isMobile(){
        $useragent=$_SERVER['HTTP_USER_AGENT'];
        if(preg_match('/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i',$useragent)||preg_match('/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i',substr($useragent,0,4))
        ) return true;
        return false;
    }
    public function hasOrderType2($idElement, $idOrderType)
    {
        $em = $this->getDoctrine()->getManager();
        $qb = $em->createQueryBuilder();
        $result = $qb->select("count(ot.id) as dispo,eot.supplement")
            ->from("App:Element", "e")
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->where('e.id = :idElement')
            ->andWhere('ot.id = :idOrderType')
            ->andWhere('eot.active = 1')
            ->setParameter('idOrderType', $idOrderType)
            ->setParameter('idElement', $idElement)
            ->getQuery()
            ->getSingleResult();
        return $result;
    }
}

