<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;

/**
 * Card
 *
 * @ORM\Table(name="card")
 * @ORM\Entity
 */
class Card
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;


    /**
     * @var \DateTime
     *
     * @ORM\Column(name="started_validity", type="date", nullable=true)
     */
    private $startedValidity;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="stopped_validity", type="date", nullable=true)
     */
    private $stoppedValidity;

    /**
     * @var int
     *
     * @ORM\Column(name="nbr_offers_authorized", type="integer")
     */
    private $nbrOffersAuthorized;

    /**
     * @var int
     *
     * @ORM\Column(name="nbr_offers_authorized_site", type="integer")
     */
    private $nbrOffersAuthorizedSite;

    /**
     * @var int
     *
     * @ORM\Column(name="display_code", type="integer")
     */
    private $displayCode;

    /**
     * @var float
     * @ORM\Column(name="delivery_charges", type="decimal", scale=2,precision= 10, options={"default" : 0})
     */
    private $deliveryCharges;

    /**
     * @var string
     *
     * @ORM\Column(name="delivery_charges_label", type="string", length=400, nullable=true)
     */
    private $deliveryChargesLabel;


    /**
     * @var float
     * @ORM\Column(name="management_fees", type="decimal", scale=2,precision= 10, options={"default" : 0})
     */
    private $managementFees;

    /**
     * @var string
     *
     * @ORM\Column(name="management_fees_label", type="string", length=400, nullable=true)
     */
    private $managementFeesLabel;

    /**
     * @var string
     *
     * @ORM\Column(name="comment", type="string", length=400)
     */
    private $comment;

    /**
     * @ORM\OneToOne(targetEntity="Pdv" , inversedBy="card" ,cascade={"persist","remove"})
     * @ORM\JoinColumn(name="pdv_id", referencedColumnName="id", nullable=false , onDelete="cascade"))
     */
    private $pdv;

    /**
     * @ORM\OneToMany(targetEntity="CardFamily", mappedBy="card", cascade={"remove"})
     */
    private $families;


    /**
     * @ORM\OneToMany(targetEntity="CardMenuCategory", mappedBy="card", cascade={"remove"})
     */
    private $menuCategories;

    /**
     * @ORM\OneToMany(targetEntity="Ingredient", mappedBy="card", cascade={"remove"})
     */
    private $ingredients;

    /**
     * @ORM\OneToMany(targetEntity="Size", mappedBy="card", cascade={"remove"})
     */
    private $sizes;

    /**
     * @ORM\OneToMany(targetEntity="Element", mappedBy="card", cascade={"remove"})
     */
    private $elements;

    /**
     * @ORM\OneToMany(targetEntity="CardOffer", mappedBy="card", cascade={"remove"})
     */
    private $offers;

    /**
     * @ORM\OneToMany(targetEntity="Menu", mappedBy="card", cascade={"remove"})
     */
    private $menus;

    public function __construct()
    {
        $this->families = new ArrayCollection();
        $this->menuCategories = new ArrayCollection();
    }



    public function addFamily(Family $famille)
    {
        $this->families[] = $famille;
        return $this;
    }

    public function removeFamily(Family $famille)
    {
        $this->families->removeElement($famille);
    }



    public function addMenuCategory(MenuCategory $menu_categorie)
    {
        $this->menuCategories[] = $menu_categorie;
        return $this;
    }

    public function removeMenuCategory(MenuCategory $menu_categorie)
    {
        $this->menuCategories->removeElement($menu_categorie);
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }


    /**
     * @return \DateTime
     */
    public function getStartedValidity()
    {
        return $this->startedValidity;
    }

    /**
     * @param \DateTime $startedValidity
     */
    public function setStartedValidity($startedValidity)
    {
        $this->startedValidity = $startedValidity;
    }

    /**
     * @return \DateTime
     */
    public function getStoppedValidity()
    {
        return $this->stoppedValidity;
    }

    /**
     * @param \DateTime $stoppedValidity
     */
    public function setStoppedValidity($stoppedValidity)
    {
        $this->stoppedValidity = $stoppedValidity;
    }

    /**
     * @return int
     */
    public function getNbrOffersAuthorized()
    {
        return $this->nbrOffersAuthorized;
    }

    /**
     * @param int $nbrOffersAuthorized
     */
    public function setNbrOffersAuthorized($nbrOffersAuthorized)
    {
        $this->nbrOffersAuthorized = $nbrOffersAuthorized;
    }

    /**
     * @return int
     */
    public function getDisplayCode()
    {
        return $this->displayCode;
    }

    /**
     * @param int $displayCode
     */
    public function setDisplayCode($displayCode)
    {
        $this->displayCode = $displayCode;
    }

    /**
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }

    /**
     * @param string $comment
     */
    public function setComment($comment)
    {
        $this->comment = $comment;
    }

    /**
     * @return mixed
     */
    /*public function getPdv()
    {
        return $this->pdv;
    }*/

    /**
     * @param mixed $pdv
     */
    public function setPdv($pdv)
    {
        $this->pdv = $pdv;
    }

    /**
     * @return mixed
     */
    public function getFamilies()
    {
        return $this->families;
    }

    /**
     * @param mixed $families
     */
    public function setFamilies($families)
    {
        $this->families = $families;
    }

    /**
     * @return mixed
     */
    public function getMenuCategories()
    {
        return $this->menuCategories;
    }

    /**
     * @param mixed $menuCategories
     */
    public function setMenuCategories($menuCategories)
    {
        $this->menuCategories = $menuCategories;
    }

    /**
     * @return int
     */
    public function getNbrOffersAuthorizedSite()
    {
        return $this->nbrOffersAuthorizedSite;
    }

    /**
     * @param int $nbrOffersAuthorizedSite
     */
    public function setNbrOffersAuthorizedSite($nbrOffersAuthorizedSite)
    {
        $this->nbrOffersAuthorizedSite = $nbrOffersAuthorizedSite;
    }

    /**
     * @return float
     */
    public function getDeliveryCharges()
    {
        return $this->deliveryCharges;
    }

    /**
     * @param float $deliveryCharges
     */
    public function setDeliveryCharges($deliveryCharges)
    {
        $this->deliveryCharges = $deliveryCharges;
    }

    /**
     * @return float
     */
    public function getManagementFees()
    {
        return $this->managementFees;
    }

    /**
     * @param float $managementFees
     */
    public function setManagementFees($managementFees)
    {
        $this->managementFees = $managementFees;
    }

    /**
     * @return string
     */
    public function getDeliveryChargesLabel()
    {
        return $this->deliveryChargesLabel;
    }

    /**
     * @param string $deliveryChargesLabel
     */
    public function setDeliveryChargesLabel($deliveryChargesLabel)
    {
        $this->deliveryChargesLabel = $deliveryChargesLabel;
    }

    /**
     * @return string
     */
    public function getManagementFeesLabel()
    {
        return $this->managementFeesLabel;
    }

    /**
     * @param string $managementFeesLabel
     */
    public function setManagementFeesLabel($managementFeesLabel)
    {
        $this->managementFeesLabel = $managementFeesLabel;
    }
}

