<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use DateTime;

/**
 * Command
 *
 * @ORM\Table(name="command")
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class Command
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var int
     * @ORM\Column(name="num", type="integer")
     */
    private $num;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="createdAt", type="datetime")
     */
    private $createdAt;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="orderProg", type="datetime")
     */
    private $orderProg;

    /**
     * @var string
     *
     * @ORM\Column(name="amount", type="decimal", precision=10, scale=2)
     */
    private $amount;

    /**
     * @var string
     *
     * @ORM\Column(name="discount", type="decimal", precision=10, scale=2)
     */
    private $discount;

    /**
     * @var string
     *
     * @ORM\Column(name="comment", type="string", length=500, nullable=true)
     */
    private $comment;


    /**
     * @var int
     *
     * @ORM\Column(name="corrected", type="integer")
     */
    private $corrected;

    /**
     * @var string
     *
     * @ORM\Column(name="createdIp", type="string", length=30)
     */
    private $createdIp;


    /**
     * @ORM\ManyToOne(targetEntity="User")
     * @ORM\JoinColumn(name="createdUser", referencedColumnName="id", nullable=true)
     */
    private $createdUser;

    /**
     * @ORM\ManyToOne(targetEntity="Team")
     * @ORM\JoinColumn(name="team", referencedColumnName="id", nullable=true, onDelete="cascade")
     */
    private $team;

    /**
     * @var string
     *
     * @ORM\Column(name="updatedIp", type="string", length=30)
     */
    private $updatedIp;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="updateAt", type="datetime")
     */
    private $updatedAt;


    /**
     * @ORM\ManyToOne(targetEntity="User")
     * @ORM\JoinColumn(name="updatedUser", referencedColumnName="id", nullable=true)
     */
    private $updatedUser;


    /**
     * @ORM\ManyToOne(targetEntity="CustomerPdv" , inversedBy="command")
     * @ORM\JoinColumn(name="customer_id", referencedColumnName="id", nullable=false, onDelete="cascade")
     */
    private $customer;

    /**
     * @ORM\ManyToOne(targetEntity="Address" , inversedBy="command")
     * @ORM\JoinColumn(name="address_id", referencedColumnName="id", nullable=true)
     */
    private $address;



    /**
     * @ORM\ManyToOne(targetEntity="Status" , inversedBy="command")
     * @ORM\JoinColumn(name="status_id", referencedColumnName="id", nullable=false)
     */
    private $status;


    /**
     * @ORM\ManyToOne(targetEntity="Platform")
     * @ORM\JoinColumn(name="platform_id", referencedColumnName="id", nullable=false)
     */
    private $platform;


    /**
     * @ORM\ManyToOne(targetEntity="OrderTypePdv" , inversedBy="command")
     * @ORM\JoinColumn(name="orderTypePdv_id", referencedColumnName="id", nullable=false, onDelete="cascade")
     */
    private $orderTypePdv;

    /**
     * @ORM\OneToMany(targetEntity="CommandElement", mappedBy="command", cascade={"remove","persist"})
     */
    private $commandElements;

    /**
     * @ORM\OneToMany(targetEntity="CommandPaymentMode", mappedBy="command", cascade={"remove"})
     */
    private $commandPaymentModes;

    public function __construct()
    {
        $this->commandPaymentModes = new ArrayCollection();

    }

    /**
     * @param mixed $platform
     */
    public function setPlatform($platform)
    {
        $this->platform = $platform;
    }

    public function getPlatform()
    {
        return $this->platform;
    }

    public function addPaymentMode(PaymentMode $payment_modes)
    {
        $this->commandPaymentModes[] = $payment_modes;
        return $this;
    }

    public function removePaymentMode(PaymentMode $payment_modes)
    {
        $this->commandPaymentModes->removeElement($payment_modes);
    }

    /**
     * @return mixed
     */
    public function getCommandPaymentModes()
    {
        return $this->commandPaymentModes;
    }

    /**
     * @param mixed $address
     */
    public function setAddress($address)
    {
        $this->address = $address;
    }

    /**
     * @param mixed $customer
     */
    public function setCustomer($customer)
    {
        $this->customer = $customer;
    }

    /**
     * @param mixed $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
    }

    /**
     * @param mixed $orderTypePdv
     */
    public function setOrderTypePdv($orderTypePdv)
    {
        $this->orderTypePdv = $orderTypePdv;
    }


    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set createdAt
     *
     * @param \DateTime $createdAt
     *
     * @return Command
     */
    public function setCreatedAt($createdAt)
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    /**
     * Get createdAt
     *
     * @return \DateTime
     */
    public function getCreatedAt()
    {
        return $this->createdAt;
    }

    /**
     * Set orderProg
     *
     * @param \DateTime $orderProg
     *
     * @return Command
     */
    public function setOrderProg($orderProg)
    {
        $this->orderProg = $orderProg;

        return $this;
    }

    /**
     * Get orderProg
     *
     * @return \DateTime
     */
    public function getOrderProg()
    {
        return $this->orderProg;
    }

    /**
     * Set amount
     *
     * @param string $amount
     *
     * @return Command
     */
    public function setAmount($amount)
    {
        $this->amount = $amount;

        return $this;
    }

    /**
     * Get amount
     *
     * @return string
     */
    public function getAmount()
    {
        return $this->amount;
    }

    /**
     * Set discount
     *
     * @param string $discount
     *
     * @return Command
     */
    public function setDiscount($discount)
    {
        $this->discount = $discount;

        return $this;
    }

    /**
     * Get discount
     *
     * @return string
     */
    public function getDiscount()
    {
        return $this->discount;
    }

    /**
     * Set comment
     *
     * @param string $comment
     *
     * @return Command
     */
    public function setComment($comment)
    {
        $this->comment = $comment;

        return $this;
    }

    /**
     * Get comment
     *
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }



    /**
     * Set corrected
     *
     * @param integer $corrected
     *
     * @return Command
     */
    public function setCorrected($corrected)
    {
        $this->corrected = $corrected;

        return $this;
    }

    /**
     * Get corrected
     *
     * @return int
     */
    public function getCorrected()
    {
        return $this->corrected;
    }

    /**
     * Set createdIp
     *
     * @param string $createdIp
     *
     * @return Command
     */
    public function setCreatedIp($createdIp)
    {
        $this->createdIp = $createdIp;

        return $this;
    }

    /**
     * Get createdIp
     *
     * @return string
     */
    public function getCreatedIp()
    {
        return $this->createdIp;
    }

    /**
     * @return mixed
     */
    public function getCreatedUser()
    {
        return $this->createdUser;
    }

    /**
     * @param mixed $createdUser
     */
    public function setCreatedUser($createdUser)
    {
        $this->createdUser = $createdUser;
    }


    /**
     * Set updatedIp
     *
     * @param string $updatedIp
     *
     * @return Command
     */
    public function setUpdatedIp($updatedIp)
    {
        $this->updatedIp = $updatedIp;

        return $this;
    }

    /**
     * Get updatedIp
     *
     * @return string
     */
    public function getUpdatedIp()
    {
        return $this->updatedIp;
    }

    /**
     * Set updatedAt
     *
     * @param \DateTime $updatedAtt
     *
     * @return Command
     */
    public function setUpdatedAt($updatedAt)
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * Get updatedAt
     *
     * @return \DateTime
     */
    public function getUpdatedAt()
    {
        return $this->updatedAt;
    }

    /**
     * @return mixed
     */
    public function getUpdatedUser()
    {
        return $this->updatedUser;
    }

    /**
     * @param mixed $updatedUser
     */
    public function setUpdatedUser($updatedUser)
    {
        $this->updatedUser = $updatedUser;
    }

    /**
     * @ORM\PrePersist
     * @ORM\PreUpdate
     */
    public function updatedTimestamps()
    {
        $dateTimeNow = new DateTime('now');
        $this->setUpdatedAt($dateTimeNow);
        if ($this->getCreatedAt() === null) {
            $this->setCreatedAt($dateTimeNow);
        }
    }

    /**
     * @return int
     */
    public function getNum()
    {
        return $this->num;
    }

    /**
     * @param int $num
     */
    public function setNum($num)
    {
        $this->num = $num;
    }

    /**
     * @return mixed
     */
    public function getAddress()
    {
        return $this->address;
    }

    /**
     * @param mixed $commandPaymentModes
     */
    public function setCommandPaymentModes($commandPaymentModes)
    {
        $this->commandPaymentModes = $commandPaymentModes;
    }



    /**
     * @return mixed
     */
    public function getTeam()
    {
        return $this->team;
    }

    /**
     * @param mixed $team
     */
    public function setTeam($team)
    {
        $this->team = $team;
    }





}

