<?php

namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * CommandElement
 *
 * @ORM\Table(name="command_element",indexes={@ORM\Index(name="name", columns={"name"})})
 * @ORM\Entity
 */
class CommandElement
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var int
     *
     * @ORM\Column(name="quantity", type="integer")
     */
    private $quantity;

    /**
     * @var string
     *
     * @ORM\Column(name="comment", type="string", length=200)
     */
    private $comment;

    /**
     * @var string
     *
     * @ORM\Column(name="total", type="decimal", precision=10, scale=2)
     */
    private $total;

    /**
     * @var string
     *
     * @ORM\Column(name="price", type="decimal", precision=10, scale=2)
     */
    private $price;

    /**
     * @var string
     *
     * @ORM\Column(name="additionalCost", type="decimal", precision=10, scale=2)
     */
    private $additionalCost;

    /**
     * @var float
     *
     * @ORM\Column(name="vat", type="float")
     */
    private $vat;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=100)
     */
    private $name;

    /**
     * @var string
     *
     * @ORM\Column(name="size", type="string", length=50)
     */
    private $size;

    /**
     * @var integer
     *
     * @ORM\Column(name="level", type="integer", length=50, nullable=true)
     */
    private $level;

    /**
     * @var string
     *
     * @ORM\Column(name="code", type="string", length=32, nullable=true)
     */
    private $code;

    /**
     * @ORM\ManyToOne(targetEntity="Command")
     * @ORM\JoinColumn(name="command_id", referencedColumnName="id", nullable=false, onDelete="cascade")
     */
    private $command;

    /**
     * @var integer
     * @ORM\Column(name="element_size_id", type="integer", nullable=true)
     */
    private $elementSize;

    /**
     * @ORM\ManyToOne(targetEntity="CommandMenu" , inversedBy="commandElement")
     * @ORM\JoinColumn(name="command_menu_id", referencedColumnName="id", nullable=true)
     */
    private $commandMenu;

    /**
     * @ORM\OneToMany(targetEntity="CommandElementIngredients", mappedBy="commandElement", cascade={"remove","persist"})
     */
    private $ingredients;

    /**
     * @param mixed $command
     */
    public function setCommand($command)
    {
        $this->command = $command;
    }


    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId(int $id)
    {
        $this->id = $id;
    }

    /**
     * @return int
     */
    public function getQuantity()
    {
        return $this->quantity;
    }

    /**
     * @param int $quantity
     */
    public function setQuantity($quantity)
    {
        $this->quantity = $quantity;
    }



    /**
     * Set comment
     *
     * @param string $comment
     *
     * @return CommandElement
     */
    public function setComment($comment)
    {
        $this->comment = $comment;

        return $this;
    }

    /**
     * Get comment
     *
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }

    /**
     * Set total
     *
     * @param string $total
     *
     * @return CommandElement
     */
    public function setTotal($total)
    {
        $this->total = $total;

        return $this;
    }

    /**
     * Get total
     *
     * @return string
     */
    public function getTotal()
    {
        return $this->total;
    }

    /**
     * Set price
     *
     * @param string $price
     *
     * @return CommandElement
     */
    public function setPrice($price)
    {
        $this->price = $price;

        return $this;
    }

    /**
     * Get price
     *
     * @return string
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * Set additionalCost
     *
     * @param string $additionalCost
     *
     * @return CommandElement
     */
    public function setAdditionalCost($additionalCost)
    {
        $this->additionalCost = $additionalCost;

        return $this;
    }

    /**
     * Get additionalCost
     *
     * @return string
     */
    public function getAdditionalCost()
    {
        return $this->additionalCost;
    }

    /**
     * Set vat
     *
     * @param float $vat
     *
     * @return CommandElement
     */
    public function setVat($vat)
    {
        $this->vat = $vat;

        return $this;
    }

    /**
     * Get vat
     *
     * @return float
     */
    public function getVat()
    {
        return $this->vat;
    }

    /**
     * Set name
     *
     * @param string $name
     *
     * @return CommandElement
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set size
     *
     * @param string $size
     *
     * @return CommandElement
     */
    public function setSize($size)
    {
        $this->size = $size;

        return $this;
    }

    /**
     * Get size
     *
     * @return string
     */
    public function getSize()
    {
        return $this->size;
    }

    /**
     * @return mixed
     */
    public function getCommandMenu()
    {
        return $this->commandMenu;
    }

    /**
     * @param mixed $commandMenu
     */
    public function setCommandMenu($commandMenu)
    {
        $this->commandMenu = $commandMenu;
    }

    /**
     * @return int
     */
    public function getLevel()
    {
        return $this->level;
    }

    /**
     * @param int $level
     */
    public function setLevel($level)
    {
        $this->level = $level;
    }

    /**
     * @return mixed
     */
    public function getElementSize()
    {
        return $this->elementSize;
    }

    /**
     * @param mixed $elementSize
     */
    public function setElementSize($elementSize)
    {
        $this->elementSize = $elementSize;
    }

    /**
     * @return string
     */
    public function getCode()
    {
        return $this->code;
    }

    /**
     * @param string $code
     */
    public function setCode($code)
    {
        $this->code = $code;
    }
}

