<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use DateTime;

/**
 * Customer
 *
 * @ORM\Table(name="customer")
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 *
 */
class Customer
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="lastname", type="string", length=100, nullable=true)
     */
    private $lastname;

    /**
     * @var string
     *
     * @ORM\Column(name="firstname", type="string", length=100, nullable=true)
     */
    private $firstname;

    /**
     * @var string
     *
     * @ORM\Column(name="society", type="string", length=100, nullable=true)
     */
    private $society;

    /**
     * @var string
     *
     * @ORM\Column(name="phone", type="string", length=10, nullable=false)
     */
    private $phone;

    /**
     * @var string
     *
     * @ORM\Column(name="secondary_phone", type="string", length=10, nullable=true)
     */
    private $secondaryPhone;

    /**
     * @var string
     *
     * @ORM\Column(name="email", type="string", length=100, nullable=true)
     */
    private $email;

    /**
     * @var int
     *
     * @ORM\Column(name="sms_authorized", type="integer", nullable=true)
     */
    private $smsAuthorized;

    /**
     * @var string
     *
     * @ORM\Column(name="created_ip", type="string", length=50, nullable=true)
     */
    private $createdIp;

    /**
     * @ORM\ManyToOne(targetEntity="User")
     * @ORM\JoinColumn(name="createdUser", referencedColumnName="id", nullable=true)
     */
    private $createdUser;


    /**
     * @var string
     *
     * @ORM\Column(name="updated_ip", type="string", length=50, nullable=true)
     */
    private $updatedIp;

    /**
     * @ORM\ManyToOne(targetEntity="User")
     * @ORM\JoinColumn(name="updatedUser", referencedColumnName="id", nullable=true)
     */
    private $updatedUser;

    /**
     * @var DateTime $created
     *
     * @ORM\Column(name="created_at", type="datetime", nullable=false, nullable=true)
     */
    protected $createdAt;
    /**
     * @var DateTime $updated
     *
     * @ORM\Column(name="updated_at", type="datetime", nullable=false, nullable=true)
     */
    protected $updatedAt;

    /**
     * @var string
     *
     * @ORM\Column(name="indicator", type="string", length=5, nullable=true)
     */
    private $indicator;

    /**
     * @ORM\PrePersist
     * @ORM\PreUpdate
     */
    public function updatedTimestamps()
    {
        $dateTimeNow = new DateTime('now');
        $this->setUpdatedAt($dateTimeNow);
        if ($this->getCreatedAt() === null) {
            $this->setCreatedAt($dateTimeNow);
        }
    }


    /**
     * @ORM\OneToMany(targetEntity="CustomerPdv", mappedBy="customer", cascade={"remove"})
     */
    private $pdvs;


    public function __construct()
    {
        $this->pdvs= new ArrayCollection();
    }

    /**
     * @return string
     */
    public function getIndicator()
    {
        return $this->indicator;
    }

    /**
     * @param string $indicator
     */
    public function setIndicator($indicator)
    {
        $this->indicator = $indicator;
    }

    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId(int $id)
    {
        $this->id = $id;
    }



    /**
     * @return string
     */
    public function getLastname()
    {
        return $this->lastname;
    }

    /**
     * @param string $lastname
     */
    public function setLastname($lastname)
    {
        $this->lastname = $lastname;
    }

    /**
     * @return string
     */
    public function getFirstname()
    {
        return $this->firstname;
    }

    /**
     * @param string $firstname
     */
    public function setFirstname($firstname)
    {
        $this->firstname = $firstname;
    }

    /**
     * @return string
     */
    public function getSociety()
    {
        return $this->society;
    }

    /**
     * @param string $society
     */
    public function setSociety($society)
    {
        $this->society = $society;
    }

    /**
     * @return string
     */
    public function getPhone()
    {
        return $this->phone;
    }

    /**
     * @param string $phone
     */
    public function setPhone($phone)
    {
        $this->phone = $phone;
    }

    /**
     * @return string
     */
    public function getSecondaryPhone()
    {
        return $this->secondaryPhone;
    }

    /**
     * @param string $secondaryPhone
     */
    public function setSecondaryPhone($secondaryPhone)
    {
        $this->secondaryPhone = $secondaryPhone;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * @return int
     */
    public function getSmsAuthorized()
    {
        return $this->smsAuthorized;
    }

    /**
     * @param int $smsAuthorized
     */
    public function setSmsAuthorized($smsAuthorized)
    {
        $this->smsAuthorized = $smsAuthorized;
    }

    /**
     * @return string
     */
    public function getCreatedIp()
    {
        return $this->createdIp;
    }

    /**
     * @param string $createdIp
     */
    public function setCreatedIp($createdIp)
    {
        $this->createdIp = $createdIp;
    }


    /**
     * @return string
     */
    public function getUpdatedIp()
    {
        return $this->updatedIp;
    }

    /**
     * @param string $updatedIp
     */
    public function setUpdatedIp($updatedIp)
    {
        $this->updatedIp = $updatedIp;
    }


    /**
     * @return DateTime
     */
    public function getCreatedAt()
    {
        return $this->createdAt;
    }

    /**
     * @param DateTime $createdAt
     */
    public function setCreatedAt($createdAt)
    {
        $this->createdAt = $createdAt;
    }

    /**
     * @return DateTime
     */
    public function getUpdatedAt()
    {
        return $this->updatedAt;
    }

    /**
     * @param DateTime $updatedAt
     */
    public function setUpdatedAt($updatedAt)
    {
        $this->updatedAt = $updatedAt;
    }

    /**
     * @return mixed
     */
    public function getCreatedUser()
    {
        return $this->createdUser;
    }

    /**
     * @param mixed $createdUser
     */
    public function setCreatedUser($createdUser)
    {
        $this->createdUser = $createdUser;
    }

    /**
     * @return mixed
     */
    public function getUpdatedUser()
    {
        return $this->updatedUser;
    }

    /**
     * @param mixed $updatedUser
     */
    public function setUpdatedUser($updatedUser)
    {
        $this->updatedUser = $updatedUser;
    }

}

