<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use DateTime;

/**
 * Element
 *
 * @ORM\Table(name="element")
 * @ORM\Entity(repositoryClass="App\Repository\ElementRepository")
 * @ORM\HasLifecycleCallbacks
 */
class Element
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="avatar", type="string", nullable=true)
     */
    private $avatar;

    /**
     * @var string
     *
     * @ORM\Column(name="code", type="string", length=32, nullable=true)
     */
    private $code;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=50, nullable=true)
     */
    private $name;

    /**
     * @var string
     *
     * @ORM\Column(name="description", type="string", length=200, nullable=true)
     */
    private $description;

    /**
     * @var int
     *
     * @ORM\Column(name="additional_option", type="integer")
     */
    private $additionalOption;

    /**
     * @var int
     *
     * @ORM\Column(name="active", type="integer")
     */
    private $active;

    /**
     * @var int
     *
     * @ORM\Column(name="active_site", type="integer")
     */
    private $activeSite;

    /**
     * @var int
     *
     * @ORM\Column(name="hasSupp", type="integer")
     */
    private $hasSupp;

    /**
     * @var int
     *
     * @ORM\Column(name="custom", type="integer")
     */
    private $custom;

    /**
     * @var float
     *
     * @ORM\Column(name="vat", type="float")
     */
    private $vat;

    /**
     * @var int
     *
     * @ORM\Column(name="max_nbr_ing_options", type="integer")
     */
    private $maxNbrIngOptions;

    /**
     * @var int
     *
     * @ORM\Column(name="price_supp_app", type="integer")
     */
    private $priceSuppApp;

    /**
     * @var string
     *
     * @ORM\Column(name="created_ip", type="string", length=50)
     */
    private $createdIp;

    /**
     * @var int
     *
     * @ORM\Column(name="created_user", type="integer")
     */
    private $createdUser;


    /**
     * @var string
     *
     * @ORM\Column(name="updated_ip", type="string", length=50)
     */
    private $updatedIp;

    /**
     * @var int
     *
     * @ORM\Column(name="updated_user", type="integer")
     */
    private $updatedUser;

    /**
     * @var DateTime $created
     *
     * @ORM\Column(name="created_at", type="datetime", nullable=false)
     */
    protected $createdAt;
    /**
     * @var DateTime $updated
     *
     * @ORM\Column(name="updated_at", type="datetime", nullable=false)
     */
    protected $updatedAt;

    /**
     * @ORM\ManyToOne(targetEntity="CardFamily", inversedBy="elements",cascade={"persist"})
     * @ORM\JoinColumn(name="family_id", referencedColumnName="id", nullable=false,onDelete="cascade" )
     */
    private $family;

    /**
     * @ORM\ManyToOne(targetEntity="Card")
     * @ORM\JoinColumn(name="card_id", referencedColumnName="id", nullable=false, onDelete="cascade")
     */
    private $card;


    /**
     * @ORM\ManyToMany(targetEntity="Ingredient",cascade={"persist"})
     * @ORM\JoinTable(name="element_composition",
     *      inverseJoinColumns={@ORM\JoinColumn(name="ingredient_id", referencedColumnName="id", onDelete="cascade")},
     *      joinColumns={@ORM\JoinColumn(name="element_id", referencedColumnName="id", onDelete="cascade")}
     *      )
     */
    private $elementComposition;



    /**
     * @ORM\ManyToMany(targetEntity="Ingredient",cascade={"persist"})
     * @ORM\JoinTable(name="element_option_ingredient",
     *      inverseJoinColumns={@ORM\JoinColumn(name="ingredient_id", referencedColumnName="id", onDelete="cascade")},
     *      joinColumns={@ORM\JoinColumn(name="element_id", referencedColumnName="id", onDelete="cascade")}
     *      )
     */
    private $elementOption;

    /**
     * @ORM\ManyToMany(targetEntity="Ingredient",cascade={"persist"})
     * @ORM\JoinTable(name="element_supplement",
     *      inverseJoinColumns={@ORM\JoinColumn(name="ingredient_id", referencedColumnName="id", onDelete="cascade")},
     *      joinColumns={@ORM\JoinColumn(name="element_id", referencedColumnName="id", onDelete="cascade")}
     *      )
     */
    private $elementSupplement;

    /**
     * @ORM\OneToMany(targetEntity="ElementSize", mappedBy="element", cascade={"remove"})
     */
    private $elementSizes;

    /**
     * @ORM\OneToMany(targetEntity="ElementOrderType", mappedBy="element", cascade={"remove"})
     */
    private $elementOrderType;

    /**
     * @var int
     *
     * @ORM\Column(name="id_origine", type="integer", nullable=true)
     */
    private $idOrigine;

    /**
     * @return int
     */
    public function getIdOrigine()
    {
        return $this->idOrigine;
    }

    /**
     * @param int $idOrigine
     */
    public function setIdOrigine(int $idOrigine)
    {
        $this->idOrigine = $idOrigine;
    }




    /**
     * Pdv constructor.
     */
    public function __construct()
    {
        $this->elementComposition = new ArrayCollection();
        $this->elementOption = new ArrayCollection();
        $this->elementSupplement = new ArrayCollection();
        $this->elementSize = new ArrayCollection();
        $this->elementOrderType = new ArrayCollection();
    }


    public function addElementComposition(Ingredient $elementComposition)
    {
        $this->elementComposition[] = $elementComposition;
        return $this;
    }

    public function removeElementComposition(Ingredient $elementComposition)
    {
        $this->elementComposition->removeElement($elementComposition);
    }

    public function addElementOption(Ingredient $elementOption)
    {
        $this->elementOption[] = $elementOption;
        return $this;
    }

    public function removeElementOption(Ingredient $elementOption)
    {
        $this->elementOption->removeElement($elementOption);
    }

    public function addElementSupplement(Ingredient $elementSupplement)
    {
        $this->elementSupplement[] = $elementSupplement;
        return $this;
    }

    public function removeElementSupplement(Ingredient $elementSupplement)
    {
        $this->elementSupplement->removeElement($elementSupplement);
    }

    /**
     * @param mixed $card
     */
    public function setCard($card)
    {
        $this->card = $card;
    }



    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getCode()
    {
        return $this->code;
    }

    /**
     * @param string $code
     */
    public function setCode($code)
    {
        $this->code = $code;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return int
     */
    public function getAdditionalOption()
    {
        return $this->additionalOption;
    }

    /**
     * @param int $additionalOption
     */
    public function setAdditionalOption($additionalOption)
    {
        $this->additionalOption = $additionalOption;
    }

    /**
     * @return int
     */
    public function getActive()
    {
        return $this->active;
    }

    /**
     * @param int $active
     */
    public function setActive($active)
    {
        $this->active = $active;
    }


    /**
     * @return int
     */
    public function getMaxNbrIngOptions()
    {
        return $this->maxNbrIngOptions;
    }

    /**
     * @param int $maxNbrIngOtions
     */
    public function setMaxNbrIngOptions($maxNbrIngOptions)
    {
        $this->maxNbrIngOptions = $maxNbrIngOptions;
    }

    /**
     * @return int
     */
    public function getPriceSuppApp()
    {
        return $this->priceSuppApp;
    }

    /**
     * @param int $priceSuppApp
     */
    public function setPriceSuppApp($priceSuppApp)
    {
        $this->priceSuppApp = $priceSuppApp;
    }

    /**
     * @return float
     */
    public function getVat()
    {
        return $this->vat;
    }

    /**
     * @param float $vat
     */
    public function setVat($vat)
    {
        $this->vat = $vat;
    }


    /**
     * @return string
     */
    public function getCreatedIp()
    {
        return $this->createdIp;
    }

    /**
     * @param string $createdIp
     */
    public function setCreatedIp($createdIp)
    {
        $this->createdIp = $createdIp;
    }

    /**
     * @return int
     */
    public function getCreatedUser()
    {
        return $this->createdUser;
    }

    /**
     * @param int $createdUser
     */
    public function setCreatedUser($createdUser)
    {
        $this->createdUser = $createdUser;
    }

    /**
     * @return string
     */
    public function getUpdatedIp()
    {
        return $this->updatedIp;
    }

    /**
     * @param string $updatedIp
     */
    public function setUpdatedIp($updatedIp)
    {
        $this->updatedIp = $updatedIp;
    }

    /**
     * @return int
     */
    public function getUpdatedUser()
    {
        return $this->updatedUser;
    }

    /**
     * @param int $updatedUser
     */
    public function setUpdatedUser($updatedUser)
    {
        $this->updatedUser = $updatedUser;
    }

    /**
     * @return DateTime
     */
    public function getCreatedAt()
    {
        return $this->createdAt;
    }

    /**
     * @param DateTime $createdAt
     */
    public function setCreatedAt($createdAt)
    {
        $this->createdAt = $createdAt;
    }

    /**
     * @return DateTime
     */
    public function getUpdatedAt()
    {
        return $this->updatedAt;
    }

    /**
     * @param DateTime $updatedAt
     */
    public function setUpdatedAt($updatedAt)
    {
        $this->updatedAt = $updatedAt;
    }

    /**
     * @return mixed
     */
    public function getFamily()
    {
        return $this->family;
    }

    /**
     * @param mixed $family
     */
    public function setFamily($family)
    {
        $this->family = $family;
    }

    /**
     * @return mixed
     */
  /* public function getElementSizes()
    {
        return $this->elementSizes;
    }*/




    /**
     * @param ArrayCollection $elementSize
     */
    public function setElementSizes($elementSize)
    {
        $this->elementSizes = $elementSize;
    }

    /**
     * @return mixed
     */
/*    public function getElementComposition()
    {
        return $this->elementComposition;
    }*/

    /**
     * @param mixed $elementComposition
     */
    public function setElementComposition($elementComposition)
    {
        $this->elementComposition = $elementComposition;
    }

    /**
     * @return mixed
     */
/*    public function getElementOption()
    {
        return $this->elementOption;
    }*/

    /**
     * @param mixed $elementOption
     */
    public function setElementOption($elementOption)
    {
        $this->elementOption = $elementOption;
    }

    /**
     * @return mixed
     */
/*    public function getElementSupplement()
    {
        return $this->elementSupplement;
    }*/

    /**
     * @param mixed $elementSupplement
     */
    public function setElementSupplement($elementSupplement)
    {
        $this->elementSupplement = $elementSupplement;
    }


    /**
     * @ORM\PrePersist
     * @ORM\PreUpdate
     */
    public function updatedTimestamps()
    {
        $dateTimeNow = new DateTime('now');
        $this->setUpdatedAt($dateTimeNow);
        if ($this->getCreatedAt() === null) {
            $this->setCreatedAt($dateTimeNow);
        }
    }

    /**
     * @return mixed
     */
    public function getElementOrderType()
    {
        return $this->elementOrderType;
    }

    /**
     * @param mixed $elementOrderType
     */
    public function setElementOrderType($elementOrderType)
    {
        $this->elementOrderType = $elementOrderType;
    }

    /**
     * @return string
     */
    public function getAvatar()
    {
        return $this->avatar;
    }

    /**
     * @param string $avatar
     */
    public function setAvatar($avatar)
    {
        $this->avatar = $avatar;
    }

    /**
     * @return int
     */
    public function getHasSupp()
    {
        return $this->hasSupp;
    }

    /**
     * @param int $hasSupp
     */
    public function setHasSupp($hasSupp)
    {
        $this->hasSupp = $hasSupp;
    }

    /**
     * @return int
     */
    public function getCustom()
    {
        return $this->custom;
    }

    /**
     * @param int $custom
     */
    public function setCustom($custom)
    {
        $this->custom = $custom;
    }

    /**
     * @return int
     */
    public function getActiveSite(): int
    {
        return $this->activeSite;
    }

    /**
     * @param int $activeSite
     */
    public function setActiveSite(int $activeSite)
    {
        $this->activeSite = $activeSite;
    }
}

