<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use DateTime;

/**
 * Ingredient
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="ingredient")
 * @ORM\Entity(repositoryClass="App\Repository\IngredientRepository")
 */
class Ingredient
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=100, nullable=true)
     */
    private $name;

    /**
     * @var string
     *
     * @ORM\Column(name="description", type="string", length=600, nullable=true)
     */
    private $description;



    /**
     * @var decimal
     *
     * @ORM\Column(name="price", type="decimal", precision=10, scale=2)
     */
    private $price;

    /**
     * @var string
     *
     * @ORM\Column(name="avatar", type="string", nullable=true)
     */
    private $avatar;

    /**
     * @var int
     *
     * @ORM\Column(name="available", type="integer")
     */
    private $available;

    /**
     * @var int
     *
     * @ORM\Column(name="active_site", type="integer")
     */
    private $activeSite;

    /**
     * @var string
     *
     * @ORM\Column(name="created_ip", type="string", length=50)
     */
    private $createdIp;

    /**
     * @var int
     *
     * @ORM\Column(name="created_user", type="integer")
     */
    private $createdUser;


    /**
     * @var string
     *
     * @ORM\Column(name="updated_ip", type="string", length=50)
     */
    private $updatedIp;

    /**
     * @var int
     *
     * @ORM\Column(name="updated_user", type="integer")
     */
    private $updatedUser;

    /**
     * @var DateTime $created
     *
     * @ORM\Column(name="created_at", type="datetime", nullable=false)
     */
    protected $createdAt;

    /**
     * @var DateTime $updated
     *
     * @ORM\Column(name="updated_at", type="datetime", nullable=false)
     */
    protected $updatedAt;

    /**
     * @var int
     *
     * @ORM\Column(name="id_origine", type="integer", nullable=true)
     */
    private $idOrigine;

    /**
     * @return int
     */
    public function getIdOrigine()
    {
        return $this->idOrigine;
    }

    /**
     * @param int $idOrigine
     */
    public function setIdOrigine(int $idOrigine)
    {
        $this->idOrigine = $idOrigine;
    }




    /**
     * @ORM\PrePersist
     * @ORM\PreUpdate
     */
    public function updatedTimestamps()
    {
        $dateTimeNow = new DateTime('now');
        $this->setUpdatedAt($dateTimeNow);
        if ($this->getCreatedAt() === null) {
            $this->setCreatedAt($dateTimeNow);
        }
    }

    /**
     * @ORM\ManyToMany(targetEntity="CardFamily", inversedBy="ingredients",cascade={"persist"})
     * @ORM\JoinTable(name="ingredient_card_family")
     */
    private $families;

    public function __construct()
    {
        $this->families = new ArrayCollection();
    }

    public function addFamily(CardFamily $famille)
    {
        $this->families[] = $famille;
        return $this;
    }

    /**
     * @ORM\ManyToOne(targetEntity="IngredientCategory")
     * @ORM\JoinColumn(name="ingredient_category_id", referencedColumnName="id")
     */
    private $ingredientCategory;

    /**
     * @ORM\ManyToOne(targetEntity="Card")
     * @ORM\JoinColumn(name="card_id", referencedColumnName="id", onDelete="cascade")
     */
    private $card;


    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return string
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * @param string $price
     */
    public function setPrice($price)
    {
        $this->price = $price;
    }

    /**
     * @return int
     */
    public function getAvailable()
    {
        return $this->available;
    }

    /**
     * @param int $available
     */
    public function setAvailable($available)
    {
        $this->available = $available;
    }

    /**
     * @return string
     */
    public function getCreatedIp()
    {
        return $this->createdIp;
    }

    /**
     * @param string $createdIp
     */
    public function setCreatedIp($createdIp)
    {
        $this->createdIp = $createdIp;
    }

    /**
     * @return int
     */
    public function getCreatedUser()
    {
        return $this->createdUser;
    }

    /**
     * @param int $createdUser
     */
    public function setCreatedUser($createdUser)
    {
        $this->createdUser = $createdUser;
    }

    /**
     * @return string
     */
    public function getUpdatedIp()
    {
        return $this->updatedIp;
    }

    /**
     * @param string $updatedIp
     */
    public function setUpdatedIp($updatedIp)
    {
        $this->updatedIp = $updatedIp;
    }

    /**
     * @return int
     */
    public function getUpdatedUser()
    {
        return $this->updatedUser;
    }

    /**
     * @param int $updatedUser
     */
    public function setUpdatedUser($updatedUser)
    {
        $this->updatedUser = $updatedUser;
    }

    /**
     * @return DateTime
     */
    public function getCreatedAt()
    {
        return $this->createdAt;
    }

    /**
     * @param DateTime $createdAt
     */
    public function setCreatedAt($createdAt)
    {
        $this->createdAt = $createdAt;
    }

    /**
     * @return DateTime
     */
    public function getUpdatedAt()
    {
        return $this->updatedAt;
    }

    /**
     * @param DateTime $updatedAt
     */
    public function setUpdatedAt($updatedAt)
    {
        $this->updatedAt = $updatedAt;
    }

    /**
     * @return mixed
     */
    public function getFamilies()
    {
        return $this->families;
    }

    /**
     * @param mixed $families
     */
    public function setFamilies($families)
    {
        $this->families = $families;
    }

    /**
     * @return mixed
     */
    public function getIngredientCategory()
    {
        return $this->ingredientCategory;
    }

    /**
     * @param mixed $ingredientCategory
     */
    public function setIngredientCategory($ingredientCategory)
    {
        $this->ingredientCategory = $ingredientCategory;
    }

    /**
     * @return mixed
     */
  /*  public function getCard()
    {
        return $this->card;
    }*/

    /**
     * @param mixed $card
     */
    public function setCard($card)
    {
        $this->card = $card;
    }

    /**
     * @return string
     */
    public function getAvatar()
    {
        return $this->avatar;
    }

    /**
     * @param string $avatar
     */
    public function setAvatar($avatar)
    {
        $this->avatar = $avatar;
    }

    /**
     * @return int
     */
    public function getActiveSite()
    {
        return $this->activeSite;
    }

    /**
     * @param int $activeSite
     */
    public function setActiveSite($activeSite)
    {
        $this->activeSite = $activeSite;
    }
}

