<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use DateTime;

/**
 * Menu
 *
 * @ORM\Table(name="menu")
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class Menu
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="avatar", type="string", nullable=true)
     */
    private $avatar;

    /**
     * @var string
     *
     * @ORM\Column(name="code", type="string", length=32, nullable=true)
     */
    private $code;

    /**
     * @var string
     *
     * @ORM\Column(name="title", type="string", length=200, nullable=false)
     */
    private $title;

    /**
     * @var text
     *
     * @ORM\Column(name="description", type="text", nullable=true)
     */
    private $description;

    /**
     * @var int
     *
     * @ORM\Column(name="available", type="integer", nullable=false)
     */
    private $available;


    /**
     * @var float
     *
     * @ORM\Column(name="vat", type="float", nullable=false)
     */
    private $vat;


    /**
     * @var float
     * @ORM\Column(name="price", type="decimal", scale=2,precision= 10)
     */
    private $price;

    /**
     * @var int
     *
     * @ORM\Column(name="active_site", type="integer")
     */
    private $activeSite;

    /**
     * @var string
     *
     * @ORM\Column(name="created_ip", type="string", length=50)
     */
    private $createdIp;

    /**
     * @var int
     *
     * @ORM\Column(name="created_user", type="integer")
     */
    private $createdUser;


    /**
     * @var string
     *
     * @ORM\Column(name="updated_ip", type="string", length=50)
     */
    private $updatedIp;

    /**
     * @var int
     *
     * @ORM\Column(name="updated_user", type="integer")
     */
    private $updatedUser;

    /**
     * @var DateTime $created
     *
     * @ORM\Column(name="created_at", type="datetime", nullable=false)
     */
    protected $createdAt;
    /**
     * @var DateTime $updated
     *
     * @ORM\Column(name="updated_at", type="datetime", nullable=false)
     */
    protected $updatedAt;


    /**
     * @ORM\ManyToOne(targetEntity="Card")
     * @ORM\JoinColumn(name="card_id", referencedColumnName="id", nullable=false, onDelete="cascade")
     */
    private $card;


    /**
     * @ORM\ManyToOne(targetEntity="MenuTime",cascade={"persist"})
     * @ORM\JoinColumn(name="menu_time_id", referencedColumnName="id", nullable=false)
     */
    private $menuTime;


    /**
     * @ORM\ManyToOne(targetEntity="MenuCategory")
     * @ORM\JoinColumn(name="menu_category_id", referencedColumnName="id", nullable=false)
     */
    private $menuCategory;

    /**
     * @ORM\ManyToOne(targetEntity="CardOffer" ,cascade={"persist","remove"})
     * @ORM\JoinColumn(name="card_offer_id", referencedColumnName="id", nullable=true, onDelete="cascade")
     */
    private $offer;

    /**
     * @ORM\OneToMany(targetEntity="MenuOrderType", mappedBy="menu", cascade={"remove"})
     */
    private $menuOrderType;

    /**
     * @ORM\ManyToMany(targetEntity="ElementSize")
     * @ORM\JoinTable(name="menu_exclusion",
     *      inverseJoinColumns={@ORM\JoinColumn(name="element_size_id", referencedColumnName="id", onDelete="cascade")},
     *      joinColumns={@ORM\JoinColumn(name="menu_id", referencedColumnName="id", onDelete="cascade")}
     *      )
     */
    private $excludedProducts;

    /**
     * @ORM\OneToMany(targetEntity="MenuOption", mappedBy="menu", cascade={"remove"})
     */
    private $options;

    /**
     * @ORM\OneToMany(targetEntity="MenuElementSize", mappedBy="menu", cascade={"remove"})
     */
    private $elements;

    /**
     * @ORM\OneToMany(targetEntity="MenuFamilySize", mappedBy="menu", cascade={"remove"})
     */
    private $families;

    /**
     * @var int
     *
     * @ORM\Column(name="id_origine", type="integer", nullable=true)
     */
    private $idOrigine;

    /**
     * @return int
     */
    public function getIdOrigine()
    {
        return $this->idOrigine;
    }

    /**
     * @param int $idOrigine
     */
    public function setIdOrigine(int $idOrigine)
    {
        $this->idOrigine = $idOrigine;
    }



    public function addExcludedProduct(ElementSize $excludedProduct)
    {
        $this->excludedProducts[] = $excludedProduct;
        return $this;
    }

    public function removeExcludedProduct(ElementSize $excludedProduct)
    {
        $this->excludedProducts->removeElement($excludedProduct);
    }

    /**
     * @param mixed $excludedProducts
     */
    public function setExcludedProducts($excludedProducts)
    {
        $this->excludedProducts = $excludedProducts;
    }


    public function getExcludedProduct()
    {
        return $this->excludedProducts;
    }


    public function __construct()
    {
        $this->excludedProducts = new ArrayCollection();
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getCode()
    {
        return $this->code;
    }

    /**
     * @param string $code
     */
    public function setCode($code)
    {
        $this->code = $code;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return int
     */
    public function getAvailable()
    {
        return $this->available;
    }

    /**
     * @param int $available
     */
    public function setAvailable($available)
    {
        $this->available = $available;
    }


    /**
     * @return float
     */
    public function getVat()
    {
        return $this->vat;
    }

    /**
     * @param float $vat
     */
    public function setVat($vat)
    {
        $this->vat = $vat;
    }

    /**
     * @return int
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * @param int $price
     */
    public function setPrice($price)
    {
        $this->price = $price;
    }

    /**
     * @return string
     */
    public function getCreatedIp()
    {
        return $this->createdIp;
    }

    /**
     * @param string $createdIp
     */
    public function setCreatedIp($createdIp)
    {
        $this->createdIp = $createdIp;
    }

    /**
     * @return int
     */
    public function getCreatedUser()
    {
        return $this->createdUser;
    }

    /**
     * @param int $createdUser
     */
    public function setCreatedUser($createdUser)
    {
        $this->createdUser = $createdUser;
    }

    /**
     * @return string
     */
    public function getUpdatedIp()
    {
        return $this->updatedIp;
    }

    /**
     * @param string $updatedIp
     */
    public function setUpdatedIp($updatedIp)
    {
        $this->updatedIp = $updatedIp;
    }

    /**
     * @return int
     */
    public function getUpdatedUser()
    {
        return $this->updatedUser;
    }

    /**
     * @param int $updatedUser
     */
    public function setUpdatedUser($updatedUser)
    {
        $this->updatedUser = $updatedUser;
    }

    /**
     * @return DateTime
     */
    public function getCreatedAt()
    {
        return $this->createdAt;
    }

    /**
     * @param DateTime $createdAt
     */
    public function setCreatedAt($createdAt)
    {
        $this->createdAt = $createdAt;
    }

    /**
     * @return DateTime
     */
    public function getUpdatedAt()
    {
        return $this->updatedAt;
    }

    /**
     * @param DateTime $updatedAt
     */
    public function setUpdatedAt($updatedAt)
    {
        $this->updatedAt = $updatedAt;
    }

    /**
     * @return mixed
     */
    /*public function getCard()
    {
        return $this->card;
    }*/

    /**
     * @param mixed $card
     */
    public function setCard($card)
    {
        $this->card = $card;
    }

    /**
     * @return mixed
     */
    public function getMenuTime()
    {
        return $this->menuTime;
    }

    /**
     * @param mixed $menuTime
     */
    public function setMenuTime($menuTime)
    {
        $this->menuTime = $menuTime;
    }

    /**
     * @return mixed
     */
    public function getOffer()
    {
        return $this->offer;
    }

    /**
     * @param mixed $offer
     */
    public function setOffer($offer)
    {
        $this->offer = $offer;
    }

    /**
     * @return mixed
     */
    public function getMenuCategory()
    {
        return $this->menuCategory;
    }

    /**
     * @param mixed $menuCategory
     */
    public function setMenuCategory($menuCategory)
    {
        $this->menuCategory = $menuCategory;
    }

    /**
     * @return mixed
     */
    public function getMenuOrderType()
    {
        return $this->menuOrderType;
    }

    /**
     * @param mixed $menuOrderType
     */
    public function setMenuOrderType($menuOrderType)
    {
        $this->menuOrderType = $menuOrderType;
    }

    /**
     * @ORM\PrePersist
     * @ORM\PreUpdate
     */
    public function updatedTimestamps()
    {
        $dateTimeNow = new DateTime('now');
        $this->setUpdatedAt($dateTimeNow);
        if ($this->getCreatedAt() === null) {
            $this->setCreatedAt($dateTimeNow);
        }
    }

    /**
     * @return string
     */
    public function getAvatar()
    {
        return $this->avatar;
    }

    /**
     * @param string $avatar
     */
    public function setAvatar($avatar)
    {
        $this->avatar = $avatar;
    }

    /**
     * @return int
     */
    public function getActiveSite(): int
    {
        return $this->activeSite;
    }

    /**
     * @param int $activeSite
     */
    public function setActiveSite(int $activeSite)
    {
        $this->activeSite = $activeSite;
    }



   /**
     * @param mixed $elements
     */
    public function setElements($elements)
    {
        $this->elements = $elements;
    }



       /**
     * @param mixed $families
     */
    public function setFamilies($families)
    {
        $this->families = $families;
    }


}

