<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use DateTime;

/**
 * Reservation
 *
 * @ORM\Table(name="reservation")
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 *
 */
class Reservation
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var datetime
     *
     * @ORM\Column(name="datetime", type="datetime")
     */
    private $datetime;

    /**
     * @var string
     *
     * @ORM\Column(name="comment", type="text",nullable=true)
     */
    private $comment;

    /**
     * @var int
     *
     * @ORM\Column(name="nbrPerson", type="integer")
     */
    private $nbrPerson;

    /**
     * @ORM\ManyToOne(targetEntity="CustomerPdv")
     * @ORM\JoinColumn(name="customer_pdv_id", referencedColumnName="id", nullable=false, onDelete="cascade")
     */
    private $customerPdv;

    /**
     * @ORM\ManyToOne(targetEntity="Service")
     */
    private $service;

    /**
     * @ORM\ManyToMany(targetEntity="Bench")
     * @ORM\JoinTable(name="reservation_table",
     *      joinColumns={@ORM\JoinColumn(name="reservation_id", referencedColumnName="id", onDelete="cascade")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="table_id", referencedColumnName="id", onDelete="cascade")}
     *      )
     */
    private $tables;

    /**
     * @var string
     *
     * @ORM\Column(name="created_ip", type="string", length=50)
     */
    private $createdIp;

    /**
     * @ORM\ManyToOne(targetEntity="User")
     * @ORM\JoinColumn(name="createdUser", referencedColumnName="id", nullable=true)
     */
    private $createdUser;


    /**
     * @var string
     *
     * @ORM\Column(name="updated_ip", type="string", length=50)
     */
    private $updatedIp;


    /**
     * @ORM\ManyToOne(targetEntity="User")
     * @ORM\JoinColumn(name="updatedUser", referencedColumnName="id", nullable=true)
     */
    private $updatedUser;

    /**
     * @var DateTime $created
     *
     * @ORM\Column(name="created_at", type="datetime", nullable=false)
     */
    protected $createdAt;
    /**
     * @var DateTime $updated
     *
     * @ORM\Column(name="updated_at", type="datetime", nullable=false)
     */
    protected $updatedAt;

    /**
     * @ORM\PrePersist
     * @ORM\PreUpdate
     */
    public function updatedTimestamps()
    {
        $dateTimeNow = new DateTime('now');
        $this->setUpdatedAt($dateTimeNow);
        if ($this->getCreatedAt() === null) {
            $this->setCreatedAt($dateTimeNow);
        }
    }

    /**
     * @return int
     */
    public function getId(): int
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId(int $id)
    {
        $this->id = $id;
    }

    /**
     * @return DateTime
     */
    public function getDatetime(): DateTime
    {
        return $this->datetime;
    }

    /**
     * @param DateTime $datetime
     */
    public function setDatetime(DateTime $datetime)
    {
        $this->datetime = $datetime;
    }

    /**
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }

    /**
     * @param string $comment
     */
    public function setComment($comment)
    {
        $this->comment = $comment;
    }

    /**
     * @return int
     */
    public function getNbrPerson(): int
    {
        return $this->nbrPerson;
    }

    /**
     * @param int $nbrPerson
     */
    public function setNbrPerson(int $nbrPerson)
    {
        $this->nbrPerson = $nbrPerson;
    }


    /**
     * @param mixed $customerPdv
     */
    public function setCustomerPdv($customerPdv)
    {
        $this->customerPdv = $customerPdv;
    }

    /**
     * @return mixed
     */
    public function getService()
    {
        return $this->service;
    }

    /**
     * @param mixed $service
     */
    public function setService($service)
    {
        $this->service = $service;
    }

    /**
     * @return string
     */
    public function getCreatedIp(): string
    {
        return $this->createdIp;
    }

    /**
     * @param string $createdIp
     */
    public function setCreatedIp(string $createdIp)
    {
        $this->createdIp = $createdIp;
    }


    /**
     * @return string
     */
    public function getUpdatedIp(): string
    {
        return $this->updatedIp;
    }

    /**
     * @param string $updatedIp
     */
    public function setUpdatedIp(string $updatedIp)
    {
        $this->updatedIp = $updatedIp;
    }


    /**
     * @return DateTime
     */
    public function getCreatedAt()
    {
        return $this->createdAt;
    }

    /**
     * @param DateTime $createdAt
     */
    public function setCreatedAt(DateTime $createdAt)
    {
        $this->createdAt = $createdAt;
    }

    /**
     * @return DateTime
     */
    public function getUpdatedAt()
    {
        return $this->updatedAt;
    }

    /**
     * @param DateTime $updatedAt
     */
    public function setUpdatedAt(DateTime $updatedAt)
    {
        $this->updatedAt = $updatedAt;
    }

    /**
     * @return mixed
     */
    public function getCreatedUser()
    {
        return $this->createdUser;
    }

    /**
     * @param mixed $createdUser
     */
    public function setCreatedUser($createdUser)
    {
        $this->createdUser = $createdUser;
    }

    /**
     * @return mixed
     */
    public function getUpdatedUser()
    {
        return $this->updatedUser;
    }

    /**
     * @param mixed $updatedUser
     */
    public function setUpdatedUser($updatedUser)
    {
        $this->updatedUser = $updatedUser;
    }

    /**
     * @return mixed
     */
    public function getTables()
    {
        $benchs=array();
        foreach ($this->tables as $b){
            $benchs[]=$b->getNum();
        }
        return $benchs;
    }

    /**
     * @param mixed $tables
     */
    public function setTables($tables)
    {
        $this->tables = $tables;
    }
}

