<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;

/**
 * Size
 *
 * @ORM\Table(name="size")
 * @ORM\Entity(repositoryClass="App\Repository\SizeRepository")
 */
class Size
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="label", type="string", length=200)
     */
    private $label;

    /**
     * @var int
     *
     * @ORM\Column(name="level", type="integer")
     */
    private $level;

    /**
     * @ORM\ManyToOne(targetEntity="Card")
     * @ORM\JoinColumn(name="card_id", referencedColumnName="id", onDelete="cascade")
     */
    private $card;

    /**
     * @ORM\ManyToMany(targetEntity="CardFamily", inversedBy="sizes",cascade={"persist"})
     * @ORM\JoinTable(name="size_card_family")
     */
    private $families;

    /**
     * @ORM\OneToMany(targetEntity="IngredientCatFamSize", mappedBy="size", cascade={"remove"})
     */
    private $ingredientCatFamSizes;

    /**
     * @ORM\OneToMany(targetEntity="MenuFamilySize", mappedBy="size", cascade={"remove"})
     */
    private $menuFamilySizes;

    /**
     * @ORM\OneToMany(targetEntity="ElementSize", mappedBy="size", cascade={"remove"})
     */
    private $elementSizes;

    /**
     * @var int
     *
     * @ORM\Column(name="id_origine", type="integer", nullable=true)
     */
    private $idOrigine;

    /**
     * @return int
     */
    public function getIdOrigine()
    {
        return $this->idOrigine;
    }

    /**
     * @param int $idOrigine
     */
    public function setIdOrigine(int $idOrigine)
    {
        $this->idOrigine = $idOrigine;
    }



    public function __construct()
    {
        $this->families = new ArrayCollection();
    }

    public function addFamily(CardFamily $family)
    {
        $this->families[] = $family;
        return $this;
    }

    public function removeFamily(CardFamily $family)
    {
        $this->families->removeElement($family);
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->label;
    }

    /**
     * @param string $label
     */
    public function setLabel($label)
    {
        $this->label = $label;
    }

    /**
     * @return int
     */
    public function getLevel()
    {
        return $this->level;
    }

    /**
     * @param int $level
     */
    public function setLevel($level)
    {
        $this->level = $level;
    }

    /**
     * @return mixed
     */
  /*  public function getCard()
    {
        return $this->card;
    }*/

    /**
     * @param mixed $card
     */
    public function setCard($card)
    {
        $this->card = $card;
    }

    /**
     * @return mixed
     */
    public function getFamilies()
    {
        return $this->families;
    }

    /**
     * @param mixed $families
     */
    public function setFamilies($families)
    {
        $this->families = $families;
    }


}

