<?php

namespace App\Repository;

/**
 * ElementRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class ElementRepository extends \Doctrine\ORM\EntityRepository
{
    public function getActiveElementsByFamilyCard($idFamily, $idCard)
    {
        $em = $this->getEntityManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("e.id,e.hasSupp,e.custom,e.avatar,e.code,e.name,e.active,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,e.priceSuppApp")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->where('cf.id = :cardFamily');
        if (!is_null($idCard)) {
            $elements = $elements->innerJoin('e.card', 'c')
                ->andWhere('c.id = :card')
                ->setParameter('card', $idCard);
        }
        $elements = $elements->andWhere('e.active = :active')
            ->setParameter('cardFamily', $idFamily)
            ->setParameter('active', 1)
            ->getQuery()
            ->getResult();
        return $elements;
    }

    public function getActiveElementsByFamilyCardAndOrderType($idFamily, $idOrderType, $idCard, $site)
    {
        $em = $this->getEntityManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("
            e.id,e.hasSupp,e.custom,e.code,e.name,eot.supplement,e.active,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp,e.avatar")
            ->from("App:Element", "e")
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->where('cf.id = :cardFamily');
        if (!is_null($idCard)) {
            $elements = $elements->innerJoin('e.card', 'c')
                ->where('c.id = :card')
                ->setParameter('card', $idCard);
        }
        $elements = $elements->andWhere('e.active=1');
        if ($site) {
            $elements = $elements->andWhere('e.activeSite=1');
        }
        $elements = $elements->andWhere('ot.id=:orderType')
            ->andWhere('eot.active=1')
            ->setParameter('cardFamily', $idFamily)
            ->setParameter('orderType', $idOrderType)
            ->getQuery()
            ->getResult();
        return $elements;
    }

    public function getActiveElementsAndKey($key, $idOrderType, $idCarte)
    {
        $em = $this->getEntityManager();
        $qb = $em->createQueryBuilder();
        $elements = $qb->select("e.id,e.hasSupp,e.custom,e.code,e.name,eot.supplement,e.active,e.vat, 1 as qty,e.description,e.maxNbrIngOptions,cf.id as family,f.label as familyName,e.priceSuppApp,e.avatar")
            ->from("App:Element", "e")
            ->innerJoin('e.card', 'c')
            ->innerJoin('e.family', 'cf')
            ->innerJoin('cf.family', 'f')
            ->innerJoin('e.elementOrderType', 'eot')
            ->innerJoin('eot.orderType', 'ot')
            ->where('e.name LIKE :key')
            ->andWhere('e.active=:active')
            ->andWhere('ot.id=:orderType')
            ->andWhere('eot.active=:active')
            ->andWhere('c.id=:card')
            ->setParameter('key', '%' . $key . '%')
            ->setParameter('orderType', $idOrderType)
            ->setParameter('card', $idCarte)
            ->setParameter('active', 1)
            ->getQuery()
            ->getResult();
        return $elements;
    }

    public function getMenusByKey($key, $idOrderType, $idCarte)
    {
        $em = $this->getEntityManager();
        $qb = $em->createQueryBuilder();
        $menus = $qb->select("m.id,mot.supplement,m.avatar,m.code,m.vat,m.title,m.description,m.price,m.available,mt.id as menuTime,c.name as category,o.id as offer,
            case when mt.id = 1 then false else true end as validPeriod");
        $menus = $menus->from("App:Menu", "m")
            ->innerJoin('m.menuCategory', 'c')
            ->leftJoin('m.offer', 'co')
            ->leftJoin('co.offer', 'o')
            ->innerJoin('m.menuTime', 'mt')
            ->innerJoin('m.card', 'ca')
            ->innerJoin('m.menuOrderType', 'mot')
            ->innerJoin('mot.orderType', 'ot')
            ->where('ca.id = :card')
            ->andWhere('ot.id=:orderType')
            ->andWhere('m.available=1')
            ->andWhere('m.title LIKE :key or m.code LIKE :key')
            ->setParameter('card', $idCarte)
            ->setParameter('orderType', $idOrderType)
            ->setParameter('key', '%' . $key . '%')
            ->groupBy('m.id')
            ->getQuery()->getResult();
        foreach ($menus as $key => $menu){
            $menus[$key]['compositions']=$this->getCompositionsMenu($menu['id']);
        }
        return $menus;
    }
    public function getCompositionsMenu($id)
    {
        $elements = array();
        $families = array();
        for ($i = 0; $i < 6; $i++) {
            $em = $this->getEntityManager();
            $qb = $em->createQueryBuilder();
            $query = $qb->select("e.name,s.label as size")
                ->from("App:Menu", "m")
                ->innerJoin('m.elements', 'mes')
                ->innerJoin('mes.elementSize', 'es')
                ->innerJoin('es.element', 'e')
                ->innerJoin('es.size', 's')
                ->where('mes.menu = :idMenu')
                ->andWhere('mes.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            $elements[$i] = $results;
            $qb2 = $em->createQueryBuilder();
            $query = $qb2->select("f.label,s.label as size")
                ->from("App:Menu", "m")
                ->leftJoin('m.families', 'mfs')
                ->leftJoin('mfs.cardFamily', 'cf')
                ->leftJoin('cf.family', 'f')
                ->leftJoin('mfs.size', 's')
                ->where('mfs.menu = :idMenu')
                ->andWhere('mfs.choice = :choice')
                ->setParameter('idMenu', $id)
                ->setParameter('choice', $i)
                ->getQuery();
            $results = $query->getResult();
            $families[$i] = $results;
        }
        $data = array('elements' => $elements, 'families' => $families);
        return $data;
    }
}